/* SourceList.c generated by valac 0.56.14, the Vala compiler
 * generated from SourceList.vala, do not modify */

/*
 * Copyright 2019 elementary, Inc. (https://elementary.io)
 * Copyright 2012-2014 Victor Martinez <victoreduardm@gmail.com>
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */
/**
     * = WORKING INTERNALS =
     *
     * In order to offer a transparent Item-based API, and avoid the need of providing methods
     * to deal with items directly on the SourceList widget, it was decided to follow a monitor-like
     * implementation, where the source list permanently monitors its root item and any other
     * child item added to it. The task of monitoring the properties of the items has been
     * divided among different objects, as shown below:
     *
     * Monitored by: Object::method that receives the signals indicating the property change.
     * Applied by: Object::method that actually updates the tree to reflect the property changes
     *             (directly or indirectly, as in the case of the tree data model).
     *
     * ---------------------------------------------------------------------------------------------
     *   PROPERTY        |  MONITORED BY                     |  APPLIED BY
     * ---------------------------------------------------------------------------------------------
     * + Item            |                                   |
     *   - parent        | Not monitored                     | N/A
     *   - name          | DataModel::on_item_prop_changed   | Tree::name_cell_data_func
     *   - editable      | DataModel::on_item_prop_changed   | Queried when needed (See Tree::start_editing_item)
     *   - visible       | DataModel::on_item_prop_changed   | DataModel::filter_visible_func
     *   - icon          | DataModel::on_item_prop_changed   | Tree::icon_cell_data_func
     *   - activatable   | Same as @icon                     | Same as @icon
     * + ExpandableItem  |                                   |
     *   - collapsible   | DataModel::on_item_prop_changed   | Tree::update_expansion
     *                   |                                   | Tree::expander_cell_data_func
     *   - expanded      | Same as @collapsible              | Same as @collapsible
     * ---------------------------------------------------------------------------------------------
     * * Only automatic properties are monitored. ExpandableItem's additions/removals are handled by
     *   DataModel::add_item() and DataModel::remove_item()
     *
     * Other features:
     * - Sorting: this happens on the tree-model level (DataModel).
     */

#include <glib-object.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <pango/pango.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>
#include <cairo-gobject.h>
#include <float.h>
#include <math.h>
#include <granite.h>

#define MAIL_SOURCE_LIST_DATA_MODEL_ITEM_PARENT_NEEDS_UPDATE "item-parent-needs-update"
#define MAIL_SOURCE_LIST_TREE_DEFAULT_STYLESHEET "\n" \
"            .sidebar.badge {\n" \
"                border-radius: 10px;\n" \
"                border-width: 0;\n" \
"                padding: 1px 2px 1px 2px;\n" \
"                font-weight: bold;\n" \
"            }\n" \
"        "
#define MAIL_SOURCE_LIST_TREE_STYLE_PROP_LEVEL_INDENTATION "level-indentation"
#define MAIL_SOURCE_LIST_TREE_STYLE_PROP_LEFT_PADDING "left-padding"
#define MAIL_SOURCE_LIST_TREE_STYLE_PROP_EXPANDER_SPACING "expander-spacing"
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define MAIL_TYPE_SOURCE_LIST_SORTABLE (mail_source_list_sortable_get_type ())
#define MAIL_SOURCE_LIST_SORTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_TYPE_SOURCE_LIST_SORTABLE, MailSourceListSortable))
#define MAIL_IS_SOURCE_LIST_SORTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_TYPE_SOURCE_LIST_SORTABLE))
#define MAIL_SOURCE_LIST_SORTABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), MAIL_TYPE_SOURCE_LIST_SORTABLE, MailSourceListSortableIface))

typedef struct _MailSourceListSortable MailSourceListSortable;
typedef struct _MailSourceListSortableIface MailSourceListSortableIface;

#define MAIL_SOURCE_LIST_TYPE_ITEM (mail_source_list_item_get_type ())
#define MAIL_SOURCE_LIST_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_SOURCE_LIST_TYPE_ITEM, MailSourceListItem))
#define MAIL_SOURCE_LIST_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_SOURCE_LIST_TYPE_ITEM, MailSourceListItemClass))
#define MAIL_SOURCE_LIST_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_SOURCE_LIST_TYPE_ITEM))
#define MAIL_SOURCE_LIST_IS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_SOURCE_LIST_TYPE_ITEM))
#define MAIL_SOURCE_LIST_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_SOURCE_LIST_TYPE_ITEM, MailSourceListItemClass))

typedef struct _MailSourceListItem MailSourceListItem;
typedef struct _MailSourceListItemClass MailSourceListItemClass;

#define MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM (mail_source_list_expandable_item_get_type ())
#define MAIL_SOURCE_LIST_EXPANDABLE_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, MailSourceListExpandableItem))
#define MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, MailSourceListExpandableItemClass))
#define MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM))
#define MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM))
#define MAIL_SOURCE_LIST_EXPANDABLE_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, MailSourceListExpandableItemClass))

typedef struct _MailSourceListExpandableItem MailSourceListExpandableItem;
typedef struct _MailSourceListExpandableItemClass MailSourceListExpandableItemClass;
enum  {
	MAIL_SOURCE_LIST_SORTABLE_USER_MOVED_ITEM_SIGNAL,
	MAIL_SOURCE_LIST_SORTABLE_NUM_SIGNALS
};
static guint mail_source_list_sortable_signals[MAIL_SOURCE_LIST_SORTABLE_NUM_SIGNALS] = {0};

#define MAIL_TYPE_SOURCE_LIST_DRAG_SOURCE (mail_source_list_drag_source_get_type ())
#define MAIL_SOURCE_LIST_DRAG_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_TYPE_SOURCE_LIST_DRAG_SOURCE, MailSourceListDragSource))
#define MAIL_IS_SOURCE_LIST_DRAG_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_TYPE_SOURCE_LIST_DRAG_SOURCE))
#define MAIL_SOURCE_LIST_DRAG_SOURCE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), MAIL_TYPE_SOURCE_LIST_DRAG_SOURCE, MailSourceListDragSourceIface))

typedef struct _MailSourceListDragSource MailSourceListDragSource;
typedef struct _MailSourceListDragSourceIface MailSourceListDragSourceIface;

#define MAIL_TYPE_SOURCE_LIST_DRAG_DEST (mail_source_list_drag_dest_get_type ())
#define MAIL_SOURCE_LIST_DRAG_DEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_TYPE_SOURCE_LIST_DRAG_DEST, MailSourceListDragDest))
#define MAIL_IS_SOURCE_LIST_DRAG_DEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_TYPE_SOURCE_LIST_DRAG_DEST))
#define MAIL_SOURCE_LIST_DRAG_DEST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), MAIL_TYPE_SOURCE_LIST_DRAG_DEST, MailSourceListDragDestIface))

typedef struct _MailSourceListDragDest MailSourceListDragDest;
typedef struct _MailSourceListDragDestIface MailSourceListDragDestIface;

#define MAIL_TYPE_SOURCE_LIST (mail_source_list_get_type ())
#define MAIL_SOURCE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_TYPE_SOURCE_LIST, MailSourceList))
#define MAIL_SOURCE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_TYPE_SOURCE_LIST, MailSourceListClass))
#define MAIL_IS_SOURCE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_TYPE_SOURCE_LIST))
#define MAIL_IS_SOURCE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_TYPE_SOURCE_LIST))
#define MAIL_SOURCE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_TYPE_SOURCE_LIST, MailSourceListClass))

typedef struct _MailSourceList MailSourceList;
typedef struct _MailSourceListClass MailSourceListClass;
typedef struct _MailSourceListPrivate MailSourceListPrivate;

#define MAIL_SOURCE_LIST_TYPE_TREE (mail_source_list_tree_get_type ())
#define MAIL_SOURCE_LIST_TREE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_SOURCE_LIST_TYPE_TREE, MailSourceListTree))
#define MAIL_SOURCE_LIST_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_SOURCE_LIST_TYPE_TREE, MailSourceListTreeClass))
#define MAIL_SOURCE_LIST_IS_TREE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_SOURCE_LIST_TYPE_TREE))
#define MAIL_SOURCE_LIST_IS_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_SOURCE_LIST_TYPE_TREE))
#define MAIL_SOURCE_LIST_TREE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_SOURCE_LIST_TYPE_TREE, MailSourceListTreeClass))

typedef struct _MailSourceListTree MailSourceListTree;
typedef struct _MailSourceListTreeClass MailSourceListTreeClass;

#define MAIL_SOURCE_LIST_TYPE_DATA_MODEL (mail_source_list_data_model_get_type ())
#define MAIL_SOURCE_LIST_DATA_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_SOURCE_LIST_TYPE_DATA_MODEL, MailSourceListDataModel))
#define MAIL_SOURCE_LIST_DATA_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_SOURCE_LIST_TYPE_DATA_MODEL, MailSourceListDataModelClass))
#define MAIL_SOURCE_LIST_IS_DATA_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_SOURCE_LIST_TYPE_DATA_MODEL))
#define MAIL_SOURCE_LIST_IS_DATA_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_SOURCE_LIST_TYPE_DATA_MODEL))
#define MAIL_SOURCE_LIST_DATA_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_SOURCE_LIST_TYPE_DATA_MODEL, MailSourceListDataModelClass))

typedef struct _MailSourceListDataModel MailSourceListDataModel;
typedef struct _MailSourceListDataModelClass MailSourceListDataModelClass;
enum  {
	MAIL_SOURCE_LIST_0_PROPERTY,
	MAIL_SOURCE_LIST_ROOT_PROPERTY,
	MAIL_SOURCE_LIST_SELECTED_PROPERTY,
	MAIL_SOURCE_LIST_NUM_PROPERTIES
};
static GParamSpec* mail_source_list_properties[MAIL_SOURCE_LIST_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef gboolean (*MailSourceListVisibleFunc) (MailSourceListItem* item, gpointer user_data);
typedef struct _MailSourceListItemPrivate MailSourceListItemPrivate;
enum  {
	MAIL_SOURCE_LIST_ITEM_0_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_PARENT_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_NAME_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_MARKUP_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_BADGE_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_EDITABLE_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_VISIBLE_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_ICON_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY,
	MAIL_SOURCE_LIST_ITEM_NUM_PROPERTIES
};
static GParamSpec* mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
enum  {
	MAIL_SOURCE_LIST_ITEM_EDITED_SIGNAL,
	MAIL_SOURCE_LIST_ITEM_ACTION_ACTIVATED_SIGNAL,
	MAIL_SOURCE_LIST_ITEM_ACTIVATED_SIGNAL,
	MAIL_SOURCE_LIST_ITEM_NUM_SIGNALS
};
static guint mail_source_list_item_signals[MAIL_SOURCE_LIST_ITEM_NUM_SIGNALS] = {0};
typedef struct _MailSourceListExpandableItemPrivate MailSourceListExpandableItemPrivate;
enum  {
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_0_PROPERTY,
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY,
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY,
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILDREN_PROPERTY,
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_NUM_PROPERTIES
};
static GParamSpec* mail_source_list_expandable_item_properties[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_NUM_PROPERTIES];
enum  {
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_ADDED_SIGNAL,
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_REMOVED_SIGNAL,
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_TOGGLED_SIGNAL,
	MAIL_SOURCE_LIST_EXPANDABLE_ITEM_NUM_SIGNALS
};
static guint mail_source_list_expandable_item_signals[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_NUM_SIGNALS] = {0};
typedef struct _MailSourceListDataModelPrivate MailSourceListDataModelPrivate;

#define MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER (mail_source_list_data_model_node_wrapper_get_type ())
#define MAIL_SOURCE_LIST_DATA_MODEL_NODE_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, MailSourceListDataModelNodeWrapper))
#define MAIL_SOURCE_LIST_DATA_MODEL_NODE_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, MailSourceListDataModelNodeWrapperClass))
#define MAIL_SOURCE_LIST_DATA_MODEL_IS_NODE_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER))
#define MAIL_SOURCE_LIST_DATA_MODEL_IS_NODE_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER))
#define MAIL_SOURCE_LIST_DATA_MODEL_NODE_WRAPPER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, MailSourceListDataModelNodeWrapperClass))

typedef struct _MailSourceListDataModelNodeWrapper MailSourceListDataModelNodeWrapper;
typedef struct _MailSourceListDataModelNodeWrapperClass MailSourceListDataModelNodeWrapperClass;

#define MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR (mail_source_list_data_model_item_monitor_get_type ())
#define MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, MailSourceListDataModelItemMonitor))
#define MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, MailSourceListDataModelItemMonitorClass))
#define MAIL_SOURCE_LIST_DATA_MODEL_IS_ITEM_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR))
#define MAIL_SOURCE_LIST_DATA_MODEL_IS_ITEM_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR))
#define MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, MailSourceListDataModelItemMonitorClass))

typedef struct _MailSourceListDataModelItemMonitor MailSourceListDataModelItemMonitor;
typedef struct _MailSourceListDataModelItemMonitorClass MailSourceListDataModelItemMonitorClass;
enum  {
	MAIL_SOURCE_LIST_DATA_MODEL_0_PROPERTY,
	MAIL_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY,
	MAIL_SOURCE_LIST_DATA_MODEL_NUM_PROPERTIES
};
static GParamSpec* mail_source_list_data_model_properties[MAIL_SOURCE_LIST_DATA_MODEL_NUM_PROPERTIES];
typedef enum  {
	MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM,
	MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_N_COLUMNS
} MailSourceListDataModelColumn;

#define MAIL_SOURCE_LIST_DATA_MODEL_TYPE_COLUMN (mail_source_list_data_model_column_get_type ())
#define __vala_GtkTreeIter_free0(var) ((var == NULL) ? NULL : (var = (_vala_GtkTreeIter_free (var), NULL)))
#define __vala_GtkTreePath_free0(var) ((var == NULL) ? NULL : (var = (_vala_GtkTreePath_free (var), NULL)))
#define _mail_source_list_data_model_node_wrapper_unref0(var) ((var == NULL) ? NULL : (var = (mail_source_list_data_model_node_wrapper_unref (var), NULL)))
#define _mail_source_list_data_model_item_monitor_unref0(var) ((var == NULL) ? NULL : (var = (mail_source_list_data_model_item_monitor_unref (var), NULL)))
typedef struct _Block48Data Block48Data;
typedef struct _MailSourceListDataModelNodeWrapperPrivate MailSourceListDataModelNodeWrapperPrivate;
#define __vala_GtkTreeRowReference_free0(var) ((var == NULL) ? NULL : (var = (_vala_GtkTreeRowReference_free (var), NULL)))
typedef struct _MailSourceListDataModelParamSpecNodeWrapper MailSourceListDataModelParamSpecNodeWrapper;
typedef struct _MailSourceListDataModelItemMonitorPrivate MailSourceListDataModelItemMonitorPrivate;
typedef struct _MailSourceListDataModelParamSpecItemMonitor MailSourceListDataModelParamSpecItemMonitor;
enum  {
	MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_CHANGED_SIGNAL,
	MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_NUM_SIGNALS
};
static guint mail_source_list_data_model_item_monitor_signals[MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_NUM_SIGNALS] = {0};
enum  {
	MAIL_SOURCE_LIST_DATA_MODEL_ITEM_UPDATED_SIGNAL,
	MAIL_SOURCE_LIST_DATA_MODEL_NUM_SIGNALS
};
static guint mail_source_list_data_model_signals[MAIL_SOURCE_LIST_DATA_MODEL_NUM_SIGNALS] = {0};

#define MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON (mail_source_list_cell_renderer_icon_get_type ())
#define MAIL_SOURCE_LIST_CELL_RENDERER_ICON(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, MailSourceListCellRendererIcon))
#define MAIL_SOURCE_LIST_CELL_RENDERER_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, MailSourceListCellRendererIconClass))
#define MAIL_SOURCE_LIST_IS_CELL_RENDERER_ICON(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON))
#define MAIL_SOURCE_LIST_IS_CELL_RENDERER_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON))
#define MAIL_SOURCE_LIST_CELL_RENDERER_ICON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, MailSourceListCellRendererIconClass))

typedef struct _MailSourceListCellRendererIcon MailSourceListCellRendererIcon;
typedef struct _MailSourceListCellRendererIconClass MailSourceListCellRendererIconClass;
typedef struct _MailSourceListCellRendererIconPrivate MailSourceListCellRendererIconPrivate;
enum  {
	MAIL_SOURCE_LIST_CELL_RENDERER_ICON_0_PROPERTY,
	MAIL_SOURCE_LIST_CELL_RENDERER_ICON_NUM_PROPERTIES
};
static GParamSpec* mail_source_list_cell_renderer_icon_properties[MAIL_SOURCE_LIST_CELL_RENDERER_ICON_NUM_PROPERTIES];
enum  {
	MAIL_SOURCE_LIST_CELL_RENDERER_ICON_ACTIVATED_SIGNAL,
	MAIL_SOURCE_LIST_CELL_RENDERER_ICON_NUM_SIGNALS
};
static guint mail_source_list_cell_renderer_icon_signals[MAIL_SOURCE_LIST_CELL_RENDERER_ICON_NUM_SIGNALS] = {0};

#define MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER (mail_source_list_cell_renderer_spacer_get_type ())
#define MAIL_SOURCE_LIST_CELL_RENDERER_SPACER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, MailSourceListCellRendererSpacer))
#define MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, MailSourceListCellRendererSpacerClass))
#define MAIL_SOURCE_LIST_IS_CELL_RENDERER_SPACER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER))
#define MAIL_SOURCE_LIST_IS_CELL_RENDERER_SPACER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER))
#define MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, MailSourceListCellRendererSpacerClass))

typedef struct _MailSourceListCellRendererSpacer MailSourceListCellRendererSpacer;
typedef struct _MailSourceListCellRendererSpacerClass MailSourceListCellRendererSpacerClass;
typedef struct _MailSourceListCellRendererSpacerPrivate MailSourceListCellRendererSpacerPrivate;
enum  {
	MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_0_PROPERTY,
	MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY,
	MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_NUM_PROPERTIES
};
static GParamSpec* mail_source_list_cell_renderer_spacer_properties[MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_NUM_PROPERTIES];
typedef struct _MailSourceListTreePrivate MailSourceListTreePrivate;

#define MAIL_TYPE_CELL_RENDERER_BADGE (mail_cell_renderer_badge_get_type ())
#define MAIL_CELL_RENDERER_BADGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_TYPE_CELL_RENDERER_BADGE, MailCellRendererBadge))
#define MAIL_CELL_RENDERER_BADGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_TYPE_CELL_RENDERER_BADGE, MailCellRendererBadgeClass))
#define MAIL_IS_CELL_RENDERER_BADGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_TYPE_CELL_RENDERER_BADGE))
#define MAIL_IS_CELL_RENDERER_BADGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_TYPE_CELL_RENDERER_BADGE))
#define MAIL_CELL_RENDERER_BADGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_TYPE_CELL_RENDERER_BADGE, MailCellRendererBadgeClass))

typedef struct _MailCellRendererBadge MailCellRendererBadge;
typedef struct _MailCellRendererBadgeClass MailCellRendererBadgeClass;

#define MAIL_TYPE_CELL_RENDERER_EXPANDER (mail_cell_renderer_expander_get_type ())
#define MAIL_CELL_RENDERER_EXPANDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAIL_TYPE_CELL_RENDERER_EXPANDER, MailCellRendererExpander))
#define MAIL_CELL_RENDERER_EXPANDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MAIL_TYPE_CELL_RENDERER_EXPANDER, MailCellRendererExpanderClass))
#define MAIL_IS_CELL_RENDERER_EXPANDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAIL_TYPE_CELL_RENDERER_EXPANDER))
#define MAIL_IS_CELL_RENDERER_EXPANDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MAIL_TYPE_CELL_RENDERER_EXPANDER))
#define MAIL_CELL_RENDERER_EXPANDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MAIL_TYPE_CELL_RENDERER_EXPANDER, MailCellRendererExpanderClass))

typedef struct _MailCellRendererExpander MailCellRendererExpander;
typedef struct _MailCellRendererExpanderClass MailCellRendererExpanderClass;
enum  {
	MAIL_SOURCE_LIST_TREE_0_PROPERTY,
	MAIL_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY,
	MAIL_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY,
	MAIL_SOURCE_LIST_TREE_EDITING_PROPERTY,
	MAIL_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY,
	MAIL_SOURCE_LIST_TREE_NUM_PROPERTIES
};
static GParamSpec* mail_source_list_tree_properties[MAIL_SOURCE_LIST_TREE_NUM_PROPERTIES];
typedef enum  {
	MAIL_SOURCE_LIST_TREE_COLUMN_ITEM,
	MAIL_SOURCE_LIST_TREE_COLUMN_N_COLS
} MailSourceListTreeColumn;

#define MAIL_SOURCE_LIST_TREE_TYPE_COLUMN (mail_source_list_tree_column_get_type ())
#define _gtk_target_list_unref0(var) ((var == NULL) ? NULL : (var = (gtk_target_list_unref (var), NULL)))
#define _g_string_free0(var) ((var == NULL) ? NULL : (var = (g_string_free (var, TRUE), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_param_spec_unref0(var) ((var == NULL) ? NULL : (var = (g_param_spec_unref (var), NULL)))
enum  {
	MAIL_SOURCE_LIST_TREE_ITEM_SELECTED_SIGNAL,
	MAIL_SOURCE_LIST_TREE_NUM_SIGNALS
};
static guint mail_source_list_tree_signals[MAIL_SOURCE_LIST_TREE_NUM_SIGNALS] = {0};
enum  {
	MAIL_SOURCE_LIST_ITEM_SELECTED_SIGNAL,
	MAIL_SOURCE_LIST_NUM_SIGNALS
};
static guint mail_source_list_signals[MAIL_SOURCE_LIST_NUM_SIGNALS] = {0};
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _MailSourceListSortableIface {
	GTypeInterface parent_iface;
	gboolean (*allow_dnd_sorting) (MailSourceListSortable* self);
	gint (*compare) (MailSourceListSortable* self, MailSourceListItem* a, MailSourceListItem* b);
};

struct _MailSourceListDragSourceIface {
	GTypeInterface parent_iface;
	gboolean (*draggable) (MailSourceListDragSource* self);
	void (*prepare_selection_data) (MailSourceListDragSource* self, GtkSelectionData* selection_data);
};

struct _MailSourceListDragDestIface {
	GTypeInterface parent_iface;
	gboolean (*data_drop_possible) (MailSourceListDragDest* self, GdkDragContext* context, GtkSelectionData* data);
	GdkDragAction (*data_received) (MailSourceListDragDest* self, GdkDragContext* context, GtkSelectionData* data);
};

struct _MailSourceList {
	GtkScrolledWindow parent_instance;
	MailSourceListPrivate * priv;
};

struct _MailSourceListClass {
	GtkScrolledWindowClass parent_class;
	void (*item_selected) (MailSourceList* self, MailSourceListItem* item);
};

struct _MailSourceListPrivate {
	MailSourceListTree* tree;
	MailSourceListDataModel* data_model;
};

struct _MailSourceListItem {
	GObject parent_instance;
	MailSourceListItemPrivate * priv;
};

struct _MailSourceListItemClass {
	GObjectClass parent_class;
	GtkMenu* (*get_context_menu) (MailSourceListItem* self);
	void (*edited) (MailSourceListItem* self, const gchar* new_name);
	void (*action_activated) (MailSourceListItem* self);
	void (*activated) (MailSourceListItem* self);
};

struct _MailSourceListItemPrivate {
	MailSourceListExpandableItem* _parent;
	gchar* _name;
	gchar* _tooltip;
	gchar* _markup;
	gchar* _badge;
	gboolean _editable;
	gboolean _visible;
	gboolean _selectable;
	GIcon* _icon;
	GIcon* _activatable;
	gchar* _activatable_tooltip;
};

struct _MailSourceListExpandableItem {
	MailSourceListItem parent_instance;
	MailSourceListExpandableItemPrivate * priv;
};

struct _MailSourceListExpandableItemClass {
	MailSourceListItemClass parent_class;
	void (*toggled) (MailSourceListExpandableItem* self);
};

struct _MailSourceListExpandableItemPrivate {
	gboolean _collapsible;
	gboolean _expanded;
	GeeCollection* children_list;
};

struct _MailSourceListDataModel {
	GtkTreeModelFilter parent_instance;
	MailSourceListDataModelPrivate * priv;
};

struct _MailSourceListDataModelClass {
	GtkTreeModelFilterClass parent_class;
};

struct _MailSourceListDataModelPrivate {
	MailSourceListExpandableItem* _root;
	GeeHashMap* items;
	GeeHashMap* monitors;
	GtkTreeStore* child_tree;
	MailSourceListVisibleFunc filter_func;
	gpointer filter_func_target;
};

struct _Block48Data {
	int _ref_count_;
	MailSourceListDataModel* self;
	MailSourceListExpandableItem* parent;
};

struct _MailSourceListDataModelNodeWrapper {
	GTypeInstance parent_instance;
	volatile int ref_count;
	MailSourceListDataModelNodeWrapperPrivate * priv;
};

struct _MailSourceListDataModelNodeWrapperClass {
	GTypeClass parent_class;
	void (*finalize) (MailSourceListDataModelNodeWrapper *self);
};

struct _MailSourceListDataModelNodeWrapperPrivate {
	GtkTreeRowReference* row_reference;
};

struct _MailSourceListDataModelParamSpecNodeWrapper {
	GParamSpec parent_instance;
};

struct _MailSourceListDataModelItemMonitor {
	GTypeInstance parent_instance;
	volatile int ref_count;
	MailSourceListDataModelItemMonitorPrivate * priv;
};

struct _MailSourceListDataModelItemMonitorClass {
	GTypeClass parent_class;
	void (*finalize) (MailSourceListDataModelItemMonitor *self);
};

struct _MailSourceListDataModelItemMonitorPrivate {
	MailSourceListItem* item;
};

struct _MailSourceListDataModelParamSpecItemMonitor {
	GParamSpec parent_instance;
};

struct _MailSourceListCellRendererIcon {
	GtkCellRendererPixbuf parent_instance;
	MailSourceListCellRendererIconPrivate * priv;
};

struct _MailSourceListCellRendererIconClass {
	GtkCellRendererPixbufClass parent_class;
};

struct _MailSourceListCellRendererSpacer {
	GtkCellRenderer parent_instance;
	MailSourceListCellRendererSpacerPrivate * priv;
};

struct _MailSourceListCellRendererSpacerClass {
	GtkCellRendererClass parent_class;
};

struct _MailSourceListCellRendererSpacerPrivate {
	gint _level;
};

struct _MailSourceListTree {
	GtkTreeView parent_instance;
	MailSourceListTreePrivate * priv;
};

struct _MailSourceListTreeClass {
	GtkTreeViewClass parent_class;
};

struct _MailSourceListTreePrivate {
	MailSourceListDataModel* _data_model;
	MailSourceListItem* selected;
	MailSourceListItem* edited;
	GtkEntry* editable_entry;
	GtkCellRendererText* text_cell;
	MailSourceListCellRendererIcon* icon_cell;
	MailSourceListCellRendererIcon* activatable_cell;
	MailCellRendererBadge* badge_cell;
	MailCellRendererExpander* primary_expander_cell;
	MailCellRendererExpander* secondary_expander_cell;
	GeeHashMap* spacer_cells;
	gboolean unselectable_item_clicked;
};

static gint MailSourceList_private_offset;
static gpointer mail_source_list_parent_class = NULL;
static gint MailSourceListItem_private_offset;
static gpointer mail_source_list_item_parent_class = NULL;
static gint MailSourceListExpandableItem_private_offset;
static gpointer mail_source_list_expandable_item_parent_class = NULL;
static gint MailSourceListDataModel_private_offset;
static gpointer mail_source_list_data_model_parent_class = NULL;
static gint MailSourceListDataModelNodeWrapper_private_offset;
static gpointer mail_source_list_data_model_node_wrapper_parent_class = NULL;
static gint MailSourceListDataModelItemMonitor_private_offset;
static gpointer mail_source_list_data_model_item_monitor_parent_class = NULL;
static GtkTreeDragSourceIface * mail_source_list_data_model_gtk_tree_drag_source_parent_iface = NULL;
static GtkTreeDragDestIface * mail_source_list_data_model_gtk_tree_drag_dest_parent_iface = NULL;
static gpointer mail_source_list_cell_renderer_icon_parent_class = NULL;
static gint MailSourceListCellRendererSpacer_private_offset;
static gpointer mail_source_list_cell_renderer_spacer_parent_class = NULL;
static gint MailSourceListTree_private_offset;
static gpointer mail_source_list_tree_parent_class = NULL;

VALA_EXTERN GType mail_source_list_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListItem, g_object_unref)
VALA_EXTERN GType mail_source_list_expandable_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListExpandableItem, g_object_unref)
VALA_EXTERN GType mail_source_list_sortable_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListSortable, g_object_unref)
VALA_EXTERN gboolean mail_source_list_sortable_allow_dnd_sorting (MailSourceListSortable* self);
VALA_EXTERN gint mail_source_list_sortable_compare (MailSourceListSortable* self,
                                        MailSourceListItem* a,
                                        MailSourceListItem* b);
static GType mail_source_list_sortable_get_type_once (void);
VALA_EXTERN GType mail_source_list_drag_source_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListDragSource, g_object_unref)
VALA_EXTERN gboolean mail_source_list_drag_source_draggable (MailSourceListDragSource* self);
VALA_EXTERN void mail_source_list_drag_source_prepare_selection_data (MailSourceListDragSource* self,
                                                          GtkSelectionData* selection_data);
static GType mail_source_list_drag_source_get_type_once (void);
VALA_EXTERN GType mail_source_list_drag_dest_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListDragDest, g_object_unref)
VALA_EXTERN gboolean mail_source_list_drag_dest_data_drop_possible (MailSourceListDragDest* self,
                                                        GdkDragContext* context,
                                                        GtkSelectionData* data);
VALA_EXTERN GdkDragAction mail_source_list_drag_dest_data_received (MailSourceListDragDest* self,
                                                        GdkDragContext* context,
                                                        GtkSelectionData* data);
static GType mail_source_list_drag_dest_get_type_once (void);
VALA_EXTERN GType mail_source_list_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceList, g_object_unref)
static GType mail_source_list_tree_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListTree, g_object_unref)
static GType mail_source_list_data_model_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListDataModel, g_object_unref)
static MailSourceListDataModel* mail_source_list_data_model_new (void);
static MailSourceListDataModel* mail_source_list_data_model_construct (GType object_type);
VALA_EXTERN MailSourceList* mail_source_list_new (MailSourceListExpandableItem* root);
VALA_EXTERN MailSourceList* mail_source_list_construct (GType object_type,
                                            MailSourceListExpandableItem* root);
VALA_EXTERN void mail_source_list_set_root (MailSourceList* self,
                                MailSourceListExpandableItem* value);
VALA_EXTERN gboolean mail_source_list_has_item (MailSourceList* self,
                                    MailSourceListItem* item);
static gboolean mail_source_list_data_model_has_item (MailSourceListDataModel* self,
                                               MailSourceListItem* item);
VALA_EXTERN void mail_source_list_set_filter_func (MailSourceList* self,
                                       MailSourceListVisibleFunc visible_func,
                                       gpointer visible_func_target,
                                       gboolean refilter);
static void mail_source_list_data_model_set_filter_func (MailSourceListDataModel* self,
                                                  MailSourceListVisibleFunc visible_func,
                                                  gpointer visible_func_target);
static void mail_source_list_refilter (MailSourceList* self);
VALA_EXTERN void mail_source_list_start_editing_item (MailSourceList* self,
                                          MailSourceListItem* item);
static gboolean mail_source_list_tree_start_editing_item (MailSourceListTree* self,
                                                   MailSourceListItem* item);
VALA_EXTERN MailSourceListExpandableItem* mail_source_list_get_root (MailSourceList* self);
static MailSourceListExpandableItem* mail_source_list_data_model_get_root (MailSourceListDataModel* self);
static void mail_source_list_data_model_set_root (MailSourceListDataModel* self,
                                           MailSourceListExpandableItem* value);
VALA_EXTERN MailSourceListItem* mail_source_list_get_selected (MailSourceList* self);
static MailSourceListItem* mail_source_list_tree_get_selected_item (MailSourceListTree* self);
VALA_EXTERN void mail_source_list_set_selected (MailSourceList* self,
                                    MailSourceListItem* value);
VALA_EXTERN MailSourceListExpandableItem* mail_source_list_item_get_parent (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_expandable_item_expand_with_parents (MailSourceListExpandableItem* self);
static void mail_source_list_tree_set_selected_item (MailSourceListTree* self,
                                              MailSourceListItem* value);
static PangoEllipsizeMode mail_source_list_get_ellipsize_mode (MailSourceList* self);
static PangoEllipsizeMode mail_source_list_tree_get_ellipsize_mode (MailSourceListTree* self);
static void mail_source_list_set_ellipsize_mode (MailSourceList* self,
                                          PangoEllipsizeMode value);
static void mail_source_list_tree_set_ellipsize_mode (MailSourceListTree* self,
                                               PangoEllipsizeMode value);
static gboolean mail_source_list_get_editing (MailSourceList* self);
static gboolean mail_source_list_tree_get_editing (MailSourceListTree* self);
static void mail_source_list_real_item_selected (MailSourceList* self,
                                          MailSourceListItem* item);
VALA_EXTERN GtkMenu* mail_source_list_item_get_context_menu (MailSourceListItem* self);
static MailSourceListItem* mail_source_list_item_new (const gchar* name);
static MailSourceListItem* mail_source_list_item_construct (GType object_type,
                                                     const gchar* name);
VALA_EXTERN void mail_source_list_item_set_name (MailSourceListItem* self,
                                     const gchar* value);
static GtkMenu* mail_source_list_item_real_get_context_menu (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_parent (MailSourceListItem* self,
                                       MailSourceListExpandableItem* value);
VALA_EXTERN const gchar* mail_source_list_item_get_name (MailSourceListItem* self);
VALA_EXTERN const gchar* mail_source_list_item_get_tooltip (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_tooltip (MailSourceListItem* self,
                                        const gchar* value);
VALA_EXTERN const gchar* mail_source_list_item_get_markup (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_markup (MailSourceListItem* self,
                                       const gchar* value);
VALA_EXTERN const gchar* mail_source_list_item_get_badge (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_badge (MailSourceListItem* self,
                                      const gchar* value);
VALA_EXTERN gboolean mail_source_list_item_get_editable (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_editable (MailSourceListItem* self,
                                         gboolean value);
VALA_EXTERN gboolean mail_source_list_item_get_visible (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_visible (MailSourceListItem* self,
                                        gboolean value);
VALA_EXTERN gboolean mail_source_list_item_get_selectable (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_selectable (MailSourceListItem* self,
                                           gboolean value);
VALA_EXTERN GIcon* mail_source_list_item_get_icon (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_icon (MailSourceListItem* self,
                                     GIcon* value);
VALA_EXTERN GIcon* mail_source_list_item_get_activatable (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_activatable (MailSourceListItem* self,
                                            GIcon* value);
VALA_EXTERN const gchar* mail_source_list_item_get_activatable_tooltip (MailSourceListItem* self);
VALA_EXTERN void mail_source_list_item_set_activatable_tooltip (MailSourceListItem* self,
                                                    const gchar* value);
static void mail_source_list_item_real_edited (MailSourceListItem* self,
                                        const gchar* new_name);
static void mail_source_list_item_real_action_activated (MailSourceListItem* self);
static void mail_source_list_item_real_activated (MailSourceListItem* self);
static void mail_source_list_item_finalize (GObject * obj);
static GType mail_source_list_item_get_type_once (void);
static void _vala_mail_source_list_item_get_property (GObject * object,
                                               guint property_id,
                                               GValue * value,
                                               GParamSpec * pspec);
static void _vala_mail_source_list_item_set_property (GObject * object,
                                               guint property_id,
                                               const GValue * value,
                                               GParamSpec * pspec);
VALA_EXTERN MailSourceListExpandableItem* mail_source_list_expandable_item_new (const gchar* name);
VALA_EXTERN MailSourceListExpandableItem* mail_source_list_expandable_item_construct (GType object_type,
                                                                          const gchar* name);
VALA_EXTERN void mail_source_list_expandable_item_add (MailSourceListExpandableItem* self,
                                           MailSourceListItem* item);
VALA_EXTERN void mail_source_list_expandable_item_remove (MailSourceListExpandableItem* self,
                                              MailSourceListItem* item);
VALA_EXTERN void mail_source_list_expandable_item_set_expanded (MailSourceListExpandableItem* self,
                                                    gboolean value);
VALA_EXTERN gboolean mail_source_list_expandable_item_get_collapsible (MailSourceListExpandableItem* self);
VALA_EXTERN void mail_source_list_expandable_item_set_collapsible (MailSourceListExpandableItem* self,
                                                       gboolean value);
VALA_EXTERN gboolean mail_source_list_expandable_item_get_expanded (MailSourceListExpandableItem* self);
static guint mail_source_list_expandable_item_get_n_children (MailSourceListExpandableItem* self);
VALA_EXTERN GeeCollection* mail_source_list_expandable_item_get_children (MailSourceListExpandableItem* self);
static void mail_source_list_expandable_item_real_toggled (MailSourceListExpandableItem* self);
static GObject * mail_source_list_expandable_item_constructor (GType type,
                                                        guint n_construct_properties,
                                                        GObjectConstructParam * construct_properties);
static void mail_source_list_expandable_item_finalize (GObject * obj);
static GType mail_source_list_expandable_item_get_type_once (void);
static void _vala_mail_source_list_expandable_item_get_property (GObject * object,
                                                          guint property_id,
                                                          GValue * value,
                                                          GParamSpec * pspec);
static void _vala_mail_source_list_expandable_item_set_property (GObject * object,
                                                          guint property_id,
                                                          const GValue * value,
                                                          GParamSpec * pspec);
static gpointer mail_source_list_data_model_node_wrapper_ref (gpointer instance);
static void mail_source_list_data_model_node_wrapper_unref (gpointer instance);
static GParamSpec* mail_source_list_data_model_param_spec_node_wrapper (const gchar* name,
                                                                 const gchar* nick,
                                                                 const gchar* blurb,
                                                                 GType object_type,
                                                                 GParamFlags flags) G_GNUC_UNUSED ;
static void mail_source_list_data_model_value_set_node_wrapper (GValue* value,
                                                         gpointer v_object) G_GNUC_UNUSED ;
static void mail_source_list_data_model_value_take_node_wrapper (GValue* value,
                                                          gpointer v_object) G_GNUC_UNUSED ;
static gpointer mail_source_list_data_model_value_get_node_wrapper (const GValue* value) G_GNUC_UNUSED ;
static GType mail_source_list_data_model_node_wrapper_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListDataModelNodeWrapper, mail_source_list_data_model_node_wrapper_unref)
static gpointer mail_source_list_data_model_item_monitor_ref (gpointer instance);
static void mail_source_list_data_model_item_monitor_unref (gpointer instance);
static GParamSpec* mail_source_list_data_model_param_spec_item_monitor (const gchar* name,
                                                                 const gchar* nick,
                                                                 const gchar* blurb,
                                                                 GType object_type,
                                                                 GParamFlags flags) G_GNUC_UNUSED ;
static void mail_source_list_data_model_value_set_item_monitor (GValue* value,
                                                         gpointer v_object) G_GNUC_UNUSED ;
static void mail_source_list_data_model_value_take_item_monitor (GValue* value,
                                                          gpointer v_object) G_GNUC_UNUSED ;
static gpointer mail_source_list_data_model_value_get_item_monitor (const GValue* value) G_GNUC_UNUSED ;
static GType mail_source_list_data_model_item_monitor_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListDataModelItemMonitor, mail_source_list_data_model_item_monitor_unref)
static GType mail_source_list_data_model_column_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static GType mail_source_list_data_model_column_type (MailSourceListDataModelColumn self);
static void mail_source_list_data_model_update_item (MailSourceListDataModel* self,
                                              MailSourceListItem* item);
static GtkTreePath* mail_source_list_data_model_node_wrapper_get_path (MailSourceListDataModelNodeWrapper* self);
static GtkTreeIter* mail_source_list_data_model_node_wrapper_get_iter (MailSourceListDataModelNodeWrapper* self);
static void _vala_GtkTreeIter_free (GtkTreeIter* self);
static void _vala_GtkTreePath_free (GtkTreePath* self);
static void mail_source_list_data_model_add_item (MailSourceListDataModel* self,
                                           MailSourceListItem* item);
static GtkTreeIter* mail_source_list_data_model_get_item_child_iter (MailSourceListDataModel* self,
                                                              MailSourceListItem* item);
static GtkTreeIter* _vala_GtkTreeIter_copy (GtkTreeIter* self);
static MailSourceListDataModelNodeWrapper* mail_source_list_data_model_node_wrapper_new (GtkTreeModel* model,
                                                                                  GtkTreeIter* iter);
static MailSourceListDataModelNodeWrapper* mail_source_list_data_model_node_wrapper_construct (GType object_type,
                                                                                        GtkTreeModel* model,
                                                                                        GtkTreeIter* iter);
static void mail_source_list_data_model_add_property_monitor (MailSourceListDataModel* self,
                                                       MailSourceListItem* item);
static void mail_source_list_data_model_push_parent_update (MailSourceListDataModel* self,
                                                     MailSourceListExpandableItem* parent);
static void mail_source_list_data_model_add_children_monitor (MailSourceListDataModel* self,
                                                       MailSourceListExpandableItem* item);
static void mail_source_list_data_model_remove_item (MailSourceListDataModel* self,
                                              MailSourceListItem* item);
static void mail_source_list_data_model_remove_property_monitor (MailSourceListDataModel* self,
                                                          MailSourceListItem* item);
static void mail_source_list_data_model_remove_children_monitor (MailSourceListDataModel* self,
                                                          MailSourceListExpandableItem* item);
static MailSourceListDataModelItemMonitor* mail_source_list_data_model_item_monitor_new (MailSourceListItem* item);
static MailSourceListDataModelItemMonitor* mail_source_list_data_model_item_monitor_construct (GType object_type,
                                                                                        MailSourceListItem* item);
static void mail_source_list_data_model_on_item_prop_changed (MailSourceListDataModel* self,
                                                       MailSourceListItem* item,
                                                       const gchar* prop_name);
static void _mail_source_list_data_model_on_item_prop_changed_mail_source_list_data_model_item_monitor_changed (MailSourceListDataModelItemMonitor* _sender,
                                                                                                         MailSourceListItem* _self_,
                                                                                                         const gchar* prop_name,
                                                                                                         gpointer self);
static void mail_source_list_data_model_on_item_child_added (MailSourceListDataModel* self,
                                                      MailSourceListItem* item);
static void _mail_source_list_data_model_on_item_child_added_mail_source_list_expandable_item_child_added (MailSourceListExpandableItem* _sender,
                                                                                                    MailSourceListItem* item,
                                                                                                    gpointer self);
static void mail_source_list_data_model_on_item_child_removed (MailSourceListDataModel* self,
                                                        MailSourceListItem* item);
static void _mail_source_list_data_model_on_item_child_removed_mail_source_list_expandable_item_child_removed (MailSourceListExpandableItem* _sender,
                                                                                                        MailSourceListItem* item,
                                                                                                        gpointer self);
static Block48Data* block48_data_ref (Block48Data* _data48_);
static void block48_data_unref (void * _userdata_);
static GtkTreePath* mail_source_list_data_model_get_item_path (MailSourceListDataModel* self,
                                                        MailSourceListItem* item);
static gboolean ___lambda4_ (Block48Data* _data48_);
static gboolean ____lambda4__gsource_func (gpointer self);
static MailSourceListItem* mail_source_list_data_model_get_item (MailSourceListDataModel* self,
                                                          GtkTreeIter* iter);
static MailSourceListItem* mail_source_list_data_model_get_item_from_path (MailSourceListDataModel* self,
                                                                    GtkTreePath* path);
static GtkTreePath* mail_source_list_data_model_get_item_child_path (MailSourceListDataModel* self,
                                                              MailSourceListItem* item);
static gboolean mail_source_list_data_model_is_category (MailSourceListDataModel* self,
                                                  MailSourceListItem* item,
                                                  GtkTreeIter* iter,
                                                  GtkTreePath* path);
static gboolean mail_source_list_data_model_is_iter_at_root_level (MailSourceListDataModel* self,
                                                            GtkTreeIter* iter);
static gboolean mail_source_list_data_model_is_path_at_root_level (MailSourceListDataModel* self,
                                                            GtkTreePath* path);
static void mail_source_list_data_model_resort (MailSourceListDataModel* self);
static gint mail_source_list_data_model_child_model_sort_func (MailSourceListDataModel* self,
                                                        GtkTreeModel* model,
                                                        GtkTreeIter* a,
                                                        GtkTreeIter* b);
static gboolean mail_source_list_data_model_filter_visible_func (MailSourceListDataModel* self,
                                                          GtkTreeModel* child_model,
                                                          GtkTreeIter* iter);
static gboolean mail_source_list_data_model_real_drag_data_received (GtkTreeDragDest* base,
                                                              GtkTreePath* dest,
                                                              GtkSelectionData* selection_data);
static gboolean mail_source_list_data_model_child_tree_drag_data_received (MailSourceListDataModel* self,
                                                                    GtkTreePath* dest,
                                                                    GtkTreePath* src_path);
static GtkTreePath* _vala_GtkTreePath_copy (GtkTreePath* self);
static void mail_source_list_data_model_recursive_node_copy (MailSourceListDataModel* self,
                                                      GtkTreeIter* src_iter,
                                                      GtkTreeIter* dest_iter);
static void mail_source_list_data_model_move_item (MailSourceListDataModel* self,
                                            GtkTreeIter* src_iter,
                                            GtkTreeIter* dest_iter);
static gboolean mail_source_list_data_model_real_row_drop_possible (GtkTreeDragDest* base,
                                                             GtkTreePath* dest,
                                                             GtkSelectionData* selection_data);
static gint* _vala_array_dup2 (gint* self,
                        gssize length);
static gint* _vala_array_dup3 (gint* self,
                        gssize length);
static gboolean mail_source_list_data_model_real_drag_data_get (GtkTreeDragSource* base,
                                                         GtkTreePath* path,
                                                         GtkSelectionData* selection_data);
static gboolean mail_source_list_data_model_real_row_draggable (GtkTreeDragSource* base,
                                                         GtkTreePath* path);
static void _vala_GtkTreeRowReference_free (GtkTreeRowReference* self);
static gboolean mail_source_list_data_model_node_wrapper_get_valid (MailSourceListDataModelNodeWrapper* self);
static void mail_source_list_data_model_node_wrapper_finalize (MailSourceListDataModelNodeWrapper * obj);
static GType mail_source_list_data_model_node_wrapper_get_type_once (void);
static void mail_source_list_data_model_item_monitor_on_notify (MailSourceListDataModelItemMonitor* self,
                                                         GParamSpec* prop);
static void _mail_source_list_data_model_item_monitor_on_notify_g_object_notify (GObject* _sender,
                                                                          GParamSpec* pspec,
                                                                          gpointer self);
static void g_cclosure_user_marshal_VOID__OBJECT_STRING (GClosure * closure,
                                                  GValue * return_value,
                                                  guint n_param_values,
                                                  const GValue * param_values,
                                                  gpointer invocation_hint,
                                                  gpointer marshal_data);
static void mail_source_list_data_model_item_monitor_finalize (MailSourceListDataModelItemMonitor * obj);
static GType mail_source_list_data_model_item_monitor_get_type_once (void);
static GObject * mail_source_list_data_model_constructor (GType type,
                                                   guint n_construct_properties,
                                                   GObjectConstructParam * construct_properties);
static gint _mail_source_list_data_model_child_model_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model,
                                                                                    GtkTreeIter* a,
                                                                                    GtkTreeIter* b,
                                                                                    gpointer self);
static gboolean _mail_source_list_data_model_filter_visible_func_gtk_tree_model_filter_visible_func (GtkTreeModel* model,
                                                                                              GtkTreeIter* iter,
                                                                                              gpointer self);
static void mail_source_list_data_model_finalize (GObject * obj);
static GType mail_source_list_data_model_get_type_once (void);
static void _vala_mail_source_list_data_model_get_property (GObject * object,
                                                     guint property_id,
                                                     GValue * value,
                                                     GParamSpec * pspec);
static void _vala_mail_source_list_data_model_set_property (GObject * object,
                                                     guint property_id,
                                                     const GValue * value,
                                                     GParamSpec * pspec);
static GType mail_source_list_cell_renderer_icon_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListCellRendererIcon, g_object_unref)
static gboolean mail_source_list_cell_renderer_icon_real_activate (GtkCellRenderer* base,
                                                            GdkEvent* event,
                                                            GtkWidget* widget,
                                                            const gchar* path,
                                                            GdkRectangle* background_area,
                                                            GdkRectangle* cell_area,
                                                            GtkCellRendererState flags);
static MailSourceListCellRendererIcon* mail_source_list_cell_renderer_icon_new (void);
static MailSourceListCellRendererIcon* mail_source_list_cell_renderer_icon_construct (GType object_type);
static GObject * mail_source_list_cell_renderer_icon_constructor (GType type,
                                                           guint n_construct_properties,
                                                           GObjectConstructParam * construct_properties);
static GType mail_source_list_cell_renderer_icon_get_type_once (void);
static GType mail_source_list_cell_renderer_spacer_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailSourceListCellRendererSpacer, g_object_unref)
static GtkSizeRequestMode mail_source_list_cell_renderer_spacer_real_get_request_mode (GtkCellRenderer* base);
static void mail_source_list_cell_renderer_spacer_real_get_preferred_width (GtkCellRenderer* base,
                                                                     GtkWidget* widget,
                                                                     gint* min_size,
                                                                     gint* natural_size);
static void mail_source_list_cell_renderer_spacer_real_get_preferred_height_for_width (GtkCellRenderer* base,
                                                                                GtkWidget* widget,
                                                                                gint width,
                                                                                gint* min_height,
                                                                                gint* natural_height);
static void mail_source_list_cell_renderer_spacer_real_render (GtkCellRenderer* base,
                                                        cairo_t* context,
                                                        GtkWidget* widget,
                                                        GdkRectangle* bg_area,
                                                        GdkRectangle* cell_area,
                                                        GtkCellRendererState flags);
static MailSourceListCellRendererSpacer* mail_source_list_cell_renderer_spacer_new (void);
static MailSourceListCellRendererSpacer* mail_source_list_cell_renderer_spacer_construct (GType object_type);
static gint mail_source_list_cell_renderer_spacer_get_level (MailSourceListCellRendererSpacer* self);
static void mail_source_list_cell_renderer_spacer_set_level (MailSourceListCellRendererSpacer* self,
                                                      gint value);
static void mail_source_list_cell_renderer_spacer_finalize (GObject * obj);
static GType mail_source_list_cell_renderer_spacer_get_type_once (void);
static void _vala_mail_source_list_cell_renderer_spacer_get_property (GObject * object,
                                                               guint property_id,
                                                               GValue * value,
                                                               GParamSpec * pspec);
static void _vala_mail_source_list_cell_renderer_spacer_set_property (GObject * object,
                                                               guint property_id,
                                                               const GValue * value,
                                                               GParamSpec * pspec);
VALA_EXTERN GType mail_cell_renderer_badge_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailCellRendererBadge, g_object_unref)
VALA_EXTERN GType mail_cell_renderer_expander_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MailCellRendererExpander, g_object_unref)
static void mail_source_list_tree_disable_item_property_monitor (MailSourceListTree* self);
static GType mail_source_list_tree_column_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static MailSourceListTree* mail_source_list_tree_new (MailSourceListDataModel* data_model);
static MailSourceListTree* mail_source_list_tree_construct (GType object_type,
                                                     MailSourceListDataModel* data_model);
static gboolean mail_source_list_tree_real_drag_motion (GtkWidget* base,
                                                 GdkDragContext* context,
                                                 gint x,
                                                 gint y,
                                                 guint time);
static MailSourceListDataModel* mail_source_list_tree_get_data_model (MailSourceListTree* self);
static void mail_source_list_tree_real_drag_data_received (GtkWidget* base,
                                                    GdkDragContext* context,
                                                    gint x,
                                                    gint y,
                                                    GtkSelectionData* selection_data,
                                                    guint info,
                                                    guint time);
static void mail_source_list_tree_configure_drag_source (MailSourceListTree* self,
                                                  GtkTargetEntry* src_entries,
                                                  gint src_entries_length1);
static GtkTargetEntry* mail_source_list_tree_append_row_target_entry (GtkTargetEntry* orig,
                                                               gint orig_length1,
                                                               gint* result_length1);
static void mail_source_list_tree_configure_drag_dest (MailSourceListTree* self,
                                                GtkTargetEntry* dest_entries,
                                                gint dest_entries_length1,
                                                GdkDragAction actions);
static gboolean mail_source_list_tree_on_query_tooltip (MailSourceListTree* self,
                                                 gint x,
                                                 gint y,
                                                 gboolean keyboard_tooltip,
                                                 GtkTooltip* tooltip);
static gboolean mail_source_list_tree_over_cell (MailSourceListTree* self,
                                          GtkTreeViewColumn* col,
                                          GtkTreePath* path,
                                          GtkCellRenderer* cell,
                                          gint x);
static void _vala_array_add10 (GtkTargetEntry* * array,
                        gint* length,
                        gint* size,
                        const GtkTargetEntry* value);
static void _vala_array_add11 (GtkTargetEntry* * array,
                        gint* length,
                        gint* size,
                        const GtkTargetEntry* value);
static void mail_source_list_tree_enable_item_property_monitor (MailSourceListTree* self);
static void mail_source_list_tree_on_model_item_updated (MailSourceListTree* self,
                                                  MailSourceListItem* item);
static void _mail_source_list_tree_on_model_item_updated_mail_source_list_data_model_item_updated (MailSourceListDataModel* _sender,
                                                                                            MailSourceListItem* item,
                                                                                            gpointer self);
static void mail_source_list_tree_update_expansion (MailSourceListTree* self,
                                             MailSourceListExpandableItem* expandable_item);
static void mail_source_list_tree_add_spacer_cell_for_level (MailSourceListTree* self,
                                                      gint level,
                                                      gboolean check_previous);
static void mail_source_list_tree_spacer_cell_data_func (GtkCellLayout* layout,
                                                  GtkCellRenderer* renderer,
                                                  GtkTreeModel* model,
                                                  GtkTreeIter* iter);
static void _mail_source_list_tree_spacer_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                             GtkCellRenderer* cell,
                                                                             GtkTreeModel* tree_model,
                                                                             GtkTreeIter* iter,
                                                                             gpointer self);
static gboolean mail_source_list_tree_select_func (MailSourceListTree* self,
                                            GtkTreeSelection* selection,
                                            GtkTreeModel* model,
                                            GtkTreePath* path,
                                            gboolean path_currently_selected);
static GtkTreePath* mail_source_list_tree_get_selected_path (MailSourceListTree* self);
static void __vala_GtkTreePath_free0_ (gpointer var);
static inline void _g_list_free___vala_GtkTreePath_free0_ (GList* self);
static void mail_source_list_tree_set_selected (MailSourceListTree* self,
                                         MailSourceListItem* item,
                                         gboolean scroll_to_item);
static gboolean mail_source_list_tree_scroll_to_item (MailSourceListTree* self,
                                               MailSourceListItem* item,
                                               gboolean use_align,
                                               gfloat row_align);
static void mail_source_list_tree_real_cursor_changed (GtkTreeView* base);
static void mail_source_list_tree_stop_editing (MailSourceListTree* self);
static void mail_source_list_tree_on_editing_started (MailSourceListTree* self,
                                               GtkCellEditable* editable,
                                               const gchar* path);
static void mail_source_list_tree_on_editing_done (MailSourceListTree* self);
static void _mail_source_list_tree_on_editing_done_gtk_cell_editable_editing_done (GtkCellEditable* _sender,
                                                                            gpointer self);
static void mail_source_list_tree_on_editing_canceled (MailSourceListTree* self);
static void mail_source_list_tree_on_activatable_activated (MailSourceListTree* self,
                                                     const gchar* item_path_str);
static MailSourceListItem* mail_source_list_tree_get_item_from_path_string (MailSourceListTree* self,
                                                                     const gchar* item_path_str);
static gboolean mail_source_list_tree_toggle_expansion (MailSourceListTree* self,
                                                 MailSourceListExpandableItem* item);
static void mail_source_list_tree_real_row_expanded (GtkTreeView* base,
                                              GtkTreeIter* iter,
                                              GtkTreePath* path);
static void mail_source_list_tree_real_row_collapsed (GtkTreeView* base,
                                               GtkTreeIter* iter,
                                               GtkTreePath* path);
static void mail_source_list_tree_real_row_activated (GtkTreeView* base,
                                               GtkTreePath* path,
                                               GtkTreeViewColumn* column);
static gboolean mail_source_list_tree_real_key_release_event (GtkWidget* base,
                                                       GdkEventKey* event);
static gboolean mail_source_list_tree_real_button_release_event (GtkWidget* base,
                                                          GdkEventButton* event);
static gboolean mail_source_list_tree_real_button_press_event (GtkWidget* base,
                                                        GdkEventButton* event);
static gboolean mail_source_list_tree_popup_context_menu (MailSourceListTree* self,
                                                   MailSourceListItem* item,
                                                   GdkEventButton* event);
static gboolean mail_source_list_tree_over_primary_expander (MailSourceListTree* self,
                                                      GtkTreeViewColumn* col,
                                                      GtkTreePath* path,
                                                      gint x);
static void mail_source_list_tree_expander_cell_data_func (MailSourceListTree* self,
                                                    GtkCellLayout* layout,
                                                    GtkCellRenderer* renderer,
                                                    GtkTreeModel* model,
                                                    GtkTreeIter* iter);
static gint mail_source_list_tree_get_cell_width (MailSourceListTree* self,
                                           GtkCellRenderer* cell_renderer);
static gboolean mail_source_list_tree_real_popup_menu (GtkWidget* base);
static MailSourceListItem* mail_source_list_tree_get_item_from_model (GtkTreeModel* model,
                                                               GtkTreeIter* iter);
static void mail_source_list_tree_name_cell_data_func (MailSourceListTree* self,
                                                GtkCellLayout* layout,
                                                GtkCellRenderer* renderer,
                                                GtkTreeModel* model,
                                                GtkTreeIter* iter);
static void mail_source_list_tree_badge_cell_data_func (MailSourceListTree* self,
                                                 GtkCellLayout* layout,
                                                 GtkCellRenderer* renderer,
                                                 GtkTreeModel* model,
                                                 GtkTreeIter* iter);
VALA_EXTERN void mail_cell_renderer_badge_set_text (MailCellRendererBadge* self,
                                        const gchar* value);
static void mail_source_list_tree_icon_cell_data_func (MailSourceListTree* self,
                                                GtkCellLayout* layout,
                                                GtkCellRenderer* renderer,
                                                GtkTreeModel* model,
                                                GtkTreeIter* iter);
static void mail_source_list_tree_set_data_model (MailSourceListTree* self,
                                           MailSourceListDataModel* value);
static GObject * mail_source_list_tree_constructor (GType type,
                                             guint n_construct_properties,
                                             GObjectConstructParam * construct_properties);
VALA_EXTERN MailCellRendererExpander* mail_cell_renderer_expander_new (void);
VALA_EXTERN MailCellRendererExpander* mail_cell_renderer_expander_construct (GType object_type);
VALA_EXTERN void mail_cell_renderer_expander_set_is_category_expander (MailCellRendererExpander* self,
                                                           gboolean value);
static void _mail_source_list_tree_expander_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                               GtkCellRenderer* cell,
                                                                               GtkTreeModel* tree_model,
                                                                               GtkTreeIter* iter,
                                                                               gpointer self);
static void _mail_source_list_tree_on_activatable_activated_mail_source_list_cell_renderer_icon_activated (MailSourceListCellRendererIcon* _sender,
                                                                                                    const gchar* path,
                                                                                                    gpointer self);
static void _mail_source_list_tree_icon_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                           GtkCellRenderer* cell,
                                                                           GtkTreeModel* tree_model,
                                                                           GtkTreeIter* iter,
                                                                           gpointer self);
VALA_EXTERN MailCellRendererBadge* mail_cell_renderer_badge_new (void);
VALA_EXTERN MailCellRendererBadge* mail_cell_renderer_badge_construct (GType object_type);
static void _mail_source_list_tree_badge_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                            GtkCellRenderer* cell,
                                                                            GtkTreeModel* tree_model,
                                                                            GtkTreeIter* iter,
                                                                            gpointer self);
static void _mail_source_list_tree_on_editing_started_gtk_cell_renderer_editing_started (GtkCellRenderer* _sender,
                                                                                  GtkCellEditable* editable,
                                                                                  const gchar* path,
                                                                                  gpointer self);
static void _mail_source_list_tree_on_editing_canceled_gtk_cell_renderer_editing_canceled (GtkCellRenderer* _sender,
                                                                                    gpointer self);
static void _mail_source_list_tree_name_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                           GtkCellRenderer* cell,
                                                                           GtkTreeModel* tree_model,
                                                                           GtkTreeIter* iter,
                                                                           gpointer self);
static gboolean _mail_source_list_tree_select_func_gtk_tree_selection_func (GtkTreeSelection* selection,
                                                                     GtkTreeModel* model,
                                                                     GtkTreePath* path,
                                                                     gboolean path_currently_selected,
                                                                     gpointer self);
static gboolean _mail_source_list_tree_on_query_tooltip_gtk_widget_query_tooltip (GtkWidget* _sender,
                                                                           gint x,
                                                                           gint y,
                                                                           gboolean keyboard_tooltip,
                                                                           GtkTooltip* tooltip,
                                                                           gpointer self);
static void mail_source_list_tree_finalize (GObject * obj);
static GType mail_source_list_tree_get_type_once (void);
static void _vala_mail_source_list_tree_get_property (GObject * object,
                                               guint property_id,
                                               GValue * value,
                                               GParamSpec * pspec);
static void _vala_mail_source_list_tree_set_property (GObject * object,
                                               guint property_id,
                                               const GValue * value,
                                               GParamSpec * pspec);
static GObject * mail_source_list_constructor (GType type,
                                        guint n_construct_properties,
                                        GObjectConstructParam * construct_properties);
static void _mail_source_list___lambda5_ (MailSourceList* self,
                                   MailSourceListItem* item);
static void __mail_source_list___lambda5__mail_source_list_tree_item_selected (MailSourceListTree* _sender,
                                                                        MailSourceListItem* item,
                                                                        gpointer self);
static void mail_source_list_finalize (GObject * obj);
static GType mail_source_list_get_type_once (void);
static void _vala_mail_source_list_get_property (GObject * object,
                                          guint property_id,
                                          GValue * value,
                                          GParamSpec * pspec);
static void _vala_mail_source_list_set_property (GObject * object,
                                          guint property_id,
                                          const GValue * value,
                                          GParamSpec * pspec);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

/**
     * Whether this item will allow users to re-arrange its children via DnD.
     *
     * This feature can co-exist with a sort algorithm (implemented
     * by {@link Granite.Widgets.SourceListSortable.compare}), but
     * the actual order of the items in the list will always
     * honor that method. The sort function has to be compatible with
     * the kind of DnD reordering the item wants to allow, since the user can
     * only reorder those items for which //compare// returns 0.
     *
     * @return Whether the item's children can be re-arranged by users.
     * @since 0.3
     */
gboolean
mail_source_list_sortable_allow_dnd_sorting (MailSourceListSortable* self)
{
	MailSourceListSortableIface* _iface_;
#line 34 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 34 "../src/SourceList/SourceList.vala"
	_iface_ = MAIL_SOURCE_LIST_SORTABLE_GET_INTERFACE (self);
#line 34 "../src/SourceList/SourceList.vala"
	if (_iface_->allow_dnd_sorting) {
#line 34 "../src/SourceList/SourceList.vala"
		return _iface_->allow_dnd_sorting (self);
#line 1183 "SourceList.c"
	}
#line 34 "../src/SourceList/SourceList.vala"
	return FALSE;
#line 1187 "SourceList.c"
}

/**
     * Should return a negative integer, zero, or a positive integer if ''a''
     * sorts //before// ''b'', ''a'' sorts //with// ''b'', or ''a'' sorts
     * //after// ''b'' respectively. If two items compare as equal, their
     * order in the sorted source list is undefined.
     *
     * In order to ensure that the source list behaves as expected, this
     * method must define a partial order on the source list tree; i.e. it
     * must be reflexive, antisymmetric and transitive. Not complying with
     * those requirements could make the program fall into an infinite loop
     * and freeze the user interface.
     *
     * Should return //0// to allow any pair of items to be sortable via DnD.
     *
     * @param a First item.
     * @param b Second item.
     * @return A //negative// integer if //a// sorts before //b//,
     *         //zero// if //a// equals //b//, or a //positive//
     *         integer if //a// sorts after //b//.
     * @since 0.3
     */
gint
mail_source_list_sortable_compare (MailSourceListSortable* self,
                                   MailSourceListItem* a,
                                   MailSourceListItem* b)
{
	MailSourceListSortableIface* _iface_;
#line 57 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 57 "../src/SourceList/SourceList.vala"
	_iface_ = MAIL_SOURCE_LIST_SORTABLE_GET_INTERFACE (self);
#line 57 "../src/SourceList/SourceList.vala"
	if (_iface_->compare) {
#line 57 "../src/SourceList/SourceList.vala"
		return _iface_->compare (self, a, b);
#line 1225 "SourceList.c"
	}
#line 57 "../src/SourceList/SourceList.vala"
	return -1;
#line 1229 "SourceList.c"
}

static void
mail_source_list_sortable_default_init (MailSourceListSortableIface * iface,
                                        gpointer iface_data)
{
	/**
	     * Emitted after a user has re-ordered an item via DnD.
	     *
	     * @param moved The item that was moved to a different position by the user.
	     * @since 0.3
	     */
#line 12 "../src/SourceList/SourceList.vala"
	mail_source_list_sortable_signals[MAIL_SOURCE_LIST_SORTABLE_USER_MOVED_ITEM_SIGNAL] = g_signal_new ("user-moved-item", MAIL_TYPE_SOURCE_LIST_SORTABLE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, MAIL_SOURCE_LIST_TYPE_ITEM);
#line 1244 "SourceList.c"
}

/**
 * An interface for sorting items.
 *
 * @since 0.3
 */
static GType
mail_source_list_sortable_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListSortableIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_sortable_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType mail_source_list_sortable_type_id;
	mail_source_list_sortable_type_id = g_type_register_static (G_TYPE_INTERFACE, "MailSourceListSortable", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (mail_source_list_sortable_type_id, MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM);
	return mail_source_list_sortable_type_id;
}

GType
mail_source_list_sortable_get_type (void)
{
	static volatile gsize mail_source_list_sortable_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_sortable_type_id__once)) {
		GType mail_source_list_sortable_type_id;
		mail_source_list_sortable_type_id = mail_source_list_sortable_get_type_once ();
		g_once_init_leave (&mail_source_list_sortable_type_id__once, mail_source_list_sortable_type_id);
	}
	return mail_source_list_sortable_type_id__once;
}

/**
     * Determines whether this item can be dragged outside the source list widget.
     *
     * Even if this method returns //false//, the item could still be dragged around
     * within the source list if its parent allows DnD reordering. This only happens
     * when the parent implements {@link Granite.Widgets.SourceListSortable}.
     *
     * @return //true// if the item can be dragged; //false// otherwise.
     * @since 0.3
     * @see Granite.Widgets.SourceListSortable
     */
gboolean
mail_source_list_drag_source_draggable (MailSourceListDragSource* self)
{
	MailSourceListDragSourceIface* _iface_;
#line 77 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 77 "../src/SourceList/SourceList.vala"
	_iface_ = MAIL_SOURCE_LIST_DRAG_SOURCE_GET_INTERFACE (self);
#line 77 "../src/SourceList/SourceList.vala"
	if (_iface_->draggable) {
#line 77 "../src/SourceList/SourceList.vala"
		return _iface_->draggable (self);
#line 1297 "SourceList.c"
	}
#line 77 "../src/SourceList/SourceList.vala"
	return FALSE;
#line 1301 "SourceList.c"
}

/**
     * This method is called when the drop site requests the data which is dragged.
     *
     * It is the responsibility of this method to fill //selection_data// with the
     * data in the format which is indicated by {@link Gtk.SelectionData.get_target}.
     *
     * @param selection_data {@link Gtk.SelectionData} containing source data.
     * @since 0.3
     * @see Gtk.SelectionData.set
     * @see Gtk.SelectionData.set_uris
     * @see Gtk.SelectionData.set_text
     */
void
mail_source_list_drag_source_prepare_selection_data (MailSourceListDragSource* self,
                                                     GtkSelectionData* selection_data)
{
	MailSourceListDragSourceIface* _iface_;
#line 91 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 91 "../src/SourceList/SourceList.vala"
	_iface_ = MAIL_SOURCE_LIST_DRAG_SOURCE_GET_INTERFACE (self);
#line 91 "../src/SourceList/SourceList.vala"
	if (_iface_->prepare_selection_data) {
#line 91 "../src/SourceList/SourceList.vala"
		_iface_->prepare_selection_data (self, selection_data);
#line 1329 "SourceList.c"
	}
}

static void
mail_source_list_drag_source_default_init (MailSourceListDragSourceIface * iface,
                                           gpointer iface_data)
{
}

/**
 * An interface for dragging items out of the source list widget.
 *
 * @since 0.3
 */
static GType
mail_source_list_drag_source_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListDragSourceIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_drag_source_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType mail_source_list_drag_source_type_id;
	mail_source_list_drag_source_type_id = g_type_register_static (G_TYPE_INTERFACE, "MailSourceListDragSource", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (mail_source_list_drag_source_type_id, MAIL_SOURCE_LIST_TYPE_ITEM);
	return mail_source_list_drag_source_type_id;
}

GType
mail_source_list_drag_source_get_type (void)
{
	static volatile gsize mail_source_list_drag_source_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_drag_source_type_id__once)) {
		GType mail_source_list_drag_source_type_id;
		mail_source_list_drag_source_type_id = mail_source_list_drag_source_get_type_once ();
		g_once_init_leave (&mail_source_list_drag_source_type_id__once, mail_source_list_drag_source_type_id);
	}
	return mail_source_list_drag_source_type_id__once;
}

/**
     * Determines whether //data// can be dropped into this item.
     *
     * @param context The drag context.
     * @param data {@link Gtk.SelectionData} containing source data.
     * @return //true// if the drop is possible; //false// otherwise.
     * @since 0.3
     */
gboolean
mail_source_list_drag_dest_data_drop_possible (MailSourceListDragDest* self,
                                               GdkDragContext* context,
                                               GtkSelectionData* data)
{
	MailSourceListDragDestIface* _iface_;
#line 108 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 108 "../src/SourceList/SourceList.vala"
	_iface_ = MAIL_SOURCE_LIST_DRAG_DEST_GET_INTERFACE (self);
#line 108 "../src/SourceList/SourceList.vala"
	if (_iface_->data_drop_possible) {
#line 108 "../src/SourceList/SourceList.vala"
		return _iface_->data_drop_possible (self, context, data);
#line 1388 "SourceList.c"
	}
#line 108 "../src/SourceList/SourceList.vala"
	return FALSE;
#line 1392 "SourceList.c"
}

/**
     * If a data drop is deemed possible, then this method is called
     * when the data is actually dropped into this item. Any actions
     * consequence of the data received should be handled here.
     *
     * @param context The drag context.
     * @param data {@link Gtk.SelectionData} containing source data.
     * @return The action taken, or //0// to indicate that the dropped data was not accepted.
     * @since 0.3
     */
GdkDragAction
mail_source_list_drag_dest_data_received (MailSourceListDragDest* self,
                                          GdkDragContext* context,
                                          GtkSelectionData* data)
{
	MailSourceListDragDestIface* _iface_;
#line 120 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 120 "../src/SourceList/SourceList.vala"
	_iface_ = MAIL_SOURCE_LIST_DRAG_DEST_GET_INTERFACE (self);
#line 120 "../src/SourceList/SourceList.vala"
	if (_iface_->data_received) {
#line 120 "../src/SourceList/SourceList.vala"
		return _iface_->data_received (self, context, data);
#line 1419 "SourceList.c"
	}
#line 120 "../src/SourceList/SourceList.vala"
	return 0U;
#line 1423 "SourceList.c"
}

static void
mail_source_list_drag_dest_default_init (MailSourceListDragDestIface * iface,
                                         gpointer iface_data)
{
}

/**
 * An interface for receiving data from other widgets via drag-and-drop.
 *
 * @since 0.3
 */
static GType
mail_source_list_drag_dest_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListDragDestIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_drag_dest_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType mail_source_list_drag_dest_type_id;
	mail_source_list_drag_dest_type_id = g_type_register_static (G_TYPE_INTERFACE, "MailSourceListDragDest", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (mail_source_list_drag_dest_type_id, MAIL_SOURCE_LIST_TYPE_ITEM);
	return mail_source_list_drag_dest_type_id;
}

GType
mail_source_list_drag_dest_get_type (void)
{
	static volatile gsize mail_source_list_drag_dest_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_drag_dest_type_id__once)) {
		GType mail_source_list_drag_dest_type_id;
		mail_source_list_drag_dest_type_id = mail_source_list_drag_dest_get_type_once ();
		g_once_init_leave (&mail_source_list_drag_dest_type_id__once, mail_source_list_drag_dest_type_id);
	}
	return mail_source_list_drag_dest_type_id__once;
}

static inline gpointer
mail_source_list_get_instance_private (MailSourceList* self)
{
	return G_STRUCT_MEMBER_P (self, MailSourceList_private_offset);
}

/**
     * Creates a new {@link Granite.Widgets.SourceList}.
     *
     * @return A new {@link Granite.Widgets.SourceList}.
     * @since 0.2
     */
MailSourceList*
mail_source_list_construct (GType object_type,
                            MailSourceListExpandableItem* root)
{
	MailSourceList * self = NULL;
#line 2548 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (root != NULL, NULL);
#line 2548 "../src/SourceList/SourceList.vala"
	self = (MailSourceList*) g_object_new (object_type, NULL);
#line 2549 "../src/SourceList/SourceList.vala"
	mail_source_list_set_root (self, root);
#line 2548 "../src/SourceList/SourceList.vala"
	return self;
#line 1484 "SourceList.c"
}

MailSourceList*
mail_source_list_new (MailSourceListExpandableItem* root)
{
#line 2548 "../src/SourceList/SourceList.vala"
	return mail_source_list_construct (MAIL_TYPE_SOURCE_LIST, root);
#line 1492 "SourceList.c"
}

/**
     * Checks whether //item// is part of the source list.
     *
     * @param item The item to query.
     * @return //true// if the item belongs to the source list; //false// otherwise.
     * @since 0.2
     */
gboolean
mail_source_list_has_item (MailSourceList* self,
                           MailSourceListItem* item)
{
	MailSourceListDataModel* _tmp0_;
	gboolean result;
#line 2569 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2569 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2570 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2570 "../src/SourceList/SourceList.vala"
	result = mail_source_list_data_model_has_item (_tmp0_, item);
#line 2570 "../src/SourceList/SourceList.vala"
	return result;
#line 1518 "SourceList.c"
}

/**
     * Sets the method used for filtering out items.
     *
     * @param visible_func The method to use for filtering items.
     * @param refilter Whether to call {@link Granite.Widgets.SourceList.refilter} using the new function.
     * @see Granite.Widgets.SourceList.VisibleFunc
     * @see Granite.Widgets.SourceList.refilter
     * @since 0.2
     */
void
mail_source_list_set_filter_func (MailSourceList* self,
                                  MailSourceListVisibleFunc visible_func,
                                  gpointer visible_func_target,
                                  gboolean refilter)
{
	MailSourceListDataModel* _tmp0_;
#line 2582 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2583 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2583 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_set_filter_func (_tmp0_, visible_func, visible_func_target);
#line 2584 "../src/SourceList/SourceList.vala"
	if (refilter) {
#line 2585 "../src/SourceList/SourceList.vala"
		mail_source_list_refilter (self);
#line 1547 "SourceList.c"
	}
}

void
mail_source_list_start_editing_item (MailSourceList* self,
                                     MailSourceListItem* item)
{
	MailSourceListTree* _tmp0_;
#line 2588 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2588 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 2589 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2589 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_start_editing_item (_tmp0_, item);
#line 1564 "SourceList.c"
}

/**
     * Applies the filter method set by {@link Granite.Widgets.SourceList.set_filter_func}
     * to all the items that are part of the current tree.
     *
     * @see Granite.Widgets.SourceList.VisibleFunc
     * @see Granite.Widgets.SourceList.set_filter_func
     * @since 0.2
     */
static void
mail_source_list_refilter (MailSourceList* self)
{
	MailSourceListDataModel* _tmp0_;
#line 2600 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2601 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2601 "../src/SourceList/SourceList.vala"
	gtk_tree_model_filter_refilter ((GtkTreeModelFilter*) _tmp0_);
#line 1585 "SourceList.c"
}

MailSourceListExpandableItem*
mail_source_list_get_root (MailSourceList* self)
{
	MailSourceListExpandableItem* result;
	MailSourceListDataModel* _tmp0_;
	MailSourceListExpandableItem* _tmp1_;
	MailSourceListExpandableItem* _tmp2_;
#line 2497 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2497 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2497 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_data_model_get_root (_tmp0_);
#line 2497 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2497 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2497 "../src/SourceList/SourceList.vala"
	return result;
#line 1607 "SourceList.c"
}

void
mail_source_list_set_root (MailSourceList* self,
                           MailSourceListExpandableItem* value)
{
	MailSourceListDataModel* _tmp0_;
#line 2498 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2498 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2498 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_set_root (_tmp0_, value);
#line 2498 "../src/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, mail_source_list_properties[MAIL_SOURCE_LIST_ROOT_PROPERTY]);
#line 1623 "SourceList.c"
}

MailSourceListItem*
mail_source_list_get_selected (MailSourceList* self)
{
	MailSourceListItem* result;
	MailSourceListTree* _tmp0_;
	MailSourceListItem* _tmp1_;
	MailSourceListItem* _tmp2_;
#line 2511 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2511 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2511 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_tree_get_selected_item (_tmp0_);
#line 2511 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2511 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2511 "../src/SourceList/SourceList.vala"
	return result;
#line 1645 "SourceList.c"
}

void
mail_source_list_set_selected (MailSourceList* self,
                               MailSourceListItem* value)
{
	gboolean _tmp0_ = FALSE;
	MailSourceListTree* _tmp5_;
#line 2512 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2513 "../src/SourceList/SourceList.vala"
	if (value != NULL) {
#line 1658 "SourceList.c"
		MailSourceListExpandableItem* _tmp1_;
		MailSourceListExpandableItem* _tmp2_;
#line 2513 "../src/SourceList/SourceList.vala"
		_tmp1_ = mail_source_list_item_get_parent (value);
#line 2513 "../src/SourceList/SourceList.vala"
		_tmp2_ = _tmp1_;
#line 2513 "../src/SourceList/SourceList.vala"
		_tmp0_ = _tmp2_ != NULL;
#line 1667 "SourceList.c"
	} else {
#line 2513 "../src/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 1671 "SourceList.c"
	}
#line 2513 "../src/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 1675 "SourceList.c"
		MailSourceListExpandableItem* _tmp3_;
		MailSourceListExpandableItem* _tmp4_;
#line 2514 "../src/SourceList/SourceList.vala"
		_tmp3_ = mail_source_list_item_get_parent (value);
#line 2514 "../src/SourceList/SourceList.vala"
		_tmp4_ = _tmp3_;
#line 2514 "../src/SourceList/SourceList.vala"
		mail_source_list_expandable_item_expand_with_parents (_tmp4_);
#line 1684 "SourceList.c"
	}
#line 2515 "../src/SourceList/SourceList.vala"
	_tmp5_ = self->priv->tree;
#line 2515 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_set_selected_item (_tmp5_, value);
#line 2512 "../src/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, mail_source_list_properties[MAIL_SOURCE_LIST_SELECTED_PROPERTY]);
#line 1692 "SourceList.c"
}

static PangoEllipsizeMode
mail_source_list_get_ellipsize_mode (MailSourceList* self)
{
	PangoEllipsizeMode result;
	MailSourceListTree* _tmp0_;
	PangoEllipsizeMode _tmp1_;
	PangoEllipsizeMode _tmp2_;
#line 2525 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 2525 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2525 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_tree_get_ellipsize_mode (_tmp0_);
#line 2525 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2525 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2525 "../src/SourceList/SourceList.vala"
	return result;
#line 1714 "SourceList.c"
}

static void
mail_source_list_set_ellipsize_mode (MailSourceList* self,
                                     PangoEllipsizeMode value)
{
	MailSourceListTree* _tmp0_;
#line 2526 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2526 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2526 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_set_ellipsize_mode (_tmp0_, value);
#line 1728 "SourceList.c"
}

static gboolean
mail_source_list_get_editing (MailSourceList* self)
{
	gboolean result;
	MailSourceListTree* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 2536 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2536 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2536 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_tree_get_editing (_tmp0_);
#line 2536 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2536 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2536 "../src/SourceList/SourceList.vala"
	return result;
#line 1750 "SourceList.c"
}

static void
mail_source_list_real_item_selected (MailSourceList* self,
                                     MailSourceListItem* item)
{
}

static inline gpointer
mail_source_list_item_get_instance_private (MailSourceListItem* self)
{
	return G_STRUCT_MEMBER_P (self, MailSourceListItem_private_offset);
}

/**
         * Creates a new {@link Granite.Widgets.SourceList.Item}.
         *
         * @param name Name of the item.
         * @return (transfer full) A new {@link Granite.Widgets.SourceList.Item}.
         * @since 0.2
         */
static MailSourceListItem*
mail_source_list_item_construct (GType object_type,
                                 const gchar* name)
{
	MailSourceListItem * self = NULL;
#line 412 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 412 "../src/SourceList/SourceList.vala"
	self = (MailSourceListItem*) g_object_new (object_type, NULL);
#line 413 "../src/SourceList/SourceList.vala"
	mail_source_list_item_set_name (self, name);
#line 412 "../src/SourceList/SourceList.vala"
	return self;
#line 1785 "SourceList.c"
}

static MailSourceListItem*
mail_source_list_item_new (const gchar* name)
{
#line 412 "../src/SourceList/SourceList.vala"
	return mail_source_list_item_construct (MAIL_SOURCE_LIST_TYPE_ITEM, name);
#line 1793 "SourceList.c"
}

/**
         * Invoked when the item is secondary-clicked or when the usual menu keys are pressed.
         *
         * Note that since Granite 5.0, right clicking on an item no longer selects/activates it, so
         * any context menu items should be actioned on the item instance rather than the selected item
         * in the SourceList
         *
         * @return A {@link Gtk.Menu} or //null// if nothing should be displayed.
         * @since 0.2
         */
static GtkMenu*
mail_source_list_item_real_get_context_menu (MailSourceListItem* self)
{
	GtkMenu* result;
#line 427 "../src/SourceList/SourceList.vala"
	result = NULL;
#line 427 "../src/SourceList/SourceList.vala"
	return result;
#line 1814 "SourceList.c"
}

GtkMenu*
mail_source_list_item_get_context_menu (MailSourceListItem* self)
{
	MailSourceListItemClass* _klass_;
#line 426 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 426 "../src/SourceList/SourceList.vala"
	_klass_ = MAIL_SOURCE_LIST_ITEM_GET_CLASS (self);
#line 426 "../src/SourceList/SourceList.vala"
	if (_klass_->get_context_menu) {
#line 426 "../src/SourceList/SourceList.vala"
		return _klass_->get_context_menu (self);
#line 1829 "SourceList.c"
	}
#line 426 "../src/SourceList/SourceList.vala"
	return NULL;
#line 1833 "SourceList.c"
}

MailSourceListExpandableItem*
mail_source_list_item_get_parent (MailSourceListItem* self)
{
	MailSourceListExpandableItem* result;
	MailSourceListExpandableItem* _tmp0_;
#line 304 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 304 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_parent;
#line 304 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 304 "../src/SourceList/SourceList.vala"
	return result;
#line 1849 "SourceList.c"
}

static gpointer
_g_object_ref0 (gpointer self)
{
#line 304 "../src/SourceList/SourceList.vala"
	return self ? g_object_ref (self) : NULL;
#line 1857 "SourceList.c"
}

void
mail_source_list_item_set_parent (MailSourceListItem* self,
                                  MailSourceListExpandableItem* value)
{
	MailSourceListExpandableItem* old_value;
#line 304 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 304 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_parent (self);
#line 304 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 1871 "SourceList.c"
		MailSourceListExpandableItem* _tmp0_;
#line 304 "../src/SourceList/SourceList.vala"
		_tmp0_ = _g_object_ref0 (value);
#line 304 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->_parent);
#line 304 "../src/SourceList/SourceList.vala"
		self->priv->_parent = _tmp0_;
#line 304 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_PARENT_PROPERTY]);
#line 1881 "SourceList.c"
	}
}

const gchar*
mail_source_list_item_get_name (MailSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 311 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 311 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_name;
#line 311 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 311 "../src/SourceList/SourceList.vala"
	return result;
#line 1898 "SourceList.c"
}

void
mail_source_list_item_set_name (MailSourceListItem* self,
                                const gchar* value)
{
	gchar* old_value;
#line 311 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 311 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_name (self);
#line 311 "../src/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 1912 "SourceList.c"
		gchar* _tmp0_;
#line 311 "../src/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 311 "../src/SourceList/SourceList.vala"
		_g_free0 (self->priv->_name);
#line 311 "../src/SourceList/SourceList.vala"
		self->priv->_name = _tmp0_;
#line 311 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_NAME_PROPERTY]);
#line 1922 "SourceList.c"
	}
}

const gchar*
mail_source_list_item_get_tooltip (MailSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 319 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 319 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_tooltip;
#line 319 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 319 "../src/SourceList/SourceList.vala"
	return result;
#line 1939 "SourceList.c"
}

void
mail_source_list_item_set_tooltip (MailSourceListItem* self,
                                   const gchar* value)
{
	gchar* old_value;
#line 319 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 319 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_tooltip (self);
#line 319 "../src/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 1953 "SourceList.c"
		gchar* _tmp0_;
#line 319 "../src/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 319 "../src/SourceList/SourceList.vala"
		_g_free0 (self->priv->_tooltip);
#line 319 "../src/SourceList/SourceList.vala"
		self->priv->_tooltip = _tmp0_;
#line 319 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY]);
#line 1963 "SourceList.c"
	}
}

const gchar*
mail_source_list_item_get_markup (MailSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 332 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 332 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_markup;
#line 332 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 332 "../src/SourceList/SourceList.vala"
	return result;
#line 1980 "SourceList.c"
}

void
mail_source_list_item_set_markup (MailSourceListItem* self,
                                  const gchar* value)
{
	gchar* old_value;
#line 332 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 332 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_markup (self);
#line 332 "../src/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 1994 "SourceList.c"
		gchar* _tmp0_;
#line 332 "../src/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 332 "../src/SourceList/SourceList.vala"
		_g_free0 (self->priv->_markup);
#line 332 "../src/SourceList/SourceList.vala"
		self->priv->_markup = _tmp0_;
#line 332 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_MARKUP_PROPERTY]);
#line 2004 "SourceList.c"
	}
}

const gchar*
mail_source_list_item_get_badge (MailSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 342 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 342 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_badge;
#line 342 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 342 "../src/SourceList/SourceList.vala"
	return result;
#line 2021 "SourceList.c"
}

void
mail_source_list_item_set_badge (MailSourceListItem* self,
                                 const gchar* value)
{
	gchar* old_value;
#line 342 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 342 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_badge (self);
#line 342 "../src/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 2035 "SourceList.c"
		gchar* _tmp0_;
#line 342 "../src/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 342 "../src/SourceList/SourceList.vala"
		_g_free0 (self->priv->_badge);
#line 342 "../src/SourceList/SourceList.vala"
		self->priv->_badge = _tmp0_;
#line 342 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_BADGE_PROPERTY]);
#line 2045 "SourceList.c"
	}
}

gboolean
mail_source_list_item_get_editable (MailSourceListItem* self)
{
	gboolean result;
#line 356 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 356 "../src/SourceList/SourceList.vala"
	result = self->priv->_editable;
#line 356 "../src/SourceList/SourceList.vala"
	return result;
#line 2059 "SourceList.c"
}

void
mail_source_list_item_set_editable (MailSourceListItem* self,
                                    gboolean value)
{
	gboolean old_value;
#line 356 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 356 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_editable (self);
#line 356 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 356 "../src/SourceList/SourceList.vala"
		self->priv->_editable = value;
#line 356 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_EDITABLE_PROPERTY]);
#line 2077 "SourceList.c"
	}
}

gboolean
mail_source_list_item_get_visible (MailSourceListItem* self)
{
	gboolean result;
#line 363 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 363 "../src/SourceList/SourceList.vala"
	result = self->priv->_visible;
#line 363 "../src/SourceList/SourceList.vala"
	return result;
#line 2091 "SourceList.c"
}

void
mail_source_list_item_set_visible (MailSourceListItem* self,
                                   gboolean value)
{
	gboolean old_value;
#line 363 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 363 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_visible (self);
#line 363 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 363 "../src/SourceList/SourceList.vala"
		self->priv->_visible = value;
#line 363 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_VISIBLE_PROPERTY]);
#line 2109 "SourceList.c"
	}
}

gboolean
mail_source_list_item_get_selectable (MailSourceListItem* self)
{
	gboolean result;
#line 376 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 376 "../src/SourceList/SourceList.vala"
	result = self->priv->_selectable;
#line 376 "../src/SourceList/SourceList.vala"
	return result;
#line 2123 "SourceList.c"
}

void
mail_source_list_item_set_selectable (MailSourceListItem* self,
                                      gboolean value)
{
	gboolean old_value;
#line 376 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 376 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_selectable (self);
#line 376 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 376 "../src/SourceList/SourceList.vala"
		self->priv->_selectable = value;
#line 376 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY]);
#line 2141 "SourceList.c"
	}
}

GIcon*
mail_source_list_item_get_icon (MailSourceListItem* self)
{
	GIcon* result;
	GIcon* _tmp0_;
#line 386 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 386 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_icon;
#line 386 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 386 "../src/SourceList/SourceList.vala"
	return result;
#line 2158 "SourceList.c"
}

void
mail_source_list_item_set_icon (MailSourceListItem* self,
                                GIcon* value)
{
	GIcon* old_value;
#line 386 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 386 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_icon (self);
#line 386 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 2172 "SourceList.c"
		GIcon* _tmp0_;
#line 386 "../src/SourceList/SourceList.vala"
		_tmp0_ = _g_object_ref0 (value);
#line 386 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->_icon);
#line 386 "../src/SourceList/SourceList.vala"
		self->priv->_icon = _tmp0_;
#line 386 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_ICON_PROPERTY]);
#line 2182 "SourceList.c"
	}
}

GIcon*
mail_source_list_item_get_activatable (MailSourceListItem* self)
{
	GIcon* result;
	GIcon* _tmp0_;
#line 396 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 396 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_activatable;
#line 396 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 396 "../src/SourceList/SourceList.vala"
	return result;
#line 2199 "SourceList.c"
}

void
mail_source_list_item_set_activatable (MailSourceListItem* self,
                                       GIcon* value)
{
	GIcon* old_value;
#line 396 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 396 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_activatable (self);
#line 396 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 2213 "SourceList.c"
		GIcon* _tmp0_;
#line 396 "../src/SourceList/SourceList.vala"
		_tmp0_ = _g_object_ref0 (value);
#line 396 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->_activatable);
#line 396 "../src/SourceList/SourceList.vala"
		self->priv->_activatable = _tmp0_;
#line 396 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY]);
#line 2223 "SourceList.c"
	}
}

const gchar*
mail_source_list_item_get_activatable_tooltip (MailSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 403 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 403 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_activatable_tooltip;
#line 403 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 403 "../src/SourceList/SourceList.vala"
	return result;
#line 2240 "SourceList.c"
}

void
mail_source_list_item_set_activatable_tooltip (MailSourceListItem* self,
                                               const gchar* value)
{
	gchar* old_value;
#line 403 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 403 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_item_get_activatable_tooltip (self);
#line 403 "../src/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 2254 "SourceList.c"
		gchar* _tmp0_;
#line 403 "../src/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 403 "../src/SourceList/SourceList.vala"
		_g_free0 (self->priv->_activatable_tooltip);
#line 403 "../src/SourceList/SourceList.vala"
		self->priv->_activatable_tooltip = _tmp0_;
#line 403 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY]);
#line 2264 "SourceList.c"
	}
}

static gchar*
string_strip (const gchar* self)
{
	gchar* _result_ = NULL;
	gchar* _tmp0_;
	gchar* result;
#line 1370 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1371 "glib-2.0.vapi"
	_tmp0_ = g_strdup (self);
#line 1371 "glib-2.0.vapi"
	_result_ = _tmp0_;
#line 1372 "glib-2.0.vapi"
	g_strstrip (_result_);
#line 1373 "glib-2.0.vapi"
	result = _result_;
#line 1373 "glib-2.0.vapi"
	return result;
#line 2286 "SourceList.c"
}

static void
mail_source_list_item_real_edited (MailSourceListItem* self,
                                   const gchar* new_name)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_;
#line 276 "../src/SourceList/SourceList.vala"
	g_return_if_fail (new_name != NULL);
#line 277 "../src/SourceList/SourceList.vala"
	_tmp1_ = self->priv->_editable;
#line 277 "../src/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 2301 "SourceList.c"
		gchar* _tmp2_;
		gchar* _tmp3_;
#line 277 "../src/SourceList/SourceList.vala"
		_tmp2_ = string_strip (new_name);
#line 277 "../src/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 277 "../src/SourceList/SourceList.vala"
		_tmp0_ = g_strcmp0 (_tmp3_, "") != 0;
#line 277 "../src/SourceList/SourceList.vala"
		_g_free0 (_tmp3_);
#line 2312 "SourceList.c"
	} else {
#line 277 "../src/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 2316 "SourceList.c"
	}
#line 277 "../src/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 278 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_name (self, new_name);
#line 2322 "SourceList.c"
	}
}

static void
mail_source_list_item_real_action_activated (MailSourceListItem* self)
{
}

static void
mail_source_list_item_real_activated (MailSourceListItem* self)
{
}

static void
mail_source_list_item_class_init (MailSourceListItemClass * klass,
                                  gpointer klass_data)
{
#line 265 "../src/SourceList/SourceList.vala"
	mail_source_list_item_parent_class = g_type_class_peek_parent (klass);
#line 265 "../src/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &MailSourceListItem_private_offset);
#line 265 "../src/SourceList/SourceList.vala"
	((MailSourceListItemClass *) klass)->get_context_menu = (GtkMenu* (*) (MailSourceListItem*)) mail_source_list_item_real_get_context_menu;
#line 265 "../src/SourceList/SourceList.vala"
	((MailSourceListItemClass *) klass)->edited = mail_source_list_item_real_edited;
#line 265 "../src/SourceList/SourceList.vala"
	((MailSourceListItemClass *) klass)->action_activated = mail_source_list_item_real_action_activated;
#line 265 "../src/SourceList/SourceList.vala"
	((MailSourceListItemClass *) klass)->activated = mail_source_list_item_real_activated;
#line 265 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_mail_source_list_item_get_property;
#line 265 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_mail_source_list_item_set_property;
#line 265 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = mail_source_list_item_finalize;
#line 2358 "SourceList.c"
	/**
	         * Parent {@link Granite.Widgets.SourceList.ExpandableItem} of the item.
	         * ''Must not'' be modified.
	         *
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_PARENT_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_PARENT_PROPERTY] = g_param_spec_object ("parent", "parent", "parent", MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2367 "SourceList.c"
	/**
	         * The item's name. Primary and most important information.
	         *
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_NAME_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_NAME_PROPERTY] = g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2375 "SourceList.c"
	/**
	         * The item's tooltip. If set to null (default), the tooltip for the item will be the
	         * contents of the {@link Granite.Widgets.SourceList.Item.name} property.
	         *
	         * @since 5.3
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY] = g_param_spec_string ("tooltip", "tooltip", "tooltip", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2384 "SourceList.c"
	/**
	         * Markup to be used instead of {@link Granite.Widgets.SourceList.ExpandableItem.name}
	         * This would mean that &, <, etc have to be escaped in the text, but basic formatting
	         * can be done on the item with HTML style tags.
	         *
	         * Note: Only the {@link Granite.Widgets.SourceList.ExpandableItem.name} property
	         * is modified for editable items. So this property will be need to updated and
	         * reformatted with editable items.
	         *
	         * @since 5.0
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_MARKUP_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_MARKUP_PROPERTY] = g_param_spec_string ("markup", "markup", "markup", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2398 "SourceList.c"
	/**
	         * A badge shown next to the item's name.
	         *
	         * It can be used for displaying the number of unread messages in the "Inbox" item,
	         * for instance.
	         *
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_BADGE_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_BADGE_PROPERTY] = g_param_spec_string ("badge", "badge", "badge", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2409 "SourceList.c"
	/**
	         * Whether the item's name can be edited from within the source list.
	         *
	         * When this property is set to //true//, users can edit the item by pressing
	         * the F2 key, or by double-clicking its name.
	         *
	         * ''This property only works for selectable items''.
	         *
	         * @see Granite.Widgets.SourceList.Item.selectable
	         * @see Granite.Widgets.SourceList.start_editing_item
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_EDITABLE_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_EDITABLE_PROPERTY] = g_param_spec_boolean ("editable", "editable", "editable", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2424 "SourceList.c"
	/**
	         * Whether the item should appear in the source list's tree or not.
	         *
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_VISIBLE_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_VISIBLE_PROPERTY] = g_param_spec_boolean ("visible", "visible", "visible", TRUE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2432 "SourceList.c"
	/**
	         * Whether the item can be selected or not.
	         *
	         * Setting this property to true doesn't guarantee that the item will actually be
	         * selectable, since there are other external factors to take into account, like the
	         * item's {@link Granite.Widgets.SourceList.Item.visible} property; whether the item is
	         * a category; the parent item is collapsed, etc.
	         *
	         * @see Granite.Widgets.SourceList.Item.visible
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY] = g_param_spec_boolean ("selectable", "selectable", "selectable", TRUE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2446 "SourceList.c"
	/**
	         * Primary icon.
	         *
	         * This property should be used to give the user an idea of what the item represents
	         * (i.e. content type.)
	         *
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_ICON_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_ICON_PROPERTY] = g_param_spec_object ("icon", "icon", "icon", g_icon_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2457 "SourceList.c"
	/**
	         * An activatable icon that works like a button.
	         *
	         * It can be used for e.g. showing an //"eject"// icon on a device's item.
	         *
	         * @see Granite.Widgets.SourceList.Item.action_activated
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY] = g_param_spec_object ("activatable", "activatable", "activatable", g_icon_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2468 "SourceList.c"
	/**
	         * The tooltip for the activatable icon.
	         *
	         * @since 5.0
	         */
#line 265 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY, mail_source_list_item_properties[MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY] = g_param_spec_string ("activatable-tooltip", "activatable-tooltip", "activatable-tooltip", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2476 "SourceList.c"
	/**
	         * Emitted when the user has finished editing the item's name.
	         *
	         * By default, if the name doesn't consist of white space, it is automatically assigned
	         * to the {@link Granite.Widgets.SourceList.Item.name} property. The default behavior can
	         * be changed by overriding this signal.
	         * @param new_name The item's new name (result of editing.)
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	mail_source_list_item_signals[MAIL_SOURCE_LIST_ITEM_EDITED_SIGNAL] = g_signal_new ("edited", MAIL_SOURCE_LIST_TYPE_ITEM, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (MailSourceListItemClass, edited), NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
#line 2488 "SourceList.c"
	/**
	         * The {@link Granite.Widgets.SourceList.Item.activatable} icon was activated.
	         *
	         * @see Granite.Widgets.SourceList.Item.activatable
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	mail_source_list_item_signals[MAIL_SOURCE_LIST_ITEM_ACTION_ACTIVATED_SIGNAL] = g_signal_new ("action-activated", MAIL_SOURCE_LIST_TYPE_ITEM, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (MailSourceListItemClass, action_activated), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 2497 "SourceList.c"
	/**
	         * Emitted when the item is double-clicked or when it is selected and one of the keys:
	         * Space, Shift+Space, Return or Enter is pressed. This signal is //also// for
	         * editable items.
	         *
	         * @since 0.2
	         */
#line 265 "../src/SourceList/SourceList.vala"
	mail_source_list_item_signals[MAIL_SOURCE_LIST_ITEM_ACTIVATED_SIGNAL] = g_signal_new ("activated", MAIL_SOURCE_LIST_TYPE_ITEM, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (MailSourceListItemClass, activated), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 2507 "SourceList.c"
}

static void
mail_source_list_item_instance_init (MailSourceListItem * self,
                                     gpointer klass)
{
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
#line 265 "../src/SourceList/SourceList.vala"
	self->priv = mail_source_list_item_get_instance_private (self);
#line 311 "../src/SourceList/SourceList.vala"
	_tmp0_ = g_strdup ("");
#line 311 "../src/SourceList/SourceList.vala"
	self->priv->_name = _tmp0_;
#line 319 "../src/SourceList/SourceList.vala"
	self->priv->_tooltip = NULL;
#line 332 "../src/SourceList/SourceList.vala"
	self->priv->_markup = NULL;
#line 342 "../src/SourceList/SourceList.vala"
	_tmp1_ = g_strdup ("");
#line 342 "../src/SourceList/SourceList.vala"
	self->priv->_badge = _tmp1_;
#line 356 "../src/SourceList/SourceList.vala"
	self->priv->_editable = FALSE;
#line 363 "../src/SourceList/SourceList.vala"
	self->priv->_visible = TRUE;
#line 376 "../src/SourceList/SourceList.vala"
	self->priv->_selectable = TRUE;
#line 403 "../src/SourceList/SourceList.vala"
	_tmp2_ = g_strdup ("");
#line 403 "../src/SourceList/SourceList.vala"
	self->priv->_activatable_tooltip = _tmp2_;
#line 2541 "SourceList.c"
}

static void
mail_source_list_item_finalize (GObject * obj)
{
	MailSourceListItem * self;
#line 265 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_ITEM, MailSourceListItem);
#line 304 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_parent);
#line 311 "../src/SourceList/SourceList.vala"
	_g_free0 (self->priv->_name);
#line 319 "../src/SourceList/SourceList.vala"
	_g_free0 (self->priv->_tooltip);
#line 332 "../src/SourceList/SourceList.vala"
	_g_free0 (self->priv->_markup);
#line 342 "../src/SourceList/SourceList.vala"
	_g_free0 (self->priv->_badge);
#line 386 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_icon);
#line 396 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_activatable);
#line 403 "../src/SourceList/SourceList.vala"
	_g_free0 (self->priv->_activatable_tooltip);
#line 265 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (mail_source_list_item_parent_class)->finalize (obj);
#line 2568 "SourceList.c"
}

/**
     * A source list entry.
     *
     * Any change made to any of its properties will be ''automatically'' reflected
     * by the {@link Granite.Widgets.SourceList} widget.
     *
     * @since 0.2
     */
static GType
mail_source_list_item_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceListItem), 0, (GInstanceInitFunc) mail_source_list_item_instance_init, NULL };
	GType mail_source_list_item_type_id;
	mail_source_list_item_type_id = g_type_register_static (G_TYPE_OBJECT, "MailSourceListItem", &g_define_type_info, 0);
	MailSourceListItem_private_offset = g_type_add_instance_private (mail_source_list_item_type_id, sizeof (MailSourceListItemPrivate));
	return mail_source_list_item_type_id;
}

GType
mail_source_list_item_get_type (void)
{
	static volatile gsize mail_source_list_item_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_item_type_id__once)) {
		GType mail_source_list_item_type_id;
		mail_source_list_item_type_id = mail_source_list_item_get_type_once ();
		g_once_init_leave (&mail_source_list_item_type_id__once, mail_source_list_item_type_id);
	}
	return mail_source_list_item_type_id__once;
}

static void
_vala_mail_source_list_item_get_property (GObject * object,
                                          guint property_id,
                                          GValue * value,
                                          GParamSpec * pspec)
{
	MailSourceListItem * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_ITEM, MailSourceListItem);
#line 265 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_PARENT_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_object (value, mail_source_list_item_get_parent (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_NAME_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_string (value, mail_source_list_item_get_name (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_string (value, mail_source_list_item_get_tooltip (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_MARKUP_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_string (value, mail_source_list_item_get_markup (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_BADGE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_string (value, mail_source_list_item_get_badge (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_EDITABLE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_boolean (value, mail_source_list_item_get_editable (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_VISIBLE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_boolean (value, mail_source_list_item_get_visible (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_boolean (value, mail_source_list_item_get_selectable (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_ICON_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_object (value, mail_source_list_item_get_icon (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_object (value, mail_source_list_item_get_activatable (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		g_value_set_string (value, mail_source_list_item_get_activatable_tooltip (self));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 2677 "SourceList.c"
		default:
#line 265 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 2683 "SourceList.c"
	}
}

static void
_vala_mail_source_list_item_set_property (GObject * object,
                                          guint property_id,
                                          const GValue * value,
                                          GParamSpec * pspec)
{
	MailSourceListItem * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_ITEM, MailSourceListItem);
#line 265 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_PARENT_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_parent (self, g_value_get_object (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_NAME_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_name (self, g_value_get_string (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_tooltip (self, g_value_get_string (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_MARKUP_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_markup (self, g_value_get_string (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_BADGE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_badge (self, g_value_get_string (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_EDITABLE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_editable (self, g_value_get_boolean (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_VISIBLE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_visible (self, g_value_get_boolean (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_selectable (self, g_value_get_boolean (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_ICON_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_icon (self, g_value_get_object (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_activatable (self, g_value_get_object (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 265 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY:
#line 265 "../src/SourceList/SourceList.vala"
		mail_source_list_item_set_activatable_tooltip (self, g_value_get_string (value));
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 2763 "SourceList.c"
		default:
#line 265 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 265 "../src/SourceList/SourceList.vala"
		break;
#line 2769 "SourceList.c"
	}
}

static inline gpointer
mail_source_list_expandable_item_get_instance_private (MailSourceListExpandableItem* self)
{
	return G_STRUCT_MEMBER_P (self, MailSourceListExpandableItem_private_offset);
}

/**
         * Creates a new {@link Granite.Widgets.SourceList.ExpandableItem}
         *
         * @param name Title of the item.
         * @return (transfer full) A new {@link Granite.Widgets.SourceList.ExpandableItem}.
         * @since 0.2
         */
MailSourceListExpandableItem*
mail_source_list_expandable_item_construct (GType object_type,
                                            const gchar* name)
{
	MailSourceListExpandableItem * self = NULL;
#line 558 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 559 "../src/SourceList/SourceList.vala"
	self = (MailSourceListExpandableItem*) mail_source_list_item_construct (object_type, name);
#line 558 "../src/SourceList/SourceList.vala"
	return self;
#line 2797 "SourceList.c"
}

MailSourceListExpandableItem*
mail_source_list_expandable_item_new (const gchar* name)
{
#line 558 "../src/SourceList/SourceList.vala"
	return mail_source_list_expandable_item_construct (MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, name);
#line 2805 "SourceList.c"
}

/**
         * Adds an item.
         *
         * {@link Granite.Widgets.SourceList.ExpandableItem.child_added} is fired after the item is added.
         *
         * While adding a child item, //the item it's being added to will set itself as the parent//.
         * Please note that items are required to have their //parent// property set to //null// before
         * being added, so make sure the item is removed from its previous parent before attempting
         * to add it to another item. For instance:
         * {{{
         * if (item.parent != null)
         *     item.parent.remove (item); // this will set item's parent to null
         * new_parent.add (item);
         * }}}
         *
         * @param item The item to add. Its parent __must__ be //null//.
         * @see Granite.Widgets.SourceList.ExpandableItem.child_added
         * @see Granite.Widgets.SourceList.ExpandableItem.remove
         * @since 0.2
         */
void
mail_source_list_expandable_item_add (MailSourceListExpandableItem* self,
                                      MailSourceListItem* item)
{
	MailSourceListExpandableItem* _tmp0_;
	MailSourceListExpandableItem* _tmp1_;
	GeeCollection* _tmp2_;
#line 586 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 586 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 586 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_item_get_parent (item);
#line 586 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 586 "../src/SourceList/SourceList.vala"
	_vala_return_if_fail (_tmp1_ == NULL, "item.parent == null");
#line 587 "../src/SourceList/SourceList.vala"
	mail_source_list_item_set_parent (item, self);
#line 588 "../src/SourceList/SourceList.vala"
	_tmp2_ = self->priv->children_list;
#line 588 "../src/SourceList/SourceList.vala"
	gee_collection_add (_tmp2_, item);
#line 589 "../src/SourceList/SourceList.vala"
	g_signal_emit (self, mail_source_list_expandable_item_signals[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_ADDED_SIGNAL], 0, item);
#line 2853 "SourceList.c"
}

/**
         * Removes an item.
         *
         * The {@link Granite.Widgets.SourceList.ExpandableItem.child_removed} signal is fired
         * //after removing the item//. Finally (i.e. after all the handlers have been invoked),
         * the item's {@link Granite.Widgets.SourceList.Item.parent} property is set to //null//.
         * This has the advantage of letting signal handlers know the parent from which //item//
         * is being removed.
         *
         * @param item The item to remove. This will fail if item has a different parent.
         * @see Granite.Widgets.SourceList.ExpandableItem.child_removed
         * @see Granite.Widgets.SourceList.ExpandableItem.clear
         * @since 0.2
         */
void
mail_source_list_expandable_item_remove (MailSourceListExpandableItem* self,
                                         MailSourceListItem* item)
{
	MailSourceListExpandableItem* _tmp0_;
	MailSourceListExpandableItem* _tmp1_;
	GeeCollection* _tmp2_;
#line 606 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 606 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 606 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_item_get_parent (item);
#line 606 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 606 "../src/SourceList/SourceList.vala"
	_vala_return_if_fail (_tmp1_ == self, "item.parent == this");
#line 607 "../src/SourceList/SourceList.vala"
	_tmp2_ = self->priv->children_list;
#line 607 "../src/SourceList/SourceList.vala"
	gee_collection_remove (_tmp2_, item);
#line 608 "../src/SourceList/SourceList.vala"
	g_signal_emit (self, mail_source_list_expandable_item_signals[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_REMOVED_SIGNAL], 0, item);
#line 609 "../src/SourceList/SourceList.vala"
	mail_source_list_item_set_parent (item, NULL);
#line 2895 "SourceList.c"
}

/**
         * Recursively expands the item along with its parent(s).
         *
         * @see Granite.Widgets.SourceList.ExpandableItem.expanded
         * @since 0.2
         */
void
mail_source_list_expandable_item_expand_with_parents (MailSourceListExpandableItem* self)
{
	MailSourceListExpandableItem* _tmp0_;
	MailSourceListExpandableItem* _tmp1_;
#line 618 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 623 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_item_get_parent ((MailSourceListItem*) self);
#line 623 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 623 "../src/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 2917 "SourceList.c"
		MailSourceListExpandableItem* _tmp2_;
		MailSourceListExpandableItem* _tmp3_;
#line 624 "../src/SourceList/SourceList.vala"
		_tmp2_ = mail_source_list_item_get_parent ((MailSourceListItem*) self);
#line 624 "../src/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 624 "../src/SourceList/SourceList.vala"
		mail_source_list_expandable_item_expand_with_parents (_tmp3_);
#line 2926 "SourceList.c"
	}
#line 625 "../src/SourceList/SourceList.vala"
	mail_source_list_expandable_item_set_expanded (self, TRUE);
#line 2930 "SourceList.c"
}

gboolean
mail_source_list_expandable_item_get_collapsible (MailSourceListExpandableItem* self)
{
	gboolean result;
#line 488 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 488 "../src/SourceList/SourceList.vala"
	result = self->priv->_collapsible;
#line 488 "../src/SourceList/SourceList.vala"
	return result;
#line 2943 "SourceList.c"
}

void
mail_source_list_expandable_item_set_collapsible (MailSourceListExpandableItem* self,
                                                  gboolean value)
{
	gboolean old_value;
#line 488 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 488 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_expandable_item_get_collapsible (self);
#line 488 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 488 "../src/SourceList/SourceList.vala"
		self->priv->_collapsible = value;
#line 488 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_expandable_item_properties[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY]);
#line 2961 "SourceList.c"
	}
}

gboolean
mail_source_list_expandable_item_get_expanded (MailSourceListExpandableItem* self)
{
	gboolean result;
	gboolean _tmp0_ = FALSE;
#line 512 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 512 "../src/SourceList/SourceList.vala"
	if (self->priv->_expanded) {
#line 512 "../src/SourceList/SourceList.vala"
		_tmp0_ = TRUE;
#line 2976 "SourceList.c"
	} else {
		gboolean _tmp1_;
#line 512 "../src/SourceList/SourceList.vala"
		_tmp1_ = self->priv->_collapsible;
#line 512 "../src/SourceList/SourceList.vala"
		_tmp0_ = !_tmp1_;
#line 2983 "SourceList.c"
	}
#line 512 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 512 "../src/SourceList/SourceList.vala"
	return result;
#line 2989 "SourceList.c"
}

void
mail_source_list_expandable_item_set_expanded (MailSourceListExpandableItem* self,
                                               gboolean value)
{
#line 513 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 514 "../src/SourceList/SourceList.vala"
	if (value != self->priv->_expanded) {
#line 515 "../src/SourceList/SourceList.vala"
		self->priv->_expanded = value;
#line 516 "../src/SourceList/SourceList.vala"
		g_signal_emit (self, mail_source_list_expandable_item_signals[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_TOGGLED_SIGNAL], 0);
#line 3004 "SourceList.c"
	}
#line 513 "../src/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, mail_source_list_expandable_item_properties[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY]);
#line 3008 "SourceList.c"
}

static guint
mail_source_list_expandable_item_get_n_children (MailSourceListExpandableItem* self)
{
	guint result;
	GeeCollection* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
#line 527 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 527 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->children_list;
#line 527 "../src/SourceList/SourceList.vala"
	_tmp1_ = gee_collection_get_size (_tmp0_);
#line 527 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 527 "../src/SourceList/SourceList.vala"
	result = (guint) _tmp2_;
#line 527 "../src/SourceList/SourceList.vala"
	return result;
#line 3030 "SourceList.c"
}

GeeCollection*
mail_source_list_expandable_item_get_children (MailSourceListExpandableItem* self)
{
	GeeCollection* result;
	GeeArrayList* children_list_copy = NULL;
	GeeArrayList* _tmp0_;
	GeeCollection* _tmp1_;
#line 540 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 543 "../src/SourceList/SourceList.vala"
	_tmp0_ = gee_array_list_new (MAIL_SOURCE_LIST_TYPE_ITEM, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
#line 543 "../src/SourceList/SourceList.vala"
	children_list_copy = _tmp0_;
#line 544 "../src/SourceList/SourceList.vala"
	_tmp1_ = self->priv->children_list;
#line 544 "../src/SourceList/SourceList.vala"
	gee_array_list_add_all (children_list_copy, _tmp1_);
#line 545 "../src/SourceList/SourceList.vala"
	result = (GeeCollection*) children_list_copy;
#line 545 "../src/SourceList/SourceList.vala"
	return result;
#line 3054 "SourceList.c"
}

static void
mail_source_list_expandable_item_real_toggled (MailSourceListExpandableItem* self)
{
}

static GObject *
mail_source_list_expandable_item_constructor (GType type,
                                              guint n_construct_properties,
                                              GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	MailSourceListExpandableItem * self;
#line 562 "../src/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (mail_source_list_expandable_item_parent_class);
#line 562 "../src/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 562 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, MailSourceListExpandableItem);
#line 563 "../src/SourceList/SourceList.vala"
	mail_source_list_item_set_editable ((MailSourceListItem*) self, FALSE);
#line 562 "../src/SourceList/SourceList.vala"
	return obj;
#line 3080 "SourceList.c"
}

static void
mail_source_list_expandable_item_class_init (MailSourceListExpandableItemClass * klass,
                                             gpointer klass_data)
{
#line 451 "../src/SourceList/SourceList.vala"
	mail_source_list_expandable_item_parent_class = g_type_class_peek_parent (klass);
#line 451 "../src/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &MailSourceListExpandableItem_private_offset);
#line 451 "../src/SourceList/SourceList.vala"
	((MailSourceListExpandableItemClass *) klass)->toggled = mail_source_list_expandable_item_real_toggled;
#line 451 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_mail_source_list_expandable_item_get_property;
#line 451 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_mail_source_list_expandable_item_set_property;
#line 451 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = mail_source_list_expandable_item_constructor;
#line 451 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = mail_source_list_expandable_item_finalize;
#line 3101 "SourceList.c"
	/**
	         * Whether the item is collapsible or not.
	         *
	         * When set to //false//, the item is //always// expanded and the expander is
	         * not shown. Please note that this will also affect the value returned by the
	         * {@link Granite.Widgets.SourceList.ExpandableItem.expanded} property.
	         *
	         * @see Granite.Widgets.SourceList.ExpandableItem.expanded
	         * @since 0.2
	         */
#line 451 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY, mail_source_list_expandable_item_properties[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY] = g_param_spec_boolean ("collapsible", "collapsible", "collapsible", TRUE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 451 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY, mail_source_list_expandable_item_properties[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY] = g_param_spec_boolean ("expanded", "expanded", "expanded", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3116 "SourceList.c"
	/**
	         * The item's children.
	         *
	         * This returns a newly-created list containing the children.
	         * It's safe to iterate it while removing items with
	         * {@link Granite.Widgets.SourceList.ExpandableItem.remove}
	         *
	         * @since 0.2
	         */
#line 451 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILDREN_PROPERTY, mail_source_list_expandable_item_properties[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILDREN_PROPERTY] = g_param_spec_object ("children", "children", "children", GEE_TYPE_COLLECTION, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
#line 3128 "SourceList.c"
	/**
	         * Emitted when an item is added.
	         *
	         * @param item Item added.
	         * @see Granite.Widgets.SourceList.ExpandableItem.add
	         * @since 0.2
	         */
#line 451 "../src/SourceList/SourceList.vala"
	mail_source_list_expandable_item_signals[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_ADDED_SIGNAL] = g_signal_new ("child-added", MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, MAIL_SOURCE_LIST_TYPE_ITEM);
#line 3138 "SourceList.c"
	/**
	         * Emitted when an item is removed.
	         *
	         * @param item Item removed.
	         * @see Granite.Widgets.SourceList.ExpandableItem.remove
	         * @since 0.2
	         */
#line 451 "../src/SourceList/SourceList.vala"
	mail_source_list_expandable_item_signals[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_REMOVED_SIGNAL] = g_signal_new ("child-removed", MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, MAIL_SOURCE_LIST_TYPE_ITEM);
#line 3148 "SourceList.c"
	/**
	         * Emitted when the item is expanded or collapsed.
	         *
	         * @since 0.2
	         */
#line 451 "../src/SourceList/SourceList.vala"
	mail_source_list_expandable_item_signals[MAIL_SOURCE_LIST_EXPANDABLE_ITEM_TOGGLED_SIGNAL] = g_signal_new ("toggled", MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (MailSourceListExpandableItemClass, toggled), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 3156 "SourceList.c"
}

static void
mail_source_list_expandable_item_instance_init (MailSourceListExpandableItem * self,
                                                gpointer klass)
{
	GeeArrayList* _tmp0_;
#line 451 "../src/SourceList/SourceList.vala"
	self->priv = mail_source_list_expandable_item_get_instance_private (self);
#line 488 "../src/SourceList/SourceList.vala"
	self->priv->_collapsible = TRUE;
#line 510 "../src/SourceList/SourceList.vala"
	self->priv->_expanded = FALSE;
#line 549 "../src/SourceList/SourceList.vala"
	_tmp0_ = gee_array_list_new (MAIL_SOURCE_LIST_TYPE_ITEM, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
#line 549 "../src/SourceList/SourceList.vala"
	self->priv->children_list = (GeeCollection*) _tmp0_;
#line 3174 "SourceList.c"
}

static void
mail_source_list_expandable_item_finalize (GObject * obj)
{
	MailSourceListExpandableItem * self;
#line 451 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, MailSourceListExpandableItem);
#line 549 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->children_list);
#line 451 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (mail_source_list_expandable_item_parent_class)->finalize (obj);
#line 3187 "SourceList.c"
}

/**
     * An item that can contain more items.
     *
     * It supports all the properties inherited from {@link Granite.Widgets.SourceList.Item},
     * and behaves like a normal item, except when it is located at the root level; in that case,
     * the following properties are ignored by the widget:
     *
     * * {@link Granite.Widgets.SourceList.Item.selectable}
     * * {@link Granite.Widgets.SourceList.Item.editable}
     * * {@link Granite.Widgets.SourceList.Item.icon}
     * * {@link Granite.Widgets.SourceList.Item.activatable}
     * * {@link Granite.Widgets.SourceList.Item.badge}
     *
     * Root-level expandable items (i.e. Main Categories) are ''not'' displayed when they contain
     * zero visible children.
     *
     * @since 0.2
     */
static GType
mail_source_list_expandable_item_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListExpandableItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_expandable_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceListExpandableItem), 0, (GInstanceInitFunc) mail_source_list_expandable_item_instance_init, NULL };
	GType mail_source_list_expandable_item_type_id;
	mail_source_list_expandable_item_type_id = g_type_register_static (MAIL_SOURCE_LIST_TYPE_ITEM, "MailSourceListExpandableItem", &g_define_type_info, 0);
	MailSourceListExpandableItem_private_offset = g_type_add_instance_private (mail_source_list_expandable_item_type_id, sizeof (MailSourceListExpandableItemPrivate));
	return mail_source_list_expandable_item_type_id;
}

GType
mail_source_list_expandable_item_get_type (void)
{
	static volatile gsize mail_source_list_expandable_item_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_expandable_item_type_id__once)) {
		GType mail_source_list_expandable_item_type_id;
		mail_source_list_expandable_item_type_id = mail_source_list_expandable_item_get_type_once ();
		g_once_init_leave (&mail_source_list_expandable_item_type_id__once, mail_source_list_expandable_item_type_id);
	}
	return mail_source_list_expandable_item_type_id__once;
}

static void
_vala_mail_source_list_expandable_item_get_property (GObject * object,
                                                     guint property_id,
                                                     GValue * value,
                                                     GParamSpec * pspec)
{
	MailSourceListExpandableItem * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, MailSourceListExpandableItem);
#line 451 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 451 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY:
#line 451 "../src/SourceList/SourceList.vala"
		g_value_set_boolean (value, mail_source_list_expandable_item_get_collapsible (self));
#line 451 "../src/SourceList/SourceList.vala"
		break;
#line 451 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY:
#line 451 "../src/SourceList/SourceList.vala"
		g_value_set_boolean (value, mail_source_list_expandable_item_get_expanded (self));
#line 451 "../src/SourceList/SourceList.vala"
		break;
#line 451 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_EXPANDABLE_ITEM_CHILDREN_PROPERTY:
#line 451 "../src/SourceList/SourceList.vala"
		g_value_take_object (value, mail_source_list_expandable_item_get_children (self));
#line 451 "../src/SourceList/SourceList.vala"
		break;
#line 3258 "SourceList.c"
		default:
#line 451 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 451 "../src/SourceList/SourceList.vala"
		break;
#line 3264 "SourceList.c"
	}
}

static void
_vala_mail_source_list_expandable_item_set_property (GObject * object,
                                                     guint property_id,
                                                     const GValue * value,
                                                     GParamSpec * pspec)
{
	MailSourceListExpandableItem * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, MailSourceListExpandableItem);
#line 451 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 451 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY:
#line 451 "../src/SourceList/SourceList.vala"
		mail_source_list_expandable_item_set_collapsible (self, g_value_get_boolean (value));
#line 451 "../src/SourceList/SourceList.vala"
		break;
#line 451 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY:
#line 451 "../src/SourceList/SourceList.vala"
		mail_source_list_expandable_item_set_expanded (self, g_value_get_boolean (value));
#line 451 "../src/SourceList/SourceList.vala"
		break;
#line 3290 "SourceList.c"
		default:
#line 451 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 451 "../src/SourceList/SourceList.vala"
		break;
#line 3296 "SourceList.c"
	}
}

static inline gpointer
mail_source_list_data_model_get_instance_private (MailSourceListDataModel* self)
{
	return G_STRUCT_MEMBER_P (self, MailSourceListDataModel_private_offset);
}

static GType
mail_source_list_data_model_column_type (MailSourceListDataModelColumn self)
{
	GType result;
#line 721 "../src/SourceList/SourceList.vala"
	switch (self) {
#line 721 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM:
#line 3314 "SourceList.c"
		{
#line 723 "../src/SourceList/SourceList.vala"
			result = MAIL_SOURCE_LIST_TYPE_ITEM;
#line 723 "../src/SourceList/SourceList.vala"
			return result;
#line 3320 "SourceList.c"
		}
		default:
		{
#line 726 "../src/SourceList/SourceList.vala"
			g_assert_not_reached ();
#line 3326 "SourceList.c"
		}
	}
}

static GType
mail_source_list_data_model_column_get_type_once (void)
{
	static const GEnumValue values[] = {{MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, "MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM", "item"}, {MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_N_COLUMNS, "MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_N_COLUMNS", "n-columns"}, {0, NULL, NULL}};
	GType mail_source_list_data_model_column_type_id;
	mail_source_list_data_model_column_type_id = g_enum_register_static ("MailSourceListDataModelColumn", values);
	return mail_source_list_data_model_column_type_id;
}

static GType
mail_source_list_data_model_column_get_type (void)
{
	static volatile gsize mail_source_list_data_model_column_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_data_model_column_type_id__once)) {
		GType mail_source_list_data_model_column_type_id;
		mail_source_list_data_model_column_type_id = mail_source_list_data_model_column_get_type_once ();
		g_once_init_leave (&mail_source_list_data_model_column_type_id__once, mail_source_list_data_model_column_type_id);
	}
	return mail_source_list_data_model_column_type_id__once;
}

static gboolean
mail_source_list_data_model_has_item (MailSourceListDataModel* self,
                                      MailSourceListItem* item)
{
	GeeHashMap* _tmp0_;
	gboolean result;
#line 784 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 784 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 785 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->items;
#line 785 "../src/SourceList/SourceList.vala"
	result = gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, item);
#line 785 "../src/SourceList/SourceList.vala"
	return result;
#line 3368 "SourceList.c"
}

static void
_vala_GtkTreeIter_free (GtkTreeIter* self)
{
#line 795 "../src/SourceList/SourceList.vala"
	g_boxed_free (gtk_tree_iter_get_type (), self);
#line 3376 "SourceList.c"
}

static void
_vala_GtkTreePath_free (GtkTreePath* self)
{
#line 795 "../src/SourceList/SourceList.vala"
	g_boxed_free (gtk_tree_path_get_type (), self);
#line 3384 "SourceList.c"
}

static void
mail_source_list_data_model_update_item (MailSourceListDataModel* self,
                                         MailSourceListItem* item)
{
	MailSourceListExpandableItem* _tmp0_;
	MailSourceListExpandableItem* _tmp1_;
	MailSourceListDataModelNodeWrapper* node_reference = NULL;
	GeeHashMap* _tmp2_;
	gpointer _tmp3_;
	MailSourceListDataModelNodeWrapper* _tmp4_;
#line 788 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 788 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 788 "../src/SourceList/SourceList.vala"
	_vala_return_if_fail (mail_source_list_data_model_has_item (self, item), "has_item (item)");
#line 789 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_get_root (self);
#line 789 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 789 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "root != null");
#line 794 "../src/SourceList/SourceList.vala"
	_tmp2_ = self->priv->items;
#line 794 "../src/SourceList/SourceList.vala"
	_tmp3_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp2_, item);
#line 794 "../src/SourceList/SourceList.vala"
	node_reference = (MailSourceListDataModelNodeWrapper*) _tmp3_;
#line 795 "../src/SourceList/SourceList.vala"
	_tmp4_ = node_reference;
#line 795 "../src/SourceList/SourceList.vala"
	if (_tmp4_ != NULL) {
#line 3419 "SourceList.c"
		GtkTreePath* path = NULL;
		MailSourceListDataModelNodeWrapper* _tmp5_;
		GtkTreePath* _tmp6_;
		GtkTreePath* _tmp7_;
		GtkTreeIter* iter = NULL;
		MailSourceListDataModelNodeWrapper* _tmp8_;
		GtkTreeIter* _tmp9_;
		GtkTreeIter* _tmp10_;
		gboolean _tmp11_ = FALSE;
		GtkTreePath* _tmp12_;
#line 796 "../src/SourceList/SourceList.vala"
		_tmp5_ = node_reference;
#line 796 "../src/SourceList/SourceList.vala"
		_tmp6_ = mail_source_list_data_model_node_wrapper_get_path (_tmp5_);
#line 796 "../src/SourceList/SourceList.vala"
		_tmp7_ = _tmp6_;
#line 796 "../src/SourceList/SourceList.vala"
		path = _tmp7_;
#line 797 "../src/SourceList/SourceList.vala"
		_tmp8_ = node_reference;
#line 797 "../src/SourceList/SourceList.vala"
		_tmp9_ = mail_source_list_data_model_node_wrapper_get_iter (_tmp8_);
#line 797 "../src/SourceList/SourceList.vala"
		_tmp10_ = _tmp9_;
#line 797 "../src/SourceList/SourceList.vala"
		iter = _tmp10_;
#line 798 "../src/SourceList/SourceList.vala"
		_tmp12_ = path;
#line 798 "../src/SourceList/SourceList.vala"
		if (_tmp12_ != NULL) {
#line 3450 "SourceList.c"
			GtkTreeIter* _tmp13_;
#line 798 "../src/SourceList/SourceList.vala"
			_tmp13_ = iter;
#line 798 "../src/SourceList/SourceList.vala"
			_tmp11_ = _tmp13_ != NULL;
#line 3456 "SourceList.c"
		} else {
#line 798 "../src/SourceList/SourceList.vala"
			_tmp11_ = FALSE;
#line 3460 "SourceList.c"
		}
#line 798 "../src/SourceList/SourceList.vala"
		if (_tmp11_) {
#line 3464 "SourceList.c"
			GtkTreeStore* _tmp14_;
			GtkTreePath* _tmp15_;
			GtkTreeIter* _tmp16_;
			GtkTreeIter _tmp17_;
#line 799 "../src/SourceList/SourceList.vala"
			_tmp14_ = self->priv->child_tree;
#line 799 "../src/SourceList/SourceList.vala"
			_tmp15_ = path;
#line 799 "../src/SourceList/SourceList.vala"
			_tmp16_ = iter;
#line 799 "../src/SourceList/SourceList.vala"
			_tmp17_ = *_tmp16_;
#line 799 "../src/SourceList/SourceList.vala"
			gtk_tree_model_row_changed ((GtkTreeModel*) _tmp14_, _tmp15_, &_tmp17_);
#line 800 "../src/SourceList/SourceList.vala"
			g_signal_emit (self, mail_source_list_data_model_signals[MAIL_SOURCE_LIST_DATA_MODEL_ITEM_UPDATED_SIGNAL], 0, item);
#line 3481 "SourceList.c"
		}
#line 795 "../src/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (iter);
#line 795 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 3487 "SourceList.c"
	}
#line 788 "../src/SourceList/SourceList.vala"
	_mail_source_list_data_model_node_wrapper_unref0 (node_reference);
#line 3491 "SourceList.c"
}

static GtkTreeIter*
_vala_GtkTreeIter_copy (GtkTreeIter* self)
{
#line 824 "../src/SourceList/SourceList.vala"
	return g_boxed_copy (gtk_tree_iter_get_type (), self);
#line 3499 "SourceList.c"
}

static gpointer
__vala_GtkTreeIter_copy0 (gpointer self)
{
#line 824 "../src/SourceList/SourceList.vala"
	return self ? _vala_GtkTreeIter_copy (self) : NULL;
#line 3507 "SourceList.c"
}

static void
mail_source_list_data_model_add_item (MailSourceListDataModel* self,
                                      MailSourceListItem* item)
{
	MailSourceListExpandableItem* _tmp0_;
	MailSourceListExpandableItem* _tmp1_;
	GtkTreeIter* parent_child_iter = NULL;
	GtkTreeIter* child_iter = NULL;
	MailSourceListExpandableItem* parent = NULL;
	MailSourceListExpandableItem* _tmp2_;
	MailSourceListExpandableItem* _tmp3_;
	MailSourceListExpandableItem* _tmp4_;
	gboolean _tmp5_ = FALSE;
	MailSourceListExpandableItem* _tmp6_;
	GtkTreeStore* _tmp15_;
	GtkTreeIter* _tmp16_;
	GtkTreeIter _tmp17_ = {0};
	GtkTreeIter _tmp18_;
	GtkTreeIter* _tmp19_;
	GtkTreeStore* _tmp20_;
	GtkTreeIter* _tmp21_;
	GtkTreeIter _tmp22_;
	GeeHashMap* _tmp23_;
	GtkTreeStore* _tmp24_;
	GtkTreeIter* _tmp25_;
	GtkTreeIter _tmp26_;
	MailSourceListDataModelNodeWrapper* _tmp27_;
	MailSourceListDataModelNodeWrapper* _tmp28_;
	MailSourceListExpandableItem* _tmp29_;
	MailSourceListExpandableItem* expandable = NULL;
	MailSourceListExpandableItem* _tmp30_;
	MailSourceListExpandableItem* _tmp31_;
#line 805 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 805 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 805 "../src/SourceList/SourceList.vala"
	_vala_return_if_fail (!mail_source_list_data_model_has_item (self, item), "!has_item (item)");
#line 806 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_get_root (self);
#line 806 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 806 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "root != null");
#line 809 "../src/SourceList/SourceList.vala"
	parent_child_iter = NULL;
#line 810 "../src/SourceList/SourceList.vala"
	_tmp2_ = mail_source_list_item_get_parent (item);
#line 810 "../src/SourceList/SourceList.vala"
	_tmp3_ = _tmp2_;
#line 810 "../src/SourceList/SourceList.vala"
	_tmp4_ = _g_object_ref0 (_tmp3_);
#line 810 "../src/SourceList/SourceList.vala"
	parent = _tmp4_;
#line 812 "../src/SourceList/SourceList.vala"
	_tmp6_ = parent;
#line 812 "../src/SourceList/SourceList.vala"
	if (_tmp6_ != NULL) {
#line 3568 "SourceList.c"
		MailSourceListExpandableItem* _tmp7_;
		MailSourceListExpandableItem* _tmp8_;
		MailSourceListExpandableItem* _tmp9_;
#line 812 "../src/SourceList/SourceList.vala"
		_tmp7_ = parent;
#line 812 "../src/SourceList/SourceList.vala"
		_tmp8_ = mail_source_list_data_model_get_root (self);
#line 812 "../src/SourceList/SourceList.vala"
		_tmp9_ = _tmp8_;
#line 812 "../src/SourceList/SourceList.vala"
		_tmp5_ = _tmp7_ != _tmp9_;
#line 3580 "SourceList.c"
	} else {
#line 812 "../src/SourceList/SourceList.vala"
		_tmp5_ = FALSE;
#line 3584 "SourceList.c"
	}
#line 812 "../src/SourceList/SourceList.vala"
	if (_tmp5_) {
#line 3588 "SourceList.c"
		MailSourceListExpandableItem* _tmp10_;
		MailSourceListExpandableItem* _tmp12_;
		GtkTreeIter* _tmp13_;
		GtkTreeIter* _tmp14_;
#line 814 "../src/SourceList/SourceList.vala"
		_tmp10_ = parent;
#line 814 "../src/SourceList/SourceList.vala"
		if (!mail_source_list_data_model_has_item (self, (MailSourceListItem*) _tmp10_)) {
#line 3597 "SourceList.c"
			MailSourceListExpandableItem* _tmp11_;
#line 815 "../src/SourceList/SourceList.vala"
			_tmp11_ = parent;
#line 815 "../src/SourceList/SourceList.vala"
			mail_source_list_data_model_add_item (self, (MailSourceListItem*) _tmp11_);
#line 3603 "SourceList.c"
		}
#line 818 "../src/SourceList/SourceList.vala"
		_tmp12_ = parent;
#line 818 "../src/SourceList/SourceList.vala"
		_tmp13_ = mail_source_list_data_model_get_item_child_iter (self, (MailSourceListItem*) _tmp12_);
#line 818 "../src/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (parent_child_iter);
#line 818 "../src/SourceList/SourceList.vala"
		parent_child_iter = _tmp13_;
#line 821 "../src/SourceList/SourceList.vala"
		_tmp14_ = parent_child_iter;
#line 821 "../src/SourceList/SourceList.vala"
		_vala_assert (_tmp14_ != NULL, "parent_child_iter != null");
#line 3617 "SourceList.c"
	}
#line 824 "../src/SourceList/SourceList.vala"
	_tmp15_ = self->priv->child_tree;
#line 824 "../src/SourceList/SourceList.vala"
	_tmp16_ = parent_child_iter;
#line 824 "../src/SourceList/SourceList.vala"
	gtk_tree_store_append (_tmp15_, &_tmp17_, _tmp16_);
#line 824 "../src/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (child_iter);
#line 824 "../src/SourceList/SourceList.vala"
	_tmp18_ = _tmp17_;
#line 824 "../src/SourceList/SourceList.vala"
	_tmp19_ = __vala_GtkTreeIter_copy0 (&_tmp18_);
#line 824 "../src/SourceList/SourceList.vala"
	child_iter = _tmp19_;
#line 825 "../src/SourceList/SourceList.vala"
	_tmp20_ = self->priv->child_tree;
#line 825 "../src/SourceList/SourceList.vala"
	_tmp21_ = child_iter;
#line 825 "../src/SourceList/SourceList.vala"
	_tmp22_ = *_tmp21_;
#line 825 "../src/SourceList/SourceList.vala"
	gtk_tree_store_set (_tmp20_, &_tmp22_, MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, item, -1, -1);
#line 827 "../src/SourceList/SourceList.vala"
	_tmp23_ = self->priv->items;
#line 827 "../src/SourceList/SourceList.vala"
	_tmp24_ = self->priv->child_tree;
#line 827 "../src/SourceList/SourceList.vala"
	_tmp25_ = child_iter;
#line 827 "../src/SourceList/SourceList.vala"
	_tmp26_ = *_tmp25_;
#line 827 "../src/SourceList/SourceList.vala"
	_tmp27_ = mail_source_list_data_model_node_wrapper_new ((GtkTreeModel*) _tmp24_, &_tmp26_);
#line 827 "../src/SourceList/SourceList.vala"
	_tmp28_ = _tmp27_;
#line 827 "../src/SourceList/SourceList.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp23_, item, _tmp28_);
#line 827 "../src/SourceList/SourceList.vala"
	_mail_source_list_data_model_node_wrapper_unref0 (_tmp28_);
#line 831 "../src/SourceList/SourceList.vala"
	g_signal_emit (self, mail_source_list_data_model_signals[MAIL_SOURCE_LIST_DATA_MODEL_ITEM_UPDATED_SIGNAL], 0, item);
#line 833 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_add_property_monitor (self, item);
#line 835 "../src/SourceList/SourceList.vala"
	_tmp29_ = parent;
#line 835 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_push_parent_update (self, _tmp29_);
#line 838 "../src/SourceList/SourceList.vala"
	_tmp30_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (item) ? ((MailSourceListExpandableItem*) item) : NULL);
#line 838 "../src/SourceList/SourceList.vala"
	expandable = _tmp30_;
#line 839 "../src/SourceList/SourceList.vala"
	_tmp31_ = expandable;
#line 839 "../src/SourceList/SourceList.vala"
	if (_tmp31_ != NULL) {
#line 3673 "SourceList.c"
		MailSourceListExpandableItem* _tmp42_;
		{
			GeeIterator* _child_item_it = NULL;
			MailSourceListExpandableItem* _tmp32_;
			GeeCollection* _tmp33_;
			GeeCollection* _tmp34_;
			GeeCollection* _tmp35_;
			GeeIterator* _tmp36_;
			GeeIterator* _tmp37_;
#line 840 "../src/SourceList/SourceList.vala"
			_tmp32_ = expandable;
#line 840 "../src/SourceList/SourceList.vala"
			_tmp33_ = mail_source_list_expandable_item_get_children (_tmp32_);
#line 840 "../src/SourceList/SourceList.vala"
			_tmp34_ = _tmp33_;
#line 840 "../src/SourceList/SourceList.vala"
			_tmp35_ = _tmp34_;
#line 840 "../src/SourceList/SourceList.vala"
			_tmp36_ = gee_iterable_iterator ((GeeIterable*) _tmp35_);
#line 840 "../src/SourceList/SourceList.vala"
			_tmp37_ = _tmp36_;
#line 840 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (_tmp35_);
#line 840 "../src/SourceList/SourceList.vala"
			_child_item_it = _tmp37_;
#line 840 "../src/SourceList/SourceList.vala"
			while (TRUE) {
#line 3701 "SourceList.c"
				GeeIterator* _tmp38_;
				MailSourceListItem* child_item = NULL;
				GeeIterator* _tmp39_;
				gpointer _tmp40_;
				MailSourceListItem* _tmp41_;
#line 840 "../src/SourceList/SourceList.vala"
				_tmp38_ = _child_item_it;
#line 840 "../src/SourceList/SourceList.vala"
				if (!gee_iterator_next (_tmp38_)) {
#line 840 "../src/SourceList/SourceList.vala"
					break;
#line 3713 "SourceList.c"
				}
#line 840 "../src/SourceList/SourceList.vala"
				_tmp39_ = _child_item_it;
#line 840 "../src/SourceList/SourceList.vala"
				_tmp40_ = gee_iterator_get (_tmp39_);
#line 840 "../src/SourceList/SourceList.vala"
				child_item = (MailSourceListItem*) _tmp40_;
#line 841 "../src/SourceList/SourceList.vala"
				_tmp41_ = child_item;
#line 841 "../src/SourceList/SourceList.vala"
				mail_source_list_data_model_add_item (self, _tmp41_);
#line 840 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (child_item);
#line 3727 "SourceList.c"
			}
#line 840 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (_child_item_it);
#line 3731 "SourceList.c"
		}
#line 844 "../src/SourceList/SourceList.vala"
		_tmp42_ = expandable;
#line 844 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_add_children_monitor (self, _tmp42_);
#line 3737 "SourceList.c"
	}
#line 805 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (expandable);
#line 805 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (parent);
#line 805 "../src/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (child_iter);
#line 805 "../src/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (parent_child_iter);
#line 3747 "SourceList.c"
}

static void
mail_source_list_data_model_remove_item (MailSourceListDataModel* self,
                                         MailSourceListItem* item)
{
	MailSourceListExpandableItem* _tmp0_;
	MailSourceListExpandableItem* _tmp1_;
	GtkTreeIter* child_iter = NULL;
	GtkTreeIter* _tmp2_;
	GeeHashMap* _tmp3_;
	GtkTreeIter* _tmp4_;
	MailSourceListExpandableItem* _tmp6_;
	MailSourceListExpandableItem* _tmp7_;
	MailSourceListExpandableItem* expandable = NULL;
	MailSourceListExpandableItem* _tmp8_;
	MailSourceListExpandableItem* _tmp9_;
#line 848 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 848 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 848 "../src/SourceList/SourceList.vala"
	_vala_return_if_fail (mail_source_list_data_model_has_item (self, item), "has_item (item)");
#line 849 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_get_root (self);
#line 849 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 849 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "root != null");
#line 851 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_remove_property_monitor (self, item);
#line 856 "../src/SourceList/SourceList.vala"
	_tmp2_ = mail_source_list_data_model_get_item_child_iter (self, item);
#line 856 "../src/SourceList/SourceList.vala"
	child_iter = _tmp2_;
#line 862 "../src/SourceList/SourceList.vala"
	_tmp3_ = self->priv->items;
#line 862 "../src/SourceList/SourceList.vala"
	gee_abstract_map_unset ((GeeAbstractMap*) _tmp3_, item, NULL);
#line 864 "../src/SourceList/SourceList.vala"
	_tmp4_ = child_iter;
#line 864 "../src/SourceList/SourceList.vala"
	if (_tmp4_ != NULL) {
#line 3791 "SourceList.c"
		GtkTreeStore* _tmp5_;
#line 865 "../src/SourceList/SourceList.vala"
		_tmp5_ = self->priv->child_tree;
#line 865 "../src/SourceList/SourceList.vala"
		gtk_tree_store_remove (_tmp5_, child_iter);
#line 3797 "SourceList.c"
	}
#line 867 "../src/SourceList/SourceList.vala"
	_tmp6_ = mail_source_list_item_get_parent (item);
#line 867 "../src/SourceList/SourceList.vala"
	_tmp7_ = _tmp6_;
#line 867 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_push_parent_update (self, _tmp7_);
#line 870 "../src/SourceList/SourceList.vala"
	_tmp8_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (item) ? ((MailSourceListExpandableItem*) item) : NULL);
#line 870 "../src/SourceList/SourceList.vala"
	expandable = _tmp8_;
#line 871 "../src/SourceList/SourceList.vala"
	_tmp9_ = expandable;
#line 871 "../src/SourceList/SourceList.vala"
	if (_tmp9_ != NULL) {
#line 3813 "SourceList.c"
		MailSourceListExpandableItem* _tmp10_;
#line 873 "../src/SourceList/SourceList.vala"
		_tmp10_ = expandable;
#line 873 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_remove_children_monitor (self, _tmp10_);
#line 3819 "SourceList.c"
		{
			GeeIterator* _child_item_it = NULL;
			MailSourceListExpandableItem* _tmp11_;
			GeeCollection* _tmp12_;
			GeeCollection* _tmp13_;
			GeeCollection* _tmp14_;
			GeeIterator* _tmp15_;
			GeeIterator* _tmp16_;
#line 875 "../src/SourceList/SourceList.vala"
			_tmp11_ = expandable;
#line 875 "../src/SourceList/SourceList.vala"
			_tmp12_ = mail_source_list_expandable_item_get_children (_tmp11_);
#line 875 "../src/SourceList/SourceList.vala"
			_tmp13_ = _tmp12_;
#line 875 "../src/SourceList/SourceList.vala"
			_tmp14_ = _tmp13_;
#line 875 "../src/SourceList/SourceList.vala"
			_tmp15_ = gee_iterable_iterator ((GeeIterable*) _tmp14_);
#line 875 "../src/SourceList/SourceList.vala"
			_tmp16_ = _tmp15_;
#line 875 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (_tmp14_);
#line 875 "../src/SourceList/SourceList.vala"
			_child_item_it = _tmp16_;
#line 875 "../src/SourceList/SourceList.vala"
			while (TRUE) {
#line 3846 "SourceList.c"
				GeeIterator* _tmp17_;
				MailSourceListItem* child_item = NULL;
				GeeIterator* _tmp18_;
				gpointer _tmp19_;
				MailSourceListItem* _tmp20_;
#line 875 "../src/SourceList/SourceList.vala"
				_tmp17_ = _child_item_it;
#line 875 "../src/SourceList/SourceList.vala"
				if (!gee_iterator_next (_tmp17_)) {
#line 875 "../src/SourceList/SourceList.vala"
					break;
#line 3858 "SourceList.c"
				}
#line 875 "../src/SourceList/SourceList.vala"
				_tmp18_ = _child_item_it;
#line 875 "../src/SourceList/SourceList.vala"
				_tmp19_ = gee_iterator_get (_tmp18_);
#line 875 "../src/SourceList/SourceList.vala"
				child_item = (MailSourceListItem*) _tmp19_;
#line 876 "../src/SourceList/SourceList.vala"
				_tmp20_ = child_item;
#line 876 "../src/SourceList/SourceList.vala"
				mail_source_list_data_model_remove_item (self, _tmp20_);
#line 875 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (child_item);
#line 3872 "SourceList.c"
			}
#line 875 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (_child_item_it);
#line 3876 "SourceList.c"
		}
	}
#line 848 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (expandable);
#line 848 "../src/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (child_iter);
#line 3883 "SourceList.c"
}

static void
_mail_source_list_data_model_on_item_prop_changed_mail_source_list_data_model_item_monitor_changed (MailSourceListDataModelItemMonitor* _sender,
                                                                                                    MailSourceListItem* _self_,
                                                                                                    const gchar* prop_name,
                                                                                                    gpointer self)
{
#line 883 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_on_item_prop_changed ((MailSourceListDataModel*) self, _self_, prop_name);
#line 3894 "SourceList.c"
}

static void
mail_source_list_data_model_add_property_monitor (MailSourceListDataModel* self,
                                                  MailSourceListItem* item)
{
	MailSourceListDataModelItemMonitor* wrapper = NULL;
	MailSourceListDataModelItemMonitor* _tmp0_;
	GeeHashMap* _tmp1_;
#line 880 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 880 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 881 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_item_monitor_new (item);
#line 881 "../src/SourceList/SourceList.vala"
	wrapper = _tmp0_;
#line 882 "../src/SourceList/SourceList.vala"
	_tmp1_ = self->priv->monitors;
#line 882 "../src/SourceList/SourceList.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp1_, item, wrapper);
#line 883 "../src/SourceList/SourceList.vala"
	g_signal_connect_object (wrapper, "changed", (GCallback) _mail_source_list_data_model_on_item_prop_changed_mail_source_list_data_model_item_monitor_changed, self, 0);
#line 880 "../src/SourceList/SourceList.vala"
	_mail_source_list_data_model_item_monitor_unref0 (wrapper);
#line 3920 "SourceList.c"
}

static void
mail_source_list_data_model_remove_property_monitor (MailSourceListDataModel* self,
                                                     MailSourceListItem* item)
{
	MailSourceListDataModelItemMonitor* wrapper = NULL;
	GeeHashMap* _tmp0_;
	gpointer _tmp1_;
	MailSourceListDataModelItemMonitor* _tmp2_;
	GeeHashMap* _tmp5_;
#line 886 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 886 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 887 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->monitors;
#line 887 "../src/SourceList/SourceList.vala"
	_tmp1_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp0_, item);
#line 887 "../src/SourceList/SourceList.vala"
	wrapper = (MailSourceListDataModelItemMonitor*) _tmp1_;
#line 888 "../src/SourceList/SourceList.vala"
	_tmp2_ = wrapper;
#line 888 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 3946 "SourceList.c"
		MailSourceListDataModelItemMonitor* _tmp3_;
		guint _tmp4_;
#line 889 "../src/SourceList/SourceList.vala"
		_tmp3_ = wrapper;
#line 889 "../src/SourceList/SourceList.vala"
		g_signal_parse_name ("changed", MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, &_tmp4_, NULL, FALSE);
#line 889 "../src/SourceList/SourceList.vala"
		g_signal_handlers_disconnect_matched (_tmp3_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp4_, 0, NULL, (GCallback) _mail_source_list_data_model_on_item_prop_changed_mail_source_list_data_model_item_monitor_changed, self);
#line 3955 "SourceList.c"
	}
#line 890 "../src/SourceList/SourceList.vala"
	_tmp5_ = self->priv->monitors;
#line 890 "../src/SourceList/SourceList.vala"
	gee_abstract_map_unset ((GeeAbstractMap*) _tmp5_, item, NULL);
#line 886 "../src/SourceList/SourceList.vala"
	_mail_source_list_data_model_item_monitor_unref0 (wrapper);
#line 3963 "SourceList.c"
}

static void
_mail_source_list_data_model_on_item_child_added_mail_source_list_expandable_item_child_added (MailSourceListExpandableItem* _sender,
                                                                                               MailSourceListItem* item,
                                                                                               gpointer self)
{
#line 894 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_on_item_child_added ((MailSourceListDataModel*) self, item);
#line 3973 "SourceList.c"
}

static void
_mail_source_list_data_model_on_item_child_removed_mail_source_list_expandable_item_child_removed (MailSourceListExpandableItem* _sender,
                                                                                                   MailSourceListItem* item,
                                                                                                   gpointer self)
{
#line 895 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_on_item_child_removed ((MailSourceListDataModel*) self, item);
#line 3983 "SourceList.c"
}

static void
mail_source_list_data_model_add_children_monitor (MailSourceListDataModel* self,
                                                  MailSourceListExpandableItem* item)
{
#line 893 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 893 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 894 "../src/SourceList/SourceList.vala"
	g_signal_connect_object (item, "child-added", (GCallback) _mail_source_list_data_model_on_item_child_added_mail_source_list_expandable_item_child_added, self, G_CONNECT_AFTER);
#line 895 "../src/SourceList/SourceList.vala"
	g_signal_connect_object (item, "child-removed", (GCallback) _mail_source_list_data_model_on_item_child_removed_mail_source_list_expandable_item_child_removed, self, G_CONNECT_AFTER);
#line 3998 "SourceList.c"
}

static void
mail_source_list_data_model_remove_children_monitor (MailSourceListDataModel* self,
                                                     MailSourceListExpandableItem* item)
{
	guint _tmp0_;
	guint _tmp1_;
#line 898 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 898 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 899 "../src/SourceList/SourceList.vala"
	g_signal_parse_name ("child-added", MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, &_tmp0_, NULL, FALSE);
#line 899 "../src/SourceList/SourceList.vala"
	g_signal_handlers_disconnect_matched (item, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp0_, 0, NULL, (GCallback) _mail_source_list_data_model_on_item_child_added_mail_source_list_expandable_item_child_added, self);
#line 900 "../src/SourceList/SourceList.vala"
	g_signal_parse_name ("child-removed", MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, &_tmp1_, NULL, FALSE);
#line 900 "../src/SourceList/SourceList.vala"
	g_signal_handlers_disconnect_matched (item, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _mail_source_list_data_model_on_item_child_removed_mail_source_list_expandable_item_child_removed, self);
#line 4019 "SourceList.c"
}

static void
mail_source_list_data_model_on_item_child_added (MailSourceListDataModel* self,
                                                 MailSourceListItem* item)
{
#line 903 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 903 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 904 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_add_item (self, item);
#line 4032 "SourceList.c"
}

static void
mail_source_list_data_model_on_item_child_removed (MailSourceListDataModel* self,
                                                   MailSourceListItem* item)
{
#line 907 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 907 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 908 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_remove_item (self, item);
#line 4045 "SourceList.c"
}

static void
mail_source_list_data_model_on_item_prop_changed (MailSourceListDataModel* self,
                                                  MailSourceListItem* item,
                                                  const gchar* prop_name)
{
#line 911 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 911 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 911 "../src/SourceList/SourceList.vala"
	g_return_if_fail (prop_name != NULL);
#line 912 "../src/SourceList/SourceList.vala"
	if (g_strcmp0 (prop_name, "parent") != 0) {
#line 913 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_update_item (self, item);
#line 4063 "SourceList.c"
	}
}

/**
         * Pushes a call to update_item() if //parent// is not //null//.
         *
         * This is needed because the visibility of categories depends on their n_children property,
         * and also because item expansion should be updated after adding or removing items.
         * If many updates are pushed, and the item has still not been updated, only one is processed.
         * This guarantees efficiency as updating a category item could trigger expensive actions.
         */
static Block48Data*
block48_data_ref (Block48Data* _data48_)
{
#line 924 "../src/SourceList/SourceList.vala"
	g_atomic_int_inc (&_data48_->_ref_count_);
#line 924 "../src/SourceList/SourceList.vala"
	return _data48_;
#line 4082 "SourceList.c"
}

static void
block48_data_unref (void * _userdata_)
{
	Block48Data* _data48_;
	_data48_ = (Block48Data*) _userdata_;
#line 924 "../src/SourceList/SourceList.vala"
	if (g_atomic_int_dec_and_test (&_data48_->_ref_count_)) {
#line 4092 "SourceList.c"
		MailSourceListDataModel* self;
#line 924 "../src/SourceList/SourceList.vala"
		self = _data48_->self;
#line 924 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (_data48_->parent);
#line 924 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (self);
#line 924 "../src/SourceList/SourceList.vala"
		g_slice_free (Block48Data, _data48_);
#line 4102 "SourceList.c"
	}
}

static gboolean
___lambda4_ (Block48Data* _data48_)
{
	MailSourceListDataModel* self;
	gboolean result;
#line 941 "../src/SourceList/SourceList.vala"
	self = _data48_->self;
#line 942 "../src/SourceList/SourceList.vala"
	if (_data48_->parent != NULL) {
#line 943 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_update_item (self, (MailSourceListItem*) _data48_->parent);
#line 946 "../src/SourceList/SourceList.vala"
		g_object_set_data_full ((GObject*) _data48_->parent, MAIL_SOURCE_LIST_DATA_MODEL_ITEM_PARENT_NEEDS_UPDATE, (gpointer) ((gintptr) FALSE), NULL);
#line 4119 "SourceList.c"
	}
#line 949 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 949 "../src/SourceList/SourceList.vala"
	return result;
#line 4125 "SourceList.c"
}

static gboolean
____lambda4__gsource_func (gpointer self)
{
	gboolean result;
	result = ___lambda4_ (self);
#line 941 "../src/SourceList/SourceList.vala"
	return result;
#line 4135 "SourceList.c"
}

static void
mail_source_list_data_model_push_parent_update (MailSourceListDataModel* self,
                                                MailSourceListExpandableItem* parent)
{
	Block48Data* _data48_;
	MailSourceListExpandableItem* _tmp0_;
	gboolean needs_update = FALSE;
	gconstpointer _tmp1_;
	GtkTreePath* path = NULL;
	GtkTreePath* _tmp2_;
	GtkTreePath* _tmp3_;
#line 924 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 924 "../src/SourceList/SourceList.vala"
	_data48_ = g_slice_new0 (Block48Data);
#line 924 "../src/SourceList/SourceList.vala"
	_data48_->_ref_count_ = 1;
#line 924 "../src/SourceList/SourceList.vala"
	_data48_->self = g_object_ref (self);
#line 924 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (parent);
#line 924 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (_data48_->parent);
#line 924 "../src/SourceList/SourceList.vala"
	_data48_->parent = _tmp0_;
#line 925 "../src/SourceList/SourceList.vala"
	if (_data48_->parent == NULL) {
#line 926 "../src/SourceList/SourceList.vala"
		block48_data_unref (_data48_);
#line 926 "../src/SourceList/SourceList.vala"
		_data48_ = NULL;
#line 926 "../src/SourceList/SourceList.vala"
		return;
#line 4171 "SourceList.c"
	}
#line 928 "../src/SourceList/SourceList.vala"
	_tmp1_ = g_object_get_data ((GObject*) _data48_->parent, MAIL_SOURCE_LIST_DATA_MODEL_ITEM_PARENT_NEEDS_UPDATE);
#line 928 "../src/SourceList/SourceList.vala"
	needs_update = (gboolean) ((gintptr) _tmp1_);
#line 932 "../src/SourceList/SourceList.vala"
	if (needs_update) {
#line 933 "../src/SourceList/SourceList.vala"
		block48_data_unref (_data48_);
#line 933 "../src/SourceList/SourceList.vala"
		_data48_ = NULL;
#line 933 "../src/SourceList/SourceList.vala"
		return;
#line 4185 "SourceList.c"
	}
#line 935 "../src/SourceList/SourceList.vala"
	_tmp2_ = mail_source_list_data_model_get_item_path (self, (MailSourceListItem*) _data48_->parent);
#line 935 "../src/SourceList/SourceList.vala"
	path = _tmp2_;
#line 937 "../src/SourceList/SourceList.vala"
	_tmp3_ = path;
#line 937 "../src/SourceList/SourceList.vala"
	if (_tmp3_ != NULL) {
#line 939 "../src/SourceList/SourceList.vala"
		g_object_set_data_full ((GObject*) _data48_->parent, MAIL_SOURCE_LIST_DATA_MODEL_ITEM_PARENT_NEEDS_UPDATE, (gpointer) ((gintptr) TRUE), NULL);
#line 941 "../src/SourceList/SourceList.vala"
		g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, ____lambda4__gsource_func, block48_data_ref (_data48_), block48_data_unref);
#line 4199 "SourceList.c"
	}
#line 924 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 924 "../src/SourceList/SourceList.vala"
	block48_data_unref (_data48_);
#line 924 "../src/SourceList/SourceList.vala"
	_data48_ = NULL;
#line 4207 "SourceList.c"
}

/**
         * Returns the Item pointed by iter, or null if the iter doesn't refer to a valid item.
         */
static MailSourceListItem*
mail_source_list_data_model_get_item (MailSourceListDataModel* self,
                                      GtkTreeIter* iter)
{
	MailSourceListItem* item = NULL;
	GtkTreeIter _tmp0_;
	MailSourceListItem* _tmp1_ = NULL;
	MailSourceListItem* result;
#line 957 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 957 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, NULL);
#line 959 "../src/SourceList/SourceList.vala"
	_tmp0_ = *iter;
#line 959 "../src/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) self, &_tmp0_, MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp1_, -1, -1);
#line 959 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 959 "../src/SourceList/SourceList.vala"
	item = _tmp1_;
#line 960 "../src/SourceList/SourceList.vala"
	result = item;
#line 960 "../src/SourceList/SourceList.vala"
	return result;
#line 4237 "SourceList.c"
}

/**
         * Returns the Item pointed by path, or null if the path doesn't refer to a valid item.
         */
static MailSourceListItem*
mail_source_list_data_model_get_item_from_path (MailSourceListDataModel* self,
                                                GtkTreePath* path)
{
	GtkTreeIter iter = {0};
	GtkTreeIter _tmp0_ = {0};
	gboolean _tmp1_;
	MailSourceListItem* result;
#line 966 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 966 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 968 "../src/SourceList/SourceList.vala"
	_tmp1_ = gtk_tree_model_get_iter ((GtkTreeModel*) self, &_tmp0_, path);
#line 968 "../src/SourceList/SourceList.vala"
	iter = _tmp0_;
#line 968 "../src/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 4261 "SourceList.c"
		GtkTreeIter _tmp2_;
		MailSourceListItem* _tmp3_;
#line 969 "../src/SourceList/SourceList.vala"
		_tmp2_ = iter;
#line 969 "../src/SourceList/SourceList.vala"
		_tmp3_ = mail_source_list_data_model_get_item (self, &_tmp2_);
#line 969 "../src/SourceList/SourceList.vala"
		result = _tmp3_;
#line 969 "../src/SourceList/SourceList.vala"
		return result;
#line 4272 "SourceList.c"
	}
#line 971 "../src/SourceList/SourceList.vala"
	result = NULL;
#line 971 "../src/SourceList/SourceList.vala"
	return result;
#line 4278 "SourceList.c"
}

/**
         * Returns a newly-created path pointing to the item, or null in case a valid path
         * is not found.
         */
static GtkTreePath*
mail_source_list_data_model_get_item_path (MailSourceListDataModel* self,
                                           MailSourceListItem* item)
{
	GtkTreePath* path = NULL;
	GtkTreePath* child_path = NULL;
	GtkTreePath* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* result;
#line 978 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 978 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 979 "../src/SourceList/SourceList.vala"
	path = NULL;
#line 979 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_get_item_child_path (self, item);
#line 979 "../src/SourceList/SourceList.vala"
	child_path = _tmp0_;
#line 982 "../src/SourceList/SourceList.vala"
	_tmp1_ = child_path;
#line 982 "../src/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 4308 "SourceList.c"
		GtkTreePath* _tmp2_;
		GtkTreePath* _tmp3_;
#line 983 "../src/SourceList/SourceList.vala"
		_tmp2_ = child_path;
#line 983 "../src/SourceList/SourceList.vala"
		_tmp3_ = gtk_tree_model_filter_convert_child_path_to_path ((GtkTreeModelFilter*) self, _tmp2_);
#line 983 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 983 "../src/SourceList/SourceList.vala"
		path = _tmp3_;
#line 4319 "SourceList.c"
	}
#line 985 "../src/SourceList/SourceList.vala"
	result = path;
#line 985 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (child_path);
#line 985 "../src/SourceList/SourceList.vala"
	return result;
#line 4327 "SourceList.c"
}

/**
         * External "extra" filter method.
         */
static void
mail_source_list_data_model_set_filter_func (MailSourceListDataModel* self,
                                             MailSourceListVisibleFunc visible_func,
                                             gpointer visible_func_target)
{
#line 991 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 992 "../src/SourceList/SourceList.vala"
	self->priv->filter_func = visible_func;
#line 992 "../src/SourceList/SourceList.vala"
	self->priv->filter_func_target = visible_func_target;
#line 4344 "SourceList.c"
}

/**
         * Checks whether an item is a category (i.e. a root-level expandable item).
         * The caller must pass an iter or path pointing to the item, but not both
         * (one of them must be null.)
         *
         * TODO: instead of checking the position of the iter or path, we should simply
         * check whether the item's parent is the root item and whether the item is
         * expandable. We don't do so right now because vala still allows client code
         * to access the Item.parent property, even though its setter is defined as internal.
         */
static gboolean
mail_source_list_data_model_is_category (MailSourceListDataModel* self,
                                         MailSourceListItem* item,
                                         GtkTreeIter* iter,
                                         GtkTreePath* path)
{
	gboolean is_category = FALSE;
	gboolean result;
#line 1005 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1005 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 1006 "../src/SourceList/SourceList.vala"
	is_category = FALSE;
#line 1008 "../src/SourceList/SourceList.vala"
	if (MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (item)) {
#line 1009 "../src/SourceList/SourceList.vala"
		if (iter != NULL) {
#line 4375 "SourceList.c"
			GtkTreeIter _tmp0_;
#line 1010 "../src/SourceList/SourceList.vala"
			_vala_assert (path == NULL, "path == null");
#line 1011 "../src/SourceList/SourceList.vala"
			_tmp0_ = *iter;
#line 1011 "../src/SourceList/SourceList.vala"
			is_category = mail_source_list_data_model_is_iter_at_root_level (self, &_tmp0_);
#line 4383 "SourceList.c"
		} else {
#line 1013 "../src/SourceList/SourceList.vala"
			_vala_assert (iter == NULL, "iter == null");
#line 1014 "../src/SourceList/SourceList.vala"
			is_category = mail_source_list_data_model_is_path_at_root_level (self, path);
#line 4389 "SourceList.c"
		}
	}
#line 1017 "../src/SourceList/SourceList.vala"
	result = is_category;
#line 1017 "../src/SourceList/SourceList.vala"
	return result;
#line 4396 "SourceList.c"
}

static gboolean
mail_source_list_data_model_is_iter_at_root_level (MailSourceListDataModel* self,
                                                   GtkTreeIter* iter)
{
	GtkTreeIter _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp2_;
	gboolean _tmp3_;
	gboolean result;
#line 1020 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1020 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, FALSE);
#line 1021 "../src/SourceList/SourceList.vala"
	_tmp0_ = *iter;
#line 1021 "../src/SourceList/SourceList.vala"
	_tmp1_ = gtk_tree_model_get_path ((GtkTreeModel*) self, &_tmp0_);
#line 1021 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 1021 "../src/SourceList/SourceList.vala"
	_tmp3_ = mail_source_list_data_model_is_path_at_root_level (self, _tmp2_);
#line 1021 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (_tmp2_);
#line 1021 "../src/SourceList/SourceList.vala"
	result = _tmp3_;
#line 1021 "../src/SourceList/SourceList.vala"
	return result;
#line 4426 "SourceList.c"
}

static gboolean
mail_source_list_data_model_is_path_at_root_level (MailSourceListDataModel* self,
                                                   GtkTreePath* path)
{
	gboolean result;
#line 1024 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1024 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1025 "../src/SourceList/SourceList.vala"
	result = gtk_tree_path_get_depth (path) == 1;
#line 1025 "../src/SourceList/SourceList.vala"
	return result;
#line 4442 "SourceList.c"
}

static void
mail_source_list_data_model_resort (MailSourceListDataModel* self)
{
	GtkTreeStore* _tmp0_;
	GtkTreeStore* _tmp1_;
#line 1028 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1029 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1029 "../src/SourceList/SourceList.vala"
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) _tmp0_, GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
#line 1030 "../src/SourceList/SourceList.vala"
	_tmp1_ = self->priv->child_tree;
#line 1030 "../src/SourceList/SourceList.vala"
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) _tmp1_, GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
#line 4460 "SourceList.c"
}

static gint
mail_source_list_data_model_child_model_sort_func (MailSourceListDataModel* self,
                                                   GtkTreeModel* model,
                                                   GtkTreeIter* a,
                                                   GtkTreeIter* b)
{
	gint order = 0;
	MailSourceListItem* item_a = NULL;
	MailSourceListItem* item_b = NULL;
	GtkTreeStore* _tmp0_;
	GtkTreeIter _tmp1_;
	MailSourceListItem* _tmp2_ = NULL;
	GtkTreeStore* _tmp3_;
	GtkTreeIter _tmp4_;
	MailSourceListItem* _tmp5_ = NULL;
	MailSourceListItem* _tmp6_;
	MailSourceListExpandableItem* _tmp7_;
	MailSourceListExpandableItem* _tmp8_;
	MailSourceListItem* _tmp9_;
	MailSourceListExpandableItem* _tmp10_;
	MailSourceListExpandableItem* _tmp11_;
	MailSourceListSortable* parent = NULL;
	MailSourceListItem* _tmp12_;
	MailSourceListExpandableItem* _tmp13_;
	MailSourceListExpandableItem* _tmp14_;
	MailSourceListSortable* _tmp15_;
	MailSourceListSortable* _tmp16_;
	gint result;
#line 1033 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 1033 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (model != NULL, 0);
#line 1033 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (a != NULL, 0);
#line 1033 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (b != NULL, 0);
#line 1034 "../src/SourceList/SourceList.vala"
	order = 0;
#line 1037 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1037 "../src/SourceList/SourceList.vala"
	_tmp1_ = *a;
#line 1037 "../src/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp0_, &_tmp1_, MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp2_, -1, -1);
#line 1037 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item_a);
#line 1037 "../src/SourceList/SourceList.vala"
	item_a = _tmp2_;
#line 1038 "../src/SourceList/SourceList.vala"
	_tmp3_ = self->priv->child_tree;
#line 1038 "../src/SourceList/SourceList.vala"
	_tmp4_ = *b;
#line 1038 "../src/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp3_, &_tmp4_, MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp5_, -1, -1);
#line 1038 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item_b);
#line 1038 "../src/SourceList/SourceList.vala"
	item_b = _tmp5_;
#line 1041 "../src/SourceList/SourceList.vala"
	_tmp6_ = item_a;
#line 1041 "../src/SourceList/SourceList.vala"
	_tmp7_ = mail_source_list_item_get_parent (_tmp6_);
#line 1041 "../src/SourceList/SourceList.vala"
	_tmp8_ = _tmp7_;
#line 1041 "../src/SourceList/SourceList.vala"
	_tmp9_ = item_b;
#line 1041 "../src/SourceList/SourceList.vala"
	_tmp10_ = mail_source_list_item_get_parent (_tmp9_);
#line 1041 "../src/SourceList/SourceList.vala"
	_tmp11_ = _tmp10_;
#line 1041 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp8_ == _tmp11_, "item_a.parent == item_b.parent");
#line 1043 "../src/SourceList/SourceList.vala"
	_tmp12_ = item_a;
#line 1043 "../src/SourceList/SourceList.vala"
	_tmp13_ = mail_source_list_item_get_parent (_tmp12_);
#line 1043 "../src/SourceList/SourceList.vala"
	_tmp14_ = _tmp13_;
#line 1043 "../src/SourceList/SourceList.vala"
	_tmp15_ = _g_object_ref0 (MAIL_IS_SOURCE_LIST_SORTABLE (_tmp14_) ? ((MailSourceListSortable*) _tmp14_) : NULL);
#line 1043 "../src/SourceList/SourceList.vala"
	parent = _tmp15_;
#line 1044 "../src/SourceList/SourceList.vala"
	_tmp16_ = parent;
#line 1044 "../src/SourceList/SourceList.vala"
	if (_tmp16_ != NULL) {
#line 4549 "SourceList.c"
		MailSourceListSortable* _tmp17_;
		MailSourceListItem* _tmp18_;
		MailSourceListItem* _tmp19_;
#line 1045 "../src/SourceList/SourceList.vala"
		_tmp17_ = parent;
#line 1045 "../src/SourceList/SourceList.vala"
		_tmp18_ = item_a;
#line 1045 "../src/SourceList/SourceList.vala"
		_tmp19_ = item_b;
#line 1045 "../src/SourceList/SourceList.vala"
		order = mail_source_list_sortable_compare (_tmp17_, _tmp18_, _tmp19_);
#line 4561 "SourceList.c"
	}
#line 1047 "../src/SourceList/SourceList.vala"
	result = order;
#line 1047 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (parent);
#line 1047 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item_b);
#line 1047 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item_a);
#line 1047 "../src/SourceList/SourceList.vala"
	return result;
#line 4573 "SourceList.c"
}

static GtkTreeIter*
mail_source_list_data_model_get_item_child_iter (MailSourceListDataModel* self,
                                                 MailSourceListItem* item)
{
	GtkTreeIter* child_iter = NULL;
	MailSourceListDataModelNodeWrapper* child_node_wrapper = NULL;
	GeeHashMap* _tmp0_;
	gpointer _tmp1_;
	MailSourceListDataModelNodeWrapper* _tmp2_;
	GtkTreeIter* result;
#line 1050 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1050 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 1051 "../src/SourceList/SourceList.vala"
	child_iter = NULL;
#line 1053 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->items;
#line 1053 "../src/SourceList/SourceList.vala"
	_tmp1_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp0_, item);
#line 1053 "../src/SourceList/SourceList.vala"
	child_node_wrapper = (MailSourceListDataModelNodeWrapper*) _tmp1_;
#line 1054 "../src/SourceList/SourceList.vala"
	_tmp2_ = child_node_wrapper;
#line 1054 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 4602 "SourceList.c"
		MailSourceListDataModelNodeWrapper* _tmp3_;
		GtkTreeIter* _tmp4_;
		GtkTreeIter* _tmp5_;
#line 1055 "../src/SourceList/SourceList.vala"
		_tmp3_ = child_node_wrapper;
#line 1055 "../src/SourceList/SourceList.vala"
		_tmp4_ = mail_source_list_data_model_node_wrapper_get_iter (_tmp3_);
#line 1055 "../src/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 1055 "../src/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (child_iter);
#line 1055 "../src/SourceList/SourceList.vala"
		child_iter = _tmp5_;
#line 4616 "SourceList.c"
	}
#line 1057 "../src/SourceList/SourceList.vala"
	result = child_iter;
#line 1057 "../src/SourceList/SourceList.vala"
	_mail_source_list_data_model_node_wrapper_unref0 (child_node_wrapper);
#line 1057 "../src/SourceList/SourceList.vala"
	return result;
#line 4624 "SourceList.c"
}

static GtkTreePath*
mail_source_list_data_model_get_item_child_path (MailSourceListDataModel* self,
                                                 MailSourceListItem* item)
{
	GtkTreePath* child_path = NULL;
	MailSourceListDataModelNodeWrapper* child_node_wrapper = NULL;
	GeeHashMap* _tmp0_;
	gpointer _tmp1_;
	MailSourceListDataModelNodeWrapper* _tmp2_;
	GtkTreePath* result;
#line 1060 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1060 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 1061 "../src/SourceList/SourceList.vala"
	child_path = NULL;
#line 1063 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->items;
#line 1063 "../src/SourceList/SourceList.vala"
	_tmp1_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp0_, item);
#line 1063 "../src/SourceList/SourceList.vala"
	child_node_wrapper = (MailSourceListDataModelNodeWrapper*) _tmp1_;
#line 1064 "../src/SourceList/SourceList.vala"
	_tmp2_ = child_node_wrapper;
#line 1064 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 4653 "SourceList.c"
		MailSourceListDataModelNodeWrapper* _tmp3_;
		GtkTreePath* _tmp4_;
		GtkTreePath* _tmp5_;
#line 1065 "../src/SourceList/SourceList.vala"
		_tmp3_ = child_node_wrapper;
#line 1065 "../src/SourceList/SourceList.vala"
		_tmp4_ = mail_source_list_data_model_node_wrapper_get_path (_tmp3_);
#line 1065 "../src/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 1065 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_path);
#line 1065 "../src/SourceList/SourceList.vala"
		child_path = _tmp5_;
#line 4667 "SourceList.c"
	}
#line 1067 "../src/SourceList/SourceList.vala"
	result = child_path;
#line 1067 "../src/SourceList/SourceList.vala"
	_mail_source_list_data_model_node_wrapper_unref0 (child_node_wrapper);
#line 1067 "../src/SourceList/SourceList.vala"
	return result;
#line 4675 "SourceList.c"
}

/**
         * Filters the child-tree items based on their "visible" property.
         */
static gboolean
mail_source_list_data_model_filter_visible_func (MailSourceListDataModel* self,
                                                 GtkTreeModel* child_model,
                                                 GtkTreeIter* iter)
{
	gboolean item_visible = FALSE;
	MailSourceListItem* item = NULL;
	GtkTreeStore* _tmp0_;
	GtkTreeIter _tmp1_;
	MailSourceListItem* _tmp2_ = NULL;
	MailSourceListItem* _tmp3_;
	MailSourceListVisibleFunc _tmp27_;
	gpointer _tmp27__target;
	gboolean result;
#line 1073 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1073 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (child_model != NULL, FALSE);
#line 1073 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, FALSE);
#line 1074 "../src/SourceList/SourceList.vala"
	item_visible = FALSE;
#line 1077 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1077 "../src/SourceList/SourceList.vala"
	_tmp1_ = *iter;
#line 1077 "../src/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp0_, &_tmp1_, MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp2_, -1, -1);
#line 1077 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1077 "../src/SourceList/SourceList.vala"
	item = _tmp2_;
#line 1079 "../src/SourceList/SourceList.vala"
	_tmp3_ = item;
#line 1079 "../src/SourceList/SourceList.vala"
	if (_tmp3_ != NULL) {
#line 4717 "SourceList.c"
		MailSourceListItem* _tmp4_;
		gboolean _tmp5_;
		gboolean _tmp6_;
		MailSourceListExpandableItem* expandable = NULL;
		MailSourceListItem* _tmp7_;
		MailSourceListExpandableItem* _tmp8_;
		gboolean _tmp9_ = FALSE;
		MailSourceListExpandableItem* _tmp10_;
#line 1080 "../src/SourceList/SourceList.vala"
		_tmp4_ = item;
#line 1080 "../src/SourceList/SourceList.vala"
		_tmp5_ = mail_source_list_item_get_visible (_tmp4_);
#line 1080 "../src/SourceList/SourceList.vala"
		_tmp6_ = _tmp5_;
#line 1080 "../src/SourceList/SourceList.vala"
		item_visible = _tmp6_;
#line 1084 "../src/SourceList/SourceList.vala"
		_tmp7_ = item;
#line 1084 "../src/SourceList/SourceList.vala"
		_tmp8_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp7_) ? ((MailSourceListExpandableItem*) _tmp7_) : NULL);
#line 1084 "../src/SourceList/SourceList.vala"
		expandable = _tmp8_;
#line 1085 "../src/SourceList/SourceList.vala"
		_tmp10_ = expandable;
#line 1085 "../src/SourceList/SourceList.vala"
		if (_tmp10_ != NULL) {
#line 4744 "SourceList.c"
			GtkTreeStore* _tmp11_;
			GtkTreeIter _tmp12_;
#line 1085 "../src/SourceList/SourceList.vala"
			_tmp11_ = self->priv->child_tree;
#line 1085 "../src/SourceList/SourceList.vala"
			_tmp12_ = *iter;
#line 1085 "../src/SourceList/SourceList.vala"
			_tmp9_ = gtk_tree_store_iter_depth (_tmp11_, &_tmp12_) == 0;
#line 4753 "SourceList.c"
		} else {
#line 1085 "../src/SourceList/SourceList.vala"
			_tmp9_ = FALSE;
#line 4757 "SourceList.c"
		}
#line 1085 "../src/SourceList/SourceList.vala"
		if (_tmp9_) {
#line 4761 "SourceList.c"
			guint n_visible_children = 0U;
			gboolean _tmp26_ = FALSE;
#line 1086 "../src/SourceList/SourceList.vala"
			n_visible_children = (guint) 0;
#line 4766 "SourceList.c"
			{
				GeeIterator* _child_item_it = NULL;
				MailSourceListExpandableItem* _tmp13_;
				GeeCollection* _tmp14_;
				GeeCollection* _tmp15_;
				GeeCollection* _tmp16_;
				GeeIterator* _tmp17_;
				GeeIterator* _tmp18_;
#line 1087 "../src/SourceList/SourceList.vala"
				_tmp13_ = expandable;
#line 1087 "../src/SourceList/SourceList.vala"
				_tmp14_ = mail_source_list_expandable_item_get_children (_tmp13_);
#line 1087 "../src/SourceList/SourceList.vala"
				_tmp15_ = _tmp14_;
#line 1087 "../src/SourceList/SourceList.vala"
				_tmp16_ = _tmp15_;
#line 1087 "../src/SourceList/SourceList.vala"
				_tmp17_ = gee_iterable_iterator ((GeeIterable*) _tmp16_);
#line 1087 "../src/SourceList/SourceList.vala"
				_tmp18_ = _tmp17_;
#line 1087 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp16_);
#line 1087 "../src/SourceList/SourceList.vala"
				_child_item_it = _tmp18_;
#line 1087 "../src/SourceList/SourceList.vala"
				while (TRUE) {
#line 4793 "SourceList.c"
					GeeIterator* _tmp19_;
					MailSourceListItem* child_item = NULL;
					GeeIterator* _tmp20_;
					gpointer _tmp21_;
					MailSourceListItem* _tmp22_;
					gboolean _tmp23_;
					gboolean _tmp24_;
#line 1087 "../src/SourceList/SourceList.vala"
					_tmp19_ = _child_item_it;
#line 1087 "../src/SourceList/SourceList.vala"
					if (!gee_iterator_next (_tmp19_)) {
#line 1087 "../src/SourceList/SourceList.vala"
						break;
#line 4807 "SourceList.c"
					}
#line 1087 "../src/SourceList/SourceList.vala"
					_tmp20_ = _child_item_it;
#line 1087 "../src/SourceList/SourceList.vala"
					_tmp21_ = gee_iterator_get (_tmp20_);
#line 1087 "../src/SourceList/SourceList.vala"
					child_item = (MailSourceListItem*) _tmp21_;
#line 1088 "../src/SourceList/SourceList.vala"
					_tmp22_ = child_item;
#line 1088 "../src/SourceList/SourceList.vala"
					_tmp23_ = mail_source_list_item_get_visible (_tmp22_);
#line 1088 "../src/SourceList/SourceList.vala"
					_tmp24_ = _tmp23_;
#line 1088 "../src/SourceList/SourceList.vala"
					if (_tmp24_) {
#line 4823 "SourceList.c"
						guint _tmp25_;
#line 1089 "../src/SourceList/SourceList.vala"
						_tmp25_ = n_visible_children;
#line 1089 "../src/SourceList/SourceList.vala"
						n_visible_children = _tmp25_ + 1;
#line 4829 "SourceList.c"
					}
#line 1087 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (child_item);
#line 4833 "SourceList.c"
				}
#line 1087 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (_child_item_it);
#line 4837 "SourceList.c"
			}
#line 1091 "../src/SourceList/SourceList.vala"
			if (item_visible) {
#line 1091 "../src/SourceList/SourceList.vala"
				_tmp26_ = n_visible_children > ((guint) 0);
#line 4843 "SourceList.c"
			} else {
#line 1091 "../src/SourceList/SourceList.vala"
				_tmp26_ = FALSE;
#line 4847 "SourceList.c"
			}
#line 1091 "../src/SourceList/SourceList.vala"
			item_visible = _tmp26_;
#line 4851 "SourceList.c"
		}
#line 1079 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (expandable);
#line 4855 "SourceList.c"
	}
#line 1095 "../src/SourceList/SourceList.vala"
	_tmp27_ = self->priv->filter_func;
#line 1095 "../src/SourceList/SourceList.vala"
	_tmp27__target = self->priv->filter_func_target;
#line 1095 "../src/SourceList/SourceList.vala"
	if (_tmp27_ != NULL) {
#line 4863 "SourceList.c"
		gboolean _tmp28_ = FALSE;
#line 1096 "../src/SourceList/SourceList.vala"
		if (item_visible) {
#line 4867 "SourceList.c"
			MailSourceListVisibleFunc _tmp29_;
			gpointer _tmp29__target;
			MailSourceListItem* _tmp30_;
#line 1096 "../src/SourceList/SourceList.vala"
			_tmp29_ = self->priv->filter_func;
#line 1096 "../src/SourceList/SourceList.vala"
			_tmp29__target = self->priv->filter_func_target;
#line 1096 "../src/SourceList/SourceList.vala"
			_tmp30_ = item;
#line 1096 "../src/SourceList/SourceList.vala"
			_tmp28_ = _tmp29_ (_tmp30_, _tmp29__target);
#line 4879 "SourceList.c"
		} else {
#line 1096 "../src/SourceList/SourceList.vala"
			_tmp28_ = FALSE;
#line 4883 "SourceList.c"
		}
#line 1096 "../src/SourceList/SourceList.vala"
		item_visible = _tmp28_;
#line 4887 "SourceList.c"
	}
#line 1098 "../src/SourceList/SourceList.vala"
	result = item_visible;
#line 1098 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1098 "../src/SourceList/SourceList.vala"
	return result;
#line 4895 "SourceList.c"
}

/**
         * TreeDragDest implementation
         */
static gboolean
mail_source_list_data_model_real_drag_data_received (GtkTreeDragDest* base,
                                                     GtkTreePath* dest,
                                                     GtkSelectionData* selection_data)
{
	MailSourceListDataModel * self;
	GtkTreeModel* model = NULL;
	GtkTreePath* src_path = NULL;
	gboolean _tmp0_ = FALSE;
	GtkTreeModel* _tmp1_ = NULL;
	GtkTreePath* _tmp2_ = NULL;
	gboolean _tmp3_;
	GtkTreeModel* _tmp4_;
	gboolean result;
#line 1105 "../src/SourceList/SourceList.vala"
	self = (MailSourceListDataModel*) base;
#line 1105 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (dest != NULL, FALSE);
#line 1105 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (selection_data != NULL, FALSE);
#line 1113 "../src/SourceList/SourceList.vala"
	_tmp3_ = gtk_tree_get_row_drag_data (selection_data, &_tmp1_, &_tmp2_);
#line 1113 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (model);
#line 1113 "../src/SourceList/SourceList.vala"
	_tmp4_ = _g_object_ref0 (_tmp1_);
#line 1113 "../src/SourceList/SourceList.vala"
	model = _tmp4_;
#line 1113 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (src_path);
#line 1113 "../src/SourceList/SourceList.vala"
	src_path = _tmp2_;
#line 1113 "../src/SourceList/SourceList.vala"
	if (_tmp3_) {
#line 4935 "SourceList.c"
		GtkTreeModel* _tmp5_;
		GtkTreeStore* _tmp6_;
#line 1113 "../src/SourceList/SourceList.vala"
		_tmp5_ = model;
#line 1113 "../src/SourceList/SourceList.vala"
		_tmp6_ = self->priv->child_tree;
#line 1113 "../src/SourceList/SourceList.vala"
		_tmp0_ = _tmp5_ == G_TYPE_CHECK_INSTANCE_CAST (_tmp6_, gtk_tree_model_get_type (), GtkTreeModel);
#line 4944 "SourceList.c"
	} else {
#line 1113 "../src/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 4948 "SourceList.c"
	}
#line 1113 "../src/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 4952 "SourceList.c"
		GtkTreePath* child_dest = NULL;
		GtkTreePath* _tmp7_;
		GtkTreePath* _tmp8_;
#line 1115 "../src/SourceList/SourceList.vala"
		_tmp7_ = gtk_tree_model_filter_convert_path_to_child_path ((GtkTreeModelFilter*) self, dest);
#line 1115 "../src/SourceList/SourceList.vala"
		child_dest = _tmp7_;
#line 1117 "../src/SourceList/SourceList.vala"
		_tmp8_ = child_dest;
#line 1117 "../src/SourceList/SourceList.vala"
		if (_tmp8_ != NULL) {
#line 4964 "SourceList.c"
			GtkTreePath* _tmp9_;
			GtkTreePath* _tmp10_;
#line 1119 "../src/SourceList/SourceList.vala"
			_tmp9_ = child_dest;
#line 1119 "../src/SourceList/SourceList.vala"
			_tmp10_ = src_path;
#line 1119 "../src/SourceList/SourceList.vala"
			if (mail_source_list_data_model_child_tree_drag_data_received (self, _tmp9_, _tmp10_)) {
#line 1120 "../src/SourceList/SourceList.vala"
				result = TRUE;
#line 1120 "../src/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (child_dest);
#line 1120 "../src/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (src_path);
#line 1120 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (model);
#line 1120 "../src/SourceList/SourceList.vala"
				return result;
#line 4983 "SourceList.c"
			}
		}
#line 1113 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 4988 "SourceList.c"
	}
#line 1125 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 1125 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (src_path);
#line 1125 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (model);
#line 1125 "../src/SourceList/SourceList.vala"
	return result;
#line 4998 "SourceList.c"
}

static GtkTreePath*
_vala_GtkTreePath_copy (GtkTreePath* self)
{
#line 1135 "../src/SourceList/SourceList.vala"
	return g_boxed_copy (gtk_tree_path_get_type (), self);
#line 5006 "SourceList.c"
}

static gpointer
__vala_GtkTreePath_copy0 (gpointer self)
{
#line 1135 "../src/SourceList/SourceList.vala"
	return self ? _vala_GtkTreePath_copy (self) : NULL;
#line 5014 "SourceList.c"
}

static gboolean
mail_source_list_data_model_child_tree_drag_data_received (MailSourceListDataModel* self,
                                                           GtkTreePath* dest,
                                                           GtkTreePath* src_path)
{
	gboolean retval = FALSE;
	GtkTreeIter src_iter = {0};
	GtkTreeIter dest_iter = {0};
	GtkTreeStore* _tmp0_;
	GtkTreeIter _tmp1_ = {0};
	gboolean _tmp2_;
	GtkTreePath* prev = NULL;
	GtkTreePath* _tmp3_;
	GtkTreePath* _tmp4_;
	gboolean result;
#line 1128 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1128 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (dest != NULL, FALSE);
#line 1128 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (src_path != NULL, FALSE);
#line 1129 "../src/SourceList/SourceList.vala"
	retval = FALSE;
#line 1132 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1132 "../src/SourceList/SourceList.vala"
	_tmp2_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp0_, &_tmp1_, src_path);
#line 1132 "../src/SourceList/SourceList.vala"
	src_iter = _tmp1_;
#line 1132 "../src/SourceList/SourceList.vala"
	if (!_tmp2_) {
#line 1133 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1133 "../src/SourceList/SourceList.vala"
		return result;
#line 5052 "SourceList.c"
	}
#line 1135 "../src/SourceList/SourceList.vala"
	_tmp3_ = __vala_GtkTreePath_copy0 (dest);
#line 1135 "../src/SourceList/SourceList.vala"
	prev = _tmp3_;
#line 1138 "../src/SourceList/SourceList.vala"
	_tmp4_ = prev;
#line 1138 "../src/SourceList/SourceList.vala"
	if (!gtk_tree_path_prev (_tmp4_)) {
#line 5062 "SourceList.c"
		GtkTreePath* parent = NULL;
		GtkTreePath* _tmp5_;
		GtkTreeIter* dest_parent = NULL;
		gboolean _tmp6_ = FALSE;
		GtkTreePath* _tmp7_;
		GtkTreeStore* _tmp14_;
		GtkTreeIter* _tmp15_;
		GtkTreeIter _tmp16_ = {0};
#line 1142 "../src/SourceList/SourceList.vala"
		_tmp5_ = __vala_GtkTreePath_copy0 (dest);
#line 1142 "../src/SourceList/SourceList.vala"
		parent = _tmp5_;
#line 1143 "../src/SourceList/SourceList.vala"
		dest_parent = NULL;
#line 1145 "../src/SourceList/SourceList.vala"
		_tmp7_ = parent;
#line 1145 "../src/SourceList/SourceList.vala"
		if (gtk_tree_path_up (_tmp7_)) {
#line 5081 "SourceList.c"
			GtkTreePath* _tmp8_;
#line 1145 "../src/SourceList/SourceList.vala"
			_tmp8_ = parent;
#line 1145 "../src/SourceList/SourceList.vala"
			_tmp6_ = gtk_tree_path_get_depth (_tmp8_) > 0;
#line 5087 "SourceList.c"
		} else {
#line 1145 "../src/SourceList/SourceList.vala"
			_tmp6_ = FALSE;
#line 5091 "SourceList.c"
		}
#line 1145 "../src/SourceList/SourceList.vala"
		if (_tmp6_) {
#line 5095 "SourceList.c"
			GtkTreeStore* _tmp9_;
			GtkTreePath* _tmp10_;
			GtkTreeIter _tmp11_ = {0};
			GtkTreeIter _tmp12_;
			GtkTreeIter* _tmp13_;
#line 1146 "../src/SourceList/SourceList.vala"
			_tmp9_ = self->priv->child_tree;
#line 1146 "../src/SourceList/SourceList.vala"
			_tmp10_ = parent;
#line 1146 "../src/SourceList/SourceList.vala"
			gtk_tree_model_get_iter ((GtkTreeModel*) _tmp9_, &_tmp11_, _tmp10_);
#line 1146 "../src/SourceList/SourceList.vala"
			__vala_GtkTreeIter_free0 (dest_parent);
#line 1146 "../src/SourceList/SourceList.vala"
			_tmp12_ = _tmp11_;
#line 1146 "../src/SourceList/SourceList.vala"
			_tmp13_ = __vala_GtkTreeIter_copy0 (&_tmp12_);
#line 1146 "../src/SourceList/SourceList.vala"
			dest_parent = _tmp13_;
#line 5115 "SourceList.c"
		}
#line 1148 "../src/SourceList/SourceList.vala"
		_tmp14_ = self->priv->child_tree;
#line 1148 "../src/SourceList/SourceList.vala"
		_tmp15_ = dest_parent;
#line 1148 "../src/SourceList/SourceList.vala"
		gtk_tree_store_prepend (_tmp14_, &_tmp16_, _tmp15_);
#line 1148 "../src/SourceList/SourceList.vala"
		dest_iter = _tmp16_;
#line 1149 "../src/SourceList/SourceList.vala"
		retval = TRUE;
#line 1138 "../src/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (dest_parent);
#line 1138 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (parent);
#line 5131 "SourceList.c"
	} else {
		GtkTreeStore* _tmp17_;
		GtkTreePath* _tmp18_;
		GtkTreeIter _tmp19_ = {0};
		gboolean _tmp20_;
#line 1150 "../src/SourceList/SourceList.vala"
		_tmp17_ = self->priv->child_tree;
#line 1150 "../src/SourceList/SourceList.vala"
		_tmp18_ = prev;
#line 1150 "../src/SourceList/SourceList.vala"
		_tmp20_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp17_, &_tmp19_, _tmp18_);
#line 1150 "../src/SourceList/SourceList.vala"
		dest_iter = _tmp19_;
#line 1150 "../src/SourceList/SourceList.vala"
		if (_tmp20_) {
#line 5147 "SourceList.c"
			GtkTreeIter tmp_iter = {0};
			GtkTreeIter _tmp21_;
			GtkTreeStore* _tmp22_;
			GtkTreeIter _tmp23_;
			GtkTreeIter _tmp24_ = {0};
#line 1151 "../src/SourceList/SourceList.vala"
			_tmp21_ = dest_iter;
#line 1151 "../src/SourceList/SourceList.vala"
			tmp_iter = _tmp21_;
#line 1152 "../src/SourceList/SourceList.vala"
			_tmp22_ = self->priv->child_tree;
#line 1152 "../src/SourceList/SourceList.vala"
			_tmp23_ = tmp_iter;
#line 1152 "../src/SourceList/SourceList.vala"
			gtk_tree_store_insert_after (_tmp22_, &_tmp24_, NULL, &_tmp23_);
#line 1152 "../src/SourceList/SourceList.vala"
			dest_iter = _tmp24_;
#line 1153 "../src/SourceList/SourceList.vala"
			retval = TRUE;
#line 5167 "SourceList.c"
		}
	}
#line 1158 "../src/SourceList/SourceList.vala"
	if (retval) {
#line 5172 "SourceList.c"
		GtkTreeIter _tmp25_;
		GtkTreeIter _tmp26_;
		MailSourceListItem* item = NULL;
		GtkTreeStore* _tmp27_;
		GtkTreeIter _tmp28_;
		MailSourceListItem* _tmp29_ = NULL;
		MailSourceListItem* _tmp30_;
		MailSourceListItem* _tmp31_;
		MailSourceListSortable* parent = NULL;
		MailSourceListItem* _tmp32_;
		MailSourceListExpandableItem* _tmp33_;
		MailSourceListExpandableItem* _tmp34_;
		MailSourceListSortable* _tmp35_;
		MailSourceListSortable* _tmp36_;
		MailSourceListSortable* _tmp37_;
		MailSourceListItem* _tmp38_;
#line 1159 "../src/SourceList/SourceList.vala"
		_tmp25_ = src_iter;
#line 1159 "../src/SourceList/SourceList.vala"
		_tmp26_ = dest_iter;
#line 1159 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_recursive_node_copy (self, &_tmp25_, &_tmp26_);
#line 1163 "../src/SourceList/SourceList.vala"
		_tmp27_ = self->priv->child_tree;
#line 1163 "../src/SourceList/SourceList.vala"
		_tmp28_ = src_iter;
#line 1163 "../src/SourceList/SourceList.vala"
		gtk_tree_model_get ((GtkTreeModel*) _tmp27_, &_tmp28_, MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp29_, -1, -1);
#line 1163 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (item);
#line 1163 "../src/SourceList/SourceList.vala"
		item = _tmp29_;
#line 1164 "../src/SourceList/SourceList.vala"
		_tmp30_ = item;
#line 1164 "../src/SourceList/SourceList.vala"
		g_return_val_if_fail (_tmp30_ != NULL, retval);
#line 1173 "../src/SourceList/SourceList.vala"
		_tmp31_ = item;
#line 1173 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_update_item (self, _tmp31_);
#line 1175 "../src/SourceList/SourceList.vala"
		_tmp32_ = item;
#line 1175 "../src/SourceList/SourceList.vala"
		_tmp33_ = mail_source_list_item_get_parent (_tmp32_);
#line 1175 "../src/SourceList/SourceList.vala"
		_tmp34_ = _tmp33_;
#line 1175 "../src/SourceList/SourceList.vala"
		_tmp35_ = _g_object_ref0 (MAIL_IS_SOURCE_LIST_SORTABLE (_tmp34_) ? ((MailSourceListSortable*) _tmp34_) : NULL);
#line 1175 "../src/SourceList/SourceList.vala"
		parent = _tmp35_;
#line 1176 "../src/SourceList/SourceList.vala"
		_tmp36_ = parent;
#line 1176 "../src/SourceList/SourceList.vala"
		g_return_val_if_fail (_tmp36_ != NULL, retval);
#line 1178 "../src/SourceList/SourceList.vala"
		_tmp37_ = parent;
#line 1178 "../src/SourceList/SourceList.vala"
		_tmp38_ = item;
#line 1178 "../src/SourceList/SourceList.vala"
		g_signal_emit (_tmp37_, mail_source_list_sortable_signals[MAIL_SOURCE_LIST_SORTABLE_USER_MOVED_ITEM_SIGNAL], 0, _tmp38_);
#line 1158 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1158 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (item);
#line 5237 "SourceList.c"
	}
#line 1181 "../src/SourceList/SourceList.vala"
	result = retval;
#line 1181 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (prev);
#line 1181 "../src/SourceList/SourceList.vala"
	return result;
#line 5245 "SourceList.c"
}

static void
mail_source_list_data_model_recursive_node_copy (MailSourceListDataModel* self,
                                                 GtkTreeIter* src_iter,
                                                 GtkTreeIter* dest_iter)
{
	GtkTreeIter _tmp0_;
	GtkTreeIter _tmp1_;
	GtkTreeIter child = {0};
	GtkTreeStore* _tmp2_;
	GtkTreeIter _tmp3_;
	GtkTreeIter _tmp4_ = {0};
	gboolean _tmp5_;
#line 1184 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1184 "../src/SourceList/SourceList.vala"
	g_return_if_fail (src_iter != NULL);
#line 1184 "../src/SourceList/SourceList.vala"
	g_return_if_fail (dest_iter != NULL);
#line 1185 "../src/SourceList/SourceList.vala"
	_tmp0_ = *src_iter;
#line 1185 "../src/SourceList/SourceList.vala"
	_tmp1_ = *dest_iter;
#line 1185 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_move_item (self, &_tmp0_, &_tmp1_);
#line 1188 "../src/SourceList/SourceList.vala"
	_tmp2_ = self->priv->child_tree;
#line 1188 "../src/SourceList/SourceList.vala"
	_tmp3_ = *src_iter;
#line 1188 "../src/SourceList/SourceList.vala"
	_tmp5_ = gtk_tree_model_iter_children ((GtkTreeModel*) _tmp2_, &_tmp4_, &_tmp3_);
#line 1188 "../src/SourceList/SourceList.vala"
	child = _tmp4_;
#line 1188 "../src/SourceList/SourceList.vala"
	if (_tmp5_) {
#line 5282 "SourceList.c"
		{
			gboolean _tmp6_ = FALSE;
#line 1191 "../src/SourceList/SourceList.vala"
			_tmp6_ = TRUE;
#line 1191 "../src/SourceList/SourceList.vala"
			while (TRUE) {
#line 5289 "SourceList.c"
				GtkTreeIter copy = {0};
				GtkTreeStore* _tmp9_;
				GtkTreeIter _tmp10_;
				GtkTreeIter _tmp11_ = {0};
				GtkTreeIter _tmp12_;
				GtkTreeIter _tmp13_;
#line 1191 "../src/SourceList/SourceList.vala"
				if (!_tmp6_) {
#line 5298 "SourceList.c"
					GtkTreeStore* _tmp7_;
					gboolean _tmp8_;
#line 1195 "../src/SourceList/SourceList.vala"
					_tmp7_ = self->priv->child_tree;
#line 1195 "../src/SourceList/SourceList.vala"
					_tmp8_ = gtk_tree_model_iter_next ((GtkTreeModel*) _tmp7_, &child);
#line 1195 "../src/SourceList/SourceList.vala"
					if (!_tmp8_) {
#line 1195 "../src/SourceList/SourceList.vala"
						break;
#line 5309 "SourceList.c"
					}
				}
#line 1191 "../src/SourceList/SourceList.vala"
				_tmp6_ = FALSE;
#line 1193 "../src/SourceList/SourceList.vala"
				_tmp9_ = self->priv->child_tree;
#line 1193 "../src/SourceList/SourceList.vala"
				_tmp10_ = *dest_iter;
#line 1193 "../src/SourceList/SourceList.vala"
				gtk_tree_store_append (_tmp9_, &_tmp11_, &_tmp10_);
#line 1193 "../src/SourceList/SourceList.vala"
				copy = _tmp11_;
#line 1194 "../src/SourceList/SourceList.vala"
				_tmp12_ = child;
#line 1194 "../src/SourceList/SourceList.vala"
				_tmp13_ = copy;
#line 1194 "../src/SourceList/SourceList.vala"
				mail_source_list_data_model_recursive_node_copy (self, &_tmp12_, &_tmp13_);
#line 5328 "SourceList.c"
			}
		}
	}
}

static void
mail_source_list_data_model_move_item (MailSourceListDataModel* self,
                                       GtkTreeIter* src_iter,
                                       GtkTreeIter* dest_iter)
{
	MailSourceListItem* item = NULL;
	GtkTreeStore* _tmp0_;
	GtkTreeIter _tmp1_;
	MailSourceListItem* _tmp2_ = NULL;
	GtkTreeStore* _tmp3_;
	GtkTreeIter _tmp4_;
	GeeHashMap* _tmp5_;
	GtkTreeStore* _tmp6_;
	GtkTreeIter _tmp7_;
	MailSourceListDataModelNodeWrapper* _tmp8_;
	MailSourceListDataModelNodeWrapper* _tmp9_;
#line 1199 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1199 "../src/SourceList/SourceList.vala"
	g_return_if_fail (src_iter != NULL);
#line 1199 "../src/SourceList/SourceList.vala"
	g_return_if_fail (dest_iter != NULL);
#line 1201 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1201 "../src/SourceList/SourceList.vala"
	_tmp1_ = *src_iter;
#line 1201 "../src/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp0_, &_tmp1_, MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp2_, -1, -1);
#line 1201 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1201 "../src/SourceList/SourceList.vala"
	item = _tmp2_;
#line 1202 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 1205 "../src/SourceList/SourceList.vala"
	_tmp3_ = self->priv->child_tree;
#line 1205 "../src/SourceList/SourceList.vala"
	_tmp4_ = *dest_iter;
#line 1205 "../src/SourceList/SourceList.vala"
	gtk_tree_store_set (_tmp3_, &_tmp4_, MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, item, -1, -1);
#line 1206 "../src/SourceList/SourceList.vala"
	_tmp5_ = self->priv->items;
#line 1206 "../src/SourceList/SourceList.vala"
	_tmp6_ = self->priv->child_tree;
#line 1206 "../src/SourceList/SourceList.vala"
	_tmp7_ = *dest_iter;
#line 1206 "../src/SourceList/SourceList.vala"
	_tmp8_ = mail_source_list_data_model_node_wrapper_new ((GtkTreeModel*) _tmp6_, &_tmp7_);
#line 1206 "../src/SourceList/SourceList.vala"
	_tmp9_ = _tmp8_;
#line 1206 "../src/SourceList/SourceList.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp5_, item, _tmp9_);
#line 1206 "../src/SourceList/SourceList.vala"
	_mail_source_list_data_model_node_wrapper_unref0 (_tmp9_);
#line 1199 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 5390 "SourceList.c"
}

static gint*
_vala_array_dup2 (gint* self,
                  gssize length)
{
#line 1253 "../src/SourceList/SourceList.vala"
	if (length > 0) {
#line 1253 "../src/SourceList/SourceList.vala"
		return _vala_memdup2 (self, length * sizeof (gint));
#line 5401 "SourceList.c"
	}
#line 1253 "../src/SourceList/SourceList.vala"
	return NULL;
#line 5405 "SourceList.c"
}

static gint*
_vala_array_dup3 (gint* self,
                  gssize length)
{
#line 1254 "../src/SourceList/SourceList.vala"
	if (length > 0) {
#line 1254 "../src/SourceList/SourceList.vala"
		return _vala_memdup2 (self, length * sizeof (gint));
#line 5416 "SourceList.c"
	}
#line 1254 "../src/SourceList/SourceList.vala"
	return NULL;
#line 5420 "SourceList.c"
}

static gboolean
mail_source_list_data_model_real_row_drop_possible (GtkTreeDragDest* base,
                                                    GtkTreePath* dest,
                                                    GtkSelectionData* selection_data)
{
	MailSourceListDataModel * self;
	GtkTreeModel* model = NULL;
	GtkTreePath* src_path = NULL;
	gboolean _tmp0_ = FALSE;
	GtkTreeModel* _tmp1_ = NULL;
	GtkTreePath* _tmp2_ = NULL;
	gboolean _tmp3_;
	GtkTreeModel* _tmp4_;
	GtkTreePath* child_dest = NULL;
	GtkTreePath* _tmp7_;
	gboolean _tmp8_ = FALSE;
	GtkTreePath* _tmp9_;
	gint src_depth = 0;
	GtkTreePath* _tmp12_;
	gint dest_depth = 0;
	GtkTreePath* _tmp13_;
	MailSourceListItem* parent = NULL;
	MailSourceListSortable* sortable = NULL;
	MailSourceListItem* _tmp35_;
	MailSourceListSortable* _tmp36_;
	gboolean _tmp37_ = FALSE;
	MailSourceListSortable* _tmp38_;
	MailSourceListItem* dest_item = NULL;
	MailSourceListItem* _tmp40_;
	MailSourceListItem* _tmp41_;
	MailSourceListItem* source_item = NULL;
	GtkTreePath* filter_src_path = NULL;
	GtkTreePath* _tmp42_;
	GtkTreePath* _tmp43_;
	GtkTreePath* _tmp44_;
	MailSourceListItem* _tmp47_;
	MailSourceListSortable* _tmp48_;
	MailSourceListItem* _tmp49_;
	MailSourceListItem* _tmp50_;
	gboolean result;
#line 1209 "../src/SourceList/SourceList.vala"
	self = (MailSourceListDataModel*) base;
#line 1209 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (dest != NULL, FALSE);
#line 1209 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (selection_data != NULL, FALSE);
#line 1216 "../src/SourceList/SourceList.vala"
	_tmp3_ = gtk_tree_get_row_drag_data (selection_data, &_tmp1_, &_tmp2_);
#line 1216 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (model);
#line 1216 "../src/SourceList/SourceList.vala"
	_tmp4_ = _g_object_ref0 (_tmp1_);
#line 1216 "../src/SourceList/SourceList.vala"
	model = _tmp4_;
#line 1216 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (src_path);
#line 1216 "../src/SourceList/SourceList.vala"
	src_path = _tmp2_;
#line 1216 "../src/SourceList/SourceList.vala"
	if (!_tmp3_) {
#line 1216 "../src/SourceList/SourceList.vala"
		_tmp0_ = TRUE;
#line 5485 "SourceList.c"
	} else {
		GtkTreeModel* _tmp5_;
		GtkTreeStore* _tmp6_;
#line 1216 "../src/SourceList/SourceList.vala"
		_tmp5_ = model;
#line 1216 "../src/SourceList/SourceList.vala"
		_tmp6_ = self->priv->child_tree;
#line 1216 "../src/SourceList/SourceList.vala"
		_tmp0_ = _tmp5_ != G_TYPE_CHECK_INSTANCE_CAST (_tmp6_, gtk_tree_model_get_type (), GtkTreeModel);
#line 5495 "SourceList.c"
	}
#line 1216 "../src/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 1217 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1217 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1217 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1217 "../src/SourceList/SourceList.vala"
		return result;
#line 5507 "SourceList.c"
	}
#line 1220 "../src/SourceList/SourceList.vala"
	_tmp7_ = gtk_tree_model_filter_convert_path_to_child_path ((GtkTreeModelFilter*) self, dest);
#line 1220 "../src/SourceList/SourceList.vala"
	child_dest = _tmp7_;
#line 1223 "../src/SourceList/SourceList.vala"
	_tmp9_ = child_dest;
#line 1223 "../src/SourceList/SourceList.vala"
	if (_tmp9_ == NULL) {
#line 1223 "../src/SourceList/SourceList.vala"
		_tmp8_ = TRUE;
#line 5519 "SourceList.c"
	} else {
		GtkTreePath* _tmp10_;
		GtkTreePath* _tmp11_;
#line 1223 "../src/SourceList/SourceList.vala"
		_tmp10_ = src_path;
#line 1223 "../src/SourceList/SourceList.vala"
		_tmp11_ = child_dest;
#line 1223 "../src/SourceList/SourceList.vala"
		_tmp8_ = gtk_tree_path_compare (_tmp10_, _tmp11_) == 0;
#line 5529 "SourceList.c"
	}
#line 1223 "../src/SourceList/SourceList.vala"
	if (_tmp8_) {
#line 1224 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1224 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1224 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1224 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1224 "../src/SourceList/SourceList.vala"
		return result;
#line 5543 "SourceList.c"
	}
#line 1228 "../src/SourceList/SourceList.vala"
	_tmp12_ = src_path;
#line 1228 "../src/SourceList/SourceList.vala"
	src_depth = gtk_tree_path_get_depth (_tmp12_);
#line 1229 "../src/SourceList/SourceList.vala"
	_tmp13_ = child_dest;
#line 1229 "../src/SourceList/SourceList.vala"
	dest_depth = gtk_tree_path_get_depth (_tmp13_);
#line 1231 "../src/SourceList/SourceList.vala"
	if (src_depth != dest_depth) {
#line 1232 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1232 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1232 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1232 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1232 "../src/SourceList/SourceList.vala"
		return result;
#line 5565 "SourceList.c"
	}
#line 1235 "../src/SourceList/SourceList.vala"
	if (src_depth < 1) {
#line 1236 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1236 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1236 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1236 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1236 "../src/SourceList/SourceList.vala"
		return result;
#line 5579 "SourceList.c"
	}
#line 1238 "../src/SourceList/SourceList.vala"
	parent = NULL;
#line 1243 "../src/SourceList/SourceList.vala"
	if (src_depth == 1) {
#line 5585 "SourceList.c"
		MailSourceListExpandableItem* _tmp14_;
		MailSourceListExpandableItem* _tmp15_;
		MailSourceListItem* _tmp16_;
#line 1244 "../src/SourceList/SourceList.vala"
		_tmp14_ = mail_source_list_data_model_get_root (self);
#line 1244 "../src/SourceList/SourceList.vala"
		_tmp15_ = _tmp14_;
#line 1244 "../src/SourceList/SourceList.vala"
		_tmp16_ = _g_object_ref0 ((MailSourceListItem*) _tmp15_);
#line 1244 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1244 "../src/SourceList/SourceList.vala"
		parent = _tmp16_;
#line 5599 "SourceList.c"
	} else {
		gint* src_indices = NULL;
		GtkTreePath* _tmp17_;
		gint _tmp18_ = 0;
		gint* _tmp19_;
		gint* _tmp20_;
		gint _tmp20__length1;
		gint src_indices_length1;
		gint _src_indices_size_;
		gint* dest_indices = NULL;
		GtkTreePath* _tmp21_;
		gint _tmp22_ = 0;
		gint* _tmp23_;
		gint* _tmp24_;
		gint _tmp24__length1;
		gint dest_indices_length1;
		gint _dest_indices_size_;
		gint src_parent_index = 0;
		gint* _tmp25_;
		gint _tmp25__length1;
		gint _tmp26_;
		gint dest_parent_index = 0;
		gint* _tmp27_;
		gint _tmp27__length1;
		gint _tmp28_;
		GtkTreePath* dest_parent = NULL;
		GtkTreePath* _tmp29_;
		gboolean _tmp30_ = FALSE;
		GtkTreePath* _tmp31_;
		GtkTreePath* _tmp33_;
		MailSourceListItem* _tmp34_;
#line 1247 "../src/SourceList/SourceList.vala"
		_vala_assert (dest_depth > 1, "dest_depth > 1");
#line 1253 "../src/SourceList/SourceList.vala"
		_tmp17_ = src_path;
#line 1253 "../src/SourceList/SourceList.vala"
		_tmp19_ = gtk_tree_path_get_indices_with_depth (_tmp17_, &_tmp18_);
#line 1253 "../src/SourceList/SourceList.vala"
		_tmp20_ = (_tmp19_ != NULL) ? _vala_array_dup2 (_tmp19_, _tmp18_) : _tmp19_;
#line 1253 "../src/SourceList/SourceList.vala"
		_tmp20__length1 = _tmp18_;
#line 1253 "../src/SourceList/SourceList.vala"
		src_indices = _tmp20_;
#line 1253 "../src/SourceList/SourceList.vala"
		src_indices_length1 = _tmp20__length1;
#line 1253 "../src/SourceList/SourceList.vala"
		_src_indices_size_ = src_indices_length1;
#line 1254 "../src/SourceList/SourceList.vala"
		_tmp21_ = child_dest;
#line 1254 "../src/SourceList/SourceList.vala"
		_tmp23_ = gtk_tree_path_get_indices_with_depth (_tmp21_, &_tmp22_);
#line 1254 "../src/SourceList/SourceList.vala"
		_tmp24_ = (_tmp23_ != NULL) ? _vala_array_dup3 (_tmp23_, _tmp22_) : _tmp23_;
#line 1254 "../src/SourceList/SourceList.vala"
		_tmp24__length1 = _tmp22_;
#line 1254 "../src/SourceList/SourceList.vala"
		dest_indices = _tmp24_;
#line 1254 "../src/SourceList/SourceList.vala"
		dest_indices_length1 = _tmp24__length1;
#line 1254 "../src/SourceList/SourceList.vala"
		_dest_indices_size_ = dest_indices_length1;
#line 1257 "../src/SourceList/SourceList.vala"
		_tmp25_ = src_indices;
#line 1257 "../src/SourceList/SourceList.vala"
		_tmp25__length1 = src_indices_length1;
#line 1257 "../src/SourceList/SourceList.vala"
		_tmp26_ = _tmp25_[src_depth - 2];
#line 1257 "../src/SourceList/SourceList.vala"
		src_parent_index = _tmp26_;
#line 1258 "../src/SourceList/SourceList.vala"
		_tmp27_ = dest_indices;
#line 1258 "../src/SourceList/SourceList.vala"
		_tmp27__length1 = dest_indices_length1;
#line 1258 "../src/SourceList/SourceList.vala"
		_tmp28_ = _tmp27_[dest_depth - 2];
#line 1258 "../src/SourceList/SourceList.vala"
		dest_parent_index = _tmp28_;
#line 1260 "../src/SourceList/SourceList.vala"
		if (src_parent_index != dest_parent_index) {
#line 1261 "../src/SourceList/SourceList.vala"
			result = FALSE;
#line 1261 "../src/SourceList/SourceList.vala"
			dest_indices = (g_free (dest_indices), NULL);
#line 1261 "../src/SourceList/SourceList.vala"
			src_indices = (g_free (src_indices), NULL);
#line 1261 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (parent);
#line 1261 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (child_dest);
#line 1261 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (src_path);
#line 1261 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (model);
#line 1261 "../src/SourceList/SourceList.vala"
			return result;
#line 5695 "SourceList.c"
		}
#line 1264 "../src/SourceList/SourceList.vala"
		_tmp29_ = __vala_GtkTreePath_copy0 (dest);
#line 1264 "../src/SourceList/SourceList.vala"
		dest_parent = _tmp29_;
#line 1266 "../src/SourceList/SourceList.vala"
		_tmp31_ = dest_parent;
#line 1266 "../src/SourceList/SourceList.vala"
		if (!gtk_tree_path_up (_tmp31_)) {
#line 1266 "../src/SourceList/SourceList.vala"
			_tmp30_ = TRUE;
#line 5707 "SourceList.c"
		} else {
			GtkTreePath* _tmp32_;
#line 1266 "../src/SourceList/SourceList.vala"
			_tmp32_ = dest_parent;
#line 1266 "../src/SourceList/SourceList.vala"
			_tmp30_ = gtk_tree_path_get_depth (_tmp32_) < 1;
#line 5714 "SourceList.c"
		}
#line 1266 "../src/SourceList/SourceList.vala"
		if (_tmp30_) {
#line 1267 "../src/SourceList/SourceList.vala"
			result = FALSE;
#line 1267 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (dest_parent);
#line 1267 "../src/SourceList/SourceList.vala"
			dest_indices = (g_free (dest_indices), NULL);
#line 1267 "../src/SourceList/SourceList.vala"
			src_indices = (g_free (src_indices), NULL);
#line 1267 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (parent);
#line 1267 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (child_dest);
#line 1267 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (src_path);
#line 1267 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (model);
#line 1267 "../src/SourceList/SourceList.vala"
			return result;
#line 5736 "SourceList.c"
		}
#line 1269 "../src/SourceList/SourceList.vala"
		_tmp33_ = dest_parent;
#line 1269 "../src/SourceList/SourceList.vala"
		_tmp34_ = mail_source_list_data_model_get_item_from_path (self, _tmp33_);
#line 1269 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1269 "../src/SourceList/SourceList.vala"
		parent = _tmp34_;
#line 1243 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (dest_parent);
#line 1243 "../src/SourceList/SourceList.vala"
		dest_indices = (g_free (dest_indices), NULL);
#line 1243 "../src/SourceList/SourceList.vala"
		src_indices = (g_free (src_indices), NULL);
#line 5752 "SourceList.c"
	}
#line 1272 "../src/SourceList/SourceList.vala"
	_tmp35_ = parent;
#line 1272 "../src/SourceList/SourceList.vala"
	_tmp36_ = _g_object_ref0 (MAIL_IS_SOURCE_LIST_SORTABLE (_tmp35_) ? ((MailSourceListSortable*) _tmp35_) : NULL);
#line 1272 "../src/SourceList/SourceList.vala"
	sortable = _tmp36_;
#line 1274 "../src/SourceList/SourceList.vala"
	_tmp38_ = sortable;
#line 1274 "../src/SourceList/SourceList.vala"
	if (_tmp38_ == NULL) {
#line 1274 "../src/SourceList/SourceList.vala"
		_tmp37_ = TRUE;
#line 5766 "SourceList.c"
	} else {
		MailSourceListSortable* _tmp39_;
#line 1274 "../src/SourceList/SourceList.vala"
		_tmp39_ = sortable;
#line 1274 "../src/SourceList/SourceList.vala"
		_tmp37_ = !mail_source_list_sortable_allow_dnd_sorting (_tmp39_);
#line 5773 "SourceList.c"
	}
#line 1274 "../src/SourceList/SourceList.vala"
	if (_tmp37_) {
#line 1275 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1275 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (sortable);
#line 1275 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1275 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1275 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1275 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1275 "../src/SourceList/SourceList.vala"
		return result;
#line 5791 "SourceList.c"
	}
#line 1277 "../src/SourceList/SourceList.vala"
	_tmp40_ = mail_source_list_data_model_get_item_from_path (self, dest);
#line 1277 "../src/SourceList/SourceList.vala"
	dest_item = _tmp40_;
#line 1279 "../src/SourceList/SourceList.vala"
	_tmp41_ = dest_item;
#line 1279 "../src/SourceList/SourceList.vala"
	if (_tmp41_ == NULL) {
#line 1280 "../src/SourceList/SourceList.vala"
		result = TRUE;
#line 1280 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (dest_item);
#line 1280 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (sortable);
#line 1280 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1280 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1280 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1280 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1280 "../src/SourceList/SourceList.vala"
		return result;
#line 5817 "SourceList.c"
	}
#line 1282 "../src/SourceList/SourceList.vala"
	source_item = NULL;
#line 1283 "../src/SourceList/SourceList.vala"
	_tmp42_ = src_path;
#line 1283 "../src/SourceList/SourceList.vala"
	_tmp43_ = gtk_tree_model_filter_convert_child_path_to_path ((GtkTreeModelFilter*) self, _tmp42_);
#line 1283 "../src/SourceList/SourceList.vala"
	filter_src_path = _tmp43_;
#line 1285 "../src/SourceList/SourceList.vala"
	_tmp44_ = filter_src_path;
#line 1285 "../src/SourceList/SourceList.vala"
	if (_tmp44_ != NULL) {
#line 5831 "SourceList.c"
		GtkTreePath* _tmp45_;
		MailSourceListItem* _tmp46_;
#line 1286 "../src/SourceList/SourceList.vala"
		_tmp45_ = filter_src_path;
#line 1286 "../src/SourceList/SourceList.vala"
		_tmp46_ = mail_source_list_data_model_get_item_from_path (self, _tmp45_);
#line 1286 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (source_item);
#line 1286 "../src/SourceList/SourceList.vala"
		source_item = _tmp46_;
#line 5842 "SourceList.c"
	}
#line 1288 "../src/SourceList/SourceList.vala"
	_tmp47_ = source_item;
#line 1288 "../src/SourceList/SourceList.vala"
	if (_tmp47_ == NULL) {
#line 1289 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1289 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (filter_src_path);
#line 1289 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (source_item);
#line 1289 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (dest_item);
#line 1289 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (sortable);
#line 1289 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1289 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1289 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1289 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1289 "../src/SourceList/SourceList.vala"
		return result;
#line 5868 "SourceList.c"
	}
#line 1297 "../src/SourceList/SourceList.vala"
	_tmp48_ = sortable;
#line 1297 "../src/SourceList/SourceList.vala"
	_tmp49_ = dest_item;
#line 1297 "../src/SourceList/SourceList.vala"
	_tmp50_ = source_item;
#line 1297 "../src/SourceList/SourceList.vala"
	if (mail_source_list_sortable_compare (_tmp48_, _tmp49_, _tmp50_) >= 0) {
#line 5878 "SourceList.c"
		MailSourceListItem* dest_item_prev = NULL;
		MailSourceListItem* _tmp51_;
		gboolean _tmp52_ = FALSE;
		gboolean _tmp53_ = FALSE;
		MailSourceListItem* _tmp54_;
#line 1298 "../src/SourceList/SourceList.vala"
		if (!gtk_tree_path_prev (dest)) {
#line 1299 "../src/SourceList/SourceList.vala"
			result = TRUE;
#line 1299 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (filter_src_path);
#line 1299 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (source_item);
#line 1299 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (dest_item);
#line 1299 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (sortable);
#line 1299 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (parent);
#line 1299 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (child_dest);
#line 1299 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (src_path);
#line 1299 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (model);
#line 1299 "../src/SourceList/SourceList.vala"
			return result;
#line 5906 "SourceList.c"
		}
#line 1303 "../src/SourceList/SourceList.vala"
		_tmp51_ = mail_source_list_data_model_get_item_from_path (self, dest);
#line 1303 "../src/SourceList/SourceList.vala"
		dest_item_prev = _tmp51_;
#line 1305 "../src/SourceList/SourceList.vala"
		_tmp54_ = dest_item_prev;
#line 1305 "../src/SourceList/SourceList.vala"
		if (_tmp54_ != NULL) {
#line 5916 "SourceList.c"
			MailSourceListItem* _tmp55_;
			MailSourceListItem* _tmp56_;
#line 1306 "../src/SourceList/SourceList.vala"
			_tmp55_ = dest_item_prev;
#line 1306 "../src/SourceList/SourceList.vala"
			_tmp56_ = source_item;
#line 1306 "../src/SourceList/SourceList.vala"
			_tmp53_ = _tmp55_ != _tmp56_;
#line 5925 "SourceList.c"
		} else {
#line 1305 "../src/SourceList/SourceList.vala"
			_tmp53_ = FALSE;
#line 5929 "SourceList.c"
		}
#line 1305 "../src/SourceList/SourceList.vala"
		if (_tmp53_) {
#line 5933 "SourceList.c"
			MailSourceListSortable* _tmp57_;
			MailSourceListItem* _tmp58_;
			MailSourceListItem* _tmp59_;
#line 1307 "../src/SourceList/SourceList.vala"
			_tmp57_ = sortable;
#line 1307 "../src/SourceList/SourceList.vala"
			_tmp58_ = dest_item_prev;
#line 1307 "../src/SourceList/SourceList.vala"
			_tmp59_ = source_item;
#line 1307 "../src/SourceList/SourceList.vala"
			_tmp52_ = mail_source_list_sortable_compare (_tmp57_, _tmp58_, _tmp59_) <= 0;
#line 5945 "SourceList.c"
		} else {
#line 1305 "../src/SourceList/SourceList.vala"
			_tmp52_ = FALSE;
#line 5949 "SourceList.c"
		}
#line 1305 "../src/SourceList/SourceList.vala"
		result = _tmp52_;
#line 1305 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (dest_item_prev);
#line 1305 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (filter_src_path);
#line 1305 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (source_item);
#line 1305 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (dest_item);
#line 1305 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (sortable);
#line 1305 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1305 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1305 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1305 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1305 "../src/SourceList/SourceList.vala"
		return result;
#line 5973 "SourceList.c"
	}
#line 1310 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 1310 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (filter_src_path);
#line 1310 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (source_item);
#line 1310 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (dest_item);
#line 1310 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (sortable);
#line 1310 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (parent);
#line 1310 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (child_dest);
#line 1310 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (src_path);
#line 1310 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (model);
#line 1310 "../src/SourceList/SourceList.vala"
	return result;
#line 5995 "SourceList.c"
}

/**
         * Override default implementation of TreeDragSource
         *
         * drag_data_delete is not overriden because the default implementation
         * does exactly what we need.
         */
static gboolean
mail_source_list_data_model_real_drag_data_get (GtkTreeDragSource* base,
                                                GtkTreePath* path,
                                                GtkSelectionData* selection_data)
{
	MailSourceListDataModel * self;
	MailSourceListDragSource* drag_source_item = NULL;
	MailSourceListItem* _tmp0_;
	MailSourceListDragSource* _tmp1_;
	gboolean _tmp2_ = FALSE;
	MailSourceListDragSource* _tmp3_;
	gboolean result;
#line 1320 "../src/SourceList/SourceList.vala"
	self = (MailSourceListDataModel*) base;
#line 1320 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1320 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (selection_data != NULL, FALSE);
#line 1323 "../src/SourceList/SourceList.vala"
	if (gtk_selection_data_get_target (selection_data) == gdk_atom_intern_static_string ("GTK_TREE_MODEL_ROW")) {
#line 1324 "../src/SourceList/SourceList.vala"
		result = mail_source_list_data_model_gtk_tree_drag_source_parent_iface->drag_data_get ((GtkTreeDragSource*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_model_filter_get_type (), GtkTreeModelFilter), path, selection_data);
#line 1324 "../src/SourceList/SourceList.vala"
		return result;
#line 6028 "SourceList.c"
	}
#line 1327 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_get_item_from_path (self, path);
#line 1327 "../src/SourceList/SourceList.vala"
	_tmp1_ = MAIL_IS_SOURCE_LIST_DRAG_SOURCE (_tmp0_) ? ((MailSourceListDragSource*) _tmp0_) : NULL;
#line 1327 "../src/SourceList/SourceList.vala"
	if (_tmp1_ == NULL) {
#line 1327 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp0_);
#line 6038 "SourceList.c"
	}
#line 1327 "../src/SourceList/SourceList.vala"
	drag_source_item = _tmp1_;
#line 1328 "../src/SourceList/SourceList.vala"
	_tmp3_ = drag_source_item;
#line 1328 "../src/SourceList/SourceList.vala"
	if (_tmp3_ != NULL) {
#line 6046 "SourceList.c"
		MailSourceListDragSource* _tmp4_;
#line 1328 "../src/SourceList/SourceList.vala"
		_tmp4_ = drag_source_item;
#line 1328 "../src/SourceList/SourceList.vala"
		_tmp2_ = mail_source_list_drag_source_draggable (_tmp4_);
#line 6052 "SourceList.c"
	} else {
#line 1328 "../src/SourceList/SourceList.vala"
		_tmp2_ = FALSE;
#line 6056 "SourceList.c"
	}
#line 1328 "../src/SourceList/SourceList.vala"
	if (_tmp2_) {
#line 6060 "SourceList.c"
		MailSourceListDragSource* _tmp5_;
#line 1329 "../src/SourceList/SourceList.vala"
		_tmp5_ = drag_source_item;
#line 1329 "../src/SourceList/SourceList.vala"
		mail_source_list_drag_source_prepare_selection_data (_tmp5_, selection_data);
#line 1330 "../src/SourceList/SourceList.vala"
		result = TRUE;
#line 1330 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (drag_source_item);
#line 1330 "../src/SourceList/SourceList.vala"
		return result;
#line 6072 "SourceList.c"
	}
#line 1333 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 1333 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (drag_source_item);
#line 1333 "../src/SourceList/SourceList.vala"
	return result;
#line 6080 "SourceList.c"
}

static gboolean
mail_source_list_data_model_real_row_draggable (GtkTreeDragSource* base,
                                                GtkTreePath* path)
{
	MailSourceListDataModel * self;
	MailSourceListItem* item = NULL;
	MailSourceListItem* _tmp0_;
	MailSourceListItem* _tmp1_;
	gboolean result;
#line 1336 "../src/SourceList/SourceList.vala"
	self = (MailSourceListDataModel*) base;
#line 1336 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1337 "../src/SourceList/SourceList.vala"
	if (!mail_source_list_data_model_gtk_tree_drag_source_parent_iface->row_draggable ((GtkTreeDragSource*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_model_filter_get_type (), GtkTreeModelFilter), path)) {
#line 1338 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1338 "../src/SourceList/SourceList.vala"
		return result;
#line 6102 "SourceList.c"
	}
#line 1340 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_get_item_from_path (self, path);
#line 1340 "../src/SourceList/SourceList.vala"
	item = _tmp0_;
#line 1342 "../src/SourceList/SourceList.vala"
	_tmp1_ = item;
#line 1342 "../src/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 6112 "SourceList.c"
		MailSourceListSortable* sortable_item = NULL;
		MailSourceListItem* _tmp2_;
		MailSourceListExpandableItem* _tmp3_;
		MailSourceListExpandableItem* _tmp4_;
		MailSourceListSortable* _tmp5_;
		gboolean _tmp6_ = FALSE;
		MailSourceListSortable* _tmp7_;
		MailSourceListDragSource* drag_source_item = NULL;
		MailSourceListItem* _tmp9_;
		MailSourceListDragSource* _tmp10_;
		gboolean _tmp11_ = FALSE;
		MailSourceListDragSource* _tmp12_;
#line 1344 "../src/SourceList/SourceList.vala"
		_tmp2_ = item;
#line 1344 "../src/SourceList/SourceList.vala"
		_tmp3_ = mail_source_list_item_get_parent (_tmp2_);
#line 1344 "../src/SourceList/SourceList.vala"
		_tmp4_ = _tmp3_;
#line 1344 "../src/SourceList/SourceList.vala"
		_tmp5_ = _g_object_ref0 (MAIL_IS_SOURCE_LIST_SORTABLE (_tmp4_) ? ((MailSourceListSortable*) _tmp4_) : NULL);
#line 1344 "../src/SourceList/SourceList.vala"
		sortable_item = _tmp5_;
#line 1346 "../src/SourceList/SourceList.vala"
		_tmp7_ = sortable_item;
#line 1346 "../src/SourceList/SourceList.vala"
		if (_tmp7_ != NULL) {
#line 6139 "SourceList.c"
			MailSourceListSortable* _tmp8_;
#line 1346 "../src/SourceList/SourceList.vala"
			_tmp8_ = sortable_item;
#line 1346 "../src/SourceList/SourceList.vala"
			_tmp6_ = mail_source_list_sortable_allow_dnd_sorting (_tmp8_);
#line 6145 "SourceList.c"
		} else {
#line 1346 "../src/SourceList/SourceList.vala"
			_tmp6_ = FALSE;
#line 6149 "SourceList.c"
		}
#line 1346 "../src/SourceList/SourceList.vala"
		if (_tmp6_) {
#line 1347 "../src/SourceList/SourceList.vala"
			result = TRUE;
#line 1347 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (sortable_item);
#line 1347 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 1347 "../src/SourceList/SourceList.vala"
			return result;
#line 6161 "SourceList.c"
		}
#line 1351 "../src/SourceList/SourceList.vala"
		_tmp9_ = item;
#line 1351 "../src/SourceList/SourceList.vala"
		_tmp10_ = _g_object_ref0 (MAIL_IS_SOURCE_LIST_DRAG_SOURCE (_tmp9_) ? ((MailSourceListDragSource*) _tmp9_) : NULL);
#line 1351 "../src/SourceList/SourceList.vala"
		drag_source_item = _tmp10_;
#line 1353 "../src/SourceList/SourceList.vala"
		_tmp12_ = drag_source_item;
#line 1353 "../src/SourceList/SourceList.vala"
		if (_tmp12_ != NULL) {
#line 6173 "SourceList.c"
			MailSourceListDragSource* _tmp13_;
#line 1353 "../src/SourceList/SourceList.vala"
			_tmp13_ = drag_source_item;
#line 1353 "../src/SourceList/SourceList.vala"
			_tmp11_ = mail_source_list_drag_source_draggable (_tmp13_);
#line 6179 "SourceList.c"
		} else {
#line 1353 "../src/SourceList/SourceList.vala"
			_tmp11_ = FALSE;
#line 6183 "SourceList.c"
		}
#line 1353 "../src/SourceList/SourceList.vala"
		if (_tmp11_) {
#line 1354 "../src/SourceList/SourceList.vala"
			result = TRUE;
#line 1354 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (drag_source_item);
#line 1354 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (sortable_item);
#line 1354 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 1354 "../src/SourceList/SourceList.vala"
			return result;
#line 6197 "SourceList.c"
		}
#line 1342 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (drag_source_item);
#line 1342 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (sortable_item);
#line 6203 "SourceList.c"
	}
#line 1357 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 1357 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1357 "../src/SourceList/SourceList.vala"
	return result;
#line 6211 "SourceList.c"
}

static MailSourceListDataModel*
mail_source_list_data_model_construct (GType object_type)
{
	MailSourceListDataModel * self = NULL;
#line 640 "../src/SourceList/SourceList.vala"
	self = (MailSourceListDataModel*) g_object_new (object_type, NULL);
#line 640 "../src/SourceList/SourceList.vala"
	return self;
#line 6222 "SourceList.c"
}

static MailSourceListDataModel*
mail_source_list_data_model_new (void)
{
#line 640 "../src/SourceList/SourceList.vala"
	return mail_source_list_data_model_construct (MAIL_SOURCE_LIST_TYPE_DATA_MODEL);
#line 6230 "SourceList.c"
}

static MailSourceListExpandableItem*
mail_source_list_data_model_get_root (MailSourceListDataModel* self)
{
	MailSourceListExpandableItem* result;
	MailSourceListExpandableItem* _tmp0_;
#line 747 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 747 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_root;
#line 747 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 747 "../src/SourceList/SourceList.vala"
	return result;
#line 6246 "SourceList.c"
}

static void
mail_source_list_data_model_set_root (MailSourceListDataModel* self,
                                      MailSourceListExpandableItem* value)
{
	MailSourceListExpandableItem* _tmp0_;
	MailSourceListExpandableItem* _tmp12_;
	MailSourceListExpandableItem* _tmp13_;
#line 748 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 749 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_root;
#line 749 "../src/SourceList/SourceList.vala"
	if (_tmp0_ != NULL) {
#line 6262 "SourceList.c"
		MailSourceListExpandableItem* _tmp1_;
#line 750 "../src/SourceList/SourceList.vala"
		_tmp1_ = self->priv->_root;
#line 750 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_remove_children_monitor (self, _tmp1_);
#line 6268 "SourceList.c"
		{
			GeeIterator* _item_it = NULL;
			MailSourceListExpandableItem* _tmp2_;
			GeeCollection* _tmp3_;
			GeeCollection* _tmp4_;
			GeeCollection* _tmp5_;
			GeeIterator* _tmp6_;
			GeeIterator* _tmp7_;
#line 751 "../src/SourceList/SourceList.vala"
			_tmp2_ = self->priv->_root;
#line 751 "../src/SourceList/SourceList.vala"
			_tmp3_ = mail_source_list_expandable_item_get_children (_tmp2_);
#line 751 "../src/SourceList/SourceList.vala"
			_tmp4_ = _tmp3_;
#line 751 "../src/SourceList/SourceList.vala"
			_tmp5_ = _tmp4_;
#line 751 "../src/SourceList/SourceList.vala"
			_tmp6_ = gee_iterable_iterator ((GeeIterable*) _tmp5_);
#line 751 "../src/SourceList/SourceList.vala"
			_tmp7_ = _tmp6_;
#line 751 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (_tmp5_);
#line 751 "../src/SourceList/SourceList.vala"
			_item_it = _tmp7_;
#line 751 "../src/SourceList/SourceList.vala"
			while (TRUE) {
#line 6295 "SourceList.c"
				GeeIterator* _tmp8_;
				MailSourceListItem* item = NULL;
				GeeIterator* _tmp9_;
				gpointer _tmp10_;
				MailSourceListItem* _tmp11_;
#line 751 "../src/SourceList/SourceList.vala"
				_tmp8_ = _item_it;
#line 751 "../src/SourceList/SourceList.vala"
				if (!gee_iterator_next (_tmp8_)) {
#line 751 "../src/SourceList/SourceList.vala"
					break;
#line 6307 "SourceList.c"
				}
#line 751 "../src/SourceList/SourceList.vala"
				_tmp9_ = _item_it;
#line 751 "../src/SourceList/SourceList.vala"
				_tmp10_ = gee_iterator_get (_tmp9_);
#line 751 "../src/SourceList/SourceList.vala"
				item = (MailSourceListItem*) _tmp10_;
#line 752 "../src/SourceList/SourceList.vala"
				_tmp11_ = item;
#line 752 "../src/SourceList/SourceList.vala"
				mail_source_list_data_model_remove_item (self, _tmp11_);
#line 751 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (item);
#line 6321 "SourceList.c"
			}
#line 751 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (_item_it);
#line 6325 "SourceList.c"
		}
	}
#line 755 "../src/SourceList/SourceList.vala"
	_tmp12_ = _g_object_ref0 (value);
#line 755 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_root);
#line 755 "../src/SourceList/SourceList.vala"
	self->priv->_root = _tmp12_;
#line 757 "../src/SourceList/SourceList.vala"
	_tmp13_ = self->priv->_root;
#line 757 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_add_children_monitor (self, _tmp13_);
#line 6338 "SourceList.c"
	{
		GeeIterator* _item_it = NULL;
		MailSourceListExpandableItem* _tmp14_;
		GeeCollection* _tmp15_;
		GeeCollection* _tmp16_;
		GeeCollection* _tmp17_;
		GeeIterator* _tmp18_;
		GeeIterator* _tmp19_;
#line 758 "../src/SourceList/SourceList.vala"
		_tmp14_ = self->priv->_root;
#line 758 "../src/SourceList/SourceList.vala"
		_tmp15_ = mail_source_list_expandable_item_get_children (_tmp14_);
#line 758 "../src/SourceList/SourceList.vala"
		_tmp16_ = _tmp15_;
#line 758 "../src/SourceList/SourceList.vala"
		_tmp17_ = _tmp16_;
#line 758 "../src/SourceList/SourceList.vala"
		_tmp18_ = gee_iterable_iterator ((GeeIterable*) _tmp17_);
#line 758 "../src/SourceList/SourceList.vala"
		_tmp19_ = _tmp18_;
#line 758 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp17_);
#line 758 "../src/SourceList/SourceList.vala"
		_item_it = _tmp19_;
#line 758 "../src/SourceList/SourceList.vala"
		while (TRUE) {
#line 6365 "SourceList.c"
			GeeIterator* _tmp20_;
			MailSourceListItem* item = NULL;
			GeeIterator* _tmp21_;
			gpointer _tmp22_;
			MailSourceListItem* _tmp23_;
#line 758 "../src/SourceList/SourceList.vala"
			_tmp20_ = _item_it;
#line 758 "../src/SourceList/SourceList.vala"
			if (!gee_iterator_next (_tmp20_)) {
#line 758 "../src/SourceList/SourceList.vala"
				break;
#line 6377 "SourceList.c"
			}
#line 758 "../src/SourceList/SourceList.vala"
			_tmp21_ = _item_it;
#line 758 "../src/SourceList/SourceList.vala"
			_tmp22_ = gee_iterator_get (_tmp21_);
#line 758 "../src/SourceList/SourceList.vala"
			item = (MailSourceListItem*) _tmp22_;
#line 759 "../src/SourceList/SourceList.vala"
			_tmp23_ = item;
#line 759 "../src/SourceList/SourceList.vala"
			mail_source_list_data_model_add_item (self, _tmp23_);
#line 758 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 6391 "SourceList.c"
		}
#line 758 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (_item_it);
#line 6395 "SourceList.c"
	}
#line 748 "../src/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, mail_source_list_data_model_properties[MAIL_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY]);
#line 6399 "SourceList.c"
}

static inline gpointer
mail_source_list_data_model_node_wrapper_get_instance_private (MailSourceListDataModelNodeWrapper* self)
{
	return G_STRUCT_MEMBER_P (self, MailSourceListDataModelNodeWrapper_private_offset);
}

static void
_vala_GtkTreeRowReference_free (GtkTreeRowReference* self)
{
#line 653 "../src/SourceList/SourceList.vala"
	g_boxed_free (gtk_tree_row_reference_get_type (), self);
#line 6413 "SourceList.c"
}

static MailSourceListDataModelNodeWrapper*
mail_source_list_data_model_node_wrapper_construct (GType object_type,
                                                    GtkTreeModel* model,
                                                    GtkTreeIter* iter)
{
	MailSourceListDataModelNodeWrapper* self = NULL;
	GtkTreeIter _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp2_;
	GtkTreeRowReference* _tmp3_;
#line 690 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (model != NULL, NULL);
#line 690 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, NULL);
#line 690 "../src/SourceList/SourceList.vala"
	self = (MailSourceListDataModelNodeWrapper*) g_type_create_instance (object_type);
#line 691 "../src/SourceList/SourceList.vala"
	_tmp0_ = *iter;
#line 691 "../src/SourceList/SourceList.vala"
	_tmp1_ = gtk_tree_model_get_path (model, &_tmp0_);
#line 691 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 691 "../src/SourceList/SourceList.vala"
	_tmp3_ = gtk_tree_row_reference_new (model, _tmp2_);
#line 691 "../src/SourceList/SourceList.vala"
	__vala_GtkTreeRowReference_free0 (self->priv->row_reference);
#line 691 "../src/SourceList/SourceList.vala"
	self->priv->row_reference = _tmp3_;
#line 691 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (_tmp2_);
#line 690 "../src/SourceList/SourceList.vala"
	return self;
#line 6448 "SourceList.c"
}

static MailSourceListDataModelNodeWrapper*
mail_source_list_data_model_node_wrapper_new (GtkTreeModel* model,
                                              GtkTreeIter* iter)
{
#line 690 "../src/SourceList/SourceList.vala"
	return mail_source_list_data_model_node_wrapper_construct (MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, model, iter);
#line 6457 "SourceList.c"
}

static GtkTreeIter*
mail_source_list_data_model_node_wrapper_get_iter (MailSourceListDataModelNodeWrapper* self)
{
	GtkTreeIter* result;
	GtkTreeIter* rv = NULL;
	gboolean _tmp0_;
	gboolean _tmp1_;
#line 659 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 660 "../src/SourceList/SourceList.vala"
	rv = NULL;
#line 662 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_node_wrapper_get_valid (self);
#line 662 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 662 "../src/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 6477 "SourceList.c"
		GtkTreePath* _path = NULL;
		GtkTreePath* _tmp2_;
		GtkTreePath* _tmp3_;
		GtkTreePath* _tmp4_;
#line 663 "../src/SourceList/SourceList.vala"
		_tmp2_ = mail_source_list_data_model_node_wrapper_get_path (self);
#line 663 "../src/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 663 "../src/SourceList/SourceList.vala"
		_path = _tmp3_;
#line 664 "../src/SourceList/SourceList.vala"
		_tmp4_ = _path;
#line 664 "../src/SourceList/SourceList.vala"
		if (_tmp4_ != NULL) {
#line 6492 "SourceList.c"
			GtkTreeIter _iter = {0};
			GtkTreeRowReference* _tmp5_;
			GtkTreeModel* _tmp6_;
			GtkTreePath* _tmp7_;
			GtkTreeIter _tmp8_ = {0};
			gboolean _tmp9_;
#line 666 "../src/SourceList/SourceList.vala"
			_tmp5_ = self->priv->row_reference;
#line 666 "../src/SourceList/SourceList.vala"
			_tmp6_ = gtk_tree_row_reference_get_model (_tmp5_);
#line 666 "../src/SourceList/SourceList.vala"
			_tmp7_ = _path;
#line 666 "../src/SourceList/SourceList.vala"
			_tmp9_ = gtk_tree_model_get_iter (_tmp6_, &_tmp8_, _tmp7_);
#line 666 "../src/SourceList/SourceList.vala"
			_iter = _tmp8_;
#line 666 "../src/SourceList/SourceList.vala"
			if (_tmp9_) {
#line 6511 "SourceList.c"
				GtkTreeIter _tmp10_;
				GtkTreeIter* _tmp11_;
#line 667 "../src/SourceList/SourceList.vala"
				_tmp10_ = _iter;
#line 667 "../src/SourceList/SourceList.vala"
				_tmp11_ = __vala_GtkTreeIter_copy0 (&_tmp10_);
#line 667 "../src/SourceList/SourceList.vala"
				__vala_GtkTreeIter_free0 (rv);
#line 667 "../src/SourceList/SourceList.vala"
				rv = _tmp11_;
#line 6522 "SourceList.c"
			}
		}
#line 662 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (_path);
#line 6527 "SourceList.c"
	}
#line 671 "../src/SourceList/SourceList.vala"
	result = rv;
#line 671 "../src/SourceList/SourceList.vala"
	return result;
#line 6533 "SourceList.c"
}

static GtkTreePath*
mail_source_list_data_model_node_wrapper_get_path (MailSourceListDataModelNodeWrapper* self)
{
	GtkTreePath* result;
	GtkTreePath* _tmp0_ = NULL;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 679 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 679 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_data_model_node_wrapper_get_valid (self);
#line 679 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 679 "../src/SourceList/SourceList.vala"
	if (_tmp2_) {
#line 6551 "SourceList.c"
		GtkTreeRowReference* _tmp3_;
		GtkTreePath* _tmp4_;
#line 679 "../src/SourceList/SourceList.vala"
		_tmp3_ = self->priv->row_reference;
#line 679 "../src/SourceList/SourceList.vala"
		_tmp4_ = gtk_tree_row_reference_get_path (_tmp3_);
#line 679 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (_tmp0_);
#line 679 "../src/SourceList/SourceList.vala"
		_tmp0_ = _tmp4_;
#line 6562 "SourceList.c"
	} else {
#line 679 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (_tmp0_);
#line 679 "../src/SourceList/SourceList.vala"
		_tmp0_ = NULL;
#line 6568 "SourceList.c"
	}
#line 679 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 679 "../src/SourceList/SourceList.vala"
	return result;
#line 6574 "SourceList.c"
}

static gboolean
mail_source_list_data_model_node_wrapper_get_valid (MailSourceListDataModelNodeWrapper* self)
{
	gboolean result;
	gboolean _tmp0_ = FALSE;
	GtkTreeRowReference* _tmp1_;
#line 687 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 687 "../src/SourceList/SourceList.vala"
	_tmp1_ = self->priv->row_reference;
#line 687 "../src/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 6589 "SourceList.c"
		GtkTreeRowReference* _tmp2_;
#line 687 "../src/SourceList/SourceList.vala"
		_tmp2_ = self->priv->row_reference;
#line 687 "../src/SourceList/SourceList.vala"
		_tmp0_ = gtk_tree_row_reference_valid (_tmp2_);
#line 6595 "SourceList.c"
	} else {
#line 687 "../src/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 6599 "SourceList.c"
	}
#line 687 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 687 "../src/SourceList/SourceList.vala"
	return result;
#line 6605 "SourceList.c"
}

static void
mail_source_list_data_model_value_node_wrapper_init (GValue* value)
{
#line 648 "../src/SourceList/SourceList.vala"
	value->data[0].v_pointer = NULL;
#line 6613 "SourceList.c"
}

static void
mail_source_list_data_model_value_node_wrapper_free_value (GValue* value)
{
#line 648 "../src/SourceList/SourceList.vala"
	if (value->data[0].v_pointer) {
#line 648 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_node_wrapper_unref (value->data[0].v_pointer);
#line 6623 "SourceList.c"
	}
}

static void
mail_source_list_data_model_value_node_wrapper_copy_value (const GValue* src_value,
                                                           GValue* dest_value)
{
#line 648 "../src/SourceList/SourceList.vala"
	if (src_value->data[0].v_pointer) {
#line 648 "../src/SourceList/SourceList.vala"
		dest_value->data[0].v_pointer = mail_source_list_data_model_node_wrapper_ref (src_value->data[0].v_pointer);
#line 6635 "SourceList.c"
	} else {
#line 648 "../src/SourceList/SourceList.vala"
		dest_value->data[0].v_pointer = NULL;
#line 6639 "SourceList.c"
	}
}

static gpointer
mail_source_list_data_model_value_node_wrapper_peek_pointer (const GValue* value)
{
#line 648 "../src/SourceList/SourceList.vala"
	return value->data[0].v_pointer;
#line 6648 "SourceList.c"
}

static gchar*
mail_source_list_data_model_value_node_wrapper_collect_value (GValue* value,
                                                              guint n_collect_values,
                                                              GTypeCValue* collect_values,
                                                              guint collect_flags)
{
#line 648 "../src/SourceList/SourceList.vala"
	if (collect_values[0].v_pointer) {
#line 6659 "SourceList.c"
		MailSourceListDataModelNodeWrapper * object;
		object = collect_values[0].v_pointer;
#line 648 "../src/SourceList/SourceList.vala"
		if (object->parent_instance.g_class == NULL) {
#line 648 "../src/SourceList/SourceList.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 6666 "SourceList.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 648 "../src/SourceList/SourceList.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 6670 "SourceList.c"
		}
#line 648 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = mail_source_list_data_model_node_wrapper_ref (object);
#line 6674 "SourceList.c"
	} else {
#line 648 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 6678 "SourceList.c"
	}
#line 648 "../src/SourceList/SourceList.vala"
	return NULL;
#line 6682 "SourceList.c"
}

static gchar*
mail_source_list_data_model_value_node_wrapper_lcopy_value (const GValue* value,
                                                            guint n_collect_values,
                                                            GTypeCValue* collect_values,
                                                            guint collect_flags)
{
	MailSourceListDataModelNodeWrapper ** object_p;
	object_p = collect_values[0].v_pointer;
#line 648 "../src/SourceList/SourceList.vala"
	if (!object_p) {
#line 648 "../src/SourceList/SourceList.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 6697 "SourceList.c"
	}
#line 648 "../src/SourceList/SourceList.vala"
	if (!value->data[0].v_pointer) {
#line 648 "../src/SourceList/SourceList.vala"
		*object_p = NULL;
#line 6703 "SourceList.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 648 "../src/SourceList/SourceList.vala"
		*object_p = value->data[0].v_pointer;
#line 6707 "SourceList.c"
	} else {
#line 648 "../src/SourceList/SourceList.vala"
		*object_p = mail_source_list_data_model_node_wrapper_ref (value->data[0].v_pointer);
#line 6711 "SourceList.c"
	}
#line 648 "../src/SourceList/SourceList.vala"
	return NULL;
#line 6715 "SourceList.c"
}

static GParamSpec*
mail_source_list_data_model_param_spec_node_wrapper (const gchar* name,
                                                     const gchar* nick,
                                                     const gchar* blurb,
                                                     GType object_type,
                                                     GParamFlags flags)
{
	MailSourceListDataModelParamSpecNodeWrapper* spec;
#line 648 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (g_type_is_a (object_type, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER), NULL);
#line 648 "../src/SourceList/SourceList.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 648 "../src/SourceList/SourceList.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 648 "../src/SourceList/SourceList.vala"
	return G_PARAM_SPEC (spec);
#line 6734 "SourceList.c"
}

static gpointer
mail_source_list_data_model_value_get_node_wrapper (const GValue* value)
{
#line 648 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER), NULL);
#line 648 "../src/SourceList/SourceList.vala"
	return value->data[0].v_pointer;
#line 6744 "SourceList.c"
}

static void
mail_source_list_data_model_value_set_node_wrapper (GValue* value,
                                                    gpointer v_object)
{
	MailSourceListDataModelNodeWrapper * old;
#line 648 "../src/SourceList/SourceList.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER));
#line 648 "../src/SourceList/SourceList.vala"
	old = value->data[0].v_pointer;
#line 648 "../src/SourceList/SourceList.vala"
	if (v_object) {
#line 648 "../src/SourceList/SourceList.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER));
#line 648 "../src/SourceList/SourceList.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 648 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = v_object;
#line 648 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_node_wrapper_ref (value->data[0].v_pointer);
#line 6766 "SourceList.c"
	} else {
#line 648 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 6770 "SourceList.c"
	}
#line 648 "../src/SourceList/SourceList.vala"
	if (old) {
#line 648 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_node_wrapper_unref (old);
#line 6776 "SourceList.c"
	}
}

static void
mail_source_list_data_model_value_take_node_wrapper (GValue* value,
                                                     gpointer v_object)
{
	MailSourceListDataModelNodeWrapper * old;
#line 648 "../src/SourceList/SourceList.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER));
#line 648 "../src/SourceList/SourceList.vala"
	old = value->data[0].v_pointer;
#line 648 "../src/SourceList/SourceList.vala"
	if (v_object) {
#line 648 "../src/SourceList/SourceList.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER));
#line 648 "../src/SourceList/SourceList.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 648 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = v_object;
#line 6797 "SourceList.c"
	} else {
#line 648 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 6801 "SourceList.c"
	}
#line 648 "../src/SourceList/SourceList.vala"
	if (old) {
#line 648 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_node_wrapper_unref (old);
#line 6807 "SourceList.c"
	}
}

static void
mail_source_list_data_model_node_wrapper_class_init (MailSourceListDataModelNodeWrapperClass * klass,
                                                     gpointer klass_data)
{
#line 648 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_node_wrapper_parent_class = g_type_class_peek_parent (klass);
#line 648 "../src/SourceList/SourceList.vala"
	((MailSourceListDataModelNodeWrapperClass *) klass)->finalize = mail_source_list_data_model_node_wrapper_finalize;
#line 648 "../src/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &MailSourceListDataModelNodeWrapper_private_offset);
#line 6821 "SourceList.c"
}

static void
mail_source_list_data_model_node_wrapper_instance_init (MailSourceListDataModelNodeWrapper * self,
                                                        gpointer klass)
{
#line 648 "../src/SourceList/SourceList.vala"
	self->priv = mail_source_list_data_model_node_wrapper_get_instance_private (self);
#line 648 "../src/SourceList/SourceList.vala"
	self->ref_count = 1;
#line 6832 "SourceList.c"
}

static void
mail_source_list_data_model_node_wrapper_finalize (MailSourceListDataModelNodeWrapper * obj)
{
	MailSourceListDataModelNodeWrapper * self;
#line 648 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, MailSourceListDataModelNodeWrapper);
#line 648 "../src/SourceList/SourceList.vala"
	g_signal_handlers_destroy (self);
#line 653 "../src/SourceList/SourceList.vala"
	__vala_GtkTreeRowReference_free0 (self->priv->row_reference);
#line 6845 "SourceList.c"
}

/**
         * An object that references a particular row in a model. This class is a wrapper built around
         * Gtk.TreeRowReference, and exists with the purpose of ensuring we never use invalid tree paths
         * or iters in the model, since most of these errors provoke failures due to GTK+ assertions
         * or, even worse, unexpected behavior.
         */
static GType
mail_source_list_data_model_node_wrapper_get_type_once (void)
{
	static const GTypeValueTable g_define_type_value_table = { mail_source_list_data_model_value_node_wrapper_init, mail_source_list_data_model_value_node_wrapper_free_value, mail_source_list_data_model_value_node_wrapper_copy_value, mail_source_list_data_model_value_node_wrapper_peek_pointer, "p", mail_source_list_data_model_value_node_wrapper_collect_value, "p", mail_source_list_data_model_value_node_wrapper_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListDataModelNodeWrapperClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_data_model_node_wrapper_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceListDataModelNodeWrapper), 0, (GInstanceInitFunc) mail_source_list_data_model_node_wrapper_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	GType mail_source_list_data_model_node_wrapper_type_id;
	mail_source_list_data_model_node_wrapper_type_id = g_type_register_fundamental (g_type_fundamental_next (), "MailSourceListDataModelNodeWrapper", &g_define_type_info, &g_define_type_fundamental_info, 0);
	MailSourceListDataModelNodeWrapper_private_offset = g_type_add_instance_private (mail_source_list_data_model_node_wrapper_type_id, sizeof (MailSourceListDataModelNodeWrapperPrivate));
	return mail_source_list_data_model_node_wrapper_type_id;
}

static GType
mail_source_list_data_model_node_wrapper_get_type (void)
{
	static volatile gsize mail_source_list_data_model_node_wrapper_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_data_model_node_wrapper_type_id__once)) {
		GType mail_source_list_data_model_node_wrapper_type_id;
		mail_source_list_data_model_node_wrapper_type_id = mail_source_list_data_model_node_wrapper_get_type_once ();
		g_once_init_leave (&mail_source_list_data_model_node_wrapper_type_id__once, mail_source_list_data_model_node_wrapper_type_id);
	}
	return mail_source_list_data_model_node_wrapper_type_id__once;
}

static gpointer
mail_source_list_data_model_node_wrapper_ref (gpointer instance)
{
	MailSourceListDataModelNodeWrapper * self;
	self = instance;
#line 648 "../src/SourceList/SourceList.vala"
	g_atomic_int_inc (&self->ref_count);
#line 648 "../src/SourceList/SourceList.vala"
	return instance;
#line 6887 "SourceList.c"
}

static void
mail_source_list_data_model_node_wrapper_unref (gpointer instance)
{
	MailSourceListDataModelNodeWrapper * self;
	self = instance;
#line 648 "../src/SourceList/SourceList.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 648 "../src/SourceList/SourceList.vala"
		MAIL_SOURCE_LIST_DATA_MODEL_NODE_WRAPPER_GET_CLASS (self)->finalize (self);
#line 648 "../src/SourceList/SourceList.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 6901 "SourceList.c"
	}
}

static inline gpointer
mail_source_list_data_model_item_monitor_get_instance_private (MailSourceListDataModelItemMonitor* self)
{
	return G_STRUCT_MEMBER_P (self, MailSourceListDataModelItemMonitor_private_offset);
}

static void
_mail_source_list_data_model_item_monitor_on_notify_g_object_notify (GObject* _sender,
                                                                     GParamSpec* pspec,
                                                                     gpointer self)
{
#line 708 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_item_monitor_on_notify ((MailSourceListDataModelItemMonitor*) self, pspec);
#line 6918 "SourceList.c"
}

static MailSourceListDataModelItemMonitor*
mail_source_list_data_model_item_monitor_construct (GType object_type,
                                                    MailSourceListItem* item)
{
	MailSourceListDataModelItemMonitor* self = NULL;
	MailSourceListItem* _tmp0_;
#line 702 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 702 "../src/SourceList/SourceList.vala"
	self = (MailSourceListDataModelItemMonitor*) g_type_create_instance (object_type);
#line 703 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (item);
#line 703 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->item);
#line 703 "../src/SourceList/SourceList.vala"
	self->priv->item = _tmp0_;
#line 704 "../src/SourceList/SourceList.vala"
	g_signal_connect_after ((GObject*) item, "notify", (GCallback) _mail_source_list_data_model_item_monitor_on_notify_g_object_notify, self);
#line 702 "../src/SourceList/SourceList.vala"
	return self;
#line 6941 "SourceList.c"
}

static MailSourceListDataModelItemMonitor*
mail_source_list_data_model_item_monitor_new (MailSourceListItem* item)
{
#line 702 "../src/SourceList/SourceList.vala"
	return mail_source_list_data_model_item_monitor_construct (MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, item);
#line 6949 "SourceList.c"
}

static void
mail_source_list_data_model_item_monitor_on_notify (MailSourceListDataModelItemMonitor* self,
                                                    GParamSpec* prop)
{
	MailSourceListItem* _tmp0_;
	const gchar* _tmp1_;
#line 711 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 711 "../src/SourceList/SourceList.vala"
	g_return_if_fail (prop != NULL);
#line 712 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->item;
#line 712 "../src/SourceList/SourceList.vala"
	_tmp1_ = prop->name;
#line 712 "../src/SourceList/SourceList.vala"
	g_signal_emit (self, mail_source_list_data_model_item_monitor_signals[MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_CHANGED_SIGNAL], 0, _tmp0_, _tmp1_);
#line 6968 "SourceList.c"
}

static void
g_cclosure_user_marshal_VOID__OBJECT_STRING (GClosure * closure,
                                             GValue * return_value,
                                             guint n_param_values,
                                             const GValue * param_values,
                                             gpointer invocation_hint,
                                             gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__OBJECT_STRING) (gpointer data1, gpointer arg_1, const char* arg_2, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_STRING callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
#line 698 "../src/SourceList/SourceList.vala"
	g_return_if_fail (n_param_values == 3);
#line 698 "../src/SourceList/SourceList.vala"
	if (G_CCLOSURE_SWAP_DATA (closure)) {
#line 698 "../src/SourceList/SourceList.vala"
		data1 = closure->data;
#line 698 "../src/SourceList/SourceList.vala"
		data2 = param_values->data[0].v_pointer;
#line 6993 "SourceList.c"
	} else {
#line 698 "../src/SourceList/SourceList.vala"
		data1 = param_values->data[0].v_pointer;
#line 698 "../src/SourceList/SourceList.vala"
		data2 = closure->data;
#line 6999 "SourceList.c"
	}
#line 698 "../src/SourceList/SourceList.vala"
	callback = (GMarshalFunc_VOID__OBJECT_STRING) (marshal_data ? marshal_data : cc->callback);
#line 698 "../src/SourceList/SourceList.vala"
	callback (data1, g_value_get_object (param_values + 1), g_value_get_string (param_values + 2), data2);
#line 7005 "SourceList.c"
}

static void
mail_source_list_data_model_value_item_monitor_init (GValue* value)
{
#line 698 "../src/SourceList/SourceList.vala"
	value->data[0].v_pointer = NULL;
#line 7013 "SourceList.c"
}

static void
mail_source_list_data_model_value_item_monitor_free_value (GValue* value)
{
#line 698 "../src/SourceList/SourceList.vala"
	if (value->data[0].v_pointer) {
#line 698 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_item_monitor_unref (value->data[0].v_pointer);
#line 7023 "SourceList.c"
	}
}

static void
mail_source_list_data_model_value_item_monitor_copy_value (const GValue* src_value,
                                                           GValue* dest_value)
{
#line 698 "../src/SourceList/SourceList.vala"
	if (src_value->data[0].v_pointer) {
#line 698 "../src/SourceList/SourceList.vala"
		dest_value->data[0].v_pointer = mail_source_list_data_model_item_monitor_ref (src_value->data[0].v_pointer);
#line 7035 "SourceList.c"
	} else {
#line 698 "../src/SourceList/SourceList.vala"
		dest_value->data[0].v_pointer = NULL;
#line 7039 "SourceList.c"
	}
}

static gpointer
mail_source_list_data_model_value_item_monitor_peek_pointer (const GValue* value)
{
#line 698 "../src/SourceList/SourceList.vala"
	return value->data[0].v_pointer;
#line 7048 "SourceList.c"
}

static gchar*
mail_source_list_data_model_value_item_monitor_collect_value (GValue* value,
                                                              guint n_collect_values,
                                                              GTypeCValue* collect_values,
                                                              guint collect_flags)
{
#line 698 "../src/SourceList/SourceList.vala"
	if (collect_values[0].v_pointer) {
#line 7059 "SourceList.c"
		MailSourceListDataModelItemMonitor * object;
		object = collect_values[0].v_pointer;
#line 698 "../src/SourceList/SourceList.vala"
		if (object->parent_instance.g_class == NULL) {
#line 698 "../src/SourceList/SourceList.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 7066 "SourceList.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 698 "../src/SourceList/SourceList.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 7070 "SourceList.c"
		}
#line 698 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = mail_source_list_data_model_item_monitor_ref (object);
#line 7074 "SourceList.c"
	} else {
#line 698 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 7078 "SourceList.c"
	}
#line 698 "../src/SourceList/SourceList.vala"
	return NULL;
#line 7082 "SourceList.c"
}

static gchar*
mail_source_list_data_model_value_item_monitor_lcopy_value (const GValue* value,
                                                            guint n_collect_values,
                                                            GTypeCValue* collect_values,
                                                            guint collect_flags)
{
	MailSourceListDataModelItemMonitor ** object_p;
	object_p = collect_values[0].v_pointer;
#line 698 "../src/SourceList/SourceList.vala"
	if (!object_p) {
#line 698 "../src/SourceList/SourceList.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 7097 "SourceList.c"
	}
#line 698 "../src/SourceList/SourceList.vala"
	if (!value->data[0].v_pointer) {
#line 698 "../src/SourceList/SourceList.vala"
		*object_p = NULL;
#line 7103 "SourceList.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 698 "../src/SourceList/SourceList.vala"
		*object_p = value->data[0].v_pointer;
#line 7107 "SourceList.c"
	} else {
#line 698 "../src/SourceList/SourceList.vala"
		*object_p = mail_source_list_data_model_item_monitor_ref (value->data[0].v_pointer);
#line 7111 "SourceList.c"
	}
#line 698 "../src/SourceList/SourceList.vala"
	return NULL;
#line 7115 "SourceList.c"
}

static GParamSpec*
mail_source_list_data_model_param_spec_item_monitor (const gchar* name,
                                                     const gchar* nick,
                                                     const gchar* blurb,
                                                     GType object_type,
                                                     GParamFlags flags)
{
	MailSourceListDataModelParamSpecItemMonitor* spec;
#line 698 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (g_type_is_a (object_type, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR), NULL);
#line 698 "../src/SourceList/SourceList.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 698 "../src/SourceList/SourceList.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 698 "../src/SourceList/SourceList.vala"
	return G_PARAM_SPEC (spec);
#line 7134 "SourceList.c"
}

static gpointer
mail_source_list_data_model_value_get_item_monitor (const GValue* value)
{
#line 698 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR), NULL);
#line 698 "../src/SourceList/SourceList.vala"
	return value->data[0].v_pointer;
#line 7144 "SourceList.c"
}

static void
mail_source_list_data_model_value_set_item_monitor (GValue* value,
                                                    gpointer v_object)
{
	MailSourceListDataModelItemMonitor * old;
#line 698 "../src/SourceList/SourceList.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR));
#line 698 "../src/SourceList/SourceList.vala"
	old = value->data[0].v_pointer;
#line 698 "../src/SourceList/SourceList.vala"
	if (v_object) {
#line 698 "../src/SourceList/SourceList.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR));
#line 698 "../src/SourceList/SourceList.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 698 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = v_object;
#line 698 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_item_monitor_ref (value->data[0].v_pointer);
#line 7166 "SourceList.c"
	} else {
#line 698 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 7170 "SourceList.c"
	}
#line 698 "../src/SourceList/SourceList.vala"
	if (old) {
#line 698 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_item_monitor_unref (old);
#line 7176 "SourceList.c"
	}
}

static void
mail_source_list_data_model_value_take_item_monitor (GValue* value,
                                                     gpointer v_object)
{
	MailSourceListDataModelItemMonitor * old;
#line 698 "../src/SourceList/SourceList.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR));
#line 698 "../src/SourceList/SourceList.vala"
	old = value->data[0].v_pointer;
#line 698 "../src/SourceList/SourceList.vala"
	if (v_object) {
#line 698 "../src/SourceList/SourceList.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR));
#line 698 "../src/SourceList/SourceList.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 698 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = v_object;
#line 7197 "SourceList.c"
	} else {
#line 698 "../src/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 7201 "SourceList.c"
	}
#line 698 "../src/SourceList/SourceList.vala"
	if (old) {
#line 698 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_item_monitor_unref (old);
#line 7207 "SourceList.c"
	}
}

static void
mail_source_list_data_model_item_monitor_class_init (MailSourceListDataModelItemMonitorClass * klass,
                                                     gpointer klass_data)
{
#line 698 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_item_monitor_parent_class = g_type_class_peek_parent (klass);
#line 698 "../src/SourceList/SourceList.vala"
	((MailSourceListDataModelItemMonitorClass *) klass)->finalize = mail_source_list_data_model_item_monitor_finalize;
#line 698 "../src/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &MailSourceListDataModelItemMonitor_private_offset);
#line 698 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_item_monitor_signals[MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_CHANGED_SIGNAL] = g_signal_new ("changed", MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_STRING, G_TYPE_NONE, 2, MAIL_SOURCE_LIST_TYPE_ITEM, G_TYPE_STRING);
#line 7223 "SourceList.c"
}

static void
mail_source_list_data_model_item_monitor_instance_init (MailSourceListDataModelItemMonitor * self,
                                                        gpointer klass)
{
#line 698 "../src/SourceList/SourceList.vala"
	self->priv = mail_source_list_data_model_item_monitor_get_instance_private (self);
#line 698 "../src/SourceList/SourceList.vala"
	self->ref_count = 1;
#line 7234 "SourceList.c"
}

static void
mail_source_list_data_model_item_monitor_finalize (MailSourceListDataModelItemMonitor * obj)
{
	MailSourceListDataModelItemMonitor * self;
	MailSourceListItem* _tmp0_;
	guint _tmp1_;
#line 698 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, MailSourceListDataModelItemMonitor);
#line 698 "../src/SourceList/SourceList.vala"
	g_signal_handlers_destroy (self);
#line 708 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->item;
#line 708 "../src/SourceList/SourceList.vala"
	g_signal_parse_name ("notify", G_TYPE_OBJECT, &_tmp1_, NULL, FALSE);
#line 708 "../src/SourceList/SourceList.vala"
	g_signal_handlers_disconnect_matched ((GObject*) _tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _mail_source_list_data_model_item_monitor_on_notify_g_object_notify, self);
#line 700 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->item);
#line 7255 "SourceList.c"
}

/**
         * Helper object used to monitor item property changes.
         */
static GType
mail_source_list_data_model_item_monitor_get_type_once (void)
{
	static const GTypeValueTable g_define_type_value_table = { mail_source_list_data_model_value_item_monitor_init, mail_source_list_data_model_value_item_monitor_free_value, mail_source_list_data_model_value_item_monitor_copy_value, mail_source_list_data_model_value_item_monitor_peek_pointer, "p", mail_source_list_data_model_value_item_monitor_collect_value, "p", mail_source_list_data_model_value_item_monitor_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListDataModelItemMonitorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_data_model_item_monitor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceListDataModelItemMonitor), 0, (GInstanceInitFunc) mail_source_list_data_model_item_monitor_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	GType mail_source_list_data_model_item_monitor_type_id;
	mail_source_list_data_model_item_monitor_type_id = g_type_register_fundamental (g_type_fundamental_next (), "MailSourceListDataModelItemMonitor", &g_define_type_info, &g_define_type_fundamental_info, 0);
	MailSourceListDataModelItemMonitor_private_offset = g_type_add_instance_private (mail_source_list_data_model_item_monitor_type_id, sizeof (MailSourceListDataModelItemMonitorPrivate));
	return mail_source_list_data_model_item_monitor_type_id;
}

static GType
mail_source_list_data_model_item_monitor_get_type (void)
{
	static volatile gsize mail_source_list_data_model_item_monitor_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_data_model_item_monitor_type_id__once)) {
		GType mail_source_list_data_model_item_monitor_type_id;
		mail_source_list_data_model_item_monitor_type_id = mail_source_list_data_model_item_monitor_get_type_once ();
		g_once_init_leave (&mail_source_list_data_model_item_monitor_type_id__once, mail_source_list_data_model_item_monitor_type_id);
	}
	return mail_source_list_data_model_item_monitor_type_id__once;
}

static gpointer
mail_source_list_data_model_item_monitor_ref (gpointer instance)
{
	MailSourceListDataModelItemMonitor * self;
	self = instance;
#line 698 "../src/SourceList/SourceList.vala"
	g_atomic_int_inc (&self->ref_count);
#line 698 "../src/SourceList/SourceList.vala"
	return instance;
#line 7294 "SourceList.c"
}

static void
mail_source_list_data_model_item_monitor_unref (gpointer instance)
{
	MailSourceListDataModelItemMonitor * self;
	self = instance;
#line 698 "../src/SourceList/SourceList.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 698 "../src/SourceList/SourceList.vala"
		MAIL_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_GET_CLASS (self)->finalize (self);
#line 698 "../src/SourceList/SourceList.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 7308 "SourceList.c"
	}
}

static gint
_mail_source_list_data_model_child_model_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model,
                                                                               GtkTreeIter* a,
                                                                               GtkTreeIter* b,
                                                                               gpointer self)
{
	gint result;
	result = mail_source_list_data_model_child_model_sort_func ((MailSourceListDataModel*) self, model, a, b);
#line 778 "../src/SourceList/SourceList.vala"
	return result;
#line 7322 "SourceList.c"
}

static gboolean
_mail_source_list_data_model_filter_visible_func_gtk_tree_model_filter_visible_func (GtkTreeModel* model,
                                                                                     GtkTreeIter* iter,
                                                                                     gpointer self)
{
	gboolean result;
	result = mail_source_list_data_model_filter_visible_func ((MailSourceListDataModel*) self, model, iter);
#line 781 "../src/SourceList/SourceList.vala"
	return result;
#line 7334 "SourceList.c"
}

static GObject *
mail_source_list_data_model_constructor (GType type,
                                         guint n_construct_properties,
                                         GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	MailSourceListDataModel * self;
	GtkTreeStore* _tmp0_;
	GtkTreeStore* _tmp1_;
	GtkTreeStore* _tmp2_;
#line 773 "../src/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (mail_source_list_data_model_parent_class);
#line 773 "../src/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 773 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_DATA_MODEL, MailSourceListDataModel);
#line 774 "../src/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_store_new ((gint) MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_N_COLUMNS, mail_source_list_data_model_column_type (MAIL_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM));
#line 774 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->child_tree);
#line 774 "../src/SourceList/SourceList.vala"
	self->priv->child_tree = _tmp0_;
#line 775 "../src/SourceList/SourceList.vala"
	_tmp1_ = self->priv->child_tree;
#line 775 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkTreeModelFilter*) self, "child-model", (GtkTreeModel*) _tmp1_, NULL);
#line 776 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkTreeModelFilter*) self, "virtual-root", NULL, NULL);
#line 778 "../src/SourceList/SourceList.vala"
	_tmp2_ = self->priv->child_tree;
#line 778 "../src/SourceList/SourceList.vala"
	gtk_tree_sortable_set_default_sort_func ((GtkTreeSortable*) _tmp2_, _mail_source_list_data_model_child_model_sort_func_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
#line 779 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_resort (self);
#line 781 "../src/SourceList/SourceList.vala"
	gtk_tree_model_filter_set_visible_func ((GtkTreeModelFilter*) self, _mail_source_list_data_model_filter_visible_func_gtk_tree_model_filter_visible_func, g_object_ref (self), g_object_unref);
#line 773 "../src/SourceList/SourceList.vala"
	return obj;
#line 7376 "SourceList.c"
}

static void
mail_source_list_data_model_class_init (MailSourceListDataModelClass * klass,
                                        gpointer klass_data)
{
#line 640 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_parent_class = g_type_class_peek_parent (klass);
#line 640 "../src/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &MailSourceListDataModel_private_offset);
#line 640 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_mail_source_list_data_model_get_property;
#line 640 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_mail_source_list_data_model_set_property;
#line 640 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = mail_source_list_data_model_constructor;
#line 640 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = mail_source_list_data_model_finalize;
#line 7395 "SourceList.c"
	/**
	         * Root item.
	         *
	         * This item is not actually part of the model. It's only used as a proxy
	         * for adding and removing items.
	         */
#line 640 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY, mail_source_list_data_model_properties[MAIL_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY] = g_param_spec_object ("root", "root", "root", MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 640 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_signals[MAIL_SOURCE_LIST_DATA_MODEL_ITEM_UPDATED_SIGNAL] = g_signal_new ("item-updated", MAIL_SOURCE_LIST_TYPE_DATA_MODEL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, MAIL_SOURCE_LIST_TYPE_ITEM);
#line 7406 "SourceList.c"
}

static void
mail_source_list_data_model_gtk_tree_drag_source_interface_init (GtkTreeDragSourceIface * iface,
                                                                 gpointer iface_data)
{
#line 640 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_gtk_tree_drag_source_parent_iface = g_type_interface_peek_parent (iface);
#line 640 "../src/SourceList/SourceList.vala"
	iface->drag_data_get = (gboolean (*) (GtkTreeDragSource*, GtkTreePath*, GtkSelectionData*)) mail_source_list_data_model_real_drag_data_get;
#line 640 "../src/SourceList/SourceList.vala"
	iface->row_draggable = (gboolean (*) (GtkTreeDragSource*, GtkTreePath*)) mail_source_list_data_model_real_row_draggable;
#line 7419 "SourceList.c"
}

static void
mail_source_list_data_model_gtk_tree_drag_dest_interface_init (GtkTreeDragDestIface * iface,
                                                               gpointer iface_data)
{
#line 640 "../src/SourceList/SourceList.vala"
	mail_source_list_data_model_gtk_tree_drag_dest_parent_iface = g_type_interface_peek_parent (iface);
#line 640 "../src/SourceList/SourceList.vala"
	iface->drag_data_received = (gboolean (*) (GtkTreeDragDest*, GtkTreePath*, GtkSelectionData*)) mail_source_list_data_model_real_drag_data_received;
#line 640 "../src/SourceList/SourceList.vala"
	iface->row_drop_possible = (gboolean (*) (GtkTreeDragDest*, GtkTreePath*, GtkSelectionData*)) mail_source_list_data_model_real_row_drop_possible;
#line 7432 "SourceList.c"
}

static void
mail_source_list_data_model_instance_init (MailSourceListDataModel * self,
                                           gpointer klass)
{
	GeeHashMap* _tmp0_;
	GeeHashMap* _tmp1_;
#line 640 "../src/SourceList/SourceList.vala"
	self->priv = mail_source_list_data_model_get_instance_private (self);
#line 766 "../src/SourceList/SourceList.vala"
	_tmp0_ = gee_hash_map_new (MAIL_SOURCE_LIST_TYPE_ITEM, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, (GBoxedCopyFunc) mail_source_list_data_model_node_wrapper_ref, (GDestroyNotify) mail_source_list_data_model_node_wrapper_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
#line 766 "../src/SourceList/SourceList.vala"
	self->priv->items = _tmp0_;
#line 768 "../src/SourceList/SourceList.vala"
	_tmp1_ = gee_hash_map_new (MAIL_SOURCE_LIST_TYPE_ITEM, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, MAIL_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, (GBoxedCopyFunc) mail_source_list_data_model_item_monitor_ref, (GDestroyNotify) mail_source_list_data_model_item_monitor_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
#line 768 "../src/SourceList/SourceList.vala"
	self->priv->monitors = _tmp1_;
#line 7451 "SourceList.c"
}

static void
mail_source_list_data_model_finalize (GObject * obj)
{
	MailSourceListDataModel * self;
#line 640 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_DATA_MODEL, MailSourceListDataModel);
#line 738 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_root);
#line 766 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->items);
#line 768 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->monitors);
#line 770 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->child_tree);
#line 640 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (mail_source_list_data_model_parent_class)->finalize (obj);
#line 7470 "SourceList.c"
}

/**
     * The model backing the SourceList tree.
     *
     * It monitors item property changes, and handles children additions and removals. It also controls
     * the visibility of the items based on their "visible" property, and on their number of children,
     * if they happen to be categories. Its main purpose is to provide an easy and practical interface
     * for sorting, adding, removing and updating items, eliminating the need of repeatedly dealing with
     * the Gtk.TreeModel API directly.
     */
static GType
mail_source_list_data_model_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListDataModelClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_data_model_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceListDataModel), 0, (GInstanceInitFunc) mail_source_list_data_model_instance_init, NULL };
	static const GInterfaceInfo gtk_tree_drag_source_info = { (GInterfaceInitFunc) mail_source_list_data_model_gtk_tree_drag_source_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	static const GInterfaceInfo gtk_tree_drag_dest_info = { (GInterfaceInitFunc) mail_source_list_data_model_gtk_tree_drag_dest_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType mail_source_list_data_model_type_id;
	mail_source_list_data_model_type_id = g_type_register_static (gtk_tree_model_filter_get_type (), "MailSourceListDataModel", &g_define_type_info, 0);
	g_type_add_interface_static (mail_source_list_data_model_type_id, gtk_tree_drag_source_get_type (), &gtk_tree_drag_source_info);
	g_type_add_interface_static (mail_source_list_data_model_type_id, gtk_tree_drag_dest_get_type (), &gtk_tree_drag_dest_info);
	MailSourceListDataModel_private_offset = g_type_add_instance_private (mail_source_list_data_model_type_id, sizeof (MailSourceListDataModelPrivate));
	return mail_source_list_data_model_type_id;
}

static GType
mail_source_list_data_model_get_type (void)
{
	static volatile gsize mail_source_list_data_model_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_data_model_type_id__once)) {
		GType mail_source_list_data_model_type_id;
		mail_source_list_data_model_type_id = mail_source_list_data_model_get_type_once ();
		g_once_init_leave (&mail_source_list_data_model_type_id__once, mail_source_list_data_model_type_id);
	}
	return mail_source_list_data_model_type_id__once;
}

static void
_vala_mail_source_list_data_model_get_property (GObject * object,
                                                guint property_id,
                                                GValue * value,
                                                GParamSpec * pspec)
{
	MailSourceListDataModel * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_DATA_MODEL, MailSourceListDataModel);
#line 640 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 640 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY:
#line 640 "../src/SourceList/SourceList.vala"
		g_value_set_object (value, mail_source_list_data_model_get_root (self));
#line 640 "../src/SourceList/SourceList.vala"
		break;
#line 7524 "SourceList.c"
		default:
#line 640 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 640 "../src/SourceList/SourceList.vala"
		break;
#line 7530 "SourceList.c"
	}
}

static void
_vala_mail_source_list_data_model_set_property (GObject * object,
                                                guint property_id,
                                                const GValue * value,
                                                GParamSpec * pspec)
{
	MailSourceListDataModel * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_DATA_MODEL, MailSourceListDataModel);
#line 640 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 640 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY:
#line 640 "../src/SourceList/SourceList.vala"
		mail_source_list_data_model_set_root (self, g_value_get_object (value));
#line 640 "../src/SourceList/SourceList.vala"
		break;
#line 7550 "SourceList.c"
		default:
#line 640 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 640 "../src/SourceList/SourceList.vala"
		break;
#line 7556 "SourceList.c"
	}
}

static gboolean
mail_source_list_cell_renderer_icon_real_activate (GtkCellRenderer* base,
                                                   GdkEvent* event,
                                                   GtkWidget* widget,
                                                   const gchar* path,
                                                   GdkRectangle* background_area,
                                                   GdkRectangle* cell_area,
                                                   GtkCellRendererState flags)
{
	MailSourceListCellRendererIcon * self;
	gboolean result;
#line 1374 "../src/SourceList/SourceList.vala"
	self = (MailSourceListCellRendererIcon*) base;
#line 1374 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 1374 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (widget != NULL, FALSE);
#line 1374 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1374 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (background_area != NULL, FALSE);
#line 1374 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (cell_area != NULL, FALSE);
#line 1382 "../src/SourceList/SourceList.vala"
	g_signal_emit (self, mail_source_list_cell_renderer_icon_signals[MAIL_SOURCE_LIST_CELL_RENDERER_ICON_ACTIVATED_SIGNAL], 0, path);
#line 1383 "../src/SourceList/SourceList.vala"
	result = TRUE;
#line 1383 "../src/SourceList/SourceList.vala"
	return result;
#line 7589 "SourceList.c"
}

static MailSourceListCellRendererIcon*
mail_source_list_cell_renderer_icon_construct (GType object_type)
{
	MailSourceListCellRendererIcon * self = NULL;
#line 1366 "../src/SourceList/SourceList.vala"
	self = (MailSourceListCellRendererIcon*) g_object_new (object_type, NULL);
#line 1366 "../src/SourceList/SourceList.vala"
	return self;
#line 7600 "SourceList.c"
}

static MailSourceListCellRendererIcon*
mail_source_list_cell_renderer_icon_new (void)
{
#line 1366 "../src/SourceList/SourceList.vala"
	return mail_source_list_cell_renderer_icon_construct (MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON);
#line 7608 "SourceList.c"
}

static GObject *
mail_source_list_cell_renderer_icon_constructor (GType type,
                                                 guint n_construct_properties,
                                                 GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	MailSourceListCellRendererIcon * self;
#line 1369 "../src/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (mail_source_list_cell_renderer_icon_parent_class);
#line 1369 "../src/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 1369 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, MailSourceListCellRendererIcon);
#line 1370 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) self, "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE, NULL);
#line 1371 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRendererPixbuf*) self, "stock-size", (guint) GTK_ICON_SIZE_MENU, NULL);
#line 1369 "../src/SourceList/SourceList.vala"
	return obj;
#line 7631 "SourceList.c"
}

static void
mail_source_list_cell_renderer_icon_class_init (MailSourceListCellRendererIconClass * klass,
                                                gpointer klass_data)
{
#line 1366 "../src/SourceList/SourceList.vala"
	mail_source_list_cell_renderer_icon_parent_class = g_type_class_peek_parent (klass);
#line 1366 "../src/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->activate = (gboolean (*) (GtkCellRenderer*, GdkEvent*, GtkWidget*, const gchar*, GdkRectangle*, GdkRectangle*, GtkCellRendererState)) mail_source_list_cell_renderer_icon_real_activate;
#line 1366 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = mail_source_list_cell_renderer_icon_constructor;
#line 1366 "../src/SourceList/SourceList.vala"
	mail_source_list_cell_renderer_icon_signals[MAIL_SOURCE_LIST_CELL_RENDERER_ICON_ACTIVATED_SIGNAL] = g_signal_new ("activated", MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
#line 7646 "SourceList.c"
}

static void
mail_source_list_cell_renderer_icon_instance_init (MailSourceListCellRendererIcon * self,
                                                   gpointer klass)
{
}

/**
     * Class responsible for rendering Item.icon and Item.activatable. It also
     * notifies about clicks through the activated() signal.
     */
static GType
mail_source_list_cell_renderer_icon_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListCellRendererIconClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_cell_renderer_icon_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceListCellRendererIcon), 0, (GInstanceInitFunc) mail_source_list_cell_renderer_icon_instance_init, NULL };
	GType mail_source_list_cell_renderer_icon_type_id;
	mail_source_list_cell_renderer_icon_type_id = g_type_register_static (gtk_cell_renderer_pixbuf_get_type (), "MailSourceListCellRendererIcon", &g_define_type_info, 0);
	return mail_source_list_cell_renderer_icon_type_id;
}

static GType
mail_source_list_cell_renderer_icon_get_type (void)
{
	static volatile gsize mail_source_list_cell_renderer_icon_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_cell_renderer_icon_type_id__once)) {
		GType mail_source_list_cell_renderer_icon_type_id;
		mail_source_list_cell_renderer_icon_type_id = mail_source_list_cell_renderer_icon_get_type_once ();
		g_once_init_leave (&mail_source_list_cell_renderer_icon_type_id__once, mail_source_list_cell_renderer_icon_type_id);
	}
	return mail_source_list_cell_renderer_icon_type_id__once;
}

static inline gpointer
mail_source_list_cell_renderer_spacer_get_instance_private (MailSourceListCellRendererSpacer* self)
{
	return G_STRUCT_MEMBER_P (self, MailSourceListCellRendererSpacer_private_offset);
}

static GtkSizeRequestMode
mail_source_list_cell_renderer_spacer_real_get_request_mode (GtkCellRenderer* base)
{
	MailSourceListCellRendererSpacer * self;
	GtkSizeRequestMode result;
#line 1396 "../src/SourceList/SourceList.vala"
	self = (MailSourceListCellRendererSpacer*) base;
#line 1397 "../src/SourceList/SourceList.vala"
	result = GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
#line 1397 "../src/SourceList/SourceList.vala"
	return result;
#line 7697 "SourceList.c"
}

static void
mail_source_list_cell_renderer_spacer_real_get_preferred_width (GtkCellRenderer* base,
                                                                GtkWidget* widget,
                                                                gint* min_size,
                                                                gint* natural_size)
{
	MailSourceListCellRendererSpacer * self;
	gint _vala_min_size = 0;
	gint _vala_natural_size = 0;
	guint _tmp0_;
	guint _tmp1_;
#line 1400 "../src/SourceList/SourceList.vala"
	self = (MailSourceListCellRendererSpacer*) base;
#line 1400 "../src/SourceList/SourceList.vala"
	g_return_if_fail (widget != NULL);
#line 1401 "../src/SourceList/SourceList.vala"
	g_object_get ((GtkCellRenderer*) self, "xpad", &_tmp0_, NULL);
#line 1401 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 1401 "../src/SourceList/SourceList.vala"
	_vala_natural_size = 2 * ((gint) _tmp1_);
#line 1401 "../src/SourceList/SourceList.vala"
	_vala_min_size = _vala_natural_size;
#line 1400 "../src/SourceList/SourceList.vala"
	if (min_size) {
#line 1400 "../src/SourceList/SourceList.vala"
		*min_size = _vala_min_size;
#line 7727 "SourceList.c"
	}
#line 1400 "../src/SourceList/SourceList.vala"
	if (natural_size) {
#line 1400 "../src/SourceList/SourceList.vala"
		*natural_size = _vala_natural_size;
#line 7733 "SourceList.c"
	}
}

static void
mail_source_list_cell_renderer_spacer_real_get_preferred_height_for_width (GtkCellRenderer* base,
                                                                           GtkWidget* widget,
                                                                           gint width,
                                                                           gint* min_height,
                                                                           gint* natural_height)
{
	MailSourceListCellRendererSpacer * self;
	gint _vala_min_height = 0;
	gint _vala_natural_height = 0;
	guint _tmp0_;
	guint _tmp1_;
#line 1404 "../src/SourceList/SourceList.vala"
	self = (MailSourceListCellRendererSpacer*) base;
#line 1404 "../src/SourceList/SourceList.vala"
	g_return_if_fail (widget != NULL);
#line 1410 "../src/SourceList/SourceList.vala"
	g_object_get ((GtkCellRenderer*) self, "ypad", &_tmp0_, NULL);
#line 1410 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 1410 "../src/SourceList/SourceList.vala"
	_vala_natural_height = 2 * ((gint) _tmp1_);
#line 1410 "../src/SourceList/SourceList.vala"
	_vala_min_height = _vala_natural_height;
#line 1404 "../src/SourceList/SourceList.vala"
	if (min_height) {
#line 1404 "../src/SourceList/SourceList.vala"
		*min_height = _vala_min_height;
#line 7765 "SourceList.c"
	}
#line 1404 "../src/SourceList/SourceList.vala"
	if (natural_height) {
#line 1404 "../src/SourceList/SourceList.vala"
		*natural_height = _vala_natural_height;
#line 7771 "SourceList.c"
	}
}

static void
mail_source_list_cell_renderer_spacer_real_render (GtkCellRenderer* base,
                                                   cairo_t* context,
                                                   GtkWidget* widget,
                                                   GdkRectangle* bg_area,
                                                   GdkRectangle* cell_area,
                                                   GtkCellRendererState flags)
{
	MailSourceListCellRendererSpacer * self;
#line 1413 "../src/SourceList/SourceList.vala"
	self = (MailSourceListCellRendererSpacer*) base;
#line 1413 "../src/SourceList/SourceList.vala"
	g_return_if_fail (context != NULL);
#line 1413 "../src/SourceList/SourceList.vala"
	g_return_if_fail (widget != NULL);
#line 1413 "../src/SourceList/SourceList.vala"
	g_return_if_fail (bg_area != NULL);
#line 1413 "../src/SourceList/SourceList.vala"
	g_return_if_fail (cell_area != NULL);
#line 7794 "SourceList.c"
}

static MailSourceListCellRendererSpacer*
mail_source_list_cell_renderer_spacer_construct (GType object_type)
{
	MailSourceListCellRendererSpacer * self = NULL;
#line 1390 "../src/SourceList/SourceList.vala"
	self = (MailSourceListCellRendererSpacer*) g_object_new (object_type, NULL);
#line 1390 "../src/SourceList/SourceList.vala"
	return self;
#line 7805 "SourceList.c"
}

static MailSourceListCellRendererSpacer*
mail_source_list_cell_renderer_spacer_new (void)
{
#line 1390 "../src/SourceList/SourceList.vala"
	return mail_source_list_cell_renderer_spacer_construct (MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER);
#line 7813 "SourceList.c"
}

static gint
mail_source_list_cell_renderer_spacer_get_level (MailSourceListCellRendererSpacer* self)
{
	gint result;
#line 1394 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 1394 "../src/SourceList/SourceList.vala"
	result = self->priv->_level;
#line 1394 "../src/SourceList/SourceList.vala"
	return result;
#line 7826 "SourceList.c"
}

static void
mail_source_list_cell_renderer_spacer_set_level (MailSourceListCellRendererSpacer* self,
                                                 gint value)
{
	gint old_value;
#line 1394 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1394 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_cell_renderer_spacer_get_level (self);
#line 1394 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 1394 "../src/SourceList/SourceList.vala"
		self->priv->_level = value;
#line 1394 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_cell_renderer_spacer_properties[MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY]);
#line 7844 "SourceList.c"
	}
}

static void
mail_source_list_cell_renderer_spacer_class_init (MailSourceListCellRendererSpacerClass * klass,
                                                  gpointer klass_data)
{
#line 1390 "../src/SourceList/SourceList.vala"
	mail_source_list_cell_renderer_spacer_parent_class = g_type_class_peek_parent (klass);
#line 1390 "../src/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &MailSourceListCellRendererSpacer_private_offset);
#line 1390 "../src/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->get_request_mode = (GtkSizeRequestMode (*) (GtkCellRenderer*)) mail_source_list_cell_renderer_spacer_real_get_request_mode;
#line 1390 "../src/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->get_preferred_width = (void (*) (GtkCellRenderer*, GtkWidget*, gint*, gint*)) mail_source_list_cell_renderer_spacer_real_get_preferred_width;
#line 1390 "../src/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->get_preferred_height_for_width = (void (*) (GtkCellRenderer*, GtkWidget*, gint, gint*, gint*)) mail_source_list_cell_renderer_spacer_real_get_preferred_height_for_width;
#line 1390 "../src/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->render = (void (*) (GtkCellRenderer*, cairo_t*, GtkWidget*, GdkRectangle*, GdkRectangle*, GtkCellRendererState)) mail_source_list_cell_renderer_spacer_real_render;
#line 1390 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_mail_source_list_cell_renderer_spacer_get_property;
#line 1390 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_mail_source_list_cell_renderer_spacer_set_property;
#line 1390 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = mail_source_list_cell_renderer_spacer_finalize;
#line 7870 "SourceList.c"
	/**
	         * Indentation level represented by this cell renderer
	         */
#line 1390 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY, mail_source_list_cell_renderer_spacer_properties[MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY] = g_param_spec_int ("level", "level", "level", G_MININT, G_MAXINT, -1, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7876 "SourceList.c"
}

static void
mail_source_list_cell_renderer_spacer_instance_init (MailSourceListCellRendererSpacer * self,
                                                     gpointer klass)
{
#line 1390 "../src/SourceList/SourceList.vala"
	self->priv = mail_source_list_cell_renderer_spacer_get_instance_private (self);
#line 1394 "../src/SourceList/SourceList.vala"
	self->priv->_level = -1;
#line 7887 "SourceList.c"
}

static void
mail_source_list_cell_renderer_spacer_finalize (GObject * obj)
{
	MailSourceListCellRendererSpacer * self;
#line 1390 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, MailSourceListCellRendererSpacer);
#line 1390 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (mail_source_list_cell_renderer_spacer_parent_class)->finalize (obj);
#line 7898 "SourceList.c"
}

/**
     * A cell renderer that only adds space.
     */
static GType
mail_source_list_cell_renderer_spacer_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListCellRendererSpacerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_cell_renderer_spacer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceListCellRendererSpacer), 0, (GInstanceInitFunc) mail_source_list_cell_renderer_spacer_instance_init, NULL };
	GType mail_source_list_cell_renderer_spacer_type_id;
	mail_source_list_cell_renderer_spacer_type_id = g_type_register_static (gtk_cell_renderer_get_type (), "MailSourceListCellRendererSpacer", &g_define_type_info, 0);
	MailSourceListCellRendererSpacer_private_offset = g_type_add_instance_private (mail_source_list_cell_renderer_spacer_type_id, sizeof (MailSourceListCellRendererSpacerPrivate));
	return mail_source_list_cell_renderer_spacer_type_id;
}

static GType
mail_source_list_cell_renderer_spacer_get_type (void)
{
	static volatile gsize mail_source_list_cell_renderer_spacer_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_cell_renderer_spacer_type_id__once)) {
		GType mail_source_list_cell_renderer_spacer_type_id;
		mail_source_list_cell_renderer_spacer_type_id = mail_source_list_cell_renderer_spacer_get_type_once ();
		g_once_init_leave (&mail_source_list_cell_renderer_spacer_type_id__once, mail_source_list_cell_renderer_spacer_type_id);
	}
	return mail_source_list_cell_renderer_spacer_type_id__once;
}

static void
_vala_mail_source_list_cell_renderer_spacer_get_property (GObject * object,
                                                          guint property_id,
                                                          GValue * value,
                                                          GParamSpec * pspec)
{
	MailSourceListCellRendererSpacer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, MailSourceListCellRendererSpacer);
#line 1390 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 1390 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY:
#line 1390 "../src/SourceList/SourceList.vala"
		g_value_set_int (value, mail_source_list_cell_renderer_spacer_get_level (self));
#line 1390 "../src/SourceList/SourceList.vala"
		break;
#line 7942 "SourceList.c"
		default:
#line 1390 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 1390 "../src/SourceList/SourceList.vala"
		break;
#line 7948 "SourceList.c"
	}
}

static void
_vala_mail_source_list_cell_renderer_spacer_set_property (GObject * object,
                                                          guint property_id,
                                                          const GValue * value,
                                                          GParamSpec * pspec)
{
	MailSourceListCellRendererSpacer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, MailSourceListCellRendererSpacer);
#line 1390 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 1390 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY:
#line 1390 "../src/SourceList/SourceList.vala"
		mail_source_list_cell_renderer_spacer_set_level (self, g_value_get_int (value));
#line 1390 "../src/SourceList/SourceList.vala"
		break;
#line 7968 "SourceList.c"
		default:
#line 1390 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 1390 "../src/SourceList/SourceList.vala"
		break;
#line 7974 "SourceList.c"
	}
}

static inline gpointer
mail_source_list_tree_get_instance_private (MailSourceListTree* self)
{
	return G_STRUCT_MEMBER_P (self, MailSourceListTree_private_offset);
}

static GType
mail_source_list_tree_column_get_type_once (void)
{
	static const GEnumValue values[] = {{MAIL_SOURCE_LIST_TREE_COLUMN_ITEM, "MAIL_SOURCE_LIST_TREE_COLUMN_ITEM", "item"}, {MAIL_SOURCE_LIST_TREE_COLUMN_N_COLS, "MAIL_SOURCE_LIST_TREE_COLUMN_N_COLS", "n-cols"}, {0, NULL, NULL}};
	GType mail_source_list_tree_column_type_id;
	mail_source_list_tree_column_type_id = g_enum_register_static ("MailSourceListTreeColumn", values);
	return mail_source_list_tree_column_type_id;
}

static GType
mail_source_list_tree_column_get_type (void)
{
	static volatile gsize mail_source_list_tree_column_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_tree_column_type_id__once)) {
		GType mail_source_list_tree_column_type_id;
		mail_source_list_tree_column_type_id = mail_source_list_tree_column_get_type_once ();
		g_once_init_leave (&mail_source_list_tree_column_type_id__once, mail_source_list_tree_column_type_id);
	}
	return mail_source_list_tree_column_type_id__once;
}

static MailSourceListTree*
mail_source_list_tree_construct (GType object_type,
                                 MailSourceListDataModel* data_model)
{
	MailSourceListTree * self = NULL;
#line 1513 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (data_model != NULL, NULL);
#line 1514 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) g_object_new (object_type, "data-model", data_model, NULL);
#line 1513 "../src/SourceList/SourceList.vala"
	return self;
#line 8016 "SourceList.c"
}

static MailSourceListTree*
mail_source_list_tree_new (MailSourceListDataModel* data_model)
{
#line 1513 "../src/SourceList/SourceList.vala"
	return mail_source_list_tree_construct (MAIL_SOURCE_LIST_TYPE_TREE, data_model);
#line 8024 "SourceList.c"
}

static gpointer
_gtk_target_list_ref0 (gpointer self)
{
#line 1662 "../src/SourceList/SourceList.vala"
	return self ? gtk_target_list_ref (self) : NULL;
#line 8032 "SourceList.c"
}

static gboolean
mail_source_list_tree_real_drag_motion (GtkWidget* base,
                                        GdkDragContext* context,
                                        gint x,
                                        gint y,
                                        guint time)
{
	MailSourceListTree * self;
	GtkTreePath* suggested_path = NULL;
	GtkTreePath* current_path = NULL;
	GtkTreeViewDropPosition suggested_pos = 0;
	GtkTreeViewDropPosition current_pos = 0;
	GtkTreePath* _tmp0_ = NULL;
	GtkTreeViewDropPosition _tmp1_ = 0;
	gboolean _tmp2_;
	gboolean result;
#line 1620 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 1620 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (context != NULL, FALSE);
#line 1622 "../src/SourceList/SourceList.vala"
	if (!GTK_WIDGET_CLASS (mail_source_list_tree_parent_class)->drag_motion ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), context, x, y, time)) {
#line 1623 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1623 "../src/SourceList/SourceList.vala"
		return result;
#line 8061 "SourceList.c"
	}
#line 1628 "../src/SourceList/SourceList.vala"
	_tmp2_ = gtk_tree_view_get_dest_row_at_pos ((GtkTreeView*) self, x, y, &_tmp0_, &_tmp1_);
#line 1628 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (suggested_path);
#line 1628 "../src/SourceList/SourceList.vala"
	suggested_path = _tmp0_;
#line 1628 "../src/SourceList/SourceList.vala"
	suggested_pos = _tmp1_;
#line 1628 "../src/SourceList/SourceList.vala"
	if (_tmp2_) {
#line 8073 "SourceList.c"
		GtkTreePath* _tmp3_ = NULL;
		GtkTreeViewDropPosition _tmp4_ = 0;
		gboolean _tmp5_ = FALSE;
		GtkTreePath* _tmp6_;
#line 1632 "../src/SourceList/SourceList.vala"
		gtk_tree_view_get_drag_dest_row ((GtkTreeView*) self, &_tmp3_, &_tmp4_);
#line 1632 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (current_path);
#line 1632 "../src/SourceList/SourceList.vala"
		current_path = _tmp3_;
#line 1632 "../src/SourceList/SourceList.vala"
		current_pos = _tmp4_;
#line 1634 "../src/SourceList/SourceList.vala"
		_tmp6_ = current_path;
#line 1634 "../src/SourceList/SourceList.vala"
		if (_tmp6_ != NULL) {
#line 8090 "SourceList.c"
			GtkTreePath* _tmp7_;
			GtkTreePath* _tmp8_;
#line 1634 "../src/SourceList/SourceList.vala"
			_tmp7_ = suggested_path;
#line 1634 "../src/SourceList/SourceList.vala"
			_tmp8_ = current_path;
#line 1634 "../src/SourceList/SourceList.vala"
			_tmp5_ = gtk_tree_path_compare (_tmp7_, _tmp8_) == 0;
#line 8099 "SourceList.c"
		} else {
#line 1634 "../src/SourceList/SourceList.vala"
			_tmp5_ = FALSE;
#line 8103 "SourceList.c"
		}
#line 1634 "../src/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 8107 "SourceList.c"
			GtkWidget* source_widget = NULL;
			GtkWidget* _tmp9_;
			GtkWidget* _tmp10_;
			gboolean dragging_treemodel_row = FALSE;
			GtkWidget* _tmp11_;
#line 1638 "../src/SourceList/SourceList.vala"
			_tmp9_ = gtk_drag_get_source_widget (context);
#line 1638 "../src/SourceList/SourceList.vala"
			_tmp10_ = _g_object_ref0 (_tmp9_);
#line 1638 "../src/SourceList/SourceList.vala"
			source_widget = _tmp10_;
#line 1639 "../src/SourceList/SourceList.vala"
			_tmp11_ = source_widget;
#line 1639 "../src/SourceList/SourceList.vala"
			dragging_treemodel_row = _tmp11_ == G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget);
#line 1641 "../src/SourceList/SourceList.vala"
			if (dragging_treemodel_row) {
#line 1644 "../src/SourceList/SourceList.vala"
				if (current_pos != GTK_TREE_VIEW_DROP_BEFORE) {
#line 1645 "../src/SourceList/SourceList.vala"
					if (current_pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE) {
#line 8129 "SourceList.c"
						GtkTreePath* _tmp12_;
#line 1646 "../src/SourceList/SourceList.vala"
						_tmp12_ = current_path;
#line 1646 "../src/SourceList/SourceList.vala"
						gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, _tmp12_, GTK_TREE_VIEW_DROP_BEFORE);
#line 8135 "SourceList.c"
					} else {
#line 1648 "../src/SourceList/SourceList.vala"
						gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, NULL, GTK_TREE_VIEW_DROP_AFTER);
#line 8139 "SourceList.c"
					}
				}
			} else {
				MailSourceListDragDest* dest = NULL;
				MailSourceListDataModel* _tmp15_;
				GtkTreePath* _tmp16_;
				MailSourceListItem* _tmp17_;
				MailSourceListDragDest* _tmp18_;
				MailSourceListDragDest* _tmp19_;
#line 1653 "../src/SourceList/SourceList.vala"
				if (current_pos == GTK_TREE_VIEW_DROP_BEFORE) {
#line 8151 "SourceList.c"
					GtkTreePath* _tmp13_;
#line 1654 "../src/SourceList/SourceList.vala"
					_tmp13_ = current_path;
#line 1654 "../src/SourceList/SourceList.vala"
					gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, _tmp13_, GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
#line 8157 "SourceList.c"
				} else {
#line 1655 "../src/SourceList/SourceList.vala"
					if (current_pos == GTK_TREE_VIEW_DROP_AFTER) {
#line 8161 "SourceList.c"
						GtkTreePath* _tmp14_;
#line 1656 "../src/SourceList/SourceList.vala"
						_tmp14_ = current_path;
#line 1656 "../src/SourceList/SourceList.vala"
						gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, _tmp14_, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
#line 8167 "SourceList.c"
					}
				}
#line 1659 "../src/SourceList/SourceList.vala"
				_tmp15_ = self->priv->_data_model;
#line 1659 "../src/SourceList/SourceList.vala"
				_tmp16_ = current_path;
#line 1659 "../src/SourceList/SourceList.vala"
				_tmp17_ = mail_source_list_data_model_get_item_from_path (_tmp15_, _tmp16_);
#line 1659 "../src/SourceList/SourceList.vala"
				_tmp18_ = MAIL_IS_SOURCE_LIST_DRAG_DEST (_tmp17_) ? ((MailSourceListDragDest*) _tmp17_) : NULL;
#line 1659 "../src/SourceList/SourceList.vala"
				if (_tmp18_ == NULL) {
#line 1659 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (_tmp17_);
#line 8182 "SourceList.c"
				}
#line 1659 "../src/SourceList/SourceList.vala"
				dest = _tmp18_;
#line 1661 "../src/SourceList/SourceList.vala"
				_tmp19_ = dest;
#line 1661 "../src/SourceList/SourceList.vala"
				if (_tmp19_ != NULL) {
#line 8190 "SourceList.c"
					GtkTargetList* target_list = NULL;
					GtkTargetList* _tmp20_;
					GtkTargetList* _tmp21_;
					GdkAtom target = (GdkAtom) 0U;
					GtkTargetList* _tmp22_;
#line 1662 "../src/SourceList/SourceList.vala"
					_tmp20_ = gtk_drag_dest_get_target_list ((GtkWidget*) self);
#line 1662 "../src/SourceList/SourceList.vala"
					_tmp21_ = _gtk_target_list_ref0 (_tmp20_);
#line 1662 "../src/SourceList/SourceList.vala"
					target_list = _tmp21_;
#line 1663 "../src/SourceList/SourceList.vala"
					_tmp22_ = target_list;
#line 1663 "../src/SourceList/SourceList.vala"
					target = gtk_drag_dest_find_target ((GtkWidget*) self, context, _tmp22_);
#line 1667 "../src/SourceList/SourceList.vala"
					g_object_set_data_full ((GObject*) context, "suggested-dnd-action", (gpointer) ((gintptr) gdk_drag_context_get_suggested_action (context)), NULL);
#line 1668 "../src/SourceList/SourceList.vala"
					gtk_drag_get_data ((GtkWidget*) self, context, target, (guint32) time);
#line 1661 "../src/SourceList/SourceList.vala"
					_gtk_target_list_unref0 (target_list);
#line 8212 "SourceList.c"
				} else {
#line 1671 "../src/SourceList/SourceList.vala"
					gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, NULL, GTK_TREE_VIEW_DROP_BEFORE);
#line 8216 "SourceList.c"
				}
#line 1641 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (dest);
#line 8220 "SourceList.c"
			}
#line 1634 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (source_widget);
#line 8224 "SourceList.c"
		}
	} else {
#line 1677 "../src/SourceList/SourceList.vala"
		gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, NULL, GTK_TREE_VIEW_DROP_AFTER);
#line 1678 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1678 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (current_path);
#line 1678 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (suggested_path);
#line 1678 "../src/SourceList/SourceList.vala"
		return result;
#line 8237 "SourceList.c"
	}
#line 1681 "../src/SourceList/SourceList.vala"
	result = TRUE;
#line 1681 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (current_path);
#line 1681 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (suggested_path);
#line 1681 "../src/SourceList/SourceList.vala"
	return result;
#line 8247 "SourceList.c"
}

static void
mail_source_list_tree_real_drag_data_received (GtkWidget* base,
                                               GdkDragContext* context,
                                               gint x,
                                               gint y,
                                               GtkSelectionData* selection_data,
                                               guint info,
                                               guint time)
{
	MailSourceListTree * self;
	GtkTargetList* target_list = NULL;
	GtkTargetList* _tmp0_;
	GtkTargetList* _tmp1_;
	GdkAtom target = (GdkAtom) 0U;
	GtkTargetList* _tmp2_;
	GtkTreePath* path = NULL;
	GtkTreeViewDropPosition pos = 0;
	gconstpointer _tmp3_;
#line 1684 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 1684 "../src/SourceList/SourceList.vala"
	g_return_if_fail (context != NULL);
#line 1684 "../src/SourceList/SourceList.vala"
	g_return_if_fail (selection_data != NULL);
#line 1692 "../src/SourceList/SourceList.vala"
	_tmp0_ = gtk_drag_dest_get_target_list ((GtkWidget*) self);
#line 1692 "../src/SourceList/SourceList.vala"
	_tmp1_ = _gtk_target_list_ref0 (_tmp0_);
#line 1692 "../src/SourceList/SourceList.vala"
	target_list = _tmp1_;
#line 1693 "../src/SourceList/SourceList.vala"
	_tmp2_ = target_list;
#line 1693 "../src/SourceList/SourceList.vala"
	target = gtk_drag_dest_find_target ((GtkWidget*) self, context, _tmp2_);
#line 1695 "../src/SourceList/SourceList.vala"
	if (target == gdk_atom_intern_static_string ("GTK_TREE_MODEL_ROW")) {
#line 1696 "../src/SourceList/SourceList.vala"
		GTK_WIDGET_CLASS (mail_source_list_tree_parent_class)->drag_data_received ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), context, x, y, selection_data, info, time);
#line 1697 "../src/SourceList/SourceList.vala"
		_gtk_target_list_unref0 (target_list);
#line 1697 "../src/SourceList/SourceList.vala"
		return;
#line 8292 "SourceList.c"
	}
#line 1703 "../src/SourceList/SourceList.vala"
	_tmp3_ = g_object_get_data ((GObject*) context, "suggested-dnd-action");
#line 1703 "../src/SourceList/SourceList.vala"
	if (((gint) ((gintptr) _tmp3_)) != 0) {
#line 8298 "SourceList.c"
		GtkTreePath* _tmp4_ = NULL;
		GtkTreeViewDropPosition _tmp5_ = 0;
		GtkTreePath* _tmp6_;
#line 1704 "../src/SourceList/SourceList.vala"
		g_object_set_data_full ((GObject*) context, "suggested-dnd-action", (gpointer) ((gintptr) 0), NULL);
#line 1706 "../src/SourceList/SourceList.vala"
		gtk_tree_view_get_drag_dest_row ((GtkTreeView*) self, &_tmp4_, &_tmp5_);
#line 1706 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 1706 "../src/SourceList/SourceList.vala"
		path = _tmp4_;
#line 1706 "../src/SourceList/SourceList.vala"
		pos = _tmp5_;
#line 1708 "../src/SourceList/SourceList.vala"
		_tmp6_ = path;
#line 1708 "../src/SourceList/SourceList.vala"
		if (_tmp6_ != NULL) {
#line 8316 "SourceList.c"
			MailSourceListDragDest* dest = NULL;
			MailSourceListDataModel* _tmp7_;
			GtkTreePath* _tmp8_;
			MailSourceListItem* _tmp9_;
			MailSourceListDragDest* _tmp10_;
			gboolean _tmp11_ = FALSE;
			MailSourceListDragDest* _tmp12_;
#line 1710 "../src/SourceList/SourceList.vala"
			_tmp7_ = self->priv->_data_model;
#line 1710 "../src/SourceList/SourceList.vala"
			_tmp8_ = path;
#line 1710 "../src/SourceList/SourceList.vala"
			_tmp9_ = mail_source_list_data_model_get_item_from_path (_tmp7_, _tmp8_);
#line 1710 "../src/SourceList/SourceList.vala"
			_tmp10_ = MAIL_IS_SOURCE_LIST_DRAG_DEST (_tmp9_) ? ((MailSourceListDragDest*) _tmp9_) : NULL;
#line 1710 "../src/SourceList/SourceList.vala"
			if (_tmp10_ == NULL) {
#line 1710 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp9_);
#line 8336 "SourceList.c"
			}
#line 1710 "../src/SourceList/SourceList.vala"
			dest = _tmp10_;
#line 1712 "../src/SourceList/SourceList.vala"
			_tmp12_ = dest;
#line 1712 "../src/SourceList/SourceList.vala"
			if (_tmp12_ == NULL) {
#line 1712 "../src/SourceList/SourceList.vala"
				_tmp11_ = TRUE;
#line 8346 "SourceList.c"
			} else {
				MailSourceListDragDest* _tmp13_;
#line 1712 "../src/SourceList/SourceList.vala"
				_tmp13_ = dest;
#line 1712 "../src/SourceList/SourceList.vala"
				_tmp11_ = !mail_source_list_drag_dest_data_drop_possible (_tmp13_, context, selection_data);
#line 8353 "SourceList.c"
			}
#line 1712 "../src/SourceList/SourceList.vala"
			if (_tmp11_) {
#line 1715 "../src/SourceList/SourceList.vala"
				gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, NULL, GTK_TREE_VIEW_DROP_BEFORE);
#line 1716 "../src/SourceList/SourceList.vala"
				gdk_drag_status (context, 0, (guint32) time);
#line 1717 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (dest);
#line 1717 "../src/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (path);
#line 1717 "../src/SourceList/SourceList.vala"
				_gtk_target_list_unref0 (target_list);
#line 1717 "../src/SourceList/SourceList.vala"
				return;
#line 8369 "SourceList.c"
			}
#line 1708 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (dest);
#line 8373 "SourceList.c"
		}
#line 1721 "../src/SourceList/SourceList.vala"
		gdk_drag_status (context, gdk_drag_context_get_suggested_action (context), (guint32) time);
#line 8377 "SourceList.c"
	} else {
		GtkTreePath* _tmp14_ = NULL;
		GtkTreeViewDropPosition _tmp15_ = 0;
		gboolean _tmp16_;
#line 1723 "../src/SourceList/SourceList.vala"
		_tmp16_ = gtk_tree_view_get_dest_row_at_pos ((GtkTreeView*) self, x, y, &_tmp14_, &_tmp15_);
#line 1723 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 1723 "../src/SourceList/SourceList.vala"
		path = _tmp14_;
#line 1723 "../src/SourceList/SourceList.vala"
		pos = _tmp15_;
#line 1723 "../src/SourceList/SourceList.vala"
		if (_tmp16_) {
#line 8392 "SourceList.c"
			MailSourceListDragDest* drag_dest = NULL;
			MailSourceListDataModel* _tmp17_;
			GtkTreePath* _tmp18_;
			MailSourceListItem* _tmp19_;
			MailSourceListDragDest* _tmp20_;
			MailSourceListDragDest* _tmp21_;
#line 1729 "../src/SourceList/SourceList.vala"
			_tmp17_ = self->priv->_data_model;
#line 1729 "../src/SourceList/SourceList.vala"
			_tmp18_ = path;
#line 1729 "../src/SourceList/SourceList.vala"
			_tmp19_ = mail_source_list_data_model_get_item_from_path (_tmp17_, _tmp18_);
#line 1729 "../src/SourceList/SourceList.vala"
			_tmp20_ = MAIL_IS_SOURCE_LIST_DRAG_DEST (_tmp19_) ? ((MailSourceListDragDest*) _tmp19_) : NULL;
#line 1729 "../src/SourceList/SourceList.vala"
			if (_tmp20_ == NULL) {
#line 1729 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp19_);
#line 8411 "SourceList.c"
			}
#line 1729 "../src/SourceList/SourceList.vala"
			drag_dest = _tmp20_;
#line 1731 "../src/SourceList/SourceList.vala"
			_tmp21_ = drag_dest;
#line 1731 "../src/SourceList/SourceList.vala"
			if (_tmp21_ != NULL) {
#line 8419 "SourceList.c"
				GdkDragAction action = 0U;
				MailSourceListDragDest* _tmp22_;
#line 1732 "../src/SourceList/SourceList.vala"
				_tmp22_ = drag_dest;
#line 1732 "../src/SourceList/SourceList.vala"
				action = mail_source_list_drag_dest_data_received (_tmp22_, context, selection_data);
#line 1733 "../src/SourceList/SourceList.vala"
				gtk_drag_finish (context, action != 0, action == GDK_ACTION_MOVE, (guint32) time);
#line 1734 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (drag_dest);
#line 1734 "../src/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (path);
#line 1734 "../src/SourceList/SourceList.vala"
				_gtk_target_list_unref0 (target_list);
#line 1734 "../src/SourceList/SourceList.vala"
				return;
#line 8436 "SourceList.c"
			}
#line 1723 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (drag_dest);
#line 8440 "SourceList.c"
		}
#line 1739 "../src/SourceList/SourceList.vala"
		gtk_drag_finish (context, FALSE, FALSE, (guint32) time);
#line 8444 "SourceList.c"
	}
#line 1684 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 1684 "../src/SourceList/SourceList.vala"
	_gtk_target_list_unref0 (target_list);
#line 8450 "SourceList.c"
}

static void
mail_source_list_tree_configure_drag_source (MailSourceListTree* self,
                                             GtkTargetEntry* src_entries,
                                             gint src_entries_length1)
{
	GtkTargetEntry* entries = NULL;
	gint _tmp0_ = 0;
	GtkTargetEntry* _tmp1_;
	gint entries_length1;
	gint _entries_size_;
#line 1743 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1745 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_tree_append_row_target_entry (src_entries, (gint) src_entries_length1, &_tmp0_);
#line 1745 "../src/SourceList/SourceList.vala"
	entries = _tmp1_;
#line 1745 "../src/SourceList/SourceList.vala"
	entries_length1 = _tmp0_;
#line 1745 "../src/SourceList/SourceList.vala"
	_entries_size_ = entries_length1;
#line 1747 "../src/SourceList/SourceList.vala"
	gtk_tree_view_unset_rows_drag_source ((GtkTreeView*) self);
#line 1748 "../src/SourceList/SourceList.vala"
	gtk_tree_view_enable_model_drag_source ((GtkTreeView*) self, GDK_BUTTON1_MASK, entries, (gint) entries_length1, GDK_ACTION_MOVE);
#line 1743 "../src/SourceList/SourceList.vala"
	entries = (g_free (entries), NULL);
#line 8479 "SourceList.c"
}

static void
mail_source_list_tree_configure_drag_dest (MailSourceListTree* self,
                                           GtkTargetEntry* dest_entries,
                                           gint dest_entries_length1,
                                           GdkDragAction actions)
{
	GtkTargetEntry* entries = NULL;
	gint _tmp0_ = 0;
	GtkTargetEntry* _tmp1_;
	gint entries_length1;
	gint _entries_size_;
#line 1751 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1753 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_tree_append_row_target_entry (dest_entries, (gint) dest_entries_length1, &_tmp0_);
#line 1753 "../src/SourceList/SourceList.vala"
	entries = _tmp1_;
#line 1753 "../src/SourceList/SourceList.vala"
	entries_length1 = _tmp0_;
#line 1753 "../src/SourceList/SourceList.vala"
	_entries_size_ = entries_length1;
#line 1755 "../src/SourceList/SourceList.vala"
	gtk_tree_view_unset_rows_drag_dest ((GtkTreeView*) self);
#line 1758 "../src/SourceList/SourceList.vala"
	gtk_tree_view_enable_model_drag_dest ((GtkTreeView*) self, entries, (gint) entries_length1, GDK_ACTION_MOVE | actions);
#line 1751 "../src/SourceList/SourceList.vala"
	entries = (g_free (entries), NULL);
#line 8509 "SourceList.c"
}

static gboolean
mail_source_list_tree_on_query_tooltip (MailSourceListTree* self,
                                        gint x,
                                        gint y,
                                        gboolean keyboard_tooltip,
                                        GtkTooltip* tooltip)
{
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* column = NULL;
	GtkTreeViewColumn* _tmp0_;
	GtkTreeViewColumn* _tmp1_;
	GtkTreePath* _tmp2_ = NULL;
	GtkTreePath* _tmp3_;
	MailSourceListItem* item = NULL;
	MailSourceListDataModel* _tmp4_;
	GtkTreePath* _tmp5_;
	MailSourceListItem* _tmp6_;
	MailSourceListItem* _tmp7_;
	gboolean result;
#line 1761 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1761 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (tooltip != NULL, FALSE);
#line 1763 "../src/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) MAIL_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 1763 "../src/SourceList/SourceList.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 1763 "../src/SourceList/SourceList.vala"
	column = _tmp1_;
#line 1765 "../src/SourceList/SourceList.vala"
	gtk_tree_view_get_tooltip_context ((GtkTreeView*) self, &x, &y, keyboard_tooltip, NULL, &_tmp2_, NULL);
#line 1765 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 1765 "../src/SourceList/SourceList.vala"
	path = _tmp2_;
#line 1766 "../src/SourceList/SourceList.vala"
	_tmp3_ = path;
#line 1766 "../src/SourceList/SourceList.vala"
	if (_tmp3_ == NULL) {
#line 1767 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1767 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (column);
#line 1767 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 1767 "../src/SourceList/SourceList.vala"
		return result;
#line 8559 "SourceList.c"
	}
#line 1770 "../src/SourceList/SourceList.vala"
	_tmp4_ = self->priv->_data_model;
#line 1770 "../src/SourceList/SourceList.vala"
	_tmp5_ = path;
#line 1770 "../src/SourceList/SourceList.vala"
	_tmp6_ = mail_source_list_data_model_get_item_from_path (_tmp4_, _tmp5_);
#line 1770 "../src/SourceList/SourceList.vala"
	item = _tmp6_;
#line 1771 "../src/SourceList/SourceList.vala"
	_tmp7_ = item;
#line 1771 "../src/SourceList/SourceList.vala"
	if (_tmp7_ != NULL) {
#line 8573 "SourceList.c"
		gboolean should_show = FALSE;
		GdkRectangle start_cell_area = {0};
		GtkTreePath* _tmp8_;
		GtkTreeViewColumn* _tmp9_;
		GdkRectangle _tmp10_ = {0};
		GtkTreePath* _tmp11_;
		MailSourceListItem* _tmp12_;
		const gchar* _tmp13_;
		const gchar* _tmp14_;
		gboolean _tmp24_ = FALSE;
		GtkTreeViewColumn* _tmp25_;
		GtkTreePath* _tmp26_;
		GtkCellRendererText* _tmp27_;
		GdkRectangle _tmp28_;
#line 1772 "../src/SourceList/SourceList.vala"
		should_show = FALSE;
#line 1775 "../src/SourceList/SourceList.vala"
		_tmp8_ = path;
#line 1775 "../src/SourceList/SourceList.vala"
		_tmp9_ = column;
#line 1775 "../src/SourceList/SourceList.vala"
		gtk_tree_view_get_cell_area ((GtkTreeView*) self, _tmp8_, _tmp9_, &_tmp10_);
#line 1775 "../src/SourceList/SourceList.vala"
		start_cell_area = _tmp10_;
#line 1777 "../src/SourceList/SourceList.vala"
		_tmp11_ = path;
#line 1777 "../src/SourceList/SourceList.vala"
		gtk_tree_view_set_tooltip_row ((GtkTreeView*) self, tooltip, _tmp11_);
#line 1779 "../src/SourceList/SourceList.vala"
		_tmp12_ = item;
#line 1779 "../src/SourceList/SourceList.vala"
		_tmp13_ = mail_source_list_item_get_tooltip (_tmp12_);
#line 1779 "../src/SourceList/SourceList.vala"
		_tmp14_ = _tmp13_;
#line 1779 "../src/SourceList/SourceList.vala"
		if (_tmp14_ == NULL) {
#line 8610 "SourceList.c"
			MailSourceListItem* _tmp15_;
			const gchar* _tmp16_;
			const gchar* _tmp17_;
#line 1780 "../src/SourceList/SourceList.vala"
			_tmp15_ = item;
#line 1780 "../src/SourceList/SourceList.vala"
			_tmp16_ = mail_source_list_item_get_name (_tmp15_);
#line 1780 "../src/SourceList/SourceList.vala"
			_tmp17_ = _tmp16_;
#line 1780 "../src/SourceList/SourceList.vala"
			gtk_tooltip_set_markup (tooltip, _tmp17_);
#line 1781 "../src/SourceList/SourceList.vala"
			should_show = TRUE;
#line 8624 "SourceList.c"
		} else {
			MailSourceListItem* _tmp18_;
			const gchar* _tmp19_;
			const gchar* _tmp20_;
#line 1782 "../src/SourceList/SourceList.vala"
			_tmp18_ = item;
#line 1782 "../src/SourceList/SourceList.vala"
			_tmp19_ = mail_source_list_item_get_tooltip (_tmp18_);
#line 1782 "../src/SourceList/SourceList.vala"
			_tmp20_ = _tmp19_;
#line 1782 "../src/SourceList/SourceList.vala"
			if (g_strcmp0 (_tmp20_, "") != 0) {
#line 8637 "SourceList.c"
				MailSourceListItem* _tmp21_;
				const gchar* _tmp22_;
				const gchar* _tmp23_;
#line 1783 "../src/SourceList/SourceList.vala"
				_tmp21_ = item;
#line 1783 "../src/SourceList/SourceList.vala"
				_tmp22_ = mail_source_list_item_get_tooltip (_tmp21_);
#line 1783 "../src/SourceList/SourceList.vala"
				_tmp23_ = _tmp22_;
#line 1783 "../src/SourceList/SourceList.vala"
				gtk_tooltip_set_markup (tooltip, _tmp23_);
#line 1784 "../src/SourceList/SourceList.vala"
				should_show = TRUE;
#line 8651 "SourceList.c"
			}
		}
#line 1787 "../src/SourceList/SourceList.vala"
		if (keyboard_tooltip) {
#line 1788 "../src/SourceList/SourceList.vala"
			result = should_show;
#line 1788 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 1788 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (column);
#line 1788 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (path);
#line 1788 "../src/SourceList/SourceList.vala"
			return result;
#line 8666 "SourceList.c"
		}
#line 1791 "../src/SourceList/SourceList.vala"
		_tmp25_ = column;
#line 1791 "../src/SourceList/SourceList.vala"
		_tmp26_ = path;
#line 1791 "../src/SourceList/SourceList.vala"
		_tmp27_ = self->priv->text_cell;
#line 1791 "../src/SourceList/SourceList.vala"
		_tmp28_ = start_cell_area;
#line 1791 "../src/SourceList/SourceList.vala"
		if (mail_source_list_tree_over_cell (self, _tmp25_, _tmp26_, (GtkCellRenderer*) _tmp27_, x - _tmp28_.x)) {
#line 1791 "../src/SourceList/SourceList.vala"
			_tmp24_ = TRUE;
#line 8680 "SourceList.c"
		} else {
			GtkTreeViewColumn* _tmp29_;
			GtkTreePath* _tmp30_;
			MailSourceListCellRendererIcon* _tmp31_;
			GdkRectangle _tmp32_;
#line 1792 "../src/SourceList/SourceList.vala"
			_tmp29_ = column;
#line 1792 "../src/SourceList/SourceList.vala"
			_tmp30_ = path;
#line 1792 "../src/SourceList/SourceList.vala"
			_tmp31_ = self->priv->icon_cell;
#line 1792 "../src/SourceList/SourceList.vala"
			_tmp32_ = start_cell_area;
#line 1792 "../src/SourceList/SourceList.vala"
			_tmp24_ = mail_source_list_tree_over_cell (self, _tmp29_, _tmp30_, (GtkCellRenderer*) _tmp31_, x - _tmp32_.x);
#line 8696 "SourceList.c"
		}
#line 1791 "../src/SourceList/SourceList.vala"
		if (_tmp24_) {
#line 1794 "../src/SourceList/SourceList.vala"
			result = should_show;
#line 1794 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 1794 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (column);
#line 1794 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (path);
#line 1794 "../src/SourceList/SourceList.vala"
			return result;
#line 8710 "SourceList.c"
		} else {
			GtkTreeViewColumn* _tmp33_;
			GtkTreePath* _tmp34_;
			MailSourceListCellRendererIcon* _tmp35_;
			GdkRectangle _tmp36_;
#line 1795 "../src/SourceList/SourceList.vala"
			_tmp33_ = column;
#line 1795 "../src/SourceList/SourceList.vala"
			_tmp34_ = path;
#line 1795 "../src/SourceList/SourceList.vala"
			_tmp35_ = self->priv->activatable_cell;
#line 1795 "../src/SourceList/SourceList.vala"
			_tmp36_ = start_cell_area;
#line 1795 "../src/SourceList/SourceList.vala"
			if (mail_source_list_tree_over_cell (self, _tmp33_, _tmp34_, (GtkCellRenderer*) _tmp35_, x - _tmp36_.x)) {
#line 8726 "SourceList.c"
				MailSourceListItem* _tmp37_;
				const gchar* _tmp38_;
				const gchar* _tmp39_;
#line 1796 "../src/SourceList/SourceList.vala"
				_tmp37_ = item;
#line 1796 "../src/SourceList/SourceList.vala"
				_tmp38_ = mail_source_list_item_get_activatable_tooltip (_tmp37_);
#line 1796 "../src/SourceList/SourceList.vala"
				_tmp39_ = _tmp38_;
#line 1796 "../src/SourceList/SourceList.vala"
				if (g_strcmp0 (_tmp39_, "") == 0) {
#line 1797 "../src/SourceList/SourceList.vala"
					result = FALSE;
#line 1797 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (item);
#line 1797 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (column);
#line 1797 "../src/SourceList/SourceList.vala"
					__vala_GtkTreePath_free0 (path);
#line 1797 "../src/SourceList/SourceList.vala"
					return result;
#line 8748 "SourceList.c"
				} else {
					MailSourceListItem* _tmp40_;
					const gchar* _tmp41_;
					const gchar* _tmp42_;
#line 1799 "../src/SourceList/SourceList.vala"
					_tmp40_ = item;
#line 1799 "../src/SourceList/SourceList.vala"
					_tmp41_ = mail_source_list_item_get_activatable_tooltip (_tmp40_);
#line 1799 "../src/SourceList/SourceList.vala"
					_tmp42_ = _tmp41_;
#line 1799 "../src/SourceList/SourceList.vala"
					gtk_tooltip_set_markup (tooltip, _tmp42_);
#line 1800 "../src/SourceList/SourceList.vala"
					result = TRUE;
#line 1800 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (item);
#line 1800 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (column);
#line 1800 "../src/SourceList/SourceList.vala"
					__vala_GtkTreePath_free0 (path);
#line 1800 "../src/SourceList/SourceList.vala"
					return result;
#line 8771 "SourceList.c"
				}
			}
		}
	}
#line 1805 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 1805 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1805 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (column);
#line 1805 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 1805 "../src/SourceList/SourceList.vala"
	return result;
#line 8786 "SourceList.c"
}

static void
_vala_array_add10 (GtkTargetEntry* * array,
                   gint* length,
                   gint* size,
                   const GtkTargetEntry* value)
{
#line 1816 "../src/SourceList/SourceList.vala"
	if ((*length) == (*size)) {
#line 1816 "../src/SourceList/SourceList.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1816 "../src/SourceList/SourceList.vala"
		*array = g_renew (GtkTargetEntry, *array, *size);
#line 8801 "SourceList.c"
	}
#line 1816 "../src/SourceList/SourceList.vala"
	(*array)[(*length)++] = *value;
#line 8805 "SourceList.c"
}

static void
_vala_array_add11 (GtkTargetEntry* * array,
                   gint* length,
                   gint* size,
                   const GtkTargetEntry* value)
{
#line 1820 "../src/SourceList/SourceList.vala"
	if ((*length) == (*size)) {
#line 1820 "../src/SourceList/SourceList.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1820 "../src/SourceList/SourceList.vala"
		*array = g_renew (GtkTargetEntry, *array, *size);
#line 8820 "SourceList.c"
	}
#line 1820 "../src/SourceList/SourceList.vala"
	(*array)[(*length)++] = *value;
#line 8824 "SourceList.c"
}

static GtkTargetEntry*
mail_source_list_tree_append_row_target_entry (GtkTargetEntry* orig,
                                               gint orig_length1,
                                               gint* result_length1)
{
#line 1809 "../src/SourceList/SourceList.vala"
	static const GtkTargetEntry row_target_entry = {"GTK_TREE_MODEL_ROW", (guint) GTK_TARGET_SAME_WIDGET, (guint) 0};
#line 8834 "SourceList.c"
	GtkTargetEntry* entries = NULL;
	GtkTargetEntry* _tmp0_;
	gint entries_length1;
	gint _entries_size_;
	GtkTargetEntry* _tmp2_;
	gint _tmp2__length1;
	GtkTargetEntry* result;
#line 1815 "../src/SourceList/SourceList.vala"
	_tmp0_ = g_new0 (GtkTargetEntry, 0);
#line 1815 "../src/SourceList/SourceList.vala"
	entries = _tmp0_;
#line 1815 "../src/SourceList/SourceList.vala"
	entries_length1 = 0;
#line 1815 "../src/SourceList/SourceList.vala"
	_entries_size_ = entries_length1;
#line 1816 "../src/SourceList/SourceList.vala"
	_vala_array_add10 (&entries, &entries_length1, &_entries_size_, &row_target_entry);
#line 1818 "../src/SourceList/SourceList.vala"
	if (orig != NULL) {
#line 8854 "SourceList.c"
		{
			GtkTargetEntry* target_entry_collection = NULL;
			gint target_entry_collection_length1 = 0;
			gint _target_entry_collection_size_ = 0;
			gint target_entry_it = 0;
#line 1819 "../src/SourceList/SourceList.vala"
			target_entry_collection = orig;
#line 1819 "../src/SourceList/SourceList.vala"
			target_entry_collection_length1 = orig_length1;
#line 1819 "../src/SourceList/SourceList.vala"
			for (target_entry_it = 0; target_entry_it < target_entry_collection_length1; target_entry_it = target_entry_it + 1) {
#line 8866 "SourceList.c"
				GtkTargetEntry target_entry = {0};
#line 1819 "../src/SourceList/SourceList.vala"
				target_entry = target_entry_collection[target_entry_it];
#line 8870 "SourceList.c"
				{
					GtkTargetEntry _tmp1_;
#line 1820 "../src/SourceList/SourceList.vala"
					_tmp1_ = target_entry;
#line 1820 "../src/SourceList/SourceList.vala"
					_vala_array_add11 (&entries, &entries_length1, &_entries_size_, &_tmp1_);
#line 8877 "SourceList.c"
				}
			}
		}
	}
#line 1823 "../src/SourceList/SourceList.vala"
	_tmp2_ = entries;
#line 1823 "../src/SourceList/SourceList.vala"
	_tmp2__length1 = entries_length1;
#line 1823 "../src/SourceList/SourceList.vala"
	if (result_length1) {
#line 1823 "../src/SourceList/SourceList.vala"
		*result_length1 = _tmp2__length1;
#line 8890 "SourceList.c"
	}
#line 1823 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 1823 "../src/SourceList/SourceList.vala"
	return result;
#line 8896 "SourceList.c"
}

static void
_mail_source_list_tree_on_model_item_updated_mail_source_list_data_model_item_updated (MailSourceListDataModel* _sender,
                                                                                       MailSourceListItem* item,
                                                                                       gpointer self)
{
#line 1827 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_on_model_item_updated ((MailSourceListTree*) self, item);
#line 8906 "SourceList.c"
}

static void
mail_source_list_tree_enable_item_property_monitor (MailSourceListTree* self)
{
	MailSourceListDataModel* _tmp0_;
#line 1826 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1827 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 1827 "../src/SourceList/SourceList.vala"
	g_signal_connect_object (_tmp0_, "item-updated", (GCallback) _mail_source_list_tree_on_model_item_updated_mail_source_list_data_model_item_updated, self, G_CONNECT_AFTER);
#line 8919 "SourceList.c"
}

static void
mail_source_list_tree_disable_item_property_monitor (MailSourceListTree* self)
{
	MailSourceListDataModel* _tmp0_;
	guint _tmp1_;
#line 1830 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1831 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 1831 "../src/SourceList/SourceList.vala"
	g_signal_parse_name ("item-updated", MAIL_SOURCE_LIST_TYPE_DATA_MODEL, &_tmp1_, NULL, FALSE);
#line 1831 "../src/SourceList/SourceList.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _mail_source_list_tree_on_model_item_updated_mail_source_list_data_model_item_updated, self);
#line 8935 "SourceList.c"
}

static void
mail_source_list_tree_on_model_item_updated (MailSourceListTree* self,
                                             MailSourceListItem* item)
{
	MailSourceListExpandableItem* expandable_item = NULL;
	MailSourceListExpandableItem* _tmp0_;
	MailSourceListExpandableItem* _tmp1_;
#line 1834 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1834 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 1837 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (item) ? ((MailSourceListExpandableItem*) item) : NULL);
#line 1837 "../src/SourceList/SourceList.vala"
	expandable_item = _tmp0_;
#line 1838 "../src/SourceList/SourceList.vala"
	_tmp1_ = expandable_item;
#line 1838 "../src/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 8957 "SourceList.c"
		MailSourceListExpandableItem* _tmp2_;
#line 1839 "../src/SourceList/SourceList.vala"
		_tmp2_ = expandable_item;
#line 1839 "../src/SourceList/SourceList.vala"
		mail_source_list_tree_update_expansion (self, _tmp2_);
#line 8963 "SourceList.c"
	}
#line 1834 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (expandable_item);
#line 8967 "SourceList.c"
}

static void
_mail_source_list_tree_spacer_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                        GtkCellRenderer* cell,
                                                                        GtkTreeModel* tree_model,
                                                                        GtkTreeIter* iter,
                                                                        gpointer self)
{
#line 1884 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_spacer_cell_data_func (cell_layout, cell, tree_model, iter);
#line 8979 "SourceList.c"
}

static void
mail_source_list_tree_add_spacer_cell_for_level (MailSourceListTree* self,
                                                 gint level,
                                                 gboolean check_previous)
{
	GeeHashMap* _tmp0_;
	GeeHashMap* _tmp2_;
#line 1842 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1842 "../src/SourceList/SourceList.vala"
	_vala_return_if_fail (level > 0, "level > 0");
#line 1846 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->spacer_cells;
#line 1846 "../src/SourceList/SourceList.vala"
	if (_tmp0_ == NULL) {
#line 8997 "SourceList.c"
		GeeHashMap* _tmp1_;
#line 1847 "../src/SourceList/SourceList.vala"
		_tmp1_ = gee_hash_map_new (G_TYPE_INT, NULL, NULL, MAIL_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
#line 1847 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->spacer_cells);
#line 1847 "../src/SourceList/SourceList.vala"
		self->priv->spacer_cells = _tmp1_;
#line 9005 "SourceList.c"
	}
#line 1849 "../src/SourceList/SourceList.vala"
	_tmp2_ = self->priv->spacer_cells;
#line 1849 "../src/SourceList/SourceList.vala"
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp2_, (gpointer) ((gintptr) level))) {
#line 9011 "SourceList.c"
		MailSourceListCellRendererSpacer* spacer_cell = NULL;
		MailSourceListCellRendererSpacer* _tmp3_;
		MailSourceListCellRendererSpacer* _tmp4_;
		GeeHashMap* _tmp5_;
		MailSourceListCellRendererSpacer* _tmp6_;
		guint cell_xpadding = 0U;
		MailSourceListCellRendererSpacer* _tmp9_;
		GtkTreeViewColumn* item_column = NULL;
		GtkTreeViewColumn* _tmp10_;
		GtkTreeViewColumn* _tmp11_;
		GtkTreeViewColumn* _tmp12_;
		MailSourceListCellRendererSpacer* _tmp13_;
		GtkTreeViewColumn* _tmp14_;
		MailSourceListCellRendererSpacer* _tmp15_;
#line 1850 "../src/SourceList/SourceList.vala"
		_tmp3_ = mail_source_list_cell_renderer_spacer_new ();
#line 1850 "../src/SourceList/SourceList.vala"
		g_object_ref_sink (_tmp3_);
#line 1850 "../src/SourceList/SourceList.vala"
		spacer_cell = _tmp3_;
#line 1851 "../src/SourceList/SourceList.vala"
		_tmp4_ = spacer_cell;
#line 1851 "../src/SourceList/SourceList.vala"
		mail_source_list_cell_renderer_spacer_set_level (_tmp4_, level);
#line 1852 "../src/SourceList/SourceList.vala"
		_tmp5_ = self->priv->spacer_cells;
#line 1852 "../src/SourceList/SourceList.vala"
		_tmp6_ = spacer_cell;
#line 1852 "../src/SourceList/SourceList.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp5_, (gpointer) ((gintptr) level), _tmp6_);
#line 1862 "../src/SourceList/SourceList.vala"
		switch (level) {
#line 1862 "../src/SourceList/SourceList.vala"
			case 1:
#line 9046 "SourceList.c"
			{
				gint left_padding = 0;
				gint _tmp7_ = 0;
#line 1865 "../src/SourceList/SourceList.vala"
				gtk_widget_style_get ((GtkWidget*) self, MAIL_SOURCE_LIST_TREE_STYLE_PROP_LEFT_PADDING, &_tmp7_, NULL);
#line 1865 "../src/SourceList/SourceList.vala"
				left_padding = _tmp7_;
#line 1866 "../src/SourceList/SourceList.vala"
				cell_xpadding = (guint) (left_padding / 2);
#line 1867 "../src/SourceList/SourceList.vala"
				break;
#line 9058 "SourceList.c"
			}
#line 1862 "../src/SourceList/SourceList.vala"
			case 2:
#line 9062 "SourceList.c"
			{
#line 1870 "../src/SourceList/SourceList.vala"
				cell_xpadding = (guint) 0;
#line 1871 "../src/SourceList/SourceList.vala"
				break;
#line 9068 "SourceList.c"
			}
			default:
			{
				gint level_indentation = 0;
				gint _tmp8_ = 0;
#line 1875 "../src/SourceList/SourceList.vala"
				gtk_widget_style_get ((GtkWidget*) self, MAIL_SOURCE_LIST_TREE_STYLE_PROP_LEVEL_INDENTATION, &_tmp8_, NULL);
#line 1875 "../src/SourceList/SourceList.vala"
				level_indentation = _tmp8_;
#line 1876 "../src/SourceList/SourceList.vala"
				cell_xpadding = (guint) (level_indentation / 2);
#line 1877 "../src/SourceList/SourceList.vala"
				break;
#line 9082 "SourceList.c"
			}
		}
#line 1880 "../src/SourceList/SourceList.vala"
		_tmp9_ = spacer_cell;
#line 1880 "../src/SourceList/SourceList.vala"
		g_object_set ((GtkCellRenderer*) _tmp9_, "xpad", cell_xpadding, NULL);
#line 1882 "../src/SourceList/SourceList.vala"
		_tmp10_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) MAIL_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 1882 "../src/SourceList/SourceList.vala"
		_tmp11_ = _g_object_ref0 (_tmp10_);
#line 1882 "../src/SourceList/SourceList.vala"
		item_column = _tmp11_;
#line 1883 "../src/SourceList/SourceList.vala"
		_tmp12_ = item_column;
#line 1883 "../src/SourceList/SourceList.vala"
		_tmp13_ = spacer_cell;
#line 1883 "../src/SourceList/SourceList.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp12_, (GtkCellRenderer*) _tmp13_, FALSE);
#line 1884 "../src/SourceList/SourceList.vala"
		_tmp14_ = item_column;
#line 1884 "../src/SourceList/SourceList.vala"
		_tmp15_ = spacer_cell;
#line 1884 "../src/SourceList/SourceList.vala"
		gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp14_, (GtkCellRenderer*) _tmp15_, _mail_source_list_tree_spacer_cell_data_func_gtk_cell_layout_data_func, NULL, NULL);
#line 1887 "../src/SourceList/SourceList.vala"
		if (check_previous) {
#line 9109 "SourceList.c"
			{
				gint i = 0;
#line 1888 "../src/SourceList/SourceList.vala"
				i = level - 1;
#line 9114 "SourceList.c"
				{
					gboolean _tmp16_ = FALSE;
#line 1888 "../src/SourceList/SourceList.vala"
					_tmp16_ = TRUE;
#line 1888 "../src/SourceList/SourceList.vala"
					while (TRUE) {
#line 1888 "../src/SourceList/SourceList.vala"
						if (!_tmp16_) {
#line 9123 "SourceList.c"
							gint _tmp17_;
#line 1888 "../src/SourceList/SourceList.vala"
							_tmp17_ = i;
#line 1888 "../src/SourceList/SourceList.vala"
							i = _tmp17_ - 1;
#line 9129 "SourceList.c"
						}
#line 1888 "../src/SourceList/SourceList.vala"
						_tmp16_ = FALSE;
#line 1888 "../src/SourceList/SourceList.vala"
						if (!(i > 0)) {
#line 1888 "../src/SourceList/SourceList.vala"
							break;
#line 9137 "SourceList.c"
						}
#line 1889 "../src/SourceList/SourceList.vala"
						mail_source_list_tree_add_spacer_cell_for_level (self, i, FALSE);
#line 9141 "SourceList.c"
					}
				}
			}
		}
#line 1849 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (item_column);
#line 1849 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (spacer_cell);
#line 9150 "SourceList.c"
	}
}

/**
         * Evaluates whether the item at the specified path can be selected or not.
         */
static gboolean
mail_source_list_tree_select_func (MailSourceListTree* self,
                                   GtkTreeSelection* selection,
                                   GtkTreeModel* model,
                                   GtkTreePath* path,
                                   gboolean path_currently_selected)
{
	gboolean selectable = FALSE;
	MailSourceListItem* item = NULL;
	MailSourceListDataModel* _tmp0_;
	MailSourceListItem* _tmp1_;
	MailSourceListItem* _tmp2_;
	gboolean result;
#line 1897 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1897 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (selection != NULL, FALSE);
#line 1897 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (model != NULL, FALSE);
#line 1897 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1903 "../src/SourceList/SourceList.vala"
	selectable = FALSE;
#line 1904 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 1904 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_data_model_get_item_from_path (_tmp0_, path);
#line 1904 "../src/SourceList/SourceList.vala"
	item = _tmp1_;
#line 1906 "../src/SourceList/SourceList.vala"
	_tmp2_ = item;
#line 1906 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 9190 "SourceList.c"
		MailSourceListDataModel* _tmp3_;
		MailSourceListItem* _tmp4_;
#line 1908 "../src/SourceList/SourceList.vala"
		_tmp3_ = self->priv->_data_model;
#line 1908 "../src/SourceList/SourceList.vala"
		_tmp4_ = item;
#line 1908 "../src/SourceList/SourceList.vala"
		if (!mail_source_list_data_model_is_category (_tmp3_, _tmp4_, NULL, path)) {
#line 9199 "SourceList.c"
			MailSourceListItem* _tmp5_;
			gboolean _tmp6_;
			gboolean _tmp7_;
#line 1909 "../src/SourceList/SourceList.vala"
			_tmp5_ = item;
#line 1909 "../src/SourceList/SourceList.vala"
			_tmp6_ = mail_source_list_item_get_selectable (_tmp5_);
#line 1909 "../src/SourceList/SourceList.vala"
			_tmp7_ = _tmp6_;
#line 1909 "../src/SourceList/SourceList.vala"
			selectable = _tmp7_;
#line 9211 "SourceList.c"
		}
	}
#line 1912 "../src/SourceList/SourceList.vala"
	result = selectable;
#line 1912 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1912 "../src/SourceList/SourceList.vala"
	return result;
#line 9220 "SourceList.c"
}

static void
__vala_GtkTreePath_free0_ (gpointer var)
{
#line 1919 "../src/SourceList/SourceList.vala"
	(var == NULL) ? NULL : (var = (_vala_GtkTreePath_free (var), NULL));
#line 9228 "SourceList.c"
}

static inline void
_g_list_free___vala_GtkTreePath_free0_ (GList* self)
{
#line 1919 "../src/SourceList/SourceList.vala"
	g_list_free_full (self, (GDestroyNotify) __vala_GtkTreePath_free0_);
#line 9236 "SourceList.c"
}

static GtkTreePath*
mail_source_list_tree_get_selected_path (MailSourceListTree* self)
{
	GtkTreePath* selected_path = NULL;
	GtkTreeSelection* selection = NULL;
	GtkTreeSelection* _tmp0_;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp2_;
	GtkTreePath* result;
#line 1915 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1916 "../src/SourceList/SourceList.vala"
	selected_path = NULL;
#line 1917 "../src/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_view_get_selection ((GtkTreeView*) self);
#line 1917 "../src/SourceList/SourceList.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 1917 "../src/SourceList/SourceList.vala"
	selection = _tmp1_;
#line 1919 "../src/SourceList/SourceList.vala"
	_tmp2_ = selection;
#line 1919 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 9262 "SourceList.c"
		GtkTreeModel* model = NULL;
		GList* selected_rows = NULL;
		GtkTreeSelection* _tmp3_;
		GtkTreeModel* _tmp4_ = NULL;
		GList* _tmp5_;
		GtkTreeModel* _tmp6_;
		GList* _tmp7_;
#line 1921 "../src/SourceList/SourceList.vala"
		_tmp3_ = selection;
#line 1921 "../src/SourceList/SourceList.vala"
		_tmp5_ = gtk_tree_selection_get_selected_rows (_tmp3_, &_tmp4_);
#line 1921 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1921 "../src/SourceList/SourceList.vala"
		_tmp6_ = _g_object_ref0 (_tmp4_);
#line 1921 "../src/SourceList/SourceList.vala"
		model = _tmp6_;
#line 1921 "../src/SourceList/SourceList.vala"
		selected_rows = _tmp5_;
#line 1922 "../src/SourceList/SourceList.vala"
		_tmp7_ = selected_rows;
#line 1922 "../src/SourceList/SourceList.vala"
		if (g_list_length (_tmp7_) == ((guint) 1)) {
#line 9286 "SourceList.c"
			GList* _tmp8_;
			gconstpointer _tmp9_;
			GtkTreePath* _tmp10_;
#line 1923 "../src/SourceList/SourceList.vala"
			_tmp8_ = selected_rows;
#line 1923 "../src/SourceList/SourceList.vala"
			_tmp9_ = g_list_nth_data (_tmp8_, (guint) 0);
#line 1923 "../src/SourceList/SourceList.vala"
			_tmp10_ = __vala_GtkTreePath_copy0 ((GtkTreePath*) _tmp9_);
#line 1923 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (selected_path);
#line 1923 "../src/SourceList/SourceList.vala"
			selected_path = _tmp10_;
#line 9300 "SourceList.c"
		}
#line 1919 "../src/SourceList/SourceList.vala"
		(selected_rows == NULL) ? NULL : (selected_rows = (_g_list_free___vala_GtkTreePath_free0_ (selected_rows), NULL));
#line 1919 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 9306 "SourceList.c"
	}
#line 1926 "../src/SourceList/SourceList.vala"
	result = selected_path;
#line 1926 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (selection);
#line 1926 "../src/SourceList/SourceList.vala"
	return result;
#line 9314 "SourceList.c"
}

static void
mail_source_list_tree_set_selected (MailSourceListTree* self,
                                    MailSourceListItem* item,
                                    gboolean scroll_to_item)
{
#line 1929 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1930 "../src/SourceList/SourceList.vala"
	if (item == NULL) {
#line 9326 "SourceList.c"
		GtkTreeSelection* selection = NULL;
		GtkTreeSelection* _tmp0_;
		GtkTreeSelection* _tmp1_;
		GtkTreeSelection* _tmp2_;
#line 1931 "../src/SourceList/SourceList.vala"
		_tmp0_ = gtk_tree_view_get_selection ((GtkTreeView*) self);
#line 1931 "../src/SourceList/SourceList.vala"
		_tmp1_ = _g_object_ref0 (_tmp0_);
#line 1931 "../src/SourceList/SourceList.vala"
		selection = _tmp1_;
#line 1932 "../src/SourceList/SourceList.vala"
		_tmp2_ = selection;
#line 1932 "../src/SourceList/SourceList.vala"
		if (_tmp2_ != NULL) {
#line 9341 "SourceList.c"
			GtkTreeSelection* _tmp3_;
#line 1933 "../src/SourceList/SourceList.vala"
			_tmp3_ = selection;
#line 1933 "../src/SourceList/SourceList.vala"
			gtk_tree_selection_unselect_all (_tmp3_);
#line 9347 "SourceList.c"
		}
#line 1944 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->selected);
#line 1944 "../src/SourceList/SourceList.vala"
		self->priv->selected = NULL;
#line 1945 "../src/SourceList/SourceList.vala"
		g_signal_emit (self, mail_source_list_tree_signals[MAIL_SOURCE_LIST_TREE_ITEM_SELECTED_SIGNAL], 0, NULL);
#line 1930 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (selection);
#line 9357 "SourceList.c"
	} else {
		gboolean _tmp4_;
		gboolean _tmp5_;
#line 1946 "../src/SourceList/SourceList.vala"
		_tmp4_ = mail_source_list_item_get_selectable (item);
#line 1946 "../src/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 1946 "../src/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 9367 "SourceList.c"
			GtkTreePath* to_select = NULL;
			MailSourceListDataModel* _tmp6_;
			GtkTreePath* _tmp7_;
			GtkTreePath* _tmp8_;
#line 1947 "../src/SourceList/SourceList.vala"
			if (scroll_to_item) {
#line 1948 "../src/SourceList/SourceList.vala"
				mail_source_list_tree_scroll_to_item (self, item, FALSE, (gfloat) 0);
#line 9376 "SourceList.c"
			}
#line 1950 "../src/SourceList/SourceList.vala"
			_tmp6_ = self->priv->_data_model;
#line 1950 "../src/SourceList/SourceList.vala"
			_tmp7_ = mail_source_list_data_model_get_item_path (_tmp6_, item);
#line 1950 "../src/SourceList/SourceList.vala"
			to_select = _tmp7_;
#line 1951 "../src/SourceList/SourceList.vala"
			_tmp8_ = to_select;
#line 1951 "../src/SourceList/SourceList.vala"
			if (_tmp8_ != NULL) {
#line 9388 "SourceList.c"
				GtkTreePath* _tmp9_;
				GtkTreeViewColumn* _tmp10_;
				GtkCellRendererText* _tmp11_;
#line 1952 "../src/SourceList/SourceList.vala"
				_tmp9_ = to_select;
#line 1952 "../src/SourceList/SourceList.vala"
				_tmp10_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) MAIL_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 1952 "../src/SourceList/SourceList.vala"
				_tmp11_ = self->priv->text_cell;
#line 1952 "../src/SourceList/SourceList.vala"
				gtk_tree_view_set_cursor_on_cell ((GtkTreeView*) self, _tmp9_, _tmp10_, (GtkCellRenderer*) _tmp11_, FALSE);
#line 9400 "SourceList.c"
			}
#line 1946 "../src/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (to_select);
#line 9404 "SourceList.c"
		}
	}
}

static void
mail_source_list_tree_real_cursor_changed (GtkTreeView* base)
{
	MailSourceListTree * self;
	GtkTreePath* path = NULL;
	GtkTreePath* _tmp0_;
	MailSourceListItem* _tmp1_ = NULL;
	GtkTreePath* _tmp2_;
	MailSourceListItem* new_item = NULL;
	MailSourceListItem* _tmp6_;
	gboolean _tmp7_ = FALSE;
	MailSourceListItem* _tmp8_;
#line 1956 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 1957 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_tree_get_selected_path (self);
#line 1957 "../src/SourceList/SourceList.vala"
	path = _tmp0_;
#line 1958 "../src/SourceList/SourceList.vala"
	_tmp2_ = path;
#line 1958 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 9431 "SourceList.c"
		MailSourceListDataModel* _tmp3_;
		GtkTreePath* _tmp4_;
		MailSourceListItem* _tmp5_;
#line 1958 "../src/SourceList/SourceList.vala"
		_tmp3_ = self->priv->_data_model;
#line 1958 "../src/SourceList/SourceList.vala"
		_tmp4_ = path;
#line 1958 "../src/SourceList/SourceList.vala"
		_tmp5_ = mail_source_list_data_model_get_item_from_path (_tmp3_, _tmp4_);
#line 1958 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp1_);
#line 1958 "../src/SourceList/SourceList.vala"
		_tmp1_ = _tmp5_;
#line 9445 "SourceList.c"
	} else {
#line 1958 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp1_);
#line 1958 "../src/SourceList/SourceList.vala"
		_tmp1_ = NULL;
#line 9451 "SourceList.c"
	}
#line 1958 "../src/SourceList/SourceList.vala"
	_tmp6_ = _g_object_ref0 (_tmp1_);
#line 1958 "../src/SourceList/SourceList.vala"
	new_item = _tmp6_;
#line 1971 "../src/SourceList/SourceList.vala"
	_tmp8_ = new_item;
#line 1971 "../src/SourceList/SourceList.vala"
	if (_tmp8_ != NULL) {
#line 9461 "SourceList.c"
		MailSourceListItem* _tmp9_;
		MailSourceListItem* _tmp10_;
#line 1971 "../src/SourceList/SourceList.vala"
		_tmp9_ = new_item;
#line 1971 "../src/SourceList/SourceList.vala"
		_tmp10_ = self->priv->selected;
#line 1971 "../src/SourceList/SourceList.vala"
		_tmp7_ = _tmp9_ != _tmp10_;
#line 9470 "SourceList.c"
	} else {
#line 1971 "../src/SourceList/SourceList.vala"
		_tmp7_ = FALSE;
#line 9474 "SourceList.c"
	}
#line 1971 "../src/SourceList/SourceList.vala"
	if (_tmp7_) {
#line 9478 "SourceList.c"
		MailSourceListItem* _tmp11_;
		MailSourceListItem* _tmp12_;
		MailSourceListItem* _tmp13_;
#line 1972 "../src/SourceList/SourceList.vala"
		_tmp11_ = new_item;
#line 1972 "../src/SourceList/SourceList.vala"
		_tmp12_ = _g_object_ref0 (_tmp11_);
#line 1972 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->selected);
#line 1972 "../src/SourceList/SourceList.vala"
		self->priv->selected = _tmp12_;
#line 1973 "../src/SourceList/SourceList.vala"
		_tmp13_ = new_item;
#line 1973 "../src/SourceList/SourceList.vala"
		g_signal_emit (self, mail_source_list_tree_signals[MAIL_SOURCE_LIST_TREE_ITEM_SELECTED_SIGNAL], 0, _tmp13_);
#line 9494 "SourceList.c"
	}
#line 1956 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (new_item);
#line 1956 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (_tmp1_);
#line 1956 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 9502 "SourceList.c"
}

static gboolean
mail_source_list_tree_scroll_to_item (MailSourceListTree* self,
                                      MailSourceListItem* item,
                                      gboolean use_align,
                                      gfloat row_align)
{
	gboolean scrolled = FALSE;
	GtkTreePath* path = NULL;
	MailSourceListDataModel* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp2_;
	gboolean result;
#line 1977 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1977 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 1978 "../src/SourceList/SourceList.vala"
	scrolled = FALSE;
#line 1980 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 1980 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_data_model_get_item_path (_tmp0_, item);
#line 1980 "../src/SourceList/SourceList.vala"
	path = _tmp1_;
#line 1981 "../src/SourceList/SourceList.vala"
	_tmp2_ = path;
#line 1981 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 9533 "SourceList.c"
		GtkTreePath* _tmp3_;
#line 1982 "../src/SourceList/SourceList.vala"
		_tmp3_ = path;
#line 1982 "../src/SourceList/SourceList.vala"
		gtk_tree_view_scroll_to_cell ((GtkTreeView*) self, _tmp3_, NULL, use_align, row_align, (gfloat) 0);
#line 1983 "../src/SourceList/SourceList.vala"
		scrolled = TRUE;
#line 9541 "SourceList.c"
	}
#line 1986 "../src/SourceList/SourceList.vala"
	result = scrolled;
#line 1986 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 1986 "../src/SourceList/SourceList.vala"
	return result;
#line 9549 "SourceList.c"
}

static gboolean
mail_source_list_tree_start_editing_item (MailSourceListTree* self,
                                          MailSourceListItem* item)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_ = FALSE;
	gboolean _tmp5_;
	gboolean _tmp6_;
	GtkTreePath* path = NULL;
	MailSourceListDataModel* _tmp8_;
	GtkTreePath* _tmp9_;
	GtkTreePath* _tmp10_;
	gboolean _tmp17_;
	gboolean _tmp18_;
	gboolean result;
#line 1989 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1989 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 1989 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_item_get_editable (item);
#line 1989 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 1989 "../src/SourceList/SourceList.vala"
	_vala_return_val_if_fail (_tmp1_, "item.editable", FALSE);
#line 1989 "../src/SourceList/SourceList.vala"
	_tmp2_ = mail_source_list_item_get_selectable (item);
#line 1989 "../src/SourceList/SourceList.vala"
	_tmp3_ = _tmp2_;
#line 1989 "../src/SourceList/SourceList.vala"
	_vala_return_val_if_fail (_tmp3_, "item.selectable", FALSE);
#line 1990 "../src/SourceList/SourceList.vala"
	_tmp5_ = mail_source_list_tree_get_editing (self);
#line 1990 "../src/SourceList/SourceList.vala"
	_tmp6_ = _tmp5_;
#line 1990 "../src/SourceList/SourceList.vala"
	if (_tmp6_) {
#line 9592 "SourceList.c"
		MailSourceListItem* _tmp7_;
#line 1990 "../src/SourceList/SourceList.vala"
		_tmp7_ = self->priv->edited;
#line 1990 "../src/SourceList/SourceList.vala"
		_tmp4_ = item == _tmp7_;
#line 9598 "SourceList.c"
	} else {
#line 1990 "../src/SourceList/SourceList.vala"
		_tmp4_ = FALSE;
#line 9602 "SourceList.c"
	}
#line 1990 "../src/SourceList/SourceList.vala"
	if (_tmp4_) {
#line 1991 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 1991 "../src/SourceList/SourceList.vala"
		return result;
#line 9610 "SourceList.c"
	}
#line 1993 "../src/SourceList/SourceList.vala"
	_tmp8_ = self->priv->_data_model;
#line 1993 "../src/SourceList/SourceList.vala"
	_tmp9_ = mail_source_list_data_model_get_item_path (_tmp8_, item);
#line 1993 "../src/SourceList/SourceList.vala"
	path = _tmp9_;
#line 1994 "../src/SourceList/SourceList.vala"
	_tmp10_ = path;
#line 1994 "../src/SourceList/SourceList.vala"
	if (_tmp10_ != NULL) {
#line 9622 "SourceList.c"
		GtkCellRendererText* _tmp11_;
		GtkTreePath* _tmp12_;
		GtkTreeViewColumn* _tmp13_;
		GtkCellRendererText* _tmp14_;
#line 1995 "../src/SourceList/SourceList.vala"
		self->priv->edited = item;
#line 1996 "../src/SourceList/SourceList.vala"
		_tmp11_ = self->priv->text_cell;
#line 1996 "../src/SourceList/SourceList.vala"
		g_object_set (_tmp11_, "editable", TRUE, NULL);
#line 1997 "../src/SourceList/SourceList.vala"
		_tmp12_ = path;
#line 1997 "../src/SourceList/SourceList.vala"
		_tmp13_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) MAIL_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 1997 "../src/SourceList/SourceList.vala"
		_tmp14_ = self->priv->text_cell;
#line 1997 "../src/SourceList/SourceList.vala"
		gtk_tree_view_set_cursor_on_cell ((GtkTreeView*) self, _tmp12_, _tmp13_, (GtkCellRenderer*) _tmp14_, TRUE);
#line 9641 "SourceList.c"
	} else {
		const gchar* _tmp15_;
		const gchar* _tmp16_;
#line 1999 "../src/SourceList/SourceList.vala"
		_tmp15_ = mail_source_list_item_get_name (item);
#line 1999 "../src/SourceList/SourceList.vala"
		_tmp16_ = _tmp15_;
#line 1999 "../src/SourceList/SourceList.vala"
		g_warning ("SourceList.vala:1999: Could not edit \"%s\": path not found", _tmp16_);
#line 9651 "SourceList.c"
	}
#line 2002 "../src/SourceList/SourceList.vala"
	_tmp17_ = mail_source_list_tree_get_editing (self);
#line 2002 "../src/SourceList/SourceList.vala"
	_tmp18_ = _tmp17_;
#line 2002 "../src/SourceList/SourceList.vala"
	result = _tmp18_;
#line 2002 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2002 "../src/SourceList/SourceList.vala"
	return result;
#line 9663 "SourceList.c"
}

static void
mail_source_list_tree_stop_editing (MailSourceListTree* self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 2005 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2006 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_tree_get_editing (self);
#line 2006 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2006 "../src/SourceList/SourceList.vala"
	if (_tmp2_) {
#line 9680 "SourceList.c"
		MailSourceListItem* _tmp3_;
#line 2006 "../src/SourceList/SourceList.vala"
		_tmp3_ = self->priv->edited;
#line 2006 "../src/SourceList/SourceList.vala"
		_tmp0_ = _tmp3_ != NULL;
#line 9686 "SourceList.c"
	} else {
#line 2006 "../src/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 9690 "SourceList.c"
	}
#line 2006 "../src/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 9694 "SourceList.c"
		GtkTreePath* path = NULL;
		MailSourceListDataModel* _tmp4_;
		MailSourceListItem* _tmp5_;
		GtkTreePath* _tmp6_;
		GtkTreePath* _tmp7_;
#line 2007 "../src/SourceList/SourceList.vala"
		_tmp4_ = self->priv->_data_model;
#line 2007 "../src/SourceList/SourceList.vala"
		_tmp5_ = self->priv->edited;
#line 2007 "../src/SourceList/SourceList.vala"
		_tmp6_ = mail_source_list_data_model_get_item_path (_tmp4_, _tmp5_);
#line 2007 "../src/SourceList/SourceList.vala"
		path = _tmp6_;
#line 2011 "../src/SourceList/SourceList.vala"
		_tmp7_ = path;
#line 2011 "../src/SourceList/SourceList.vala"
		if (_tmp7_ != NULL) {
#line 9712 "SourceList.c"
			GtkTreePath* _tmp8_;
			GtkTreeViewColumn* _tmp9_;
			GtkCellRendererText* _tmp10_;
#line 2012 "../src/SourceList/SourceList.vala"
			_tmp8_ = path;
#line 2012 "../src/SourceList/SourceList.vala"
			_tmp9_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) MAIL_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 2012 "../src/SourceList/SourceList.vala"
			_tmp10_ = self->priv->text_cell;
#line 2012 "../src/SourceList/SourceList.vala"
			gtk_tree_view_set_cursor_on_cell ((GtkTreeView*) self, _tmp8_, _tmp9_, (GtkCellRenderer*) _tmp10_, FALSE);
#line 9724 "SourceList.c"
		}
#line 2006 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 9728 "SourceList.c"
	}
}

static void
_mail_source_list_tree_on_editing_done_gtk_cell_editable_editing_done (GtkCellEditable* _sender,
                                                                       gpointer self)
{
#line 2019 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_on_editing_done ((MailSourceListTree*) self);
#line 9738 "SourceList.c"
}

static void
mail_source_list_tree_on_editing_started (MailSourceListTree* self,
                                          GtkCellEditable* editable,
                                          const gchar* path)
{
	GtkEntry* _tmp0_;
	GtkEntry* _tmp1_;
#line 2016 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2016 "../src/SourceList/SourceList.vala"
	g_return_if_fail (editable != NULL);
#line 2016 "../src/SourceList/SourceList.vala"
	g_return_if_fail (path != NULL);
#line 2017 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (editable, gtk_entry_get_type ()) ? ((GtkEntry*) editable) : NULL);
#line 2017 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->editable_entry);
#line 2017 "../src/SourceList/SourceList.vala"
	self->priv->editable_entry = _tmp0_;
#line 2018 "../src/SourceList/SourceList.vala"
	_tmp1_ = self->priv->editable_entry;
#line 2018 "../src/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 9764 "SourceList.c"
		GtkEntry* _tmp2_;
		GtkEntry* _tmp3_;
#line 2019 "../src/SourceList/SourceList.vala"
		_tmp2_ = self->priv->editable_entry;
#line 2019 "../src/SourceList/SourceList.vala"
		g_signal_connect_object ((GtkCellEditable*) _tmp2_, "editing-done", (GCallback) _mail_source_list_tree_on_editing_done_gtk_cell_editable_editing_done, self, 0);
#line 2020 "../src/SourceList/SourceList.vala"
		_tmp3_ = self->priv->editable_entry;
#line 2020 "../src/SourceList/SourceList.vala"
		g_object_set (_tmp3_, "editable", TRUE, NULL);
#line 9775 "SourceList.c"
	}
}

static void
mail_source_list_tree_on_editing_canceled (MailSourceListTree* self)
{
	GtkEntry* _tmp0_;
	GtkCellRendererText* _tmp4_;
#line 2024 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2025 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->editable_entry;
#line 2025 "../src/SourceList/SourceList.vala"
	if (_tmp0_ != NULL) {
#line 9790 "SourceList.c"
		GtkEntry* _tmp1_;
		GtkEntry* _tmp2_;
		guint _tmp3_;
#line 2026 "../src/SourceList/SourceList.vala"
		_tmp1_ = self->priv->editable_entry;
#line 2026 "../src/SourceList/SourceList.vala"
		g_object_set (_tmp1_, "editable", FALSE, NULL);
#line 2027 "../src/SourceList/SourceList.vala"
		_tmp2_ = self->priv->editable_entry;
#line 2027 "../src/SourceList/SourceList.vala"
		g_signal_parse_name ("editing-done", gtk_cell_editable_get_type (), &_tmp3_, NULL, FALSE);
#line 2027 "../src/SourceList/SourceList.vala"
		g_signal_handlers_disconnect_matched ((GtkCellEditable*) _tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _mail_source_list_tree_on_editing_done_gtk_cell_editable_editing_done, self);
#line 9804 "SourceList.c"
	}
#line 2030 "../src/SourceList/SourceList.vala"
	_tmp4_ = self->priv->text_cell;
#line 2030 "../src/SourceList/SourceList.vala"
	g_object_set (_tmp4_, "editable", FALSE, NULL);
#line 2031 "../src/SourceList/SourceList.vala"
	self->priv->edited = NULL;
#line 9812 "SourceList.c"
}

static void
mail_source_list_tree_on_editing_done (MailSourceListTree* self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	MailSourceListItem* _tmp2_;
#line 2034 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2035 "../src/SourceList/SourceList.vala"
	_tmp2_ = self->priv->edited;
#line 2035 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 9827 "SourceList.c"
		MailSourceListItem* _tmp3_;
		gboolean _tmp4_;
		gboolean _tmp5_;
#line 2035 "../src/SourceList/SourceList.vala"
		_tmp3_ = self->priv->edited;
#line 2035 "../src/SourceList/SourceList.vala"
		_tmp4_ = mail_source_list_item_get_editable (_tmp3_);
#line 2035 "../src/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 2035 "../src/SourceList/SourceList.vala"
		_tmp1_ = _tmp5_;
#line 9839 "SourceList.c"
	} else {
#line 2035 "../src/SourceList/SourceList.vala"
		_tmp1_ = FALSE;
#line 9843 "SourceList.c"
	}
#line 2035 "../src/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 9847 "SourceList.c"
		GtkEntry* _tmp6_;
#line 2035 "../src/SourceList/SourceList.vala"
		_tmp6_ = self->priv->editable_entry;
#line 2035 "../src/SourceList/SourceList.vala"
		_tmp0_ = _tmp6_ != NULL;
#line 9853 "SourceList.c"
	} else {
#line 2035 "../src/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 9857 "SourceList.c"
	}
#line 2035 "../src/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 9861 "SourceList.c"
		MailSourceListItem* _tmp7_;
		GtkEntry* _tmp8_;
		const gchar* _tmp9_;
#line 2036 "../src/SourceList/SourceList.vala"
		_tmp7_ = self->priv->edited;
#line 2036 "../src/SourceList/SourceList.vala"
		_tmp8_ = self->priv->editable_entry;
#line 2036 "../src/SourceList/SourceList.vala"
		_tmp9_ = gtk_entry_get_text (_tmp8_);
#line 2036 "../src/SourceList/SourceList.vala"
		g_signal_emit (_tmp7_, mail_source_list_item_signals[MAIL_SOURCE_LIST_ITEM_EDITED_SIGNAL], 0, _tmp9_);
#line 9873 "SourceList.c"
	}
#line 2039 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_on_editing_canceled (self);
#line 9877 "SourceList.c"
}

static void
mail_source_list_tree_on_activatable_activated (MailSourceListTree* self,
                                                const gchar* item_path_str)
{
	MailSourceListItem* item = NULL;
	MailSourceListItem* _tmp0_;
	MailSourceListItem* _tmp1_;
#line 2042 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2042 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item_path_str != NULL);
#line 2043 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_tree_get_item_from_path_string (self, item_path_str);
#line 2043 "../src/SourceList/SourceList.vala"
	item = _tmp0_;
#line 2044 "../src/SourceList/SourceList.vala"
	_tmp1_ = item;
#line 2044 "../src/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 9899 "SourceList.c"
		MailSourceListItem* _tmp2_;
#line 2045 "../src/SourceList/SourceList.vala"
		_tmp2_ = item;
#line 2045 "../src/SourceList/SourceList.vala"
		g_signal_emit (_tmp2_, mail_source_list_item_signals[MAIL_SOURCE_LIST_ITEM_ACTION_ACTIVATED_SIGNAL], 0);
#line 9905 "SourceList.c"
	}
#line 2042 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 9909 "SourceList.c"
}

static MailSourceListItem*
mail_source_list_tree_get_item_from_path_string (MailSourceListTree* self,
                                                 const gchar* item_path_str)
{
	GtkTreePath* item_path = NULL;
	GtkTreePath* _tmp0_;
	MailSourceListDataModel* _tmp1_;
	MailSourceListItem* _tmp2_;
	MailSourceListItem* result;
#line 2048 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2048 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item_path_str != NULL, NULL);
#line 2049 "../src/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_path_new_from_string (item_path_str);
#line 2049 "../src/SourceList/SourceList.vala"
	item_path = _tmp0_;
#line 2050 "../src/SourceList/SourceList.vala"
	_tmp1_ = self->priv->_data_model;
#line 2050 "../src/SourceList/SourceList.vala"
	_tmp2_ = mail_source_list_data_model_get_item_from_path (_tmp1_, item_path);
#line 2050 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2050 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (item_path);
#line 2050 "../src/SourceList/SourceList.vala"
	return result;
#line 9939 "SourceList.c"
}

static gboolean
mail_source_list_tree_toggle_expansion (MailSourceListTree* self,
                                        MailSourceListExpandableItem* item)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean result;
#line 2053 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2053 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2054 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_expandable_item_get_collapsible (item);
#line 2054 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 2054 "../src/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 9959 "SourceList.c"
		gboolean _tmp2_;
		gboolean _tmp3_;
#line 2055 "../src/SourceList/SourceList.vala"
		_tmp2_ = mail_source_list_expandable_item_get_expanded (item);
#line 2055 "../src/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 2055 "../src/SourceList/SourceList.vala"
		mail_source_list_expandable_item_set_expanded (item, !_tmp3_);
#line 2056 "../src/SourceList/SourceList.vala"
		result = TRUE;
#line 2056 "../src/SourceList/SourceList.vala"
		return result;
#line 9972 "SourceList.c"
	}
#line 2058 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 2058 "../src/SourceList/SourceList.vala"
	return result;
#line 9978 "SourceList.c"
}

/**
         * Updates the tree to reflect the ''expanded'' property of expandable_item.
         */
static void
mail_source_list_tree_update_expansion (MailSourceListTree* self,
                                        MailSourceListExpandableItem* expandable_item)
{
	GtkTreePath* path = NULL;
	MailSourceListDataModel* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp2_;
#line 2064 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2064 "../src/SourceList/SourceList.vala"
	g_return_if_fail (expandable_item != NULL);
#line 2065 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 2065 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_data_model_get_item_path (_tmp0_, (MailSourceListItem*) expandable_item);
#line 2065 "../src/SourceList/SourceList.vala"
	path = _tmp1_;
#line 2067 "../src/SourceList/SourceList.vala"
	_tmp2_ = path;
#line 2067 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 10006 "SourceList.c"
		GtkTreePath* _tmp3_;
		gboolean _tmp4_;
		gboolean _tmp5_;
#line 2071 "../src/SourceList/SourceList.vala"
		_tmp3_ = path;
#line 2071 "../src/SourceList/SourceList.vala"
		mail_source_list_tree_add_spacer_cell_for_level (self, gtk_tree_path_get_depth (_tmp3_) + 1, TRUE);
#line 2073 "../src/SourceList/SourceList.vala"
		_tmp4_ = mail_source_list_expandable_item_get_expanded (expandable_item);
#line 2073 "../src/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 2073 "../src/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 10020 "SourceList.c"
			GtkTreePath* _tmp6_;
			gboolean _tmp7_ = FALSE;
			MailSourceListItem* _tmp8_;
#line 2074 "../src/SourceList/SourceList.vala"
			_tmp6_ = path;
#line 2074 "../src/SourceList/SourceList.vala"
			gtk_tree_view_expand_row ((GtkTreeView*) self, _tmp6_, FALSE);
#line 2082 "../src/SourceList/SourceList.vala"
			_tmp8_ = self->priv->selected;
#line 2082 "../src/SourceList/SourceList.vala"
			if (_tmp8_ != NULL) {
#line 10032 "SourceList.c"
				MailSourceListItem* _tmp9_;
				MailSourceListExpandableItem* _tmp10_;
				MailSourceListExpandableItem* _tmp11_;
#line 2082 "../src/SourceList/SourceList.vala"
				_tmp9_ = self->priv->selected;
#line 2082 "../src/SourceList/SourceList.vala"
				_tmp10_ = mail_source_list_item_get_parent (_tmp9_);
#line 2082 "../src/SourceList/SourceList.vala"
				_tmp11_ = _tmp10_;
#line 2082 "../src/SourceList/SourceList.vala"
				_tmp7_ = _tmp11_ == expandable_item;
#line 10044 "SourceList.c"
			} else {
#line 2082 "../src/SourceList/SourceList.vala"
				_tmp7_ = FALSE;
#line 10048 "SourceList.c"
			}
#line 2082 "../src/SourceList/SourceList.vala"
			if (_tmp7_) {
#line 10052 "SourceList.c"
				MailSourceListItem* _tmp12_;
#line 2083 "../src/SourceList/SourceList.vala"
				_tmp12_ = self->priv->selected;
#line 2083 "../src/SourceList/SourceList.vala"
				mail_source_list_tree_set_selected (self, _tmp12_, TRUE);
#line 10058 "SourceList.c"
			}
			{
				GeeIterator* _child_item_it = NULL;
				GeeCollection* _tmp13_;
				GeeCollection* _tmp14_;
				GeeCollection* _tmp15_;
				GeeIterator* _tmp16_;
				GeeIterator* _tmp17_;
#line 2088 "../src/SourceList/SourceList.vala"
				_tmp13_ = mail_source_list_expandable_item_get_children (expandable_item);
#line 2088 "../src/SourceList/SourceList.vala"
				_tmp14_ = _tmp13_;
#line 2088 "../src/SourceList/SourceList.vala"
				_tmp15_ = _tmp14_;
#line 2088 "../src/SourceList/SourceList.vala"
				_tmp16_ = gee_iterable_iterator ((GeeIterable*) _tmp15_);
#line 2088 "../src/SourceList/SourceList.vala"
				_tmp17_ = _tmp16_;
#line 2088 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp15_);
#line 2088 "../src/SourceList/SourceList.vala"
				_child_item_it = _tmp17_;
#line 2088 "../src/SourceList/SourceList.vala"
				while (TRUE) {
#line 10083 "SourceList.c"
					GeeIterator* _tmp18_;
					MailSourceListItem* child_item = NULL;
					GeeIterator* _tmp19_;
					gpointer _tmp20_;
					MailSourceListExpandableItem* child_expandable_item = NULL;
					MailSourceListItem* _tmp21_;
					MailSourceListExpandableItem* _tmp22_;
					MailSourceListExpandableItem* _tmp23_;
#line 2088 "../src/SourceList/SourceList.vala"
					_tmp18_ = _child_item_it;
#line 2088 "../src/SourceList/SourceList.vala"
					if (!gee_iterator_next (_tmp18_)) {
#line 2088 "../src/SourceList/SourceList.vala"
						break;
#line 10098 "SourceList.c"
					}
#line 2088 "../src/SourceList/SourceList.vala"
					_tmp19_ = _child_item_it;
#line 2088 "../src/SourceList/SourceList.vala"
					_tmp20_ = gee_iterator_get (_tmp19_);
#line 2088 "../src/SourceList/SourceList.vala"
					child_item = (MailSourceListItem*) _tmp20_;
#line 2089 "../src/SourceList/SourceList.vala"
					_tmp21_ = child_item;
#line 2089 "../src/SourceList/SourceList.vala"
					_tmp22_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp21_) ? ((MailSourceListExpandableItem*) _tmp21_) : NULL);
#line 2089 "../src/SourceList/SourceList.vala"
					child_expandable_item = _tmp22_;
#line 2090 "../src/SourceList/SourceList.vala"
					_tmp23_ = child_expandable_item;
#line 2090 "../src/SourceList/SourceList.vala"
					if (_tmp23_ != NULL) {
#line 10116 "SourceList.c"
						MailSourceListExpandableItem* _tmp24_;
#line 2091 "../src/SourceList/SourceList.vala"
						_tmp24_ = child_expandable_item;
#line 2091 "../src/SourceList/SourceList.vala"
						mail_source_list_tree_update_expansion (self, _tmp24_);
#line 10122 "SourceList.c"
					}
#line 2088 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (child_expandable_item);
#line 2088 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (child_item);
#line 10128 "SourceList.c"
				}
#line 2088 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (_child_item_it);
#line 10132 "SourceList.c"
			}
		} else {
			GtkTreePath* _tmp25_;
#line 2094 "../src/SourceList/SourceList.vala"
			_tmp25_ = path;
#line 2094 "../src/SourceList/SourceList.vala"
			gtk_tree_view_collapse_row ((GtkTreeView*) self, _tmp25_);
#line 10140 "SourceList.c"
		}
	}
#line 2064 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 10145 "SourceList.c"
}

static void
mail_source_list_tree_real_row_expanded (GtkTreeView* base,
                                         GtkTreeIter* iter,
                                         GtkTreePath* path)
{
	MailSourceListTree * self;
	MailSourceListExpandableItem* item = NULL;
	MailSourceListDataModel* _tmp0_;
	GtkTreeIter _tmp1_;
	MailSourceListItem* _tmp2_;
	MailSourceListExpandableItem* _tmp3_;
#line 2099 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 2099 "../src/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2099 "../src/SourceList/SourceList.vala"
	g_return_if_fail (path != NULL);
#line 2100 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 2100 "../src/SourceList/SourceList.vala"
	_tmp1_ = *iter;
#line 2100 "../src/SourceList/SourceList.vala"
	_tmp2_ = mail_source_list_data_model_get_item (_tmp0_, &_tmp1_);
#line 2100 "../src/SourceList/SourceList.vala"
	_tmp3_ = MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp2_) ? ((MailSourceListExpandableItem*) _tmp2_) : NULL;
#line 2100 "../src/SourceList/SourceList.vala"
	if (_tmp3_ == NULL) {
#line 2100 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp2_);
#line 10177 "SourceList.c"
	}
#line 2100 "../src/SourceList/SourceList.vala"
	item = _tmp3_;
#line 2101 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 2103 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_disable_item_property_monitor (self);
#line 2104 "../src/SourceList/SourceList.vala"
	mail_source_list_expandable_item_set_expanded (item, TRUE);
#line 2105 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_enable_item_property_monitor (self);
#line 2099 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 10191 "SourceList.c"
}

static void
mail_source_list_tree_real_row_collapsed (GtkTreeView* base,
                                          GtkTreeIter* iter,
                                          GtkTreePath* path)
{
	MailSourceListTree * self;
	MailSourceListExpandableItem* item = NULL;
	MailSourceListDataModel* _tmp0_;
	GtkTreeIter _tmp1_;
	MailSourceListItem* _tmp2_;
	MailSourceListExpandableItem* _tmp3_;
#line 2108 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 2108 "../src/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2108 "../src/SourceList/SourceList.vala"
	g_return_if_fail (path != NULL);
#line 2109 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 2109 "../src/SourceList/SourceList.vala"
	_tmp1_ = *iter;
#line 2109 "../src/SourceList/SourceList.vala"
	_tmp2_ = mail_source_list_data_model_get_item (_tmp0_, &_tmp1_);
#line 2109 "../src/SourceList/SourceList.vala"
	_tmp3_ = MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp2_) ? ((MailSourceListExpandableItem*) _tmp2_) : NULL;
#line 2109 "../src/SourceList/SourceList.vala"
	if (_tmp3_ == NULL) {
#line 2109 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp2_);
#line 10223 "SourceList.c"
	}
#line 2109 "../src/SourceList/SourceList.vala"
	item = _tmp3_;
#line 2110 "../src/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 2112 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_disable_item_property_monitor (self);
#line 2113 "../src/SourceList/SourceList.vala"
	mail_source_list_expandable_item_set_expanded (item, FALSE);
#line 2114 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_enable_item_property_monitor (self);
#line 2108 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 10237 "SourceList.c"
}

static void
mail_source_list_tree_real_row_activated (GtkTreeView* base,
                                          GtkTreePath* path,
                                          GtkTreeViewColumn* column)
{
	MailSourceListTree * self;
	GtkTreeViewColumn* _tmp0_;
#line 2117 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 2117 "../src/SourceList/SourceList.vala"
	g_return_if_fail (path != NULL);
#line 2117 "../src/SourceList/SourceList.vala"
	g_return_if_fail (column != NULL);
#line 2118 "../src/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) MAIL_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 2118 "../src/SourceList/SourceList.vala"
	if (column == _tmp0_) {
#line 10257 "SourceList.c"
		MailSourceListItem* item = NULL;
		MailSourceListDataModel* _tmp1_;
		MailSourceListItem* _tmp2_;
		MailSourceListItem* _tmp3_;
#line 2119 "../src/SourceList/SourceList.vala"
		_tmp1_ = self->priv->_data_model;
#line 2119 "../src/SourceList/SourceList.vala"
		_tmp2_ = mail_source_list_data_model_get_item_from_path (_tmp1_, path);
#line 2119 "../src/SourceList/SourceList.vala"
		item = _tmp2_;
#line 2120 "../src/SourceList/SourceList.vala"
		_tmp3_ = item;
#line 2120 "../src/SourceList/SourceList.vala"
		if (_tmp3_ != NULL) {
#line 10272 "SourceList.c"
			MailSourceListItem* _tmp4_;
#line 2121 "../src/SourceList/SourceList.vala"
			_tmp4_ = item;
#line 2121 "../src/SourceList/SourceList.vala"
			g_signal_emit (_tmp4_, mail_source_list_item_signals[MAIL_SOURCE_LIST_ITEM_ACTIVATED_SIGNAL], 0);
#line 10278 "SourceList.c"
		}
#line 2118 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (item);
#line 10282 "SourceList.c"
	}
}

static gboolean
mail_source_list_tree_real_key_release_event (GtkWidget* base,
                                              GdkEventKey* event)
{
	MailSourceListTree * self;
	MailSourceListItem* _tmp0_;
	MailSourceListItem* _tmp1_;
	gboolean result;
#line 2125 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 2125 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 2126 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_tree_get_selected_item (self);
#line 2126 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 2126 "../src/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 2127 "../src/SourceList/SourceList.vala"
		switch (event->keyval) {
#line 2127 "../src/SourceList/SourceList.vala"
			case GDK_KEY_F2:
#line 10308 "SourceList.c"
			{
				GdkModifierType modifiers = 0U;
				gboolean _tmp2_ = FALSE;
#line 2129 "../src/SourceList/SourceList.vala"
				modifiers = gtk_accelerator_get_default_mod_mask ();
#line 2131 "../src/SourceList/SourceList.vala"
				if ((event->state & modifiers) == 0) {
#line 10316 "SourceList.c"
					MailSourceListItem* _tmp3_;
					MailSourceListItem* _tmp4_;
					gboolean _tmp5_;
					gboolean _tmp6_;
#line 2131 "../src/SourceList/SourceList.vala"
					_tmp3_ = mail_source_list_tree_get_selected_item (self);
#line 2131 "../src/SourceList/SourceList.vala"
					_tmp4_ = _tmp3_;
#line 2131 "../src/SourceList/SourceList.vala"
					_tmp5_ = mail_source_list_item_get_editable (_tmp4_);
#line 2131 "../src/SourceList/SourceList.vala"
					_tmp6_ = _tmp5_;
#line 2131 "../src/SourceList/SourceList.vala"
					_tmp2_ = _tmp6_;
#line 10331 "SourceList.c"
				} else {
#line 2131 "../src/SourceList/SourceList.vala"
					_tmp2_ = FALSE;
#line 10335 "SourceList.c"
				}
#line 2131 "../src/SourceList/SourceList.vala"
				if (_tmp2_) {
#line 10339 "SourceList.c"
					MailSourceListItem* _tmp7_;
					MailSourceListItem* _tmp8_;
#line 2132 "../src/SourceList/SourceList.vala"
					_tmp7_ = mail_source_list_tree_get_selected_item (self);
#line 2132 "../src/SourceList/SourceList.vala"
					_tmp8_ = _tmp7_;
#line 2132 "../src/SourceList/SourceList.vala"
					mail_source_list_tree_start_editing_item (self, _tmp8_);
#line 10348 "SourceList.c"
				}
#line 2133 "../src/SourceList/SourceList.vala"
				break;
#line 10352 "SourceList.c"
			}
			default:
#line 2127 "../src/SourceList/SourceList.vala"
			break;
#line 10357 "SourceList.c"
		}
	}
#line 2137 "../src/SourceList/SourceList.vala"
	result = GTK_WIDGET_CLASS (mail_source_list_tree_parent_class)->key_release_event ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), event);
#line 2137 "../src/SourceList/SourceList.vala"
	return result;
#line 10364 "SourceList.c"
}

static gboolean
mail_source_list_tree_real_button_release_event (GtkWidget* base,
                                                 GdkEventButton* event)
{
	MailSourceListTree * self;
	gboolean _tmp0_ = FALSE;
	gboolean result;
#line 2140 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 2140 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 2141 "../src/SourceList/SourceList.vala"
	if (self->priv->unselectable_item_clicked) {
#line 10380 "SourceList.c"
		GdkWindow* _tmp1_;
		GdkWindow* _tmp2_;
#line 2141 "../src/SourceList/SourceList.vala"
		_tmp1_ = event->window;
#line 2141 "../src/SourceList/SourceList.vala"
		_tmp2_ = gtk_tree_view_get_bin_window ((GtkTreeView*) self);
#line 2141 "../src/SourceList/SourceList.vala"
		_tmp0_ = _tmp1_ == _tmp2_;
#line 10389 "SourceList.c"
	} else {
#line 2141 "../src/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 10393 "SourceList.c"
	}
#line 2141 "../src/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 10397 "SourceList.c"
		GtkTreePath* path = NULL;
		GtkTreeViewColumn* column = NULL;
		gint x = 0;
		gint y = 0;
		gint cell_x = 0;
		gint cell_y = 0;
		GtkTreePath* _tmp3_ = NULL;
		GtkTreeViewColumn* _tmp4_ = NULL;
		gint _tmp5_ = 0;
		gint _tmp6_ = 0;
		gboolean _tmp7_;
		GtkTreeViewColumn* _tmp8_;
#line 2142 "../src/SourceList/SourceList.vala"
		self->priv->unselectable_item_clicked = FALSE;
#line 2146 "../src/SourceList/SourceList.vala"
		x = (gint) event->x;
#line 2146 "../src/SourceList/SourceList.vala"
		y = (gint) event->y;
#line 2148 "../src/SourceList/SourceList.vala"
		_tmp7_ = gtk_tree_view_get_path_at_pos ((GtkTreeView*) self, x, y, &_tmp3_, &_tmp4_, &_tmp5_, &_tmp6_);
#line 2148 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 2148 "../src/SourceList/SourceList.vala"
		path = _tmp3_;
#line 2148 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (column);
#line 2148 "../src/SourceList/SourceList.vala"
		_tmp8_ = _g_object_ref0 (_tmp4_);
#line 2148 "../src/SourceList/SourceList.vala"
		column = _tmp8_;
#line 2148 "../src/SourceList/SourceList.vala"
		cell_x = _tmp5_;
#line 2148 "../src/SourceList/SourceList.vala"
		cell_y = _tmp6_;
#line 2148 "../src/SourceList/SourceList.vala"
		if (_tmp7_) {
#line 10434 "SourceList.c"
			MailSourceListExpandableItem* item = NULL;
			MailSourceListDataModel* _tmp9_;
			GtkTreePath* _tmp10_;
			MailSourceListItem* _tmp11_;
			MailSourceListExpandableItem* _tmp12_;
			MailSourceListExpandableItem* _tmp13_;
#line 2149 "../src/SourceList/SourceList.vala"
			_tmp9_ = self->priv->_data_model;
#line 2149 "../src/SourceList/SourceList.vala"
			_tmp10_ = path;
#line 2149 "../src/SourceList/SourceList.vala"
			_tmp11_ = mail_source_list_data_model_get_item_from_path (_tmp9_, _tmp10_);
#line 2149 "../src/SourceList/SourceList.vala"
			_tmp12_ = MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp11_) ? ((MailSourceListExpandableItem*) _tmp11_) : NULL;
#line 2149 "../src/SourceList/SourceList.vala"
			if (_tmp12_ == NULL) {
#line 2149 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp11_);
#line 10453 "SourceList.c"
			}
#line 2149 "../src/SourceList/SourceList.vala"
			item = _tmp12_;
#line 2151 "../src/SourceList/SourceList.vala"
			_tmp13_ = item;
#line 2151 "../src/SourceList/SourceList.vala"
			if (_tmp13_ != NULL) {
#line 10461 "SourceList.c"
				gboolean _tmp14_ = FALSE;
				MailSourceListExpandableItem* _tmp15_;
				gboolean _tmp16_;
				gboolean _tmp17_;
#line 2152 "../src/SourceList/SourceList.vala"
				_tmp15_ = item;
#line 2152 "../src/SourceList/SourceList.vala"
				_tmp16_ = mail_source_list_item_get_selectable ((MailSourceListItem*) _tmp15_);
#line 2152 "../src/SourceList/SourceList.vala"
				_tmp17_ = _tmp16_;
#line 2152 "../src/SourceList/SourceList.vala"
				if (!_tmp17_) {
#line 2152 "../src/SourceList/SourceList.vala"
					_tmp14_ = TRUE;
#line 10476 "SourceList.c"
				} else {
					MailSourceListDataModel* _tmp18_;
					MailSourceListExpandableItem* _tmp19_;
					GtkTreePath* _tmp20_;
#line 2152 "../src/SourceList/SourceList.vala"
					_tmp18_ = self->priv->_data_model;
#line 2152 "../src/SourceList/SourceList.vala"
					_tmp19_ = item;
#line 2152 "../src/SourceList/SourceList.vala"
					_tmp20_ = path;
#line 2152 "../src/SourceList/SourceList.vala"
					_tmp14_ = mail_source_list_data_model_is_category (_tmp18_, (MailSourceListItem*) _tmp19_, NULL, _tmp20_);
#line 10489 "SourceList.c"
				}
#line 2152 "../src/SourceList/SourceList.vala"
				if (_tmp14_) {
#line 10493 "SourceList.c"
					MailSourceListExpandableItem* _tmp21_;
#line 2153 "../src/SourceList/SourceList.vala"
					_tmp21_ = item;
#line 2153 "../src/SourceList/SourceList.vala"
					mail_source_list_tree_toggle_expansion (self, _tmp21_);
#line 10499 "SourceList.c"
				}
			}
#line 2148 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 10504 "SourceList.c"
		}
#line 2141 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (column);
#line 2141 "../src/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 10510 "SourceList.c"
	}
#line 2158 "../src/SourceList/SourceList.vala"
	result = GTK_WIDGET_CLASS (mail_source_list_tree_parent_class)->button_release_event ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), event);
#line 2158 "../src/SourceList/SourceList.vala"
	return result;
#line 10516 "SourceList.c"
}

static gboolean
mail_source_list_tree_real_button_press_event (GtkWidget* base,
                                               GdkEventButton* event)
{
	MailSourceListTree * self;
	GdkWindow* _tmp0_;
	GdkWindow* _tmp1_;
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* column = NULL;
	gint x = 0;
	gint y = 0;
	gint cell_x = 0;
	gint cell_y = 0;
	GtkTreePath* _tmp2_ = NULL;
	GtkTreeViewColumn* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	gboolean _tmp6_;
	GtkTreeViewColumn* _tmp7_;
	gboolean result;
#line 2161 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 2161 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 2162 "../src/SourceList/SourceList.vala"
	_tmp0_ = event->window;
#line 2162 "../src/SourceList/SourceList.vala"
	_tmp1_ = gtk_tree_view_get_bin_window ((GtkTreeView*) self);
#line 2162 "../src/SourceList/SourceList.vala"
	if (_tmp0_ != _tmp1_) {
#line 2163 "../src/SourceList/SourceList.vala"
		result = GTK_WIDGET_CLASS (mail_source_list_tree_parent_class)->button_press_event ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), event);
#line 2163 "../src/SourceList/SourceList.vala"
		return result;
#line 10553 "SourceList.c"
	}
#line 2167 "../src/SourceList/SourceList.vala"
	x = (gint) event->x;
#line 2167 "../src/SourceList/SourceList.vala"
	y = (gint) event->y;
#line 2169 "../src/SourceList/SourceList.vala"
	_tmp6_ = gtk_tree_view_get_path_at_pos ((GtkTreeView*) self, x, y, &_tmp2_, &_tmp3_, &_tmp4_, &_tmp5_);
#line 2169 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2169 "../src/SourceList/SourceList.vala"
	path = _tmp2_;
#line 2169 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (column);
#line 2169 "../src/SourceList/SourceList.vala"
	_tmp7_ = _g_object_ref0 (_tmp3_);
#line 2169 "../src/SourceList/SourceList.vala"
	column = _tmp7_;
#line 2169 "../src/SourceList/SourceList.vala"
	cell_x = _tmp4_;
#line 2169 "../src/SourceList/SourceList.vala"
	cell_y = _tmp5_;
#line 2169 "../src/SourceList/SourceList.vala"
	if (_tmp6_) {
#line 10577 "SourceList.c"
		MailSourceListItem* item = NULL;
		MailSourceListDataModel* _tmp8_;
		GtkTreePath* _tmp9_;
		MailSourceListItem* _tmp10_;
		GdkRectangle start_cell_area = {0};
		GtkTreePath* _tmp11_;
		GtkTreeViewColumn* _tmp12_;
		GdkRectangle _tmp13_ = {0};
		GdkRectangle _tmp14_;
		gboolean _tmp15_ = FALSE;
		MailSourceListItem* _tmp16_;
#line 2170 "../src/SourceList/SourceList.vala"
		_tmp8_ = self->priv->_data_model;
#line 2170 "../src/SourceList/SourceList.vala"
		_tmp9_ = path;
#line 2170 "../src/SourceList/SourceList.vala"
		_tmp10_ = mail_source_list_data_model_get_item_from_path (_tmp8_, _tmp9_);
#line 2170 "../src/SourceList/SourceList.vala"
		item = _tmp10_;
#line 2174 "../src/SourceList/SourceList.vala"
		_tmp11_ = path;
#line 2174 "../src/SourceList/SourceList.vala"
		_tmp12_ = gtk_tree_view_get_column ((GtkTreeView*) self, 0);
#line 2174 "../src/SourceList/SourceList.vala"
		gtk_tree_view_get_cell_area ((GtkTreeView*) self, _tmp11_, _tmp12_, &_tmp13_);
#line 2174 "../src/SourceList/SourceList.vala"
		start_cell_area = _tmp13_;
#line 2175 "../src/SourceList/SourceList.vala"
		_tmp14_ = start_cell_area;
#line 2175 "../src/SourceList/SourceList.vala"
		cell_x -= _tmp14_.x;
#line 2177 "../src/SourceList/SourceList.vala"
		_tmp16_ = item;
#line 2177 "../src/SourceList/SourceList.vala"
		if (_tmp16_ != NULL) {
#line 10613 "SourceList.c"
			GtkTreeViewColumn* _tmp17_;
			GtkTreeViewColumn* _tmp18_;
#line 2177 "../src/SourceList/SourceList.vala"
			_tmp17_ = column;
#line 2177 "../src/SourceList/SourceList.vala"
			_tmp18_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) MAIL_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 2177 "../src/SourceList/SourceList.vala"
			_tmp15_ = _tmp17_ == _tmp18_;
#line 10622 "SourceList.c"
		} else {
#line 2177 "../src/SourceList/SourceList.vala"
			_tmp15_ = FALSE;
#line 10626 "SourceList.c"
		}
#line 2177 "../src/SourceList/SourceList.vala"
		if (_tmp15_) {
#line 2179 "../src/SourceList/SourceList.vala"
			mail_source_list_tree_stop_editing (self);
#line 2181 "../src/SourceList/SourceList.vala"
			if (event->button == ((guint) GDK_BUTTON_SECONDARY)) {
#line 10634 "SourceList.c"
				MailSourceListItem* _tmp19_;
#line 2182 "../src/SourceList/SourceList.vala"
				_tmp19_ = item;
#line 2182 "../src/SourceList/SourceList.vala"
				mail_source_list_tree_popup_context_menu (self, _tmp19_, event);
#line 2183 "../src/SourceList/SourceList.vala"
				result = TRUE;
#line 2183 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (item);
#line 2183 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (column);
#line 2183 "../src/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (path);
#line 2183 "../src/SourceList/SourceList.vala"
				return result;
#line 10650 "SourceList.c"
			} else {
#line 2184 "../src/SourceList/SourceList.vala"
				if (event->button == ((guint) GDK_BUTTON_PRIMARY)) {
#line 10654 "SourceList.c"
					gboolean is_expandable = FALSE;
					MailSourceListItem* _tmp20_;
					gboolean _tmp21_ = FALSE;
					gboolean is_category = FALSE;
#line 2186 "../src/SourceList/SourceList.vala"
					_tmp20_ = item;
#line 2186 "../src/SourceList/SourceList.vala"
					is_expandable = MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp20_);
#line 2187 "../src/SourceList/SourceList.vala"
					if (is_expandable) {
#line 10665 "SourceList.c"
						MailSourceListDataModel* _tmp22_;
						MailSourceListItem* _tmp23_;
						GtkTreePath* _tmp24_;
#line 2187 "../src/SourceList/SourceList.vala"
						_tmp22_ = self->priv->_data_model;
#line 2187 "../src/SourceList/SourceList.vala"
						_tmp23_ = item;
#line 2187 "../src/SourceList/SourceList.vala"
						_tmp24_ = path;
#line 2187 "../src/SourceList/SourceList.vala"
						_tmp21_ = mail_source_list_data_model_is_category (_tmp22_, _tmp23_, NULL, _tmp24_);
#line 10677 "SourceList.c"
					} else {
#line 2187 "../src/SourceList/SourceList.vala"
						_tmp21_ = FALSE;
#line 10681 "SourceList.c"
					}
#line 2187 "../src/SourceList/SourceList.vala"
					is_category = _tmp21_;
#line 2189 "../src/SourceList/SourceList.vala"
					if (event->type == GDK_BUTTON_PRESS) {
#line 2190 "../src/SourceList/SourceList.vala"
						if (is_expandable) {
#line 10689 "SourceList.c"
							gboolean _tmp25_ = FALSE;
							gboolean _tmp33_ = FALSE;
							gboolean _tmp34_ = FALSE;
#line 2196 "../src/SourceList/SourceList.vala"
							if (is_category) {
#line 2196 "../src/SourceList/SourceList.vala"
								_tmp25_ = TRUE;
#line 10697 "SourceList.c"
							} else {
								gboolean _tmp26_ = FALSE;
								MailSourceListItem* _tmp27_;
								gboolean _tmp28_;
								gboolean _tmp29_;
#line 2197 "../src/SourceList/SourceList.vala"
								_tmp27_ = item;
#line 2197 "../src/SourceList/SourceList.vala"
								_tmp28_ = mail_source_list_item_get_selectable (_tmp27_);
#line 2197 "../src/SourceList/SourceList.vala"
								_tmp29_ = _tmp28_;
#line 2197 "../src/SourceList/SourceList.vala"
								if (!_tmp29_) {
#line 10711 "SourceList.c"
									GtkTreeViewColumn* _tmp30_;
									GtkTreePath* _tmp31_;
									MailSourceListCellRendererIcon* _tmp32_;
#line 2197 "../src/SourceList/SourceList.vala"
									_tmp30_ = column;
#line 2197 "../src/SourceList/SourceList.vala"
									_tmp31_ = path;
#line 2197 "../src/SourceList/SourceList.vala"
									_tmp32_ = self->priv->activatable_cell;
#line 2197 "../src/SourceList/SourceList.vala"
									_tmp26_ = !mail_source_list_tree_over_cell (self, _tmp30_, _tmp31_, (GtkCellRenderer*) _tmp32_, cell_x);
#line 10723 "SourceList.c"
								} else {
#line 2197 "../src/SourceList/SourceList.vala"
									_tmp26_ = FALSE;
#line 10727 "SourceList.c"
								}
#line 2197 "../src/SourceList/SourceList.vala"
								_tmp25_ = _tmp26_;
#line 10731 "SourceList.c"
							}
#line 2196 "../src/SourceList/SourceList.vala"
							self->priv->unselectable_item_clicked = _tmp25_;
#line 2199 "../src/SourceList/SourceList.vala"
							if (!self->priv->unselectable_item_clicked) {
#line 10737 "SourceList.c"
								GtkTreeViewColumn* _tmp35_;
								GtkTreePath* _tmp36_;
#line 2200 "../src/SourceList/SourceList.vala"
								_tmp35_ = column;
#line 2200 "../src/SourceList/SourceList.vala"
								_tmp36_ = path;
#line 2200 "../src/SourceList/SourceList.vala"
								_tmp34_ = mail_source_list_tree_over_primary_expander (self, _tmp35_, _tmp36_, cell_x);
#line 10746 "SourceList.c"
							} else {
#line 2199 "../src/SourceList/SourceList.vala"
								_tmp34_ = FALSE;
#line 10750 "SourceList.c"
							}
#line 2199 "../src/SourceList/SourceList.vala"
							if (_tmp34_) {
#line 10754 "SourceList.c"
								MailSourceListItem* _tmp37_;
#line 2201 "../src/SourceList/SourceList.vala"
								_tmp37_ = item;
#line 2201 "../src/SourceList/SourceList.vala"
								_tmp33_ = mail_source_list_tree_toggle_expansion (self, MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp37_) ? ((MailSourceListExpandableItem*) _tmp37_) : NULL);
#line 10760 "SourceList.c"
							} else {
#line 2199 "../src/SourceList/SourceList.vala"
								_tmp33_ = FALSE;
#line 10764 "SourceList.c"
							}
#line 2199 "../src/SourceList/SourceList.vala"
							if (_tmp33_) {
#line 2202 "../src/SourceList/SourceList.vala"
								result = TRUE;
#line 2202 "../src/SourceList/SourceList.vala"
								_g_object_unref0 (item);
#line 2202 "../src/SourceList/SourceList.vala"
								_g_object_unref0 (column);
#line 2202 "../src/SourceList/SourceList.vala"
								__vala_GtkTreePath_free0 (path);
#line 2202 "../src/SourceList/SourceList.vala"
								return result;
#line 10778 "SourceList.c"
							}
						}
					} else {
						gboolean _tmp38_ = FALSE;
						gboolean _tmp39_ = FALSE;
						gboolean _tmp40_ = FALSE;
						gboolean _tmp41_ = FALSE;
						gboolean _tmp42_ = FALSE;
#line 2205 "../src/SourceList/SourceList.vala"
						if (event->type == GDK_2BUTTON_PRESS) {
#line 2206 "../src/SourceList/SourceList.vala"
							_tmp42_ = !is_category;
#line 10791 "SourceList.c"
						} else {
#line 2205 "../src/SourceList/SourceList.vala"
							_tmp42_ = FALSE;
#line 10795 "SourceList.c"
						}
#line 2205 "../src/SourceList/SourceList.vala"
						if (_tmp42_) {
#line 10799 "SourceList.c"
							MailSourceListItem* _tmp43_;
							gboolean _tmp44_;
							gboolean _tmp45_;
#line 2207 "../src/SourceList/SourceList.vala"
							_tmp43_ = item;
#line 2207 "../src/SourceList/SourceList.vala"
							_tmp44_ = mail_source_list_item_get_editable (_tmp43_);
#line 2207 "../src/SourceList/SourceList.vala"
							_tmp45_ = _tmp44_;
#line 2207 "../src/SourceList/SourceList.vala"
							_tmp41_ = _tmp45_;
#line 10811 "SourceList.c"
						} else {
#line 2205 "../src/SourceList/SourceList.vala"
							_tmp41_ = FALSE;
#line 10815 "SourceList.c"
						}
#line 2205 "../src/SourceList/SourceList.vala"
						if (_tmp41_) {
#line 10819 "SourceList.c"
							MailSourceListItem* _tmp46_;
							gboolean _tmp47_;
							gboolean _tmp48_;
#line 2208 "../src/SourceList/SourceList.vala"
							_tmp46_ = item;
#line 2208 "../src/SourceList/SourceList.vala"
							_tmp47_ = mail_source_list_item_get_selectable (_tmp46_);
#line 2208 "../src/SourceList/SourceList.vala"
							_tmp48_ = _tmp47_;
#line 2208 "../src/SourceList/SourceList.vala"
							_tmp40_ = _tmp48_;
#line 10831 "SourceList.c"
						} else {
#line 2205 "../src/SourceList/SourceList.vala"
							_tmp40_ = FALSE;
#line 10835 "SourceList.c"
						}
#line 2205 "../src/SourceList/SourceList.vala"
						if (_tmp40_) {
#line 10839 "SourceList.c"
							GtkTreeViewColumn* _tmp49_;
							GtkTreePath* _tmp50_;
							GtkCellRendererText* _tmp51_;
#line 2209 "../src/SourceList/SourceList.vala"
							_tmp49_ = column;
#line 2209 "../src/SourceList/SourceList.vala"
							_tmp50_ = path;
#line 2209 "../src/SourceList/SourceList.vala"
							_tmp51_ = self->priv->text_cell;
#line 2209 "../src/SourceList/SourceList.vala"
							_tmp39_ = mail_source_list_tree_over_cell (self, _tmp49_, _tmp50_, (GtkCellRenderer*) _tmp51_, cell_x);
#line 10851 "SourceList.c"
						} else {
#line 2205 "../src/SourceList/SourceList.vala"
							_tmp39_ = FALSE;
#line 10855 "SourceList.c"
						}
#line 2205 "../src/SourceList/SourceList.vala"
						if (_tmp39_) {
#line 10859 "SourceList.c"
							MailSourceListItem* _tmp52_;
#line 2210 "../src/SourceList/SourceList.vala"
							_tmp52_ = item;
#line 2210 "../src/SourceList/SourceList.vala"
							_tmp38_ = mail_source_list_tree_start_editing_item (self, _tmp52_);
#line 10865 "SourceList.c"
						} else {
#line 2205 "../src/SourceList/SourceList.vala"
							_tmp38_ = FALSE;
#line 10869 "SourceList.c"
						}
#line 2204 "../src/SourceList/SourceList.vala"
						if (_tmp38_) {
#line 2213 "../src/SourceList/SourceList.vala"
							result = TRUE;
#line 2213 "../src/SourceList/SourceList.vala"
							_g_object_unref0 (item);
#line 2213 "../src/SourceList/SourceList.vala"
							_g_object_unref0 (column);
#line 2213 "../src/SourceList/SourceList.vala"
							__vala_GtkTreePath_free0 (path);
#line 2213 "../src/SourceList/SourceList.vala"
							return result;
#line 10883 "SourceList.c"
						}
					}
				}
			}
		}
#line 2169 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (item);
#line 10891 "SourceList.c"
	}
#line 2219 "../src/SourceList/SourceList.vala"
	result = GTK_WIDGET_CLASS (mail_source_list_tree_parent_class)->button_press_event ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), event);
#line 2219 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (column);
#line 2219 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2219 "../src/SourceList/SourceList.vala"
	return result;
#line 10901 "SourceList.c"
}

static gboolean
mail_source_list_tree_over_primary_expander (MailSourceListTree* self,
                                             GtkTreeViewColumn* col,
                                             GtkTreePath* path,
                                             gint x)
{
	GtkTreeIter iter = {0};
	GtkTreeModel* _tmp0_;
	GtkTreeModel* _tmp1_;
	GtkTreeIter _tmp2_ = {0};
	gboolean _tmp3_;
	MailCellRendererExpander* _tmp4_;
	GtkTreeModel* _tmp5_;
	GtkTreeModel* _tmp6_;
	GtkTreeIter _tmp7_;
	MailCellRendererExpander* _tmp8_;
	gboolean _tmp9_;
	gboolean _tmp10_;
	GtkTreeModel* _tmp11_;
	GtkTreeModel* _tmp12_;
	GtkTreeIter _tmp13_;
	gint indentation_level = 0;
	MailSourceListCellRendererSpacer* last_spacer_cell = NULL;
	GeeHashMap* _tmp14_;
	gpointer _tmp15_;
	MailSourceListCellRendererSpacer* _tmp16_;
	gboolean result;
#line 2222 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2222 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (col != NULL, FALSE);
#line 2222 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 2224 "../src/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_view_get_model ((GtkTreeView*) self);
#line 2224 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 2224 "../src/SourceList/SourceList.vala"
	_tmp3_ = gtk_tree_model_get_iter (_tmp1_, &_tmp2_, path);
#line 2224 "../src/SourceList/SourceList.vala"
	iter = _tmp2_;
#line 2224 "../src/SourceList/SourceList.vala"
	if (!_tmp3_) {
#line 2225 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 2225 "../src/SourceList/SourceList.vala"
		return result;
#line 10951 "SourceList.c"
	}
#line 2228 "../src/SourceList/SourceList.vala"
	_tmp4_ = self->priv->primary_expander_cell;
#line 2228 "../src/SourceList/SourceList.vala"
	_tmp5_ = gtk_tree_view_get_model ((GtkTreeView*) self);
#line 2228 "../src/SourceList/SourceList.vala"
	_tmp6_ = _tmp5_;
#line 2228 "../src/SourceList/SourceList.vala"
	_tmp7_ = iter;
#line 2228 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_expander_cell_data_func (self, (GtkCellLayout*) col, (GtkCellRenderer*) _tmp4_, _tmp6_, &_tmp7_);
#line 2230 "../src/SourceList/SourceList.vala"
	_tmp8_ = self->priv->primary_expander_cell;
#line 2230 "../src/SourceList/SourceList.vala"
	_tmp9_ = gtk_cell_renderer_get_visible ((GtkCellRenderer*) _tmp8_);
#line 2230 "../src/SourceList/SourceList.vala"
	_tmp10_ = _tmp9_;
#line 2230 "../src/SourceList/SourceList.vala"
	if (!_tmp10_) {
#line 2231 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 2231 "../src/SourceList/SourceList.vala"
		return result;
#line 10975 "SourceList.c"
	}
#line 2234 "../src/SourceList/SourceList.vala"
	_tmp11_ = gtk_tree_view_get_model ((GtkTreeView*) self);
#line 2234 "../src/SourceList/SourceList.vala"
	_tmp12_ = _tmp11_;
#line 2234 "../src/SourceList/SourceList.vala"
	_tmp13_ = iter;
#line 2234 "../src/SourceList/SourceList.vala"
	if (gtk_tree_model_iter_n_children (_tmp12_, &_tmp13_) < 1) {
#line 2235 "../src/SourceList/SourceList.vala"
		result = FALSE;
#line 2235 "../src/SourceList/SourceList.vala"
		return result;
#line 10989 "SourceList.c"
	}
#line 2243 "../src/SourceList/SourceList.vala"
	indentation_level = gtk_tree_path_get_depth (path);
#line 2244 "../src/SourceList/SourceList.vala"
	_tmp14_ = self->priv->spacer_cells;
#line 2244 "../src/SourceList/SourceList.vala"
	_tmp15_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp14_, (gpointer) ((gintptr) indentation_level));
#line 2244 "../src/SourceList/SourceList.vala"
	last_spacer_cell = (MailSourceListCellRendererSpacer*) _tmp15_;
#line 2246 "../src/SourceList/SourceList.vala"
	_tmp16_ = last_spacer_cell;
#line 2246 "../src/SourceList/SourceList.vala"
	if (_tmp16_ != NULL) {
#line 11003 "SourceList.c"
		gint cell_x = 0;
		gint cell_width = 0;
		MailSourceListCellRendererSpacer* _tmp17_;
		gint _tmp18_ = 0;
		gint _tmp19_ = 0;
		gboolean _tmp20_;
#line 2249 "../src/SourceList/SourceList.vala"
		_tmp17_ = last_spacer_cell;
#line 2249 "../src/SourceList/SourceList.vala"
		_tmp20_ = gtk_tree_view_column_cell_get_position (col, (GtkCellRenderer*) _tmp17_, &_tmp18_, &_tmp19_);
#line 2249 "../src/SourceList/SourceList.vala"
		cell_x = _tmp18_;
#line 2249 "../src/SourceList/SourceList.vala"
		cell_width = _tmp19_;
#line 2249 "../src/SourceList/SourceList.vala"
		if (_tmp20_) {
#line 11020 "SourceList.c"
			gint expander_width = 0;
			MailCellRendererExpander* _tmp21_;
			GtkTextDirection dir = 0;
			gboolean _tmp23_ = FALSE;
#line 2251 "../src/SourceList/SourceList.vala"
			_tmp21_ = self->priv->primary_expander_cell;
#line 2251 "../src/SourceList/SourceList.vala"
			expander_width = mail_source_list_tree_get_cell_width (self, (GtkCellRenderer*) _tmp21_) + 1;
#line 2253 "../src/SourceList/SourceList.vala"
			dir = gtk_widget_get_direction ((GtkWidget*) self);
#line 2254 "../src/SourceList/SourceList.vala"
			if (dir == GTK_TEXT_DIR_NONE) {
#line 2255 "../src/SourceList/SourceList.vala"
				dir = gtk_widget_get_default_direction ();
#line 11035 "SourceList.c"
			}
#line 2258 "../src/SourceList/SourceList.vala"
			if (dir == GTK_TEXT_DIR_LTR) {
#line 11039 "SourceList.c"
				gint indentation_offset = 0;
				gboolean _tmp22_ = FALSE;
#line 2259 "../src/SourceList/SourceList.vala"
				indentation_offset = cell_x + cell_width;
#line 2260 "../src/SourceList/SourceList.vala"
				if (x >= indentation_offset) {
#line 2260 "../src/SourceList/SourceList.vala"
					_tmp22_ = x <= (indentation_offset + expander_width);
#line 11048 "SourceList.c"
				} else {
#line 2260 "../src/SourceList/SourceList.vala"
					_tmp22_ = FALSE;
#line 11052 "SourceList.c"
				}
#line 2260 "../src/SourceList/SourceList.vala"
				result = _tmp22_;
#line 2260 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (last_spacer_cell);
#line 2260 "../src/SourceList/SourceList.vala"
				return result;
#line 11060 "SourceList.c"
			}
#line 2263 "../src/SourceList/SourceList.vala"
			if (x <= cell_x) {
#line 2263 "../src/SourceList/SourceList.vala"
				_tmp23_ = x >= (cell_x - expander_width);
#line 11066 "SourceList.c"
			} else {
#line 2263 "../src/SourceList/SourceList.vala"
				_tmp23_ = FALSE;
#line 11070 "SourceList.c"
			}
#line 2263 "../src/SourceList/SourceList.vala"
			result = _tmp23_;
#line 2263 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (last_spacer_cell);
#line 2263 "../src/SourceList/SourceList.vala"
			return result;
#line 11078 "SourceList.c"
		}
	}
#line 2267 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 2267 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (last_spacer_cell);
#line 2267 "../src/SourceList/SourceList.vala"
	return result;
#line 11087 "SourceList.c"
}

static gboolean
mail_source_list_tree_over_cell (MailSourceListTree* self,
                                 GtkTreeViewColumn* col,
                                 GtkTreePath* path,
                                 GtkCellRenderer* cell,
                                 gint x)
{
	gint cell_x = 0;
	gint cell_width = 0;
	gboolean found = FALSE;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gboolean _tmp2_;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gboolean result;
#line 2270 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2270 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (col != NULL, FALSE);
#line 2270 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 2270 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (cell != NULL, FALSE);
#line 2272 "../src/SourceList/SourceList.vala"
	_tmp2_ = gtk_tree_view_column_cell_get_position (col, cell, &_tmp0_, &_tmp1_);
#line 2272 "../src/SourceList/SourceList.vala"
	cell_x = _tmp0_;
#line 2272 "../src/SourceList/SourceList.vala"
	cell_width = _tmp1_;
#line 2272 "../src/SourceList/SourceList.vala"
	found = _tmp2_;
#line 2273 "../src/SourceList/SourceList.vala"
	if (found) {
#line 2273 "../src/SourceList/SourceList.vala"
		_tmp4_ = x > cell_x;
#line 11126 "SourceList.c"
	} else {
#line 2273 "../src/SourceList/SourceList.vala"
		_tmp4_ = FALSE;
#line 11130 "SourceList.c"
	}
#line 2273 "../src/SourceList/SourceList.vala"
	if (_tmp4_) {
#line 2273 "../src/SourceList/SourceList.vala"
		_tmp3_ = x < (cell_x + cell_width);
#line 11136 "SourceList.c"
	} else {
#line 2273 "../src/SourceList/SourceList.vala"
		_tmp3_ = FALSE;
#line 11140 "SourceList.c"
	}
#line 2273 "../src/SourceList/SourceList.vala"
	result = _tmp3_;
#line 2273 "../src/SourceList/SourceList.vala"
	return result;
#line 11146 "SourceList.c"
}

static gint
mail_source_list_tree_get_cell_width (MailSourceListTree* self,
                                      GtkCellRenderer* cell_renderer)
{
	GtkRequisition min_req = {0};
	GtkRequisition _tmp0_ = {0};
	GtkRequisition _tmp1_;
	gint result;
#line 2276 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 2276 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (cell_renderer != NULL, 0);
#line 2278 "../src/SourceList/SourceList.vala"
	gtk_cell_renderer_get_preferred_size (cell_renderer, (GtkWidget*) self, &_tmp0_, NULL);
#line 2278 "../src/SourceList/SourceList.vala"
	min_req = _tmp0_;
#line 2279 "../src/SourceList/SourceList.vala"
	_tmp1_ = min_req;
#line 2279 "../src/SourceList/SourceList.vala"
	result = _tmp1_.width;
#line 2279 "../src/SourceList/SourceList.vala"
	return result;
#line 11171 "SourceList.c"
}

static gboolean
mail_source_list_tree_real_popup_menu (GtkWidget* base)
{
	MailSourceListTree * self;
	gboolean result;
#line 2282 "../src/SourceList/SourceList.vala"
	self = (MailSourceListTree*) base;
#line 2283 "../src/SourceList/SourceList.vala"
	result = mail_source_list_tree_popup_context_menu (self, NULL, NULL);
#line 2283 "../src/SourceList/SourceList.vala"
	return result;
#line 11185 "SourceList.c"
}

static gboolean
mail_source_list_tree_popup_context_menu (MailSourceListTree* self,
                                          MailSourceListItem* item,
                                          GdkEventButton* event)
{
	gboolean result;
#line 2286 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2287 "../src/SourceList/SourceList.vala"
	if (item == NULL) {
#line 11198 "SourceList.c"
		MailSourceListItem* _tmp0_;
		MailSourceListItem* _tmp1_;
#line 2288 "../src/SourceList/SourceList.vala"
		_tmp0_ = mail_source_list_tree_get_selected_item (self);
#line 2288 "../src/SourceList/SourceList.vala"
		_tmp1_ = _tmp0_;
#line 2288 "../src/SourceList/SourceList.vala"
		item = _tmp1_;
#line 11207 "SourceList.c"
	}
#line 2290 "../src/SourceList/SourceList.vala"
	if (item != NULL) {
#line 11211 "SourceList.c"
		GtkMenu* menu = NULL;
		GtkMenu* _tmp2_;
		GtkMenu* _tmp3_;
#line 2291 "../src/SourceList/SourceList.vala"
		_tmp2_ = mail_source_list_item_get_context_menu (item);
#line 2291 "../src/SourceList/SourceList.vala"
		menu = _tmp2_;
#line 2292 "../src/SourceList/SourceList.vala"
		_tmp3_ = menu;
#line 2292 "../src/SourceList/SourceList.vala"
		if (_tmp3_ != NULL) {
#line 11223 "SourceList.c"
			GtkMenu* _tmp4_;
			GtkMenu* _tmp5_;
#line 2293 "../src/SourceList/SourceList.vala"
			_tmp4_ = menu;
#line 2293 "../src/SourceList/SourceList.vala"
			g_object_set (_tmp4_, "attach-widget", (GtkWidget*) self, NULL);
#line 2294 "../src/SourceList/SourceList.vala"
			_tmp5_ = menu;
#line 2294 "../src/SourceList/SourceList.vala"
			gtk_menu_popup_at_pointer (_tmp5_, (GdkEvent*) event);
#line 2295 "../src/SourceList/SourceList.vala"
			if (event == NULL) {
#line 11236 "SourceList.c"
				GtkMenu* _tmp6_;
#line 2296 "../src/SourceList/SourceList.vala"
				_tmp6_ = menu;
#line 2296 "../src/SourceList/SourceList.vala"
				gtk_menu_shell_select_first ((GtkMenuShell*) _tmp6_, FALSE);
#line 11242 "SourceList.c"
			}
#line 2299 "../src/SourceList/SourceList.vala"
			result = TRUE;
#line 2299 "../src/SourceList/SourceList.vala"
			_g_object_unref0 (menu);
#line 2299 "../src/SourceList/SourceList.vala"
			return result;
#line 11250 "SourceList.c"
		}
#line 2290 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (menu);
#line 11254 "SourceList.c"
	}
#line 2303 "../src/SourceList/SourceList.vala"
	result = FALSE;
#line 2303 "../src/SourceList/SourceList.vala"
	return result;
#line 11260 "SourceList.c"
}

static MailSourceListItem*
mail_source_list_tree_get_item_from_model (GtkTreeModel* model,
                                           GtkTreeIter* iter)
{
	MailSourceListDataModel* data_model = NULL;
	MailSourceListDataModel* _tmp0_;
	GtkTreeIter _tmp1_;
	MailSourceListItem* _tmp2_;
	MailSourceListItem* result;
#line 2306 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (model != NULL, NULL);
#line 2306 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, NULL);
#line 2307 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_DATA_MODEL (model) ? ((MailSourceListDataModel*) model) : NULL);
#line 2307 "../src/SourceList/SourceList.vala"
	data_model = _tmp0_;
#line 2308 "../src/SourceList/SourceList.vala"
	_vala_assert (data_model != NULL, "data_model != null");
#line 2309 "../src/SourceList/SourceList.vala"
	_tmp1_ = *iter;
#line 2309 "../src/SourceList/SourceList.vala"
	_tmp2_ = mail_source_list_data_model_get_item (data_model, &_tmp1_);
#line 2309 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2309 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (data_model);
#line 2309 "../src/SourceList/SourceList.vala"
	return result;
#line 11292 "SourceList.c"
}

static void
mail_source_list_tree_spacer_cell_data_func (GtkCellLayout* layout,
                                             GtkCellRenderer* renderer,
                                             GtkTreeModel* model,
                                             GtkTreeIter* iter)
{
	MailSourceListCellRendererSpacer* spacer = NULL;
	MailSourceListCellRendererSpacer* _tmp0_;
	MailSourceListCellRendererSpacer* _tmp1_;
	MailSourceListCellRendererSpacer* _tmp2_;
	gint _tmp3_;
	gint _tmp4_;
	GtkTreePath* path = NULL;
	GtkTreeIter _tmp5_;
	GtkTreePath* _tmp6_;
	gint level = 0;
	GtkTreePath* _tmp7_;
	MailSourceListCellRendererSpacer* _tmp9_;
	gint _tmp10_;
	gint _tmp11_;
#line 2312 "../src/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2312 "../src/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2312 "../src/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2312 "../src/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2318 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_CELL_RENDERER_SPACER (renderer) ? ((MailSourceListCellRendererSpacer*) renderer) : NULL);
#line 2318 "../src/SourceList/SourceList.vala"
	spacer = _tmp0_;
#line 2319 "../src/SourceList/SourceList.vala"
	_tmp1_ = spacer;
#line 2319 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "spacer != null");
#line 2320 "../src/SourceList/SourceList.vala"
	_tmp2_ = spacer;
#line 2320 "../src/SourceList/SourceList.vala"
	_tmp3_ = mail_source_list_cell_renderer_spacer_get_level (_tmp2_);
#line 2320 "../src/SourceList/SourceList.vala"
	_tmp4_ = _tmp3_;
#line 2320 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp4_ > 0, "spacer.level > 0");
#line 2322 "../src/SourceList/SourceList.vala"
	_tmp5_ = *iter;
#line 2322 "../src/SourceList/SourceList.vala"
	_tmp6_ = gtk_tree_model_get_path (model, &_tmp5_);
#line 2322 "../src/SourceList/SourceList.vala"
	path = _tmp6_;
#line 2324 "../src/SourceList/SourceList.vala"
	level = -1;
#line 2325 "../src/SourceList/SourceList.vala"
	_tmp7_ = path;
#line 2325 "../src/SourceList/SourceList.vala"
	if (_tmp7_ != NULL) {
#line 11351 "SourceList.c"
		GtkTreePath* _tmp8_;
#line 2326 "../src/SourceList/SourceList.vala"
		_tmp8_ = path;
#line 2326 "../src/SourceList/SourceList.vala"
		level = gtk_tree_path_get_depth (_tmp8_);
#line 11357 "SourceList.c"
	}
#line 2328 "../src/SourceList/SourceList.vala"
	_tmp9_ = spacer;
#line 2328 "../src/SourceList/SourceList.vala"
	_tmp10_ = mail_source_list_cell_renderer_spacer_get_level (_tmp9_);
#line 2328 "../src/SourceList/SourceList.vala"
	_tmp11_ = _tmp10_;
#line 2328 "../src/SourceList/SourceList.vala"
	gtk_cell_renderer_set_visible (renderer, _tmp11_ <= level);
#line 2312 "../src/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2312 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (spacer);
#line 11371 "SourceList.c"
}

static void
mail_source_list_tree_name_cell_data_func (MailSourceListTree* self,
                                           GtkCellLayout* layout,
                                           GtkCellRenderer* renderer,
                                           GtkTreeModel* model,
                                           GtkTreeIter* iter)
{
	GtkCellRendererText* text_renderer = NULL;
	GtkCellRendererText* _tmp0_;
	GtkCellRendererText* _tmp1_;
	GString* text = NULL;
	GString* _tmp2_;
	PangoWeight weight = 0;
	gboolean use_markup = FALSE;
	MailSourceListItem* item = NULL;
	GtkTreeIter _tmp3_;
	MailSourceListItem* _tmp4_;
	MailSourceListItem* _tmp5_;
	GtkCellRendererText* _tmp20_;
#line 2331 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2331 "../src/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2331 "../src/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2331 "../src/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2331 "../src/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2337 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (renderer, gtk_cell_renderer_text_get_type ()) ? ((GtkCellRendererText*) renderer) : NULL);
#line 2337 "../src/SourceList/SourceList.vala"
	text_renderer = _tmp0_;
#line 2338 "../src/SourceList/SourceList.vala"
	_tmp1_ = text_renderer;
#line 2338 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "text_renderer != null");
#line 2340 "../src/SourceList/SourceList.vala"
	_tmp2_ = g_string_new ("");
#line 2340 "../src/SourceList/SourceList.vala"
	text = _tmp2_;
#line 2341 "../src/SourceList/SourceList.vala"
	weight = PANGO_WEIGHT_NORMAL;
#line 2342 "../src/SourceList/SourceList.vala"
	use_markup = FALSE;
#line 2344 "../src/SourceList/SourceList.vala"
	_tmp3_ = *iter;
#line 2344 "../src/SourceList/SourceList.vala"
	_tmp4_ = mail_source_list_tree_get_item_from_model (model, &_tmp3_);
#line 2344 "../src/SourceList/SourceList.vala"
	item = _tmp4_;
#line 2345 "../src/SourceList/SourceList.vala"
	_tmp5_ = item;
#line 2345 "../src/SourceList/SourceList.vala"
	if (_tmp5_ != NULL) {
#line 11429 "SourceList.c"
		MailSourceListItem* _tmp6_;
		const gchar* _tmp7_;
		const gchar* _tmp8_;
		MailSourceListDataModel* _tmp17_;
		MailSourceListItem* _tmp18_;
		GtkTreeIter _tmp19_;
#line 2346 "../src/SourceList/SourceList.vala"
		_tmp6_ = item;
#line 2346 "../src/SourceList/SourceList.vala"
		_tmp7_ = mail_source_list_item_get_markup (_tmp6_);
#line 2346 "../src/SourceList/SourceList.vala"
		_tmp8_ = _tmp7_;
#line 2346 "../src/SourceList/SourceList.vala"
		if (_tmp8_ != NULL) {
#line 11444 "SourceList.c"
			GString* _tmp9_;
			MailSourceListItem* _tmp10_;
			const gchar* _tmp11_;
			const gchar* _tmp12_;
#line 2347 "../src/SourceList/SourceList.vala"
			_tmp9_ = text;
#line 2347 "../src/SourceList/SourceList.vala"
			_tmp10_ = item;
#line 2347 "../src/SourceList/SourceList.vala"
			_tmp11_ = mail_source_list_item_get_markup (_tmp10_);
#line 2347 "../src/SourceList/SourceList.vala"
			_tmp12_ = _tmp11_;
#line 2347 "../src/SourceList/SourceList.vala"
			g_string_append (_tmp9_, _tmp12_);
#line 2348 "../src/SourceList/SourceList.vala"
			use_markup = TRUE;
#line 11461 "SourceList.c"
		} else {
			GString* _tmp13_;
			MailSourceListItem* _tmp14_;
			const gchar* _tmp15_;
			const gchar* _tmp16_;
#line 2350 "../src/SourceList/SourceList.vala"
			_tmp13_ = text;
#line 2350 "../src/SourceList/SourceList.vala"
			_tmp14_ = item;
#line 2350 "../src/SourceList/SourceList.vala"
			_tmp15_ = mail_source_list_item_get_name (_tmp14_);
#line 2350 "../src/SourceList/SourceList.vala"
			_tmp16_ = _tmp15_;
#line 2350 "../src/SourceList/SourceList.vala"
			g_string_append (_tmp13_, _tmp16_);
#line 11477 "SourceList.c"
		}
#line 2353 "../src/SourceList/SourceList.vala"
		_tmp17_ = self->priv->_data_model;
#line 2353 "../src/SourceList/SourceList.vala"
		_tmp18_ = item;
#line 2353 "../src/SourceList/SourceList.vala"
		_tmp19_ = *iter;
#line 2353 "../src/SourceList/SourceList.vala"
		if (mail_source_list_data_model_is_category (_tmp17_, _tmp18_, &_tmp19_, NULL)) {
#line 2354 "../src/SourceList/SourceList.vala"
			weight = PANGO_WEIGHT_BOLD;
#line 11489 "SourceList.c"
		}
	}
#line 2357 "../src/SourceList/SourceList.vala"
	_tmp20_ = text_renderer;
#line 2357 "../src/SourceList/SourceList.vala"
	g_object_set (_tmp20_, "weight", (gint) weight, NULL);
#line 2358 "../src/SourceList/SourceList.vala"
	if (use_markup) {
#line 11498 "SourceList.c"
		GtkCellRendererText* _tmp21_;
		GString* _tmp22_;
		const gchar* _tmp23_;
#line 2359 "../src/SourceList/SourceList.vala"
		_tmp21_ = text_renderer;
#line 2359 "../src/SourceList/SourceList.vala"
		_tmp22_ = text;
#line 2359 "../src/SourceList/SourceList.vala"
		_tmp23_ = _tmp22_->str;
#line 2359 "../src/SourceList/SourceList.vala"
		g_object_set (_tmp21_, "markup", _tmp23_, NULL);
#line 11510 "SourceList.c"
	} else {
		GtkCellRendererText* _tmp24_;
		GString* _tmp25_;
		const gchar* _tmp26_;
#line 2361 "../src/SourceList/SourceList.vala"
		_tmp24_ = text_renderer;
#line 2361 "../src/SourceList/SourceList.vala"
		_tmp25_ = text;
#line 2361 "../src/SourceList/SourceList.vala"
		_tmp26_ = _tmp25_->str;
#line 2361 "../src/SourceList/SourceList.vala"
		g_object_set (_tmp24_, "text", _tmp26_, NULL);
#line 11523 "SourceList.c"
	}
#line 2331 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 2331 "../src/SourceList/SourceList.vala"
	_g_string_free0 (text);
#line 2331 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (text_renderer);
#line 11531 "SourceList.c"
}

static void
mail_source_list_tree_badge_cell_data_func (MailSourceListTree* self,
                                            GtkCellLayout* layout,
                                            GtkCellRenderer* renderer,
                                            GtkTreeModel* model,
                                            GtkTreeIter* iter)
{
	MailCellRendererBadge* badge_renderer = NULL;
	MailCellRendererBadge* _tmp0_;
	MailCellRendererBadge* _tmp1_;
	gchar* text = NULL;
	gchar* _tmp2_;
	gboolean visible = FALSE;
	MailSourceListItem* item = NULL;
	GtkTreeIter _tmp3_;
	MailSourceListItem* _tmp4_;
	MailSourceListItem* _tmp5_;
	MailCellRendererBadge* _tmp23_;
	MailCellRendererBadge* _tmp24_;
	const gchar* _tmp25_;
#line 2365 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2365 "../src/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2365 "../src/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2365 "../src/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2365 "../src/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2371 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (MAIL_IS_CELL_RENDERER_BADGE (renderer) ? ((MailCellRendererBadge*) renderer) : NULL);
#line 2371 "../src/SourceList/SourceList.vala"
	badge_renderer = _tmp0_;
#line 2372 "../src/SourceList/SourceList.vala"
	_tmp1_ = badge_renderer;
#line 2372 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "badge_renderer != null");
#line 2374 "../src/SourceList/SourceList.vala"
	_tmp2_ = g_strdup ("");
#line 2374 "../src/SourceList/SourceList.vala"
	text = _tmp2_;
#line 2375 "../src/SourceList/SourceList.vala"
	visible = FALSE;
#line 2377 "../src/SourceList/SourceList.vala"
	_tmp3_ = *iter;
#line 2377 "../src/SourceList/SourceList.vala"
	_tmp4_ = mail_source_list_tree_get_item_from_model (model, &_tmp3_);
#line 2377 "../src/SourceList/SourceList.vala"
	item = _tmp4_;
#line 2378 "../src/SourceList/SourceList.vala"
	_tmp5_ = item;
#line 2378 "../src/SourceList/SourceList.vala"
	if (_tmp5_ != NULL) {
#line 11588 "SourceList.c"
		gboolean _tmp6_ = FALSE;
		gboolean _tmp7_ = FALSE;
		MailSourceListDataModel* _tmp8_;
		MailSourceListItem* _tmp9_;
		GtkTreeIter _tmp10_;
#line 2380 "../src/SourceList/SourceList.vala"
		_tmp8_ = self->priv->_data_model;
#line 2380 "../src/SourceList/SourceList.vala"
		_tmp9_ = item;
#line 2380 "../src/SourceList/SourceList.vala"
		_tmp10_ = *iter;
#line 2380 "../src/SourceList/SourceList.vala"
		if (!mail_source_list_data_model_is_category (_tmp8_, _tmp9_, &_tmp10_, NULL)) {
#line 11602 "SourceList.c"
			MailSourceListItem* _tmp11_;
			const gchar* _tmp12_;
			const gchar* _tmp13_;
#line 2381 "../src/SourceList/SourceList.vala"
			_tmp11_ = item;
#line 2381 "../src/SourceList/SourceList.vala"
			_tmp12_ = mail_source_list_item_get_badge (_tmp11_);
#line 2381 "../src/SourceList/SourceList.vala"
			_tmp13_ = _tmp12_;
#line 2381 "../src/SourceList/SourceList.vala"
			_tmp7_ = _tmp13_ != NULL;
#line 11614 "SourceList.c"
		} else {
#line 2380 "../src/SourceList/SourceList.vala"
			_tmp7_ = FALSE;
#line 11618 "SourceList.c"
		}
#line 2380 "../src/SourceList/SourceList.vala"
		if (_tmp7_) {
#line 11622 "SourceList.c"
			MailSourceListItem* _tmp14_;
			const gchar* _tmp15_;
			const gchar* _tmp16_;
			gchar* _tmp17_;
			gchar* _tmp18_;
#line 2382 "../src/SourceList/SourceList.vala"
			_tmp14_ = item;
#line 2382 "../src/SourceList/SourceList.vala"
			_tmp15_ = mail_source_list_item_get_badge (_tmp14_);
#line 2382 "../src/SourceList/SourceList.vala"
			_tmp16_ = _tmp15_;
#line 2382 "../src/SourceList/SourceList.vala"
			_tmp17_ = string_strip (_tmp16_);
#line 2382 "../src/SourceList/SourceList.vala"
			_tmp18_ = _tmp17_;
#line 2382 "../src/SourceList/SourceList.vala"
			_tmp6_ = g_strcmp0 (_tmp18_, "") != 0;
#line 2382 "../src/SourceList/SourceList.vala"
			_g_free0 (_tmp18_);
#line 11642 "SourceList.c"
		} else {
#line 2380 "../src/SourceList/SourceList.vala"
			_tmp6_ = FALSE;
#line 11646 "SourceList.c"
		}
#line 2380 "../src/SourceList/SourceList.vala"
		visible = _tmp6_;
#line 2384 "../src/SourceList/SourceList.vala"
		if (visible) {
#line 11652 "SourceList.c"
			MailSourceListItem* _tmp19_;
			const gchar* _tmp20_;
			const gchar* _tmp21_;
			gchar* _tmp22_;
#line 2385 "../src/SourceList/SourceList.vala"
			_tmp19_ = item;
#line 2385 "../src/SourceList/SourceList.vala"
			_tmp20_ = mail_source_list_item_get_badge (_tmp19_);
#line 2385 "../src/SourceList/SourceList.vala"
			_tmp21_ = _tmp20_;
#line 2385 "../src/SourceList/SourceList.vala"
			_tmp22_ = g_strdup (_tmp21_);
#line 2385 "../src/SourceList/SourceList.vala"
			_g_free0 (text);
#line 2385 "../src/SourceList/SourceList.vala"
			text = _tmp22_;
#line 11669 "SourceList.c"
		}
	}
#line 2388 "../src/SourceList/SourceList.vala"
	_tmp23_ = badge_renderer;
#line 2388 "../src/SourceList/SourceList.vala"
	gtk_cell_renderer_set_visible ((GtkCellRenderer*) _tmp23_, visible);
#line 2389 "../src/SourceList/SourceList.vala"
	_tmp24_ = badge_renderer;
#line 2389 "../src/SourceList/SourceList.vala"
	_tmp25_ = text;
#line 2389 "../src/SourceList/SourceList.vala"
	mail_cell_renderer_badge_set_text (_tmp24_, _tmp25_);
#line 2365 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 2365 "../src/SourceList/SourceList.vala"
	_g_free0 (text);
#line 2365 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (badge_renderer);
#line 11688 "SourceList.c"
}

static void
mail_source_list_tree_icon_cell_data_func (MailSourceListTree* self,
                                           GtkCellLayout* layout,
                                           GtkCellRenderer* renderer,
                                           GtkTreeModel* model,
                                           GtkTreeIter* iter)
{
	MailSourceListCellRendererIcon* icon_renderer = NULL;
	MailSourceListCellRendererIcon* _tmp0_;
	MailSourceListCellRendererIcon* _tmp1_;
	gboolean visible = FALSE;
	GIcon* icon = NULL;
	MailSourceListItem* item = NULL;
	GtkTreeIter _tmp2_;
	MailSourceListItem* _tmp3_;
	MailSourceListItem* _tmp4_;
	gboolean _tmp20_ = FALSE;
	MailSourceListCellRendererIcon* _tmp22_;
	GIcon* _tmp23_ = NULL;
	MailSourceListCellRendererIcon* _tmp25_;
#line 2392 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2392 "../src/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2392 "../src/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2392 "../src/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2392 "../src/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2397 "../src/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_CELL_RENDERER_ICON (renderer) ? ((MailSourceListCellRendererIcon*) renderer) : NULL);
#line 2397 "../src/SourceList/SourceList.vala"
	icon_renderer = _tmp0_;
#line 2398 "../src/SourceList/SourceList.vala"
	_tmp1_ = icon_renderer;
#line 2398 "../src/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "icon_renderer != null");
#line 2400 "../src/SourceList/SourceList.vala"
	visible = FALSE;
#line 2401 "../src/SourceList/SourceList.vala"
	icon = NULL;
#line 2403 "../src/SourceList/SourceList.vala"
	_tmp2_ = *iter;
#line 2403 "../src/SourceList/SourceList.vala"
	_tmp3_ = mail_source_list_tree_get_item_from_model (model, &_tmp2_);
#line 2403 "../src/SourceList/SourceList.vala"
	item = _tmp3_;
#line 2404 "../src/SourceList/SourceList.vala"
	_tmp4_ = item;
#line 2404 "../src/SourceList/SourceList.vala"
	if (_tmp4_ != NULL) {
#line 11743 "SourceList.c"
		MailSourceListDataModel* _tmp5_;
		MailSourceListItem* _tmp6_;
		GtkTreeIter _tmp7_;
#line 2406 "../src/SourceList/SourceList.vala"
		_tmp5_ = self->priv->_data_model;
#line 2406 "../src/SourceList/SourceList.vala"
		_tmp6_ = item;
#line 2406 "../src/SourceList/SourceList.vala"
		_tmp7_ = *iter;
#line 2406 "../src/SourceList/SourceList.vala"
		visible = !mail_source_list_data_model_is_category (_tmp5_, _tmp6_, &_tmp7_, NULL);
#line 2408 "../src/SourceList/SourceList.vala"
		if (visible) {
#line 11757 "SourceList.c"
			MailSourceListCellRendererIcon* _tmp8_;
			MailSourceListCellRendererIcon* _tmp9_;
#line 2409 "../src/SourceList/SourceList.vala"
			_tmp8_ = icon_renderer;
#line 2409 "../src/SourceList/SourceList.vala"
			_tmp9_ = self->priv->icon_cell;
#line 2409 "../src/SourceList/SourceList.vala"
			if (_tmp8_ == _tmp9_) {
#line 11766 "SourceList.c"
				MailSourceListItem* _tmp10_;
				GIcon* _tmp11_;
				GIcon* _tmp12_;
				GIcon* _tmp13_;
#line 2410 "../src/SourceList/SourceList.vala"
				_tmp10_ = item;
#line 2410 "../src/SourceList/SourceList.vala"
				_tmp11_ = mail_source_list_item_get_icon (_tmp10_);
#line 2410 "../src/SourceList/SourceList.vala"
				_tmp12_ = _tmp11_;
#line 2410 "../src/SourceList/SourceList.vala"
				_tmp13_ = _g_object_ref0 (_tmp12_);
#line 2410 "../src/SourceList/SourceList.vala"
				_g_object_unref0 (icon);
#line 2410 "../src/SourceList/SourceList.vala"
				icon = _tmp13_;
#line 11783 "SourceList.c"
			} else {
				MailSourceListCellRendererIcon* _tmp14_;
				MailSourceListCellRendererIcon* _tmp15_;
#line 2411 "../src/SourceList/SourceList.vala"
				_tmp14_ = icon_renderer;
#line 2411 "../src/SourceList/SourceList.vala"
				_tmp15_ = self->priv->activatable_cell;
#line 2411 "../src/SourceList/SourceList.vala"
				if (_tmp14_ == _tmp15_) {
#line 11793 "SourceList.c"
					MailSourceListItem* _tmp16_;
					GIcon* _tmp17_;
					GIcon* _tmp18_;
					GIcon* _tmp19_;
#line 2412 "../src/SourceList/SourceList.vala"
					_tmp16_ = item;
#line 2412 "../src/SourceList/SourceList.vala"
					_tmp17_ = mail_source_list_item_get_activatable (_tmp16_);
#line 2412 "../src/SourceList/SourceList.vala"
					_tmp18_ = _tmp17_;
#line 2412 "../src/SourceList/SourceList.vala"
					_tmp19_ = _g_object_ref0 (_tmp18_);
#line 2412 "../src/SourceList/SourceList.vala"
					_g_object_unref0 (icon);
#line 2412 "../src/SourceList/SourceList.vala"
					icon = _tmp19_;
#line 11810 "SourceList.c"
				} else {
#line 2414 "../src/SourceList/SourceList.vala"
					g_assert_not_reached ();
#line 11814 "SourceList.c"
				}
			}
		}
	}
#line 2418 "../src/SourceList/SourceList.vala"
	if (visible) {
#line 11821 "SourceList.c"
		GIcon* _tmp21_;
#line 2418 "../src/SourceList/SourceList.vala"
		_tmp21_ = icon;
#line 2418 "../src/SourceList/SourceList.vala"
		_tmp20_ = _tmp21_ != NULL;
#line 11827 "SourceList.c"
	} else {
#line 2418 "../src/SourceList/SourceList.vala"
		_tmp20_ = FALSE;
#line 11831 "SourceList.c"
	}
#line 2418 "../src/SourceList/SourceList.vala"
	visible = _tmp20_;
#line 2420 "../src/SourceList/SourceList.vala"
	_tmp22_ = icon_renderer;
#line 2420 "../src/SourceList/SourceList.vala"
	gtk_cell_renderer_set_visible ((GtkCellRenderer*) _tmp22_, visible);
#line 2421 "../src/SourceList/SourceList.vala"
	if (visible) {
#line 11841 "SourceList.c"
		GIcon* _tmp24_;
#line 2421 "../src/SourceList/SourceList.vala"
		_tmp24_ = icon;
#line 2421 "../src/SourceList/SourceList.vala"
		_tmp23_ = _tmp24_;
#line 11847 "SourceList.c"
	} else {
#line 2421 "../src/SourceList/SourceList.vala"
		_tmp23_ = NULL;
#line 11851 "SourceList.c"
	}
#line 2421 "../src/SourceList/SourceList.vala"
	_tmp25_ = icon_renderer;
#line 2421 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRendererPixbuf*) _tmp25_, "gicon", _tmp23_, NULL);
#line 2392 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 2392 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (icon);
#line 2392 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (icon_renderer);
#line 11863 "SourceList.c"
}

/**
         * Controls expander visibility.
         */
static void
mail_source_list_tree_expander_cell_data_func (MailSourceListTree* self,
                                               GtkCellLayout* layout,
                                               GtkCellRenderer* renderer,
                                               GtkTreeModel* model,
                                               GtkTreeIter* iter)
{
	MailSourceListItem* item = NULL;
	GtkTreeIter _tmp0_;
	MailSourceListItem* _tmp1_;
	MailSourceListItem* _tmp2_;
	MailCellRendererExpander* _tmp12_;
#line 2427 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2427 "../src/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2427 "../src/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2427 "../src/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2427 "../src/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2433 "../src/SourceList/SourceList.vala"
	_tmp0_ = *iter;
#line 2433 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_tree_get_item_from_model (model, &_tmp0_);
#line 2433 "../src/SourceList/SourceList.vala"
	item = _tmp1_;
#line 2434 "../src/SourceList/SourceList.vala"
	_tmp2_ = item;
#line 2434 "../src/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 11901 "SourceList.c"
		MailSourceListExpandableItem* expandable_item = NULL;
		MailSourceListItem* _tmp3_;
		MailSourceListExpandableItem* _tmp4_;
		MailSourceListExpandableItem* _tmp5_;
#line 2438 "../src/SourceList/SourceList.vala"
		_tmp3_ = item;
#line 2438 "../src/SourceList/SourceList.vala"
		_tmp4_ = _g_object_ref0 (MAIL_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp3_) ? ((MailSourceListExpandableItem*) _tmp3_) : NULL);
#line 2438 "../src/SourceList/SourceList.vala"
		expandable_item = _tmp4_;
#line 2439 "../src/SourceList/SourceList.vala"
		_tmp5_ = expandable_item;
#line 2439 "../src/SourceList/SourceList.vala"
		if (_tmp5_ != NULL) {
#line 11916 "SourceList.c"
			gboolean _tmp6_ = FALSE;
			gboolean _tmp7_;
			gboolean _tmp8_;
#line 2440 "../src/SourceList/SourceList.vala"
			g_object_get (renderer, "is-expander", &_tmp7_, NULL);
#line 2440 "../src/SourceList/SourceList.vala"
			_tmp8_ = _tmp7_;
#line 2440 "../src/SourceList/SourceList.vala"
			if (_tmp8_) {
#line 11926 "SourceList.c"
				MailSourceListExpandableItem* _tmp9_;
				gboolean _tmp10_;
				gboolean _tmp11_;
#line 2440 "../src/SourceList/SourceList.vala"
				_tmp9_ = expandable_item;
#line 2440 "../src/SourceList/SourceList.vala"
				_tmp10_ = mail_source_list_expandable_item_get_collapsible (_tmp9_);
#line 2440 "../src/SourceList/SourceList.vala"
				_tmp11_ = _tmp10_;
#line 2440 "../src/SourceList/SourceList.vala"
				_tmp6_ = _tmp11_;
#line 11938 "SourceList.c"
			} else {
#line 2440 "../src/SourceList/SourceList.vala"
				_tmp6_ = FALSE;
#line 11942 "SourceList.c"
			}
#line 2440 "../src/SourceList/SourceList.vala"
			g_object_set (renderer, "is-expander", _tmp6_, NULL);
#line 11946 "SourceList.c"
		}
#line 2434 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (expandable_item);
#line 11950 "SourceList.c"
	}
#line 2443 "../src/SourceList/SourceList.vala"
	_tmp12_ = self->priv->primary_expander_cell;
#line 2443 "../src/SourceList/SourceList.vala"
	if (renderer == G_TYPE_CHECK_INSTANCE_CAST (_tmp12_, gtk_cell_renderer_get_type (), GtkCellRenderer)) {
#line 11956 "SourceList.c"
		MailSourceListDataModel* _tmp13_;
		GtkTreeIter _tmp14_;
#line 2444 "../src/SourceList/SourceList.vala"
		_tmp13_ = self->priv->_data_model;
#line 2444 "../src/SourceList/SourceList.vala"
		_tmp14_ = *iter;
#line 2444 "../src/SourceList/SourceList.vala"
		gtk_cell_renderer_set_visible (renderer, !mail_source_list_data_model_is_iter_at_root_level (_tmp13_, &_tmp14_));
#line 11965 "SourceList.c"
	} else {
		MailCellRendererExpander* _tmp15_;
#line 2445 "../src/SourceList/SourceList.vala"
		_tmp15_ = self->priv->secondary_expander_cell;
#line 2445 "../src/SourceList/SourceList.vala"
		if (renderer == G_TYPE_CHECK_INSTANCE_CAST (_tmp15_, gtk_cell_renderer_get_type (), GtkCellRenderer)) {
#line 11972 "SourceList.c"
			MailSourceListDataModel* _tmp16_;
			MailSourceListItem* _tmp17_;
			GtkTreeIter _tmp18_;
#line 2446 "../src/SourceList/SourceList.vala"
			_tmp16_ = self->priv->_data_model;
#line 2446 "../src/SourceList/SourceList.vala"
			_tmp17_ = item;
#line 2446 "../src/SourceList/SourceList.vala"
			_tmp18_ = *iter;
#line 2446 "../src/SourceList/SourceList.vala"
			gtk_cell_renderer_set_visible (renderer, mail_source_list_data_model_is_category (_tmp16_, _tmp17_, &_tmp18_, NULL));
#line 11984 "SourceList.c"
		} else {
#line 2448 "../src/SourceList/SourceList.vala"
			g_assert_not_reached ();
#line 11988 "SourceList.c"
		}
	}
#line 2427 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 11993 "SourceList.c"
}

static MailSourceListDataModel*
mail_source_list_tree_get_data_model (MailSourceListTree* self)
{
	MailSourceListDataModel* result;
	MailSourceListDataModel* _tmp0_;
#line 1432 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1432 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 1432 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 1432 "../src/SourceList/SourceList.vala"
	return result;
#line 12009 "SourceList.c"
}

static void
mail_source_list_tree_set_data_model (MailSourceListTree* self,
                                      MailSourceListDataModel* value)
{
	MailSourceListDataModel* old_value;
#line 1432 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1432 "../src/SourceList/SourceList.vala"
	old_value = mail_source_list_tree_get_data_model (self);
#line 1432 "../src/SourceList/SourceList.vala"
	if (old_value != value) {
#line 12023 "SourceList.c"
		MailSourceListDataModel* _tmp0_;
#line 1432 "../src/SourceList/SourceList.vala"
		_tmp0_ = _g_object_ref0 (value);
#line 1432 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->_data_model);
#line 1432 "../src/SourceList/SourceList.vala"
		self->priv->_data_model = _tmp0_;
#line 1432 "../src/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, mail_source_list_tree_properties[MAIL_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY]);
#line 12033 "SourceList.c"
	}
}

static MailSourceListItem*
mail_source_list_tree_get_selected_item (MailSourceListTree* self)
{
	MailSourceListItem* result;
	MailSourceListItem* _tmp0_;
#line 1437 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1437 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->selected;
#line 1437 "../src/SourceList/SourceList.vala"
	result = _tmp0_;
#line 1437 "../src/SourceList/SourceList.vala"
	return result;
#line 12050 "SourceList.c"
}

static void
mail_source_list_tree_set_selected_item (MailSourceListTree* self,
                                         MailSourceListItem* value)
{
#line 1438 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1438 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_set_selected (self, value, TRUE);
#line 1438 "../src/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, mail_source_list_tree_properties[MAIL_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY]);
#line 12063 "SourceList.c"
}

static gboolean
mail_source_list_tree_get_editing (MailSourceListTree* self)
{
	gboolean result;
	GtkCellRendererText* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 1442 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1442 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->text_cell;
#line 1442 "../src/SourceList/SourceList.vala"
	g_object_get ((GtkCellRenderer*) _tmp0_, "editing", &_tmp1_, NULL);
#line 1442 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 1442 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 1442 "../src/SourceList/SourceList.vala"
	return result;
#line 12085 "SourceList.c"
}

static PangoEllipsizeMode
mail_source_list_tree_get_ellipsize_mode (MailSourceListTree* self)
{
	PangoEllipsizeMode result;
	GtkCellRendererText* _tmp0_;
	PangoEllipsizeMode _tmp1_;
	PangoEllipsizeMode _tmp2_;
#line 1446 "../src/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 1446 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->text_cell;
#line 1446 "../src/SourceList/SourceList.vala"
	g_object_get (_tmp0_, "ellipsize", &_tmp1_, NULL);
#line 1446 "../src/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 1446 "../src/SourceList/SourceList.vala"
	result = _tmp2_;
#line 1446 "../src/SourceList/SourceList.vala"
	return result;
#line 12107 "SourceList.c"
}

static void
mail_source_list_tree_set_ellipsize_mode (MailSourceListTree* self,
                                          PangoEllipsizeMode value)
{
	GtkCellRendererText* _tmp0_;
#line 1447 "../src/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1447 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->text_cell;
#line 1447 "../src/SourceList/SourceList.vala"
	g_object_set (_tmp0_, "ellipsize", value, NULL);
#line 1447 "../src/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, mail_source_list_tree_properties[MAIL_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY]);
#line 12123 "SourceList.c"
}

static void
_mail_source_list_tree_expander_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                          GtkCellRenderer* cell,
                                                                          GtkTreeModel* tree_model,
                                                                          GtkTreeIter* iter,
                                                                          gpointer self)
{
#line 1558 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_expander_cell_data_func ((MailSourceListTree*) self, cell_layout, cell, tree_model, iter);
#line 12135 "SourceList.c"
}

static void
_mail_source_list_tree_on_activatable_activated_mail_source_list_cell_renderer_icon_activated (MailSourceListCellRendererIcon* _sender,
                                                                                               const gchar* path,
                                                                                               gpointer self)
{
#line 1562 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_on_activatable_activated ((MailSourceListTree*) self, path);
#line 12145 "SourceList.c"
}

static void
_mail_source_list_tree_icon_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                      GtkCellRenderer* cell,
                                                                      GtkTreeModel* tree_model,
                                                                      GtkTreeIter* iter,
                                                                      gpointer self)
{
#line 1564 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_icon_cell_data_func ((MailSourceListTree*) self, cell_layout, cell, tree_model, iter);
#line 12157 "SourceList.c"
}

static void
_mail_source_list_tree_badge_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                       GtkCellRenderer* cell,
                                                                       GtkTreeModel* tree_model,
                                                                       GtkTreeIter* iter,
                                                                       gpointer self)
{
#line 1570 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_badge_cell_data_func ((MailSourceListTree*) self, cell_layout, cell, tree_model, iter);
#line 12169 "SourceList.c"
}

static void
_mail_source_list_tree_on_editing_started_gtk_cell_renderer_editing_started (GtkCellRenderer* _sender,
                                                                             GtkCellEditable* editable,
                                                                             const gchar* path,
                                                                             gpointer self)
{
#line 1575 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_on_editing_started ((MailSourceListTree*) self, editable, path);
#line 12180 "SourceList.c"
}

static void
_mail_source_list_tree_on_editing_canceled_gtk_cell_renderer_editing_canceled (GtkCellRenderer* _sender,
                                                                               gpointer self)
{
#line 1576 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_on_editing_canceled ((MailSourceListTree*) self);
#line 12189 "SourceList.c"
}

static void
_mail_source_list_tree_name_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                      GtkCellRenderer* cell,
                                                                      GtkTreeModel* tree_model,
                                                                      GtkTreeIter* iter,
                                                                      gpointer self)
{
#line 1580 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_name_cell_data_func ((MailSourceListTree*) self, cell_layout, cell, tree_model, iter);
#line 12201 "SourceList.c"
}

static gboolean
_mail_source_list_tree_select_func_gtk_tree_selection_func (GtkTreeSelection* selection,
                                                            GtkTreeModel* model,
                                                            GtkTreePath* path,
                                                            gboolean path_currently_selected,
                                                            gpointer self)
{
	gboolean result;
	result = mail_source_list_tree_select_func ((MailSourceListTree*) self, selection, model, path, path_currently_selected);
#line 1600 "../src/SourceList/SourceList.vala"
	return result;
#line 12215 "SourceList.c"
}

static gboolean
_mail_source_list_tree_on_query_tooltip_gtk_widget_query_tooltip (GtkWidget* _sender,
                                                                  gint x,
                                                                  gint y,
                                                                  gboolean keyboard_tooltip,
                                                                  GtkTooltip* tooltip,
                                                                  gpointer self)
{
	gboolean result;
	result = mail_source_list_tree_on_query_tooltip ((MailSourceListTree*) self, x, y, keyboard_tooltip, tooltip);
#line 1612 "../src/SourceList/SourceList.vala"
	return result;
#line 12230 "SourceList.c"
}

static GObject *
mail_source_list_tree_constructor (GType type,
                                   guint n_construct_properties,
                                   GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	MailSourceListTree * self;
	GtkStyleContext* style_context = NULL;
	GtkStyleContext* _tmp0_;
	GtkStyleContext* _tmp1_;
	GtkStyleContext* _tmp2_;
	GtkCssProvider* css_provider = NULL;
	GtkCssProvider* _tmp3_;
	MailSourceListDataModel* _tmp9_;
	GtkTreeViewColumn* item_column = NULL;
	GtkTreeViewColumn* _tmp10_;
	GtkTreeViewColumn* _tmp11_;
	GtkTreeViewColumn* _tmp12_;
	MailCellRendererExpander* _tmp13_;
	MailCellRendererExpander* _tmp14_;
	MailCellRendererExpander* _tmp15_;
	GtkTreeViewColumn* _tmp16_;
	MailCellRendererExpander* _tmp17_;
	GtkTreeViewColumn* _tmp18_;
	MailCellRendererExpander* _tmp19_;
	MailSourceListCellRendererIcon* _tmp20_;
	MailSourceListCellRendererIcon* _tmp21_;
	MailSourceListCellRendererIcon* _tmp22_;
	GtkTreeViewColumn* _tmp23_;
	MailSourceListCellRendererIcon* _tmp24_;
	GtkTreeViewColumn* _tmp25_;
	MailSourceListCellRendererIcon* _tmp26_;
	MailCellRendererBadge* _tmp27_;
	MailCellRendererBadge* _tmp28_;
	MailCellRendererBadge* _tmp29_;
	GtkTreeViewColumn* _tmp30_;
	MailCellRendererBadge* _tmp31_;
	GtkTreeViewColumn* _tmp32_;
	MailCellRendererBadge* _tmp33_;
	GtkCellRendererText* _tmp34_;
	GtkCellRendererText* _tmp35_;
	GtkCellRendererText* _tmp36_;
	GtkCellRendererText* _tmp37_;
	GtkCellRendererText* _tmp38_;
	GtkCellRendererText* _tmp39_;
	GtkCellRendererText* _tmp40_;
	GtkTreeViewColumn* _tmp41_;
	GtkCellRendererText* _tmp42_;
	GtkTreeViewColumn* _tmp43_;
	GtkCellRendererText* _tmp44_;
	MailSourceListCellRendererIcon* _tmp45_;
	MailSourceListCellRendererIcon* _tmp46_;
	GtkTreeViewColumn* _tmp47_;
	MailSourceListCellRendererIcon* _tmp48_;
	GtkTreeViewColumn* _tmp49_;
	MailSourceListCellRendererIcon* _tmp50_;
	MailCellRendererExpander* _tmp51_;
	gint expander_spacing = 0;
	gint _tmp52_ = 0;
	MailCellRendererExpander* _tmp53_;
	GtkTreeViewColumn* _tmp54_;
	MailCellRendererExpander* _tmp55_;
	GtkTreeViewColumn* _tmp56_;
	MailCellRendererExpander* _tmp57_;
	GtkTreeSelection* selection = NULL;
	GtkTreeSelection* _tmp58_;
	GtkTreeSelection* _tmp59_;
	GtkTreeSelection* _tmp60_;
	GtkTreeSelection* _tmp61_;
	GError* _inner_error0_ = NULL;
#line 1517 "../src/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (mail_source_list_tree_parent_class);
#line 1517 "../src/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 1517 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_TREE, MailSourceListTree);
#line 1518 "../src/SourceList/SourceList.vala"
	_tmp0_ = gtk_widget_get_style_context ((GtkWidget*) self);
#line 1518 "../src/SourceList/SourceList.vala"
	style_context = _tmp0_;
#line 1519 "../src/SourceList/SourceList.vala"
	_tmp1_ = style_context;
#line 1519 "../src/SourceList/SourceList.vala"
	gtk_style_context_add_class (_tmp1_, GTK_STYLE_CLASS_SIDEBAR);
#line 1520 "../src/SourceList/SourceList.vala"
	_tmp2_ = style_context;
#line 1520 "../src/SourceList/SourceList.vala"
	gtk_style_context_add_class (_tmp2_, GRANITE_STYLE_CLASS_SOURCE_LIST);
#line 1522 "../src/SourceList/SourceList.vala"
	_tmp3_ = gtk_css_provider_new ();
#line 1522 "../src/SourceList/SourceList.vala"
	css_provider = _tmp3_;
#line 12326 "SourceList.c"
	{
		GtkCssProvider* _tmp4_;
		GtkStyleContext* _tmp5_;
		GtkCssProvider* _tmp6_;
#line 1524 "../src/SourceList/SourceList.vala"
		_tmp4_ = css_provider;
#line 1524 "../src/SourceList/SourceList.vala"
		gtk_css_provider_load_from_data (_tmp4_, MAIL_SOURCE_LIST_TREE_DEFAULT_STYLESHEET, (gssize) -1, &_inner_error0_);
#line 1524 "../src/SourceList/SourceList.vala"
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
#line 12337 "SourceList.c"
			goto __catch0_g_error;
		}
#line 1525 "../src/SourceList/SourceList.vala"
		_tmp5_ = style_context;
#line 1525 "../src/SourceList/SourceList.vala"
		_tmp6_ = css_provider;
#line 1525 "../src/SourceList/SourceList.vala"
		gtk_style_context_add_provider (_tmp5_, (GtkStyleProvider*) _tmp6_, (guint) GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
#line 12346 "SourceList.c"
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError* e = NULL;
		GError* _tmp7_;
		const gchar* _tmp8_;
#line 1523 "../src/SourceList/SourceList.vala"
		e = _inner_error0_;
#line 1523 "../src/SourceList/SourceList.vala"
		_inner_error0_ = NULL;
#line 1527 "../src/SourceList/SourceList.vala"
		_tmp7_ = e;
#line 1527 "../src/SourceList/SourceList.vala"
		_tmp8_ = _tmp7_->message;
#line 1527 "../src/SourceList/SourceList.vala"
		g_warning ("SourceList.vala:1527: Could not create CSS Provider: %s\n" \
"Stylesheet:\n" \
"%s", _tmp8_, MAIL_SOURCE_LIST_TREE_DEFAULT_STYLESHEET);
#line 1523 "../src/SourceList/SourceList.vala"
		_g_error_free0 (e);
#line 12366 "SourceList.c"
	}
	__finally0:
#line 1523 "../src/SourceList/SourceList.vala"
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
#line 1523 "../src/SourceList/SourceList.vala"
		_g_object_unref0 (css_provider);
#line 1523 "../src/SourceList/SourceList.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
#line 1523 "../src/SourceList/SourceList.vala"
		g_clear_error (&_inner_error0_);
#line 12377 "SourceList.c"
	}
#line 1530 "../src/SourceList/SourceList.vala"
	_tmp9_ = self->priv->_data_model;
#line 1530 "../src/SourceList/SourceList.vala"
	gtk_tree_view_set_model ((GtkTreeView*) self, (GtkTreeModel*) _tmp9_);
#line 1532 "../src/SourceList/SourceList.vala"
	gtk_widget_set_valign ((GtkWidget*) self, GTK_ALIGN_FILL);
#line 1532 "../src/SourceList/SourceList.vala"
	gtk_widget_set_halign ((GtkWidget*) self, GTK_ALIGN_FILL);
#line 1533 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkWidget*) self, "expand", TRUE, NULL);
#line 1535 "../src/SourceList/SourceList.vala"
	gtk_tree_view_set_enable_search ((GtkTreeView*) self, FALSE);
#line 1536 "../src/SourceList/SourceList.vala"
	gtk_tree_view_set_headers_visible ((GtkTreeView*) self, FALSE);
#line 1537 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkTreeView*) self, "enable-grid-lines", GTK_TREE_VIEW_GRID_LINES_NONE, NULL);
#line 1540 "../src/SourceList/SourceList.vala"
	gtk_tree_view_set_expander_column ((GtkTreeView*) self, NULL);
#line 1541 "../src/SourceList/SourceList.vala"
	gtk_tree_view_set_show_expanders ((GtkTreeView*) self, FALSE);
#line 1543 "../src/SourceList/SourceList.vala"
	_tmp10_ = gtk_tree_view_column_new ();
#line 1543 "../src/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp10_);
#line 1543 "../src/SourceList/SourceList.vala"
	item_column = _tmp10_;
#line 1544 "../src/SourceList/SourceList.vala"
	_tmp11_ = item_column;
#line 1544 "../src/SourceList/SourceList.vala"
	gtk_tree_view_column_set_expand (_tmp11_, TRUE);
#line 1546 "../src/SourceList/SourceList.vala"
	_tmp12_ = item_column;
#line 1546 "../src/SourceList/SourceList.vala"
	gtk_tree_view_insert_column ((GtkTreeView*) self, _tmp12_, (gint) MAIL_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 1554 "../src/SourceList/SourceList.vala"
	_tmp13_ = mail_cell_renderer_expander_new ();
#line 1554 "../src/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp13_);
#line 1554 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->secondary_expander_cell);
#line 1554 "../src/SourceList/SourceList.vala"
	self->priv->secondary_expander_cell = _tmp13_;
#line 1555 "../src/SourceList/SourceList.vala"
	_tmp14_ = self->priv->secondary_expander_cell;
#line 1555 "../src/SourceList/SourceList.vala"
	mail_cell_renderer_expander_set_is_category_expander (_tmp14_, TRUE);
#line 1556 "../src/SourceList/SourceList.vala"
	_tmp15_ = self->priv->secondary_expander_cell;
#line 1556 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp15_, "xpad", (guint) 10, NULL);
#line 1557 "../src/SourceList/SourceList.vala"
	_tmp16_ = item_column;
#line 1557 "../src/SourceList/SourceList.vala"
	_tmp17_ = self->priv->secondary_expander_cell;
#line 1557 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp16_, (GtkCellRenderer*) _tmp17_, FALSE);
#line 1558 "../src/SourceList/SourceList.vala"
	_tmp18_ = item_column;
#line 1558 "../src/SourceList/SourceList.vala"
	_tmp19_ = self->priv->secondary_expander_cell;
#line 1558 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp18_, (GtkCellRenderer*) _tmp19_, _mail_source_list_tree_expander_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1560 "../src/SourceList/SourceList.vala"
	_tmp20_ = mail_source_list_cell_renderer_icon_new ();
#line 1560 "../src/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp20_);
#line 1560 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->activatable_cell);
#line 1560 "../src/SourceList/SourceList.vala"
	self->priv->activatable_cell = _tmp20_;
#line 1561 "../src/SourceList/SourceList.vala"
	_tmp21_ = self->priv->activatable_cell;
#line 1561 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp21_, "xpad", (guint) 6, NULL);
#line 1562 "../src/SourceList/SourceList.vala"
	_tmp22_ = self->priv->activatable_cell;
#line 1562 "../src/SourceList/SourceList.vala"
	g_signal_connect_object (_tmp22_, "activated", (GCallback) _mail_source_list_tree_on_activatable_activated_mail_source_list_cell_renderer_icon_activated, self, 0);
#line 1563 "../src/SourceList/SourceList.vala"
	_tmp23_ = item_column;
#line 1563 "../src/SourceList/SourceList.vala"
	_tmp24_ = self->priv->activatable_cell;
#line 1563 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp23_, (GtkCellRenderer*) _tmp24_, FALSE);
#line 1564 "../src/SourceList/SourceList.vala"
	_tmp25_ = item_column;
#line 1564 "../src/SourceList/SourceList.vala"
	_tmp26_ = self->priv->activatable_cell;
#line 1564 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp25_, (GtkCellRenderer*) _tmp26_, _mail_source_list_tree_icon_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1566 "../src/SourceList/SourceList.vala"
	_tmp27_ = mail_cell_renderer_badge_new ();
#line 1566 "../src/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp27_);
#line 1566 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->badge_cell);
#line 1566 "../src/SourceList/SourceList.vala"
	self->priv->badge_cell = _tmp27_;
#line 1567 "../src/SourceList/SourceList.vala"
	_tmp28_ = self->priv->badge_cell;
#line 1567 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp28_, "xpad", (guint) 1, NULL);
#line 1568 "../src/SourceList/SourceList.vala"
	_tmp29_ = self->priv->badge_cell;
#line 1568 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp29_, "xalign", (gfloat) 1, NULL);
#line 1569 "../src/SourceList/SourceList.vala"
	_tmp30_ = item_column;
#line 1569 "../src/SourceList/SourceList.vala"
	_tmp31_ = self->priv->badge_cell;
#line 1569 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp30_, (GtkCellRenderer*) _tmp31_, FALSE);
#line 1570 "../src/SourceList/SourceList.vala"
	_tmp32_ = item_column;
#line 1570 "../src/SourceList/SourceList.vala"
	_tmp33_ = self->priv->badge_cell;
#line 1570 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp32_, (GtkCellRenderer*) _tmp33_, _mail_source_list_tree_badge_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1572 "../src/SourceList/SourceList.vala"
	_tmp34_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
#line 1572 "../src/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp34_);
#line 1572 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->text_cell);
#line 1572 "../src/SourceList/SourceList.vala"
	self->priv->text_cell = _tmp34_;
#line 1573 "../src/SourceList/SourceList.vala"
	_tmp35_ = self->priv->text_cell;
#line 1573 "../src/SourceList/SourceList.vala"
	g_object_set (_tmp35_, "editable-set", TRUE, NULL);
#line 1574 "../src/SourceList/SourceList.vala"
	_tmp36_ = self->priv->text_cell;
#line 1574 "../src/SourceList/SourceList.vala"
	g_object_set (_tmp36_, "editable", FALSE, NULL);
#line 1575 "../src/SourceList/SourceList.vala"
	_tmp37_ = self->priv->text_cell;
#line 1575 "../src/SourceList/SourceList.vala"
	g_signal_connect_object ((GtkCellRenderer*) _tmp37_, "editing-started", (GCallback) _mail_source_list_tree_on_editing_started_gtk_cell_renderer_editing_started, self, 0);
#line 1576 "../src/SourceList/SourceList.vala"
	_tmp38_ = self->priv->text_cell;
#line 1576 "../src/SourceList/SourceList.vala"
	g_signal_connect_object ((GtkCellRenderer*) _tmp38_, "editing-canceled", (GCallback) _mail_source_list_tree_on_editing_canceled_gtk_cell_renderer_editing_canceled, self, 0);
#line 1577 "../src/SourceList/SourceList.vala"
	_tmp39_ = self->priv->text_cell;
#line 1577 "../src/SourceList/SourceList.vala"
	g_object_set (_tmp39_, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
#line 1578 "../src/SourceList/SourceList.vala"
	_tmp40_ = self->priv->text_cell;
#line 1578 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp40_, "xalign", (gfloat) 0, NULL);
#line 1579 "../src/SourceList/SourceList.vala"
	_tmp41_ = item_column;
#line 1579 "../src/SourceList/SourceList.vala"
	_tmp42_ = self->priv->text_cell;
#line 1579 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp41_, (GtkCellRenderer*) _tmp42_, TRUE);
#line 1580 "../src/SourceList/SourceList.vala"
	_tmp43_ = item_column;
#line 1580 "../src/SourceList/SourceList.vala"
	_tmp44_ = self->priv->text_cell;
#line 1580 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp43_, (GtkCellRenderer*) _tmp44_, _mail_source_list_tree_name_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1582 "../src/SourceList/SourceList.vala"
	_tmp45_ = mail_source_list_cell_renderer_icon_new ();
#line 1582 "../src/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp45_);
#line 1582 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->icon_cell);
#line 1582 "../src/SourceList/SourceList.vala"
	self->priv->icon_cell = _tmp45_;
#line 1583 "../src/SourceList/SourceList.vala"
	_tmp46_ = self->priv->icon_cell;
#line 1583 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp46_, "xpad", (guint) 2, NULL);
#line 1584 "../src/SourceList/SourceList.vala"
	_tmp47_ = item_column;
#line 1584 "../src/SourceList/SourceList.vala"
	_tmp48_ = self->priv->icon_cell;
#line 1584 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp47_, (GtkCellRenderer*) _tmp48_, FALSE);
#line 1585 "../src/SourceList/SourceList.vala"
	_tmp49_ = item_column;
#line 1585 "../src/SourceList/SourceList.vala"
	_tmp50_ = self->priv->icon_cell;
#line 1585 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp49_, (GtkCellRenderer*) _tmp50_, _mail_source_list_tree_icon_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1588 "../src/SourceList/SourceList.vala"
	_tmp51_ = mail_cell_renderer_expander_new ();
#line 1588 "../src/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp51_);
#line 1588 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->primary_expander_cell);
#line 1588 "../src/SourceList/SourceList.vala"
	self->priv->primary_expander_cell = _tmp51_;
#line 1591 "../src/SourceList/SourceList.vala"
	gtk_widget_style_get ((GtkWidget*) self, MAIL_SOURCE_LIST_TREE_STYLE_PROP_EXPANDER_SPACING, &_tmp52_, NULL);
#line 1591 "../src/SourceList/SourceList.vala"
	expander_spacing = _tmp52_;
#line 1592 "../src/SourceList/SourceList.vala"
	_tmp53_ = self->priv->primary_expander_cell;
#line 1592 "../src/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp53_, "xpad", (guint) (expander_spacing / 2), NULL);
#line 1594 "../src/SourceList/SourceList.vala"
	_tmp54_ = item_column;
#line 1594 "../src/SourceList/SourceList.vala"
	_tmp55_ = self->priv->primary_expander_cell;
#line 1594 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp54_, (GtkCellRenderer*) _tmp55_, FALSE);
#line 1595 "../src/SourceList/SourceList.vala"
	_tmp56_ = item_column;
#line 1595 "../src/SourceList/SourceList.vala"
	_tmp57_ = self->priv->primary_expander_cell;
#line 1595 "../src/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp56_, (GtkCellRenderer*) _tmp57_, _mail_source_list_tree_expander_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1598 "../src/SourceList/SourceList.vala"
	_tmp58_ = gtk_tree_view_get_selection ((GtkTreeView*) self);
#line 1598 "../src/SourceList/SourceList.vala"
	_tmp59_ = _g_object_ref0 (_tmp58_);
#line 1598 "../src/SourceList/SourceList.vala"
	selection = _tmp59_;
#line 1599 "../src/SourceList/SourceList.vala"
	_tmp60_ = selection;
#line 1599 "../src/SourceList/SourceList.vala"
	gtk_tree_selection_set_mode (_tmp60_, GTK_SELECTION_BROWSE);
#line 1600 "../src/SourceList/SourceList.vala"
	_tmp61_ = selection;
#line 1600 "../src/SourceList/SourceList.vala"
	gtk_tree_selection_set_select_function (_tmp61_, _mail_source_list_tree_select_func_gtk_tree_selection_func, g_object_ref (self), g_object_unref);
#line 1603 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_enable_item_property_monitor (self);
#line 1606 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_add_spacer_cell_for_level (self, 1, TRUE);
#line 1609 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_configure_drag_source (self, NULL, (gint) 0);
#line 1610 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_configure_drag_dest (self, NULL, (gint) 0, 0);
#line 1612 "../src/SourceList/SourceList.vala"
	g_signal_connect_object ((GtkWidget*) self, "query-tooltip", (GCallback) _mail_source_list_tree_on_query_tooltip_gtk_widget_query_tooltip, self, G_CONNECT_AFTER);
#line 1613 "../src/SourceList/SourceList.vala"
	gtk_widget_set_has_tooltip ((GtkWidget*) self, TRUE);
#line 1517 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (selection);
#line 1517 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (item_column);
#line 1517 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (css_provider);
#line 1517 "../src/SourceList/SourceList.vala"
	return obj;
#line 12627 "SourceList.c"
}

static void
mail_source_list_tree_class_init (MailSourceListTreeClass * klass,
                                  gpointer klass_data)
{
	GParamSpecInt* _tmp0_;
	GParamSpecInt* _tmp1_;
	GParamSpecInt* _tmp2_;
	GParamSpecInt* _tmp3_;
	GParamSpecInt* _tmp4_;
	GParamSpecInt* _tmp5_;
#line 1431 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_parent_class = g_type_class_peek_parent (klass);
#line 1431 "../src/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &MailSourceListTree_private_offset);
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->drag_motion = (gboolean (*) (GtkWidget*, GdkDragContext*, gint, gint, guint)) mail_source_list_tree_real_drag_motion;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->drag_data_received = (void (*) (GtkWidget*, GdkDragContext*, gint, gint, GtkSelectionData*, guint, guint)) mail_source_list_tree_real_drag_data_received;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkTreeViewClass *) klass)->cursor_changed = (void (*) (GtkTreeView*)) mail_source_list_tree_real_cursor_changed;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkTreeViewClass *) klass)->row_expanded = (void (*) (GtkTreeView*, GtkTreeIter*, GtkTreePath*)) mail_source_list_tree_real_row_expanded;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkTreeViewClass *) klass)->row_collapsed = (void (*) (GtkTreeView*, GtkTreeIter*, GtkTreePath*)) mail_source_list_tree_real_row_collapsed;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkTreeViewClass *) klass)->row_activated = (void (*) (GtkTreeView*, GtkTreePath*, GtkTreeViewColumn*)) mail_source_list_tree_real_row_activated;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->key_release_event = (gboolean (*) (GtkWidget*, GdkEventKey*)) mail_source_list_tree_real_key_release_event;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->button_release_event = (gboolean (*) (GtkWidget*, GdkEventButton*)) mail_source_list_tree_real_button_release_event;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->button_press_event = (gboolean (*) (GtkWidget*, GdkEventButton*)) mail_source_list_tree_real_button_press_event;
#line 1431 "../src/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->popup_menu = (gboolean (*) (GtkWidget*)) mail_source_list_tree_real_popup_menu;
#line 1431 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_mail_source_list_tree_get_property;
#line 1431 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_mail_source_list_tree_set_property;
#line 1431 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = mail_source_list_tree_constructor;
#line 1431 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = mail_source_list_tree_finalize;
#line 1431 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY, mail_source_list_tree_properties[MAIL_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY] = g_param_spec_object ("data-model", "data-model", "data-model", MAIL_SOURCE_LIST_TYPE_DATA_MODEL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
#line 1431 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY, mail_source_list_tree_properties[MAIL_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY] = g_param_spec_object ("selected-item", "selected-item", "selected-item", MAIL_SOURCE_LIST_TYPE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1431 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_TREE_EDITING_PROPERTY, mail_source_list_tree_properties[MAIL_SOURCE_LIST_TREE_EDITING_PROPERTY] = g_param_spec_boolean ("editing", "editing", "editing", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
#line 1431 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY, mail_source_list_tree_properties[MAIL_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY] = g_param_spec_enum ("ellipsize-mode", "ellipsize-mode", "ellipsize-mode", pango_ellipsize_mode_get_type (), 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1431 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_signals[MAIL_SOURCE_LIST_TREE_ITEM_SELECTED_SIGNAL] = g_signal_new ("item-selected", MAIL_SOURCE_LIST_TYPE_TREE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, MAIL_SOURCE_LIST_TYPE_ITEM);
#line 1482 "../src/SourceList/SourceList.vala"
	_tmp0_ = (GParamSpecInt*) g_param_spec_int (MAIL_SOURCE_LIST_TREE_STYLE_PROP_LEVEL_INDENTATION, "Level Indentation", "Space to add at the beginning of every indentation level. Must be an e" \
"ven number.", 1, 50, 6, G_PARAM_READABLE);
#line 1482 "../src/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 1482 "../src/SourceList/SourceList.vala"
	gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (klass), (GParamSpec*) _tmp1_);
#line 1482 "../src/SourceList/SourceList.vala"
	_g_param_spec_unref0 (_tmp1_);
#line 1492 "../src/SourceList/SourceList.vala"
	_tmp2_ = (GParamSpecInt*) g_param_spec_int (MAIL_SOURCE_LIST_TREE_STYLE_PROP_LEFT_PADDING, "Left Padding", "Padding added to the left side of the tree. Must be an even number.", 1, 50, 4, G_PARAM_READABLE);
#line 1492 "../src/SourceList/SourceList.vala"
	_tmp3_ = _tmp2_;
#line 1492 "../src/SourceList/SourceList.vala"
	gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (klass), (GParamSpec*) _tmp3_);
#line 1492 "../src/SourceList/SourceList.vala"
	_g_param_spec_unref0 (_tmp3_);
#line 1502 "../src/SourceList/SourceList.vala"
	_tmp4_ = (GParamSpecInt*) g_param_spec_int (MAIL_SOURCE_LIST_TREE_STYLE_PROP_EXPANDER_SPACING, "Expander Spacing", "Space added between an item and its expander. Must be an even number.", 1, 50, 4, G_PARAM_READABLE);
#line 1502 "../src/SourceList/SourceList.vala"
	_tmp5_ = _tmp4_;
#line 1502 "../src/SourceList/SourceList.vala"
	gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (klass), (GParamSpec*) _tmp5_);
#line 1502 "../src/SourceList/SourceList.vala"
	_g_param_spec_unref0 (_tmp5_);
#line 12706 "SourceList.c"
}

static void
mail_source_list_tree_instance_init (MailSourceListTree * self,
                                     gpointer klass)
{
#line 1431 "../src/SourceList/SourceList.vala"
	self->priv = mail_source_list_tree_get_instance_private (self);
#line 1466 "../src/SourceList/SourceList.vala"
	self->priv->unselectable_item_clicked = FALSE;
#line 12717 "SourceList.c"
}

static void
mail_source_list_tree_finalize (GObject * obj)
{
	MailSourceListTree * self;
#line 1431 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_SOURCE_LIST_TYPE_TREE, MailSourceListTree);
#line 1617 "../src/SourceList/SourceList.vala"
	mail_source_list_tree_disable_item_property_monitor (self);
#line 1432 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_data_model);
#line 1455 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->selected);
#line 1458 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->editable_entry);
#line 1459 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->text_cell);
#line 1460 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->icon_cell);
#line 1461 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->activatable_cell);
#line 1462 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->badge_cell);
#line 1463 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->primary_expander_cell);
#line 1464 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->secondary_expander_cell);
#line 1465 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->spacer_cells);
#line 1431 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (mail_source_list_tree_parent_class)->finalize (obj);
#line 12750 "SourceList.c"
}

/**
     * The tree that actually displays the items.
     *
     * All the user interaction happens here.
     */
static GType
mail_source_list_tree_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListTreeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_tree_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceListTree), 0, (GInstanceInitFunc) mail_source_list_tree_instance_init, NULL };
	GType mail_source_list_tree_type_id;
	mail_source_list_tree_type_id = g_type_register_static (gtk_tree_view_get_type (), "MailSourceListTree", &g_define_type_info, 0);
	MailSourceListTree_private_offset = g_type_add_instance_private (mail_source_list_tree_type_id, sizeof (MailSourceListTreePrivate));
	return mail_source_list_tree_type_id;
}

static GType
mail_source_list_tree_get_type (void)
{
	static volatile gsize mail_source_list_tree_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_tree_type_id__once)) {
		GType mail_source_list_tree_type_id;
		mail_source_list_tree_type_id = mail_source_list_tree_get_type_once ();
		g_once_init_leave (&mail_source_list_tree_type_id__once, mail_source_list_tree_type_id);
	}
	return mail_source_list_tree_type_id__once;
}

static void
_vala_mail_source_list_tree_get_property (GObject * object,
                                          guint property_id,
                                          GValue * value,
                                          GParamSpec * pspec)
{
	MailSourceListTree * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_TREE, MailSourceListTree);
#line 1431 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 1431 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY:
#line 1431 "../src/SourceList/SourceList.vala"
		g_value_set_object (value, mail_source_list_tree_get_data_model (self));
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 1431 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY:
#line 1431 "../src/SourceList/SourceList.vala"
		g_value_set_object (value, mail_source_list_tree_get_selected_item (self));
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 1431 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_TREE_EDITING_PROPERTY:
#line 1431 "../src/SourceList/SourceList.vala"
		g_value_set_boolean (value, mail_source_list_tree_get_editing (self));
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 1431 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY:
#line 1431 "../src/SourceList/SourceList.vala"
		g_value_set_enum (value, mail_source_list_tree_get_ellipsize_mode (self));
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 12814 "SourceList.c"
		default:
#line 1431 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 12820 "SourceList.c"
	}
}

static void
_vala_mail_source_list_tree_set_property (GObject * object,
                                          guint property_id,
                                          const GValue * value,
                                          GParamSpec * pspec)
{
	MailSourceListTree * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_SOURCE_LIST_TYPE_TREE, MailSourceListTree);
#line 1431 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 1431 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY:
#line 1431 "../src/SourceList/SourceList.vala"
		mail_source_list_tree_set_data_model (self, g_value_get_object (value));
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 1431 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY:
#line 1431 "../src/SourceList/SourceList.vala"
		mail_source_list_tree_set_selected_item (self, g_value_get_object (value));
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 1431 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY:
#line 1431 "../src/SourceList/SourceList.vala"
		mail_source_list_tree_set_ellipsize_mode (self, g_value_get_enum (value));
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 12852 "SourceList.c"
		default:
#line 1431 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 1431 "../src/SourceList/SourceList.vala"
		break;
#line 12858 "SourceList.c"
	}
}

static void
_mail_source_list___lambda5_ (MailSourceList* self,
                              MailSourceListItem* item)
{
#line 2559 "../src/SourceList/SourceList.vala"
	g_signal_emit (self, mail_source_list_signals[MAIL_SOURCE_LIST_ITEM_SELECTED_SIGNAL], 0, item);
#line 12868 "SourceList.c"
}

static void
__mail_source_list___lambda5__mail_source_list_tree_item_selected (MailSourceListTree* _sender,
                                                                   MailSourceListItem* item,
                                                                   gpointer self)
{
#line 2559 "../src/SourceList/SourceList.vala"
	_mail_source_list___lambda5_ ((MailSourceList*) self, item);
#line 12878 "SourceList.c"
}

static GObject *
mail_source_list_constructor (GType type,
                              guint n_construct_properties,
                              GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	MailSourceList * self;
	MailSourceListDataModel* _tmp0_;
	MailSourceListTree* _tmp1_;
	MailSourceListTree* _tmp2_;
	MailSourceListTree* _tmp3_;
#line 2552 "../src/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (mail_source_list_parent_class);
#line 2552 "../src/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 2552 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_TYPE_SOURCE_LIST, MailSourceList);
#line 2553 "../src/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2553 "../src/SourceList/SourceList.vala"
	_tmp1_ = mail_source_list_tree_new (_tmp0_);
#line 2553 "../src/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp1_);
#line 2553 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->tree);
#line 2553 "../src/SourceList/SourceList.vala"
	self->priv->tree = _tmp1_;
#line 2555 "../src/SourceList/SourceList.vala"
	gtk_scrolled_window_set_policy ((GtkScrolledWindow*) self, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
#line 2556 "../src/SourceList/SourceList.vala"
	_tmp2_ = self->priv->tree;
#line 2556 "../src/SourceList/SourceList.vala"
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) _tmp2_);
#line 2557 "../src/SourceList/SourceList.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 2559 "../src/SourceList/SourceList.vala"
	_tmp3_ = self->priv->tree;
#line 2559 "../src/SourceList/SourceList.vala"
	g_signal_connect_object (_tmp3_, "item-selected", (GCallback) __mail_source_list___lambda5__mail_source_list_tree_item_selected, self, 0);
#line 2552 "../src/SourceList/SourceList.vala"
	return obj;
#line 12923 "SourceList.c"
}

static void
mail_source_list_class_init (MailSourceListClass * klass,
                             gpointer klass_data)
{
#line 218 "../src/SourceList/SourceList.vala"
	mail_source_list_parent_class = g_type_class_peek_parent (klass);
#line 218 "../src/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &MailSourceList_private_offset);
#line 218 "../src/SourceList/SourceList.vala"
	((MailSourceListClass *) klass)->item_selected = mail_source_list_real_item_selected;
#line 218 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_mail_source_list_get_property;
#line 218 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_mail_source_list_set_property;
#line 218 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = mail_source_list_constructor;
#line 218 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = mail_source_list_finalize;
#line 12944 "SourceList.c"
	/**
	     * Root-level expandable item.
	     *
	     * This item contains the first-level source list items. It //only serves as an item container//.
	     * It is used to add and remove items to/from the widget.
	     *
	     * Internally, it allows the source list to connect to its {@link Granite.Widgets.SourceList.ExpandableItem.child_added}
	     * and {@link Granite.Widgets.SourceList.ExpandableItem.child_removed} signals in order to monitor
	     * new children additions/removals.
	     *
	     * @since 0.2
	     */
#line 218 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_ROOT_PROPERTY, mail_source_list_properties[MAIL_SOURCE_LIST_ROOT_PROPERTY] = g_param_spec_object ("root", "root", "root", MAIL_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 12959 "SourceList.c"
	/**
	     * The current selected item.
	     *
	     * Setting it to //null// un-selects the previously selected item, if there was any.
	     * {@link Granite.Widgets.SourceList.ExpandableItem.expand_with_parents} is called on the
	     * item's parent to make sure it's possible to select it.
	     *
	     * @since 0.2
	     */
#line 218 "../src/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAIL_SOURCE_LIST_SELECTED_PROPERTY, mail_source_list_properties[MAIL_SOURCE_LIST_SELECTED_PROPERTY] = g_param_spec_object ("selected", "selected", "selected", MAIL_SOURCE_LIST_TYPE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 12971 "SourceList.c"
	/**
	     * Emitted when the source list selection changes.
	     *
	     * @param item Selected item; //null// if nothing is selected.
	     * @since 0.2
	     */
#line 218 "../src/SourceList/SourceList.vala"
	mail_source_list_signals[MAIL_SOURCE_LIST_ITEM_SELECTED_SIGNAL] = g_signal_new ("item-selected", MAIL_TYPE_SOURCE_LIST, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (MailSourceListClass, item_selected), NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, MAIL_SOURCE_LIST_TYPE_ITEM);
#line 12980 "SourceList.c"
}

static void
mail_source_list_instance_init (MailSourceList * self,
                                gpointer klass)
{
	MailSourceListDataModel* _tmp0_;
#line 218 "../src/SourceList/SourceList.vala"
	self->priv = mail_source_list_get_instance_private (self);
#line 2540 "../src/SourceList/SourceList.vala"
	_tmp0_ = mail_source_list_data_model_new ();
#line 2540 "../src/SourceList/SourceList.vala"
	self->priv->data_model = _tmp0_;
#line 12994 "SourceList.c"
}

static void
mail_source_list_finalize (GObject * obj)
{
	MailSourceList * self;
#line 218 "../src/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MAIL_TYPE_SOURCE_LIST, MailSourceList);
#line 2539 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->tree);
#line 2540 "../src/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->data_model);
#line 218 "../src/SourceList/SourceList.vala"
	G_OBJECT_CLASS (mail_source_list_parent_class)->finalize (obj);
#line 13009 "SourceList.c"
}

/**
 * A widget that can display a list of items organized in categories.
 *
 * The source list widget consists of a collection of items, some of which are also expandable (and
 * thus can contain more items). All the items displayed in the source list are children of the widget's
 * root item. The API is meant to be used as follows:
 *
 * 1. Create the items you want to display in the source list, setting the appropriate values for their
 * properties. The desired hierarchy is achieved by creating expandable items and adding items to them.
 * These will be displayed as descendants in the widget's tree structure. The expandable items that are
 * not nested inside any other item are considered to be at root level, and should be added to
 * the widget's root item.<<BR>>
 *
 * Expandable items located at the root level are treated as categories, and only support text.
 *
 * ''Example''<<BR>>
 * The final tree will have the following structure:
 * {{{
 * Libraries
 *   Music
 * Stores
 *   My Store
 *      Music
 *      Podcasts
 * Devices
 *   Player 1
 *   Player 2
 * }}}
 *
 * {{{
 * var library_category = new Granite.Widgets.SourceList.ExpandableItem ("Libraries");
 * var store_category = new Granite.Widgets.SourceList.ExpandableItem ("Stores");
 * var device_category = new Granite.Widgets.SourceList.ExpandableItem ("Devices");
 *
 * var music_item = new Granite.Widgets.SourceList.Item ("Music");
 *
 * // "Libraries" will be the parent category of "Music"
 * library_category.add (music_item);
 *
 * // We plan to add sub-items to the store, so let's use an expandable item
 * var my_store_item = new Granite.Widgets.SourceList.ExpandableItem ("My Store");
 * store_category.add (my_store_item);
 *
 * var my_store_podcast_item = new Granite.Widgets.SourceList.Item ("Podcasts");
 * var my_store_music_item = new Granite.Widgets.SourceList.Item ("Music");
 *
 * my_store_item.add (my_store_music_item);
 * my_store_item.add (my_store_podcast_item);
 *
 * var player1_item = new Granite.Widgets.SourceList.Item ("Player 1");
 * var player2_item = new Granite.Widgets.SourceList.Item ("Player 2");
 *
 * device_category.add (player1_item);
 * device_category.add (player2_item);
 * }}}
 *
 * 2. Create a source list widget.<<BR>>
 * {{{
 * var source_list = new Granite.Widgets.SourceList ();
 * }}}
 *
 * 3. Add root-level items to the {@link Granite.Widgets.SourceList.root} item.
 * This item only serves as a container, and all its properties are ignored by the widget.
 *
 * {{{
 * // This will add the main categories (including their children) to the source list. After
 * // having being added to be widget, any other item added to any of these items
 * // (or any other child item in a deeper level) will be automatically added too.
 * // There's no need to deal with the source list widget directly.
 *
 * var root = source_list.root;
 *
 * root.add (library_category);
 * root.add (store_category);
 * root.add (device_category);
 * }}}
 *
 * The steps mentioned above are enough for initializing the source list. Future changes to the items'
 * properties are ''automatically'' reflected by the widget.
 *
 * Final steps would involve connecting handlers to the source list events, being
 * {@link Granite.Widgets.SourceList.item_selected} the most important, as it indicates that
 * the selection was modified.
 *
 * Pack the source list into the GUI using the {@link Gtk.Paned} widget.
 * This is usually done as follows:
 * {{{
 * var pane = new Gtk.Paned (Gtk.Orientation.HORIZONTAL);
 * pane.pack1 (source_list, false, false);
 * pane.pack2 (content_area, true, false);
 * }}}
 *
 * @since 0.2
 * @see Gtk.Paned
 */
static GType
mail_source_list_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (MailSourceListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) mail_source_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MailSourceList), 0, (GInstanceInitFunc) mail_source_list_instance_init, NULL };
	GType mail_source_list_type_id;
	mail_source_list_type_id = g_type_register_static (gtk_scrolled_window_get_type (), "MailSourceList", &g_define_type_info, 0);
	MailSourceList_private_offset = g_type_add_instance_private (mail_source_list_type_id, sizeof (MailSourceListPrivate));
	return mail_source_list_type_id;
}

GType
mail_source_list_get_type (void)
{
	static volatile gsize mail_source_list_type_id__once = 0;
	if (g_once_init_enter (&mail_source_list_type_id__once)) {
		GType mail_source_list_type_id;
		mail_source_list_type_id = mail_source_list_get_type_once ();
		g_once_init_leave (&mail_source_list_type_id__once, mail_source_list_type_id);
	}
	return mail_source_list_type_id__once;
}

static void
_vala_mail_source_list_get_property (GObject * object,
                                     guint property_id,
                                     GValue * value,
                                     GParamSpec * pspec)
{
	MailSourceList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_TYPE_SOURCE_LIST, MailSourceList);
#line 218 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 218 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ROOT_PROPERTY:
#line 218 "../src/SourceList/SourceList.vala"
		g_value_set_object (value, mail_source_list_get_root (self));
#line 218 "../src/SourceList/SourceList.vala"
		break;
#line 218 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_SELECTED_PROPERTY:
#line 218 "../src/SourceList/SourceList.vala"
		g_value_set_object (value, mail_source_list_get_selected (self));
#line 218 "../src/SourceList/SourceList.vala"
		break;
#line 13151 "SourceList.c"
		default:
#line 218 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 218 "../src/SourceList/SourceList.vala"
		break;
#line 13157 "SourceList.c"
	}
}

static void
_vala_mail_source_list_set_property (GObject * object,
                                     guint property_id,
                                     const GValue * value,
                                     GParamSpec * pspec)
{
	MailSourceList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MAIL_TYPE_SOURCE_LIST, MailSourceList);
#line 218 "../src/SourceList/SourceList.vala"
	switch (property_id) {
#line 218 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_ROOT_PROPERTY:
#line 218 "../src/SourceList/SourceList.vala"
		mail_source_list_set_root (self, g_value_get_object (value));
#line 218 "../src/SourceList/SourceList.vala"
		break;
#line 218 "../src/SourceList/SourceList.vala"
		case MAIL_SOURCE_LIST_SELECTED_PROPERTY:
#line 218 "../src/SourceList/SourceList.vala"
		mail_source_list_set_selected (self, g_value_get_object (value));
#line 218 "../src/SourceList/SourceList.vala"
		break;
#line 13183 "SourceList.c"
		default:
#line 218 "../src/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 218 "../src/SourceList/SourceList.vala"
		break;
#line 13189 "SourceList.c"
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

